=== modified file '.bzrignore'
--- .bzrignore	2009-02-10 01:57:06 +0000
+++ .bzrignore	2011-03-10 14:22:12 +0000
@@ -1,8 +1,10 @@
 ./dist
 ./loggerhead.egg-info
-./loggerhead.conf
 ./loggerhead.pid
 ./logs
 build
 *.log
 _trial_temp
+loggerhead-memprofile
+./docs/_build/
+tags

=== added file 'HACKING'
--- HACKING	1970-01-01 00:00:00 +0000
+++ HACKING	2011-02-10 23:05:31 +0000
@@ -0,0 +1,59 @@
+Loggerhead
+==========
+
+Overview
+--------
+
+This document attempts to give some hints for people that are wanting to work
+on Loggerhead.
+
+
+Testing
+-------
+
+You can run the loggerhead test suite as a bzr plugin. To run just the
+loggerhead tests::
+
+  bzr selftest -s bp.loggerhead
+
+
+Load Testing
+------------
+
+As a web service, Loggerhead will often be hit by multiple requests. We want
+to make sure that loggerhead can scale with many requests, without performing
+poorly or crashing under the load.
+
+There is a command ``bzr load-test-loggerhead`` that can be run to stress
+loggerhead. A script is given, describing what requests to make, against what
+URLs, and for what level of parallel activity.
+
+
+Load Testing Multiple Instances
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+One way that Launchpad provides both high availability and performance scaling
+is by running multiple instances of loggerhead, serving the same content. A
+proxy is then used to load balance the requests. This also allows us to shut
+down one instance for upgrading, without interupting service (requests are
+just routed to the other instance).
+
+However, multiple processes poses an even greater risk that caches will
+conflict. As such, it is useful to test that changes don't introduce coherency
+issues at load. ``bzr load-test-loggerhead`` can be configured with a script
+that will make requests against multiple loggerhead instances concurrently.
+
+To run multiple instances, it is often sufficient to just spawn multiple
+servers on different ports. For example::
+
+  $ bzr serve --http --port=8080 &
+  $ bzr serve --http --port=8081 &
+
+There is a simple example script already in the source tree::
+
+  $ bzr load-test-loggerhead load_test_scripts/multiple_instances.script
+
+
+
+.. vim: ft=rst tw=78
+

=== modified file 'MANIFEST.in'
--- MANIFEST.in	2008-06-22 18:04:15 +0000
+++ MANIFEST.in	2011-03-10 14:22:12 +0000
@@ -1,2 +1,2 @@
-include loggerhead.conf.example README.txt NEWS
+include README.txt NEWS
 recursive-include loggerhead/static *

=== modified file 'Makefile'
--- Makefile	2007-10-29 16:19:30 +0000
+++ Makefile	2011-03-14 08:47:03 +0000
@@ -8,5 +8,4 @@
 	rm -rf dist/
 
 check:
-	# XXX MichaelHudson 2007-10-29 bug=158361: We can't run the loggerhead
-	# tests in PQM yet :(
+	BZR_PLUGINS_AT=loggerhead@$$(pwd) bzr selftest -s bp.loggerhead

=== modified file 'NEWS'
--- NEWS	2009-01-23 21:40:04 +0000
+++ NEWS	2011-03-24 23:02:29 +0000
@@ -1,18 +1,242 @@
 What's changed in loggerhead?
 =============================
 
-dev [future]
----------------
-    - Loggerhead now serves bzr branches over HTTP and exposes the URL to
-      branch them. (Jonathan Lange)
+1.19 [???]
+----------------
+
+    - Add ``bzr load-test-loggerhead`` as a way to make sure loggerhead can
+      handle concurrent requests, etc. Scripts can be written that spawn
+      multiple threads, and issue concurrent requests.
+      (John Arbash Meinel)
+
+    - HEAD requests should not return body content. This is done by adding
+      another wsgi middleware that strips the body when the REQUEST_METHOD is
+      HEAD. Note that you have to add the middleware into your pipeline, and
+      it does not decrease the actual work done.
+      (John Arbash Meinel, #716201)
+
+    - If we get a HEAD request, there is no reason to expand the template, we
+      shouldn't be returning body content anyway.
+      (John Arbash Meinel, #716201, #716217)
+
+    - Merge the pqm changes back into trunk, after trunk was reverted to an old
+      revision. (John Arbash Meinel, #716152)
+
+    - Redirect ``/files/file.txt`` to ``/view/file.txt`` and ``/view/dir`` to
+      ``/files/dir``. (Jasper St. Pierre, #569358)
+
+    - Remove ``start-loggerhead`` and ``stop-loggerhead`` which were already
+      deprecated. (John Arbash Meinel)
+
+    - Show "Author(s)" as separate from "Committer". And label them
+      correctly. (John Arbash Meinel, #733015)
+
+    - The json module is no longer claimed to be supported as alternative for 
+      simplejson. (Jelmer Vernooij, #586611)
+
+    - Viewing the ``/changes`` page only iterates enough of the history to show
+      the actual revisions displayed, rather than walking the whole mainline.
+      Improves performance on projects with long histories like emacs.
+      (John Arbash Meinel)
+
+    - Fix escaping of filenames in revision views.
+      (William Grant, #740142)
+
+
+1.18 [10Nov2010]
+----------------
+
+    - Syntax highlighting is no longer applied for files greater than 512K,
+      reducing codebrowse.launchpad.net overloading.
+      (Max Kanat-Alexander, #513044)
+
+    - Documentation added in the docs directory. README simplified
+      accordingly. (Tres Seaver).
+
+    - Show svn/git/hg revision ids in loggerhead revision view.
+      (Jelmer Vernooij)
+
+    - Fix .bzr/smart access to branches in shared repos. (You also need
+      a version of bzr with bug #348308 fixed.) (Andrew Bennetts)
+
+    - Support FastCGI, SCGI and AJP using flup. (Denis Martinez)
+
+    - Repository.get_revision_inventory() was removed in bzr 2.2; use
+      Repository.get_inventory() instead. (Matt Nordhoff, #528194)
+
+    - Ignore readonly+ prefix when checking if Loggerhead is serving a
+      local location. (Reported by Tres Seaver.) (Matt Nordhoff)
+
+    - Set Cache-Control and Expires headers on static pages.
+      (John Arbash Meinel)
+
+    - Generate relative links where possible (everywhere but HTTP
+      redirects and feed IDs). (Michael Hudson, Matt Nordhoff)
+
+    - Fix bad redirect when visiting "/download" or "/download/".
+      (Matt Nordhoff, #247992)
+
+
+1.17 [20Aug2009]
+----------------
+
+    - Add bug links in revision informations (Alexandre Garnier, #314052)
+
+    - Make sure that binary files aren't annotated. (Martin Albisetti,
+      #258848)
+
+    - Loggerhead now serves bzr branches over HTTP and exposes the URL
+      to branch them. Addresses bug #240577. (Jonathan Lange)
+
+    - Leading blank lines in commit messages no longer result in an
+      empty summary. (Colin Watson)
+
+    - Added optional syntax highlighting to annotate view using
+      python-pygments.  Partially addresses bug #306631. (Peter Bui)
+
+    - Convert newlines in commit messages to HTML line breaks for
+      annotate and changelog views.  Addresses bug #273688. (Peter
+      Bui)
+
+    - serve-branches now errors if run behind a proxy without
+      paste.deploy installed. (Michael Hudson)
+
+    - Loggerhead should now handle file and directory names that need
+      URL escaping without crashing.
+
+    - The start-loggerhead script properly sets the wsgi.url_scheme
+      from the server.webpath option. (neror, #260547)
+
+    - The revision page defaults to unified style again, and can
+      convert to a side-by-side view using JavaScript. (Michael Hudson)
+
+    - Clean up and improve performance of the annotate view. (Michael
+      Hudson)
+
+    - Finish converting JavaScript from MooTools to YUI 3. (Michael
+      Hudson)
+
+    - Improve compatibility with IE 6. (Michael Hudson)
+
+    - Leading blank lines in commit messages no longer result in an
+      empty summary. (Colin Watson)
+
+    - Clip long lines in side-by-side diff view. (Michael Hudson,
+      #334837)
+
+    - The user-confusing "next" and "previous" links now read "older"
+      and "newer" respectively. (Michael Hudson, #297930)
+
+    - The annotate view now contains line number anchors. (Michael
+      Hudson)
+
+    - Fix inventory pages using "//" in links. (Michael Hudson, #329668)
+
+    - Fix problems viewing files and directories containing spaces and
+      other funny characters. (Peter Bui)
+
+    - Changelog messages are now displayed with newlines preserved.
+      (Peter Bui, #273688)
+
+    - Offer a link to see the full file diffs for a file path. (Michael
+      Hudson, #333797)
+
+    - Fix annotate error caused by Pygments stripping trailing
+      whitespace. (Michael Hudson, #338762)
+
+    - Loggerhead can be installed as a Bazaar plugin and run by
+      'bzr serve --http'. (Martin Pool)
+
+    - Load parts of the changelog and revision pages via XMLHttpRequest
+      to improve performance. This adds a dependency on simplejson or
+      json. Partially addresses bug #253950. (Michael Hudson)
+
+    - Various improvements to the animation JavaScript. (Michael Hudson)
+
+    - Fix HTML content of source files being displayed unescaped when
+      Pygments was unavailable. (Michael Hudson, #344970)
+
+    - Fix serve-branches's path argument. (Michael Hudson, #353230)
+
+    - serve-branches now has an option, --use-cdn, to load YUI from
+      Yahoo!'s CDN. (Matt Nordhoff)
+
+    - Fix certain race conditions for loading bzr-search. (Robert
+      Collins, #334250)
+
+    - Fix errors when using serve-branches --log-folder or --user-dirs.
+      (It was calling config.get_option() incorrectly.) (Matt Nordhoff,
+      bug #361238)
+
+    - Move some caching from RAM to the disk, and other caching and
+      memory usage improvements. (Michael Hudson)
+
+    - Add a --cache-dir option to serve-branches to choose where to
+      place the SQL cache, and only create one temporary SQL dir per
+      process. (Matt Nordhoff, #358322)
+
+    - Replace homebrew memory profiling code with Dozer. (Paul Hummer)
+
+    - Use the branch's public_branch as the default suggested URL to
+      branch from (Matt Nordhoff, #369767)
+
+    - Fix a file descriptor leak (Matt Nordhoff, #370845)
+
+    - Use transport API internally, so it is possible to specify a remote
+      URL to serve-branches. (Jelmer Vernooij, #371787)
+
+    - Fix internal server errors when using start-loggerhead. (Matt
+      Nordhoff, #375948)
+
+    - Fix annotating non-UTF-8 files when Pygments is disabled. (Matt
+      Nordhoff, #376957)
+
+    - Fix 'bzr serve --http' errors. (Matt Nordhoff, #377551)
+
+    - Added the option to hide branches by setting http_serve = False
+      in locations.conf (Martin Albisetti)
+
+    - Fix serving branches over HTTP. (Matt Nordhoff, Jelmer Vernooij,
+      #380026)
+
+    - Install loggerhead as a bzr plugin by default (Jelmer Vernooij)
+
+    - Fix logging 404 Not Found responses (Matt Nordhoff, #381029)
+
+    - Bumped minimunm bzrlib version to 1.13 (Martin Albisetti)
+
+    - Make sure the Atom feeds (nearly) validate. (Matt Nordhoff, #247162)
+
+    - Support serving branches over HTTP using the smart server protocol.
+      (Jelmer Vernooij, #306853)
+
+    - Serving branch data was broken when --allow-writes was *not*
+      passed. (Michael Hudson, #388730)
+
+    - http_serve config values are interpreted more forgivingly.
+      (Michael Hudson)
+
+    - When specifying a remote url to serve-branches, do not share
+      connections between threads. (Michael Hudson, #390972)
+
+    - http_serve values from locations.conf are now applied to
+      non-branch .bzr data (e.g shared repositories). (Michael Hudson)
+
+    - tags are now displayed. (Cris Boylan, Alexandre Garnier, Michael
+      Hudson, #246739)
+
+    - Display Loggerhead's version number at the bottom of the page, and
+      add a <meta> generator tag also including the version numbers of
+      its dependencies. (Matt Nordhoff, #370155)
 
 
 1.10 [22Dec2008]
----------------
+----------------
+
     - Add startup deamon script for Linux (Marius Kruger)
 
-    - Switch navigation from file_ids to paths. Fixes bugs #260363, #269365
-      and #128926. (Martin Albisetti)
+    - Switch navigation from file_ids to paths. Fixes bugs #260363,
+      #269365 and #128926. (Martin Albisetti)
 
     - Fix bug #258710 ("the /files page explodes in an empty branch").
       Also minor improvements to the /files and /changes pages.

=== renamed file 'README.txt' => 'README'
--- README.txt	2008-10-02 21:12:58 +0000
+++ README	2010-04-23 01:30:29 +0000
@@ -1,117 +1,28 @@
-LOGGERHEAD
+Loggerhead
 ==========
 
-[ Version 1.6 for Bazaar 1.6 ]
-
-Loggerhead is a web viewer for Bazaar branches.  It can be used to
-navigate a branch history, annotate files, perform searches... all the
-usual things.
-
-
-GETTING STARTED
----------------
-
-Loggerhead depends on 
-1) SimpleTAL for templating.
-   on Ubuntu package `sudo apt-get install python-simpletal`
-   or download from http://www.owlfish.com/software/simpleTAL/download.html
-2) Paste for the server. (You need version 1.2 or newer of Paste.) 
-   on Ubuntu package `sudo apt-get install python-paste`
-   or use `easy_install Paste`
-3) Paste Deploy  (optional, needed when proxying through Apache)
-   on Ubuntu package `sudo apt-get install python-pastedeploy`
-   or use `easy_install PasteDeploy`
-
-Then simply run the 'serve-branches' with the branch you want to
-serve on the command line:
-
-    ./serve-branches ~/path/to/branch
-
-The script listens on port 8080 so head to http://localhost:8080/ in
-your browser to see the branch.  You can also pass a directory that
-contains branches to the script, and it will serve a very simple
-directory listing at other pages.
-
-You may update the Bazaar branches being viewed at any time.
-Loggerhead will notice and refresh, and Bazaar uses its own branch
-locking to prevent corruption.
-
-To run loggerhead as a linux daemon: 
-1) Copy loggerheadd to /etc/init.d
-2) Edit the file to configure where your loggerhead is installed, and which
-   serve-branches options you would like.
-3) Register the service
-   cd /etc/init.d
-   a) on upstart based systems like Ubuntu run: 
-      update-rc.d loggerheadd defaults
-   b) on Sysvinit based systems like Centos or SuSE run:
-      chkconfig --add loggerheadd
-
-USING A CONFIG FILE
--------------------
-
-Previous versions of Loggerhead read their configuration from a config
-file.  This mode of operation is still supported by the
-'start-loggerhead' script.  A 'loggerhead.conf.example' file is
-included in the source which has comments explaining the various
-options.
-
-Loggerhead can then be started by running::
-
-    $ ./start-loggerhead
-
-This will run loggerhead in the background, listening on port 8080 by
-default.
-
-To stop Loggerhead, run::
-
-    $ ./stop-loggerhead
-
-In the configuration file you can configure projects, and branches per
-project.  The idea is that you could be publishing several (possibly
-unrelated) projects through the same loggerhead instance, and several
-branches for the same project.  See the "loggerhead.conf.example" file
-included with the source.
-
-A debug and access log are stored in the logs/ folder, relative to
-the location of the start-loggerhead script.
-
-
-SERVING LOGGERHEAD FROM BEHIND APACHE
--------------------------------------
-
-If you want to view Bazaar branches from your existing Apache
-installation, you'll need to configure Apache to proxy certain
-requests to Loggerhead.  Adding lines like this to you Apache
-configuration is one way to do this::
-
-    <Location "/branches/">
-        ProxyPass http://127.0.0.1:8080/
-        ProxyPassReverse http://127.0.0.1:8080/
-    </Location>
-
-If Paste Deploy is installed, the 'serve-branches' script can be
-run behind a proxy at the root of a site, but if you're running it at
-some path into the site, you'll need to specify is using '--prefix=/some_path'.
-
-FILES CHANGED CACHE
--------------------
-
-To speed up the display of the changelog view for large trees,
-loggerhead can be configured to cache the files changes between
-revisions.  Set the 'cachepath' value in the config file.
-
-
-SUPPORT
--------
-
-Loggerhead is loosely based on bazaar-webserve, which was loosely
-based on hgweb.  Discussion should take place on the bazaar-dev
-mailing list at bazaar@lists.canonical.com.  You can join the list at
-<https://lists.ubuntu.com/mailman/listinfo/bazaar>.  You don't need to
-subscribe to post, but your first post will be held briefly for manual
-moderation.
-
-Bugs are tracked on Launchpad; start at:
-
-    https://bugs.launchpad.net/loggerhead
+Overview
+--------
+
+Loggerhead is a web viewer for Bazaar branches. Its lets users do
+stuff like:
+
+* navigate through branch history
+* view the files in a given revision
+* annotate files (i.e. find out the origin of each line of a file)
+* perform searches.
+
+
+Documentation
+-------------
+
+See docs/index.rst for installation instructions and basic
+documentation.
+
+
+Licensing
+---------
+
+This software is (C) Copyright Canonical Limited 2006-10 under the
+GPL Version 2 or later. Please see the file COPYING.txt for the
+licence details.

=== added file '__init__.py'
--- __init__.py	1970-01-01 00:00:00 +0000
+++ __init__.py	2011-03-19 08:35:57 +0000
@@ -0,0 +1,119 @@
+# Copyright 2009, 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+# This file allows loggerhead to be treated as a plugin for bzr.
+#
+# XXX: Because loggerhead already contains a loggerhead directory, much of the
+# code is going to appear loaded at bzrlib.plugins.loggerhead.loggerhead.
+# This seems like the easiest thing, because bzrlib wants the top-level plugin
+# directory to be the module, but when it's used as a library people expect
+# the source directory to contain a directory called loggerhead.  -- mbp
+# 20090123
+
+"""Loggerhead web viewer for Bazaar branches.
+
+This provides a new option "--http" to the "bzr serve" command, that
+starts a web server to browse the contents of a branch.
+"""
+
+import sys
+
+from info import (
+    bzr_plugin_version as version_info,
+    bzr_compatible_versions,
+    )
+
+if __name__ == 'bzrlib.plugins.loggerhead':
+    import bzrlib
+    from bzrlib.api import require_any_api
+    from bzrlib import commands
+
+    require_any_api(bzrlib, bzr_compatible_versions)
+
+    from bzrlib.transport import transport_server_registry
+
+    DEFAULT_HOST = '0.0.0.0'
+    DEFAULT_PORT = 8080
+    HELP = ('Loggerhead, a web-based code viewer and server. (default port: %d)' %
+            (DEFAULT_PORT,))
+
+    def _ensure_loggerhead_path():
+        """Ensure that you can 'import loggerhead' and get the root."""
+        # loggerhead internal code will try to 'import loggerhead', so
+        # let's put it on the path if we can't find it in the existing path
+        try:
+            import loggerhead.apps.transport
+        except ImportError:
+            import os.path, sys
+            sys.path.append(os.path.dirname(__file__))
+
+    def serve_http(transport, host=None, port=None, inet=None):
+        # TODO: if we supported inet to pass requests in and respond to them,
+        #       then it would be easier to test the full stack, but it probably
+        #       means routing around paste.httpserver.serve which probably
+        #       isn't testing the full stack
+        from paste.httpexceptions import HTTPExceptionHandler
+        from paste.httpserver import serve
+
+        _ensure_loggerhead_path()
+
+        from loggerhead.apps.http_head import HeadMiddleware
+        from loggerhead.apps.transport import BranchesFromTransportRoot
+        from loggerhead.config import LoggerheadConfig
+        from loggerhead.main import setup_logging
+
+        if host is None:
+            host = DEFAULT_HOST
+        if port is None:
+            port = DEFAULT_PORT
+        argv = ['--host', host, '--port', str(port), '--', transport.base]
+        if not transport.is_readonly():
+            argv.insert(0, '--allow-writes')
+        config = LoggerheadConfig(argv)
+        setup_logging(config, init_logging=False, log_file=sys.stderr)
+        app = BranchesFromTransportRoot(transport.base, config)
+        app = HeadMiddleware(app)
+        app = HTTPExceptionHandler(app)
+        serve(app, host=host, port=port)
+
+    transport_server_registry.register('http', serve_http, help=HELP)
+
+    class cmd_load_test_loggerhead(commands.Command):
+        """Run a load test against a live loggerhead instance.
+
+        Pass in the name of a script file to run. See loggerhead/load_test.py
+        for a description of the file format.
+        """
+
+        takes_args = ["filename"]
+
+        def run(self, filename):
+            from bzrlib.plugins.loggerhead.loggerhead import load_test
+            script = load_test.run_script(filename)
+            for thread_id in sorted(script._threads):
+                worker = script._threads[thread_id][0]
+                for url, success, time in worker.stats:
+                    self.outf.write(' %5.3fs %s %s\n'
+                                    % (time, str(success)[0], url))
+
+    commands.register_command(cmd_load_test_loggerhead)
+
+    def load_tests(standard_tests, module, loader):
+        _ensure_loggerhead_path()
+        standard_tests.addTests(loader.loadTestsFromModuleNames(
+            ['bzrlib.plugins.loggerhead.loggerhead.tests']))
+        return standard_tests

=== added directory 'docs'
=== added file 'docs/Makefile'
--- docs/Makefile	1970-01-01 00:00:00 +0000
+++ docs/Makefile	2010-03-23 15:44:51 +0000
@@ -0,0 +1,89 @@
+# Makefile for Sphinx documentation
+#
+
+# You can set these variables from the command line.
+SPHINXOPTS    =
+SPHINXBUILD   = sphinx-build
+PAPER         =
+BUILDDIR      = _build
+
+# Internal variables.
+PAPEROPT_a4     = -D latex_paper_size=a4
+PAPEROPT_letter = -D latex_paper_size=letter
+ALLSPHINXOPTS   = -d $(BUILDDIR)/doctrees $(PAPEROPT_$(PAPER)) $(SPHINXOPTS) .
+
+.PHONY: help clean html dirhtml pickle json htmlhelp qthelp latex changes linkcheck doctest
+
+help:
+	@echo "Please use \`make <target>' where <target> is one of"
+	@echo "  html      to make standalone HTML files"
+	@echo "  dirhtml   to make HTML files named index.html in directories"
+	@echo "  pickle    to make pickle files"
+	@echo "  json      to make JSON files"
+	@echo "  htmlhelp  to make HTML files and a HTML help project"
+	@echo "  qthelp    to make HTML files and a qthelp project"
+	@echo "  latex     to make LaTeX files, you can set PAPER=a4 or PAPER=letter"
+	@echo "  changes   to make an overview of all changed/added/deprecated items"
+	@echo "  linkcheck to check all external links for integrity"
+	@echo "  doctest   to run all doctests embedded in the documentation (if enabled)"
+
+clean:
+	-rm -rf $(BUILDDIR)/*
+
+html:
+	$(SPHINXBUILD) -b html $(ALLSPHINXOPTS) $(BUILDDIR)/html
+	@echo
+	@echo "Build finished. The HTML pages are in $(BUILDDIR)/html."
+
+dirhtml:
+	$(SPHINXBUILD) -b dirhtml $(ALLSPHINXOPTS) $(BUILDDIR)/dirhtml
+	@echo
+	@echo "Build finished. The HTML pages are in $(BUILDDIR)/dirhtml."
+
+pickle:
+	$(SPHINXBUILD) -b pickle $(ALLSPHINXOPTS) $(BUILDDIR)/pickle
+	@echo
+	@echo "Build finished; now you can process the pickle files."
+
+json:
+	$(SPHINXBUILD) -b json $(ALLSPHINXOPTS) $(BUILDDIR)/json
+	@echo
+	@echo "Build finished; now you can process the JSON files."
+
+htmlhelp:
+	$(SPHINXBUILD) -b htmlhelp $(ALLSPHINXOPTS) $(BUILDDIR)/htmlhelp
+	@echo
+	@echo "Build finished; now you can run HTML Help Workshop with the" \
+	      ".hhp project file in $(BUILDDIR)/htmlhelp."
+
+qthelp:
+	$(SPHINXBUILD) -b qthelp $(ALLSPHINXOPTS) $(BUILDDIR)/qthelp
+	@echo
+	@echo "Build finished; now you can run "qcollectiongenerator" with the" \
+	      ".qhcp project file in $(BUILDDIR)/qthelp, like this:"
+	@echo "# qcollectiongenerator $(BUILDDIR)/qthelp/loggerhead.qhcp"
+	@echo "To view the help file:"
+	@echo "# assistant -collectionFile $(BUILDDIR)/qthelp/loggerhead.qhc"
+
+latex:
+	$(SPHINXBUILD) -b latex $(ALLSPHINXOPTS) $(BUILDDIR)/latex
+	@echo
+	@echo "Build finished; the LaTeX files are in $(BUILDDIR)/latex."
+	@echo "Run \`make all-pdf' or \`make all-ps' in that directory to" \
+	      "run these through (pdf)latex."
+
+changes:
+	$(SPHINXBUILD) -b changes $(ALLSPHINXOPTS) $(BUILDDIR)/changes
+	@echo
+	@echo "The overview file is in $(BUILDDIR)/changes."
+
+linkcheck:
+	$(SPHINXBUILD) -b linkcheck $(ALLSPHINXOPTS) $(BUILDDIR)/linkcheck
+	@echo
+	@echo "Link check complete; look for any errors in the above output " \
+	      "or in $(BUILDDIR)/linkcheck/output.txt."
+
+doctest:
+	$(SPHINXBUILD) -b doctest $(ALLSPHINXOPTS) $(BUILDDIR)/doctest
+	@echo "Testing of doctests in the sources finished, look at the " \
+	      "results in $(BUILDDIR)/doctest/output.txt."

=== added directory 'docs/_build'
=== added directory 'docs/_static'
=== added directory 'docs/_templates'
=== added file 'docs/conf.py'
--- docs/conf.py	1970-01-01 00:00:00 +0000
+++ docs/conf.py	2010-03-25 10:25:13 +0000
@@ -0,0 +1,194 @@
+# -*- coding: utf-8 -*-
+#
+# loggerhead documentation build configuration file, created by
+# sphinx-quickstart on Tue Mar 23 10:49:50 2010.
+#
+# This file is execfile()d with the current directory set to its containing dir.
+#
+# Note that not all possible configuration values are present in this
+# autogenerated file.
+#
+# All configuration values have a default; values that are commented out
+# serve to show the default.
+
+import sys, os
+
+# If extensions (or modules to document with autodoc) are in another directory,
+# add these directories to sys.path here. If the directory is relative to the
+# documentation root, use os.path.abspath to make it absolute, like shown here.
+#sys.path.append(os.path.abspath('.'))
+
+# -- General configuration -----------------------------------------------------
+
+# Add any Sphinx extension module names here, as strings. They can be extensions
+# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
+extensions = []
+
+# Add any paths that contain templates here, relative to this directory.
+templates_path = ['_templates']
+
+# The suffix of source filenames.
+source_suffix = '.rst'
+
+# The encoding of source files.
+#source_encoding = 'utf-8'
+
+# The master toctree document.
+master_doc = 'index'
+
+# General information about the project.
+project = u'Loggerhead'
+copyright = u'2010, Loggerhead team (https://launchpad.net/~loggerhead-team)'
+
+# The version info for the project you're documenting, acts as replacement for
+# |version| and |release|, also used in various other places throughout the
+# built documents.
+#
+# The short X.Y version.
+version = '1.17'
+# The full version, including alpha/beta/rc tags.
+release = '1.17'
+
+# The language for content autogenerated by Sphinx. Refer to documentation
+# for a list of supported languages.
+#language = None
+
+# There are two options for replacing |today|: either, you set today to some
+# non-false value, then it is used:
+#today = ''
+# Else, today_fmt is used as the format for a strftime call.
+#today_fmt = '%B %d, %Y'
+
+# List of documents that shouldn't be included in the build.
+#unused_docs = []
+
+# List of directories, relative to source directory, that shouldn't be searched
+# for source files.
+exclude_trees = ['_build']
+
+# The reST default role (used for this markup: `text`) to use for all documents.
+#default_role = None
+
+# If true, '()' will be appended to :func: etc. cross-reference text.
+#add_function_parentheses = True
+
+# If true, the current module name will be prepended to all description
+# unit titles (such as .. function::).
+#add_module_names = True
+
+# If true, sectionauthor and moduleauthor directives will be shown in the
+# output. They are ignored by default.
+#show_authors = False
+
+# The name of the Pygments (syntax highlighting) style to use.
+pygments_style = 'sphinx'
+
+# A list of ignored prefixes for module index sorting.
+#modindex_common_prefix = []
+
+
+# -- Options for HTML output ---------------------------------------------------
+
+# The theme to use for HTML and HTML Help pages.  Major themes that come with
+# Sphinx are currently 'default' and 'sphinxdoc'.
+html_theme = 'default'
+
+# Theme options are theme-specific and customize the look and feel of a theme
+# further.  For a list of options available for each theme, see the
+# documentation.
+#html_theme_options = {}
+
+# Add any paths that contain custom themes here, relative to this directory.
+#html_theme_path = []
+
+# The name for this set of Sphinx documents.  If None, it defaults to
+# "<project> v<release> documentation".
+#html_title = None
+
+# A shorter title for the navigation bar.  Default is the same as html_title.
+#html_short_title = None
+
+# The name of an image file (relative to this directory) to place at the top
+# of the sidebar.
+#html_logo = None
+
+# The name of an image file (within the static path) to use as favicon of the
+# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
+# pixels large.
+#html_favicon = None
+
+# Add any paths that contain custom static files (such as style sheets) here,
+# relative to this directory. They are copied after the builtin static files,
+# so a file named "default.css" will overwrite the builtin "default.css".
+html_static_path = ['_static']
+
+# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
+# using the given strftime format.
+#html_last_updated_fmt = '%b %d, %Y'
+
+# If true, SmartyPants will be used to convert quotes and dashes to
+# typographically correct entities.
+#html_use_smartypants = True
+
+# Custom sidebar templates, maps document names to template names.
+#html_sidebars = {}
+
+# Additional templates that should be rendered to pages, maps page names to
+# template names.
+#html_additional_pages = {}
+
+# If false, no module index is generated.
+#html_use_modindex = True
+
+# If false, no index is generated.
+#html_use_index = True
+
+# If true, the index is split into individual pages for each letter.
+#html_split_index = False
+
+# If true, links to the reST sources are added to the pages.
+#html_show_sourcelink = True
+
+# If true, an OpenSearch description file will be output, and all pages will
+# contain a <link> tag referring to it.  The value of this option must be the
+# base URL from which the finished HTML is served.
+#html_use_opensearch = ''
+
+# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
+#html_file_suffix = ''
+
+# Output file base name for HTML help builder.
+htmlhelp_basename = 'loggerheaddoc'
+
+
+# -- Options for LaTeX output --------------------------------------------------
+
+# The paper size ('letter' or 'a4').
+#latex_paper_size = 'letter'
+
+# The font size ('10pt', '11pt' or '12pt').
+#latex_font_size = '10pt'
+
+# Grouping the document tree into LaTeX files. List of tuples
+# (source start file, target name, title, author, documentclass [howto/manual]).
+latex_documents = [
+  ('index', 'loggerhead.tex', u'Loggerhead Documentation',
+   u'Loggerhead team (https://launchpad.net/\\textasciitilde{}loggerhead-team)', 'manual'),
+]
+
+# The name of an image file (relative to this directory) to place at the top of
+# the title page.
+#latex_logo = None
+
+# For "manual" documents, if this is true, then toplevel headings are parts,
+# not chapters.
+#latex_use_parts = False
+
+# Additional stuff for the LaTeX preamble.
+#latex_preamble = ''
+
+# Documents to append as an appendix to all manuals.
+#latex_appendices = []
+
+# If false, no module index is generated.
+#latex_use_modindex = True

=== added file 'docs/index.rst'
--- docs/index.rst	1970-01-01 00:00:00 +0000
+++ docs/index.rst	2010-05-14 07:33:29 +0000
@@ -0,0 +1,209 @@
+Loggerhead:  A web viewer for ``bzr`` branches
+==============================================
+
+Loggerhead is a web viewer for projects in Bazaar. It can be used to navigate 
+a branch history, annotate files, view patches, perform searches, etc.
+Loggerhead is heavily based on `bazaar-webserve
+<https://launchpad.net/bzr-webserve>`_, which was, in turn, loosely
+based on `hgweb <http://mercurial.selenic.com/wiki/HgWebDirStepByStep>`_.
+
+
+Getting Started
+---------------
+
+Loggerhead depends on the following Python libraries.:
+
+- SimpleTAL for templating.
+
+- simplejson for producing JSON data.
+
+- Paste for the server. (You need version 1.2 or newer of Paste).
+
+- Paste Deploy  (optional, needed when proxying through Apache).
+
+- flup (optional, needed to use FastCGI, SCGI or AJP).
+
+
+Installing Dependencies Using Ubuntu Packages
+#############################################
+
+.. code-block:: sh
+
+   $ sudo apt-get install python-simpletal
+   $ sudo apt-get install python-simplejson
+   $ sudo apt-get install python-paste
+   $ sudo apt-get install python-pastedeploy
+   $ sudo apt-get install python-flup
+
+Installing Dependencies Using :command:`easy_install`
+#####################################################
+
+.. code-block:: sh
+
+   $ easy_install \
+     -f http://www.owlfish.com/software/simpleTAL/py2compatible/download.html \
+     SimpleTAL
+   $ easy_install simplejson
+   $ easy_install Paste
+   $ easy_install PasteDeploy
+   $ easy_install flup
+
+
+Running the Standalone Loggerhead Server
+----------------------------------------
+
+After installing all the dependencies, you should be able to run
+:command:`serve-branches` with the branch you want to serve on the
+command line:
+
+.. code-block:: sh
+
+    ./serve-branches ~/path/to/branch
+
+By default, the script listens on port 8080, so head to
+http://localhost:8080/ in your browser to see the branch.
+
+You can also pass a directory that contains branches to the script,
+and it will serve a very simple directory listing at other pages.
+
+You may update the Bazaar branches being viewed at any time.
+Loggerhead will notice and refresh, and Bazaar uses its own branch
+locking to prevent corruption.
+
+See :doc:`serve-branches` for all command line options.
+
+Running Loggerhead as a Daemon
+------------------------------
+
+To run Loggerhead as a linux daemon:
+
+1) Copy the ``loggerheadd`` scipt to ``/etc/init.d``
+
+.. code-block:: sh
+
+   $ sudo cp ./loggerheadd /etc/init.d
+
+2) Edit the file to configure where your Loggerhead is installed, and which
+   serve-branches options you would like.
+
+.. code-block:: sh
+
+   $ sudo vim /etc/init.d/loggerheadd
+
+3) Register the service
+
+.. code-block:: sh
+
+   # on upstart based systems like Ubuntu run: 
+   $ sudo update-rc.d loggerheadd defaults
+
+   # on Sysvinit based systems like Centos or SuSE run:
+   $ sudo chkconfig --add loggerheadd
+
+
+Using Loggerhead as a Bazaar Plugin
+------------------------------------
+
+This branch contains experimental support for using Loggerhead as a Bazaar
+plugin.  To use it, place the top-level Loggerhead directory (the one
+containing COPYING.txt) at ``~/.bazaar/plugins/loggerhead``.  E.g.:
+
+.. code-block:: sh
+
+   $ bzr branch lp:loggerhead ~/.bazaar/plugins/loggerhead
+   $ cd ~/myproject
+   $ bzr serve --http
+
+
+Using a Config File
+-------------------
+
+To hide branches from being displayed, add to ``~/.bazaar/locations.conf``,
+under the branch's section:
+
+.. code-block:: ini
+
+    [/path/to/branch]
+    http_serve = False
+
+More configuration options to come soon.
+
+
+Serving Loggerhead behind Apache
+--------------------------------
+
+If you want to view Bazaar branches from your existing Apache
+installation, you'll need to configure Apache to proxy certain
+requests to Loggerhead.  Adding lines like this to your Apache
+configuration is one way to do this:
+
+.. code-block:: apache
+
+    <Location "/branches/">
+        ProxyPass http://127.0.0.1:8080/branches/
+        ProxyPassReverse http://127.0.0.1:8080/branches/
+    </Location>
+
+If Paste Deploy is installed, the :command:`serve-branches` script can be
+run behind a proxy at the root of a site, but if you're running it at
+some path into the site, you'll need to specify it using
+``--prefix=/some_path``.
+
+
+Search
+------
+
+Search is currently supported by using the bzr-search plugin (available
+at: https://launchpad.net/bzr-search ).
+
+You need to have the plugin installed and each branch indexed to allow
+searching on branches.
+
+Command-Line Reference
+----------------------
+
+.. toctree::
+   :maxdepth: 2
+
+   serve-branches
+
+
+Support
+-------
+
+Discussion should take place on the bazaar-dev mailing list at
+mailto:bazaar@lists.canonical.com.  You can join the list at
+<https://lists.ubuntu.com/mailman/listinfo/bazaar>.  You don't need to
+subscribe to post, but your first post will be held briefly for manual
+moderation.
+
+Bugs, support questions and merge proposals are tracked on Launchpad, e.g:
+
+    https://bugs.launchpad.net/loggerhead
+
+
+Hacking
+-------
+
+To run Loggerhead tests, you will need to install the package ``python-nose``,
+and run its :command:`nosetests` script in the Loggerhead directory:
+
+.. code-block:: sh
+
+    nosetests
+
+
+License
+-------
+
+GNU GPLv2 or later.
+
+See Also
+--------
+
+https://launchpad.net/loggerhead
+
+Index
+=====
+
+- :ref:`genindex`

=== added file 'docs/make.bat'
--- docs/make.bat	1970-01-01 00:00:00 +0000
+++ docs/make.bat	2010-03-23 15:44:51 +0000
@@ -0,0 +1,113 @@
+@ECHO OFF
+
+REM Command file for Sphinx documentation
+
+set SPHINXBUILD=sphinx-build
+set BUILDDIR=_build
+set ALLSPHINXOPTS=-d %BUILDDIR%/doctrees %SPHINXOPTS% .
+if NOT "%PAPER%" == "" (
+	set ALLSPHINXOPTS=-D latex_paper_size=%PAPER% %ALLSPHINXOPTS%
+)
+
+if "%1" == "" goto help
+
+if "%1" == "help" (
+	:help
+	echo.Please use `make ^<target^>` where ^<target^> is one of
+	echo.  html      to make standalone HTML files
+	echo.  dirhtml   to make HTML files named index.html in directories
+	echo.  pickle    to make pickle files
+	echo.  json      to make JSON files
+	echo.  htmlhelp  to make HTML files and a HTML help project
+	echo.  qthelp    to make HTML files and a qthelp project
+	echo.  latex     to make LaTeX files, you can set PAPER=a4 or PAPER=letter
+	echo.  changes   to make an overview over all changed/added/deprecated items
+	echo.  linkcheck to check all external links for integrity
+	echo.  doctest   to run all doctests embedded in the documentation if enabled
+	goto end
+)
+
+if "%1" == "clean" (
+	for /d %%i in (%BUILDDIR%\*) do rmdir /q /s %%i
+	del /q /s %BUILDDIR%\*
+	goto end
+)
+
+if "%1" == "html" (
+	%SPHINXBUILD% -b html %ALLSPHINXOPTS% %BUILDDIR%/html
+	echo.
+	echo.Build finished. The HTML pages are in %BUILDDIR%/html.
+	goto end
+)
+
+if "%1" == "dirhtml" (
+	%SPHINXBUILD% -b dirhtml %ALLSPHINXOPTS% %BUILDDIR%/dirhtml
+	echo.
+	echo.Build finished. The HTML pages are in %BUILDDIR%/dirhtml.
+	goto end
+)
+
+if "%1" == "pickle" (
+	%SPHINXBUILD% -b pickle %ALLSPHINXOPTS% %BUILDDIR%/pickle
+	echo.
+	echo.Build finished; now you can process the pickle files.
+	goto end
+)
+
+if "%1" == "json" (
+	%SPHINXBUILD% -b json %ALLSPHINXOPTS% %BUILDDIR%/json
+	echo.
+	echo.Build finished; now you can process the JSON files.
+	goto end
+)
+
+if "%1" == "htmlhelp" (
+	%SPHINXBUILD% -b htmlhelp %ALLSPHINXOPTS% %BUILDDIR%/htmlhelp
+	echo.
+	echo.Build finished; now you can run HTML Help Workshop with the ^
+.hhp project file in %BUILDDIR%/htmlhelp.
+	goto end
+)
+
+if "%1" == "qthelp" (
+	%SPHINXBUILD% -b qthelp %ALLSPHINXOPTS% %BUILDDIR%/qthelp
+	echo.
+	echo.Build finished; now you can run "qcollectiongenerator" with the ^
+.qhcp project file in %BUILDDIR%/qthelp, like this:
+	echo.^> qcollectiongenerator %BUILDDIR%\qthelp\loggerhead.qhcp
+	echo.To view the help file:
+	echo.^> assistant -collectionFile %BUILDDIR%\qthelp\loggerhead.ghc
+	goto end
+)
+
+if "%1" == "latex" (
+	%SPHINXBUILD% -b latex %ALLSPHINXOPTS% %BUILDDIR%/latex
+	echo.
+	echo.Build finished; the LaTeX files are in %BUILDDIR%/latex.
+	goto end
+)
+
+if "%1" == "changes" (
+	%SPHINXBUILD% -b changes %ALLSPHINXOPTS% %BUILDDIR%/changes
+	echo.
+	echo.The overview file is in %BUILDDIR%/changes.
+	goto end
+)
+
+if "%1" == "linkcheck" (
+	%SPHINXBUILD% -b linkcheck %ALLSPHINXOPTS% %BUILDDIR%/linkcheck
+	echo.
+	echo.Link check complete; look for any errors in the above output ^
+or in %BUILDDIR%/linkcheck/output.txt.
+	goto end
+)
+
+if "%1" == "doctest" (
+	%SPHINXBUILD% -b doctest %ALLSPHINXOPTS% %BUILDDIR%/doctest
+	echo.
+	echo.Testing of doctests in the sources finished, look at the ^
+results in %BUILDDIR%/doctest/output.txt.
+	goto end
+)
+
+:end

=== added file 'docs/serve-branches.rst'
--- docs/serve-branches.rst	1970-01-01 00:00:00 +0000
+++ docs/serve-branches.rst	2010-03-25 10:32:47 +0000
@@ -0,0 +1,116 @@
+:command:`serve-branches`
+=========================
+
+The :command:`serve-branches` script runs a standalone Loggerhead server in
+the foreground.
+
+.. program:: serve-branches
+
+Usage
+-----
+
+.. code-block:: sh
+
+   serve-branches [OPTIONS] <target directory>
+
+Options
+-------
+
+.. cmdoption:: --user-dirs
+
+    Serve user directories as ``~user`` (requires ``--trunk-dir``).
+
+    If both options are set, then for requests where the CGI ``PATH_INFO``
+    starts with "/~<name>", serve branches under the <name> directory.
+
+.. cmdoption:: --trunk-dir=DIR
+
+    The directory that contains the trunk branches (requires ``--user-dirs``).
+
+    If both options are set, then for requests where the CGI ``PATH_INFO``
+    does not start with "/~<name>", serve branches under DIR.
+
+.. cmdoption:: --port
+
+    Listen on the given port.
+    
+    Defaults to 8080.
+
+.. cmdoption:: --host
+
+    Listen on the interface corresponding to the given IP. 
+    
+    Defaults to listening on all interfaces, i.e., "0.0.0.0".
+
+.. cmdoption:: --protocol
+
+    Serve the application using the specified protocol.
+    
+    Can be one of: "http", "scgi", "fcgi", "ajp" (defaults to "http").
+
+.. cmdoption:: --prefix
+
+    Set the supplied value as the CGI ``SCRIPT_NAME`` for the application.
+
+    This option is intended for use when serving Loggerhead behind a
+    reverse proxy, with Loggerhead being "mounted" at a directory below
+    the root.  E.g., if the reverse proxy translates requests for
+    ``http://example.com/loggerhead`` onto the standalone Loggerhead process,
+    that process should be run with ``--prefix=/loggerhead``.
+
+.. cmdoption:: --log-folder=LOG_FOLDER
+
+    The directory in which to place Loggerhead's log files.
+    
+    Defaults to the current directory.
+
+.. cmdoption:: --cache-dir=SQL_CACHE_DIR
+
+    The directory in which to place the SQL cache.
+
+    Defaults to the current directory.
+
+.. cmdoption:: --use-cdn
+   
+    Serve YUI javascript libraries from Yahoo!'s CDN.
+
+.. cmdoption:: --allow-writes
+   
+    Allow writing to the Bazaar server.
+    
+    Setting this option keeps Loggerhead from adding a 'readonly+' prefix
+    to the base URL of the branch.  The only effect of suppressing this prefix
+    is to make visible the display of instructions for checking out the
+    'public_branch' URL for the branch being browsed.
+
+.. cmdoption:: -h, --help
+
+    Print the help message and exit
+
+.. cmdoption:: --version
+
+    Print the software version and exit.
+
+Debugging Options
+-----------------
+
+The following options are only useful when developing / debugging Loggerhead
+itself.
+
+.. cmdoption:: --profile
+   
+    Generate per-request callgrind profile data.
+    
+    Data for each request is written to a file ``%d-stats.callgrind``,
+    where ``%d`` is replaced by the sequence number of the request.
+
+.. cmdoption:: --memory-profile
+
+    Profile the memory usage using the `Dozer
+    <http://pypi.python.org/pypi/Dozer>`_ middleware.
+
+.. cmdoption:: --reload
+
+    Restart the application when any of its python file change.
+    
+    This option should only used for development purposes.

=== added file 'info.py'
--- info.py	1970-01-01 00:00:00 +0000
+++ info.py	2011-03-19 08:35:57 +0000
@@ -0,0 +1,14 @@
+#!/usr/bin/env python
+# API Info for loggerhead
+
+bzr_plugin_name = "loggerhead"
+
+bzr_plugin_version = (1, 18, 0)
+
+bzr_compatible_versions = [
+    (1, 17, 0), (1, 18, 0), (2, 0, 0), (2, 1, 0), (2, 2, 0), (2, 3, 0),
+    (2, 4, 0)]
+
+bzr_minimum_version = bzr_compatible_versions[0]
+
+bzr_maximum_version = bzr_compatible_versions[-1]

=== added directory 'load_test_scripts'
=== added file 'load_test_scripts/multiple_instances.script'
--- load_test_scripts/multiple_instances.script	1970-01-01 00:00:00 +0000
+++ load_test_scripts/multiple_instances.script	2011-02-10 01:00:32 +0000
@@ -0,0 +1,19 @@
+{
+    "comment": "Connect to multiple loggerhead instances and make requests on each. One should be on :8080, one should be on :8081. Multiple threads will place requests on each.",
+    "parameters": {"base_url": "http://localhost"},
+    "requests": [
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"},
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"},
+        {"thread": "1", "relpath": ":8080/changes"},
+        {"thread": "2", "relpath": ":8080/files"},
+        {"thread": "3", "relpath": ":8081/files"},
+        {"thread": "4", "relpath": ":8081/changes"}
+    ]
+}
+

=== added file 'load_test_scripts/simple.script'
--- load_test_scripts/simple.script	1970-01-01 00:00:00 +0000
+++ load_test_scripts/simple.script	2011-02-10 01:00:32 +0000
@@ -0,0 +1,9 @@
+{
+    "comment": "A fairly trivial load test script. It just loads the main two pages from a loggerhead install running directly on a branch.",
+    "parameters": {"base_url": "http://localhost:8080"},
+    "requests": [
+        {"relpath": "/changes"},
+        {"relpath": "/files"}
+    ]
+}
+

=== removed file 'loggerhead.conf.example'
--- loggerhead.conf.example	2008-07-22 00:40:03 +0000
+++ loggerhead.conf.example	1970-01-01 00:00:00 +0000
@@ -1,76 +0,0 @@
-
-# use this to change the server's port
-#server.socket_port = 8080
-
-# use this if you're mapping loggerhead within apache via proxy
-#server.webpath = 'http://example.com/branches'
-
-# use this if you want loggerhead to listen to another interface than 127.0.0.1
-# server.socket_host = 'your_hostname_or_ip'
-
-# where's bzrlib?
-# (you only need this if you don't install bzr.)
-#bzrpath = '/Users/robey/code/bzr/bzr'
-
-# if you want a special title on the front page, set it here:
-title = 'branches in loggerhead'
-
-# the access and debug logs can be set up to roll 'daily', 'weekly', or 'never':
-log.roll = 'weekly'
-
-# group branches by project:
-[bazaar]
-
-    # name of this project
-    name = 'bazaar'
-    
-    # for the top browsing page, you can include a description, in raw HTML.
-    #description = 'Bazaar is a <i>revision control system</i>.'
-
-    # should we cache files changed? (recommended for large trees)
-    # this will be a folder containing one file.  if the folder doesn't
-    # exist, it'll be created.
-    #
-    # all branches in this project will share the same cache, which is usually
-    # what you want if they share a repository.  if not, you can also specify
-    # this per-branch.
-    cachepath = '/Users/robey/code/bzr/.bzr/loggerhead-files'
-
-    # each branch is in a sub-folder of the website, and the config
-    # key describes that folder name:
-    [[bzr.dev]]
-
-        # where is the bazaar branch located?
-        folder = '/Users/robey/code/bzr/bzr.dev'
-
-        # (optional) is there a friendier name for the branch that i should
-        # use on the web pages?
-        # you can also define this using the branch's nickname.
-        #branch_name = 'bazaar-dev'
-
-        # (optional) short description for the top-level page
-        # you can also define this using the "description" config key in the
-        # branch's 'branch.conf'.
-        description = "bazaar vcs development branch"
-
-        # (optional) public branch url for the top-level page
-        # you can also define this using the "public_url" config key in the
-        # branch's 'branch.conf'.
-        url = 'http://bazaar-ng.org/bzr/bzr.dev'
-
-
-# here's an example of an auto-published folder:
-[paramiko]
-    name = 'paramiko'
-    cachepath = '/Users/robey/code/paramiko/.bzr/loggerhead-files'
-    
-    # if an auto_publish_folder is given, any bazaar branches discovered in
-    # that folder will be published.  using each branch's 'branch.conf' to
-    # provide optional descriptions and public urls is a nice way to avoid
-    # having to do much configuration here.
-    auto_publish_folder = '/Users/robey/code/paramiko'
-    
-    # if all of the branches under an auto_publish_folder are exported to
-    # the same (base) public url, you can specify that url prefix here, and
-    # each discovered branch's public url will be generated.
-    url_prefix = 'http://www.example.com/code/paramiko'

=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2009-02-10 21:11:43 +0000
+++ loggerhead/__init__.py	2010-11-11 03:30:43 +0000
@@ -1,3 +1,6 @@
+#
+# Copyright (C) 2008, 2009 Canonical Ltd.
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -19,8 +22,8 @@
 
 import pkg_resources
 
-__version__ = '1.10'
-required_bzrlib = (1, 6)
+__version__ = '1.18'
+required_bzrlib = (1, 17)
 
 pkg_resources.get_distribution('Paste>=1.6')
 try:

=== modified file 'loggerhead/apps/__init__.py'
--- loggerhead/apps/__init__.py	2008-12-05 05:07:33 +0000
+++ loggerhead/apps/__init__.py	2010-04-30 21:30:18 +0000
@@ -4,15 +4,17 @@
 
 from paste import urlparser, fileapp
 
-from bzrlib.plugin import load_plugins
+from loggerhead.util import convert_file_errors
 
 static = os.path.join(
     os.path.dirname(os.path.dirname(__file__)), 'static')
 
-static_app = urlparser.make_static(None, static)
-
-favicon_app = fileapp.FileApp(os.path.join(static, 'images', 'favicon.ico'))
-robots_app = fileapp.FileApp(os.path.join(static, 'robots.txt'))
-
-# load plugins - such as svn:// support, extra formats and so on.
-load_plugins()
+# Static things can be cached for half a day, we could probably make this
+# longer, except for just before rollout times.
+static_app = urlparser.make_static(None, static, cache_max_age=12*60*60)
+
+favicon_app = convert_file_errors(fileapp.FileApp(
+    os.path.join(static, 'images', 'favicon.ico')))
+
+robots_app = convert_file_errors(fileapp.FileApp(
+    os.path.join(static, 'robots.txt')))

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2009-02-18 10:36:48 +0000
+++ loggerhead/apps/branch.py	2011-03-16 11:40:05 +0000
@@ -1,3 +1,19 @@
+# Copyright (C) 2008-2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
 """The WSGI application for serving a Bazaar branch."""
 
 import logging
@@ -5,20 +21,24 @@
 import sys
 
 import bzrlib.branch
+import bzrlib.errors
 import bzrlib.lru_cache
 
 from paste import request
 from paste import httpexceptions
 
 from loggerhead.apps import static_app
+from loggerhead.controllers.annotate_ui import AnnotateUI
+from loggerhead.controllers.view_ui import ViewUI
+from loggerhead.controllers.atom_ui import AtomUI
 from loggerhead.controllers.changelog_ui import ChangeLogUI
+from loggerhead.controllers.diff_ui import DiffUI
+from loggerhead.controllers.download_ui import DownloadUI
+from loggerhead.controllers.filediff_ui import FileDiffUI
 from loggerhead.controllers.inventory_ui import InventoryUI
-from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.revision_ui import RevisionUI
-from loggerhead.controllers.atom_ui import AtomUI
-from loggerhead.controllers.download_ui import DownloadUI
+from loggerhead.controllers.revlog_ui import RevLogUI
 from loggerhead.controllers.search_ui import SearchUI
-from loggerhead.controllers.diff_ui import DiffUI
 from loggerhead.history import History
 from loggerhead import util
 
@@ -29,33 +49,38 @@
 
     def __init__(self, branch, friendly_name=None, config={},
                  graph_cache=None, branch_link=None, is_root=False,
-                 served_url=_DEFAULT):
+                 served_url=_DEFAULT, use_cdn=False):
         self.branch = branch
         self._config = config
         self.friendly_name = friendly_name
         self.branch_link = branch_link  # Currently only used in Launchpad
-        self.log = logging.getLogger('loggerhead.%s' % friendly_name)
+        self.log = logging.getLogger('loggerhead.%s' % (friendly_name,))
         if graph_cache is None:
-            graph_cache = bzrlib.lru_cache.LRUCache()
+            graph_cache = bzrlib.lru_cache.LRUCache(10)
         self.graph_cache = graph_cache
         self.is_root = is_root
         self.served_url = served_url
+        self.use_cdn = use_cdn
 
     def get_history(self):
-        _history = History(self.branch, self.graph_cache)
+        file_cache = None
+        revinfo_disk_cache = None
         cache_path = self._config.get('cachepath', None)
         if cache_path is not None:
             # Only import the cache if we're going to use it.
             # This makes sqlite optional
             try:
-                from loggerhead.changecache import FileChangeCache
+                from loggerhead.changecache import (
+                    FileChangeCache, RevInfoDiskCache)
             except ImportError:
                 self.log.debug("Couldn't load python-sqlite,"
                                " continuing without using a cache")
             else:
-                _history.use_file_cache(
-                    FileChangeCache(_history, cache_path))
-        return _history
+                file_cache = FileChangeCache(cache_path)
+                revinfo_disk_cache = RevInfoDiskCache(cache_path)
+        return History(
+            self.branch, self.graph_cache, file_cache=file_cache,
+            revinfo_disk_cache=revinfo_disk_cache, cache_key=self.friendly_name)
 
     def url(self, *args, **kw):
         if isinstance(args[0], list):
@@ -63,12 +88,17 @@
         qs = []
         for k, v in kw.iteritems():
             if v is not None:
-                qs.append('%s=%s'%(k, urllib.quote(v)))
+                qs.append('%s=%s' % (k, urllib.quote(v)))
         qs = '&'.join(qs)
-        return request.construct_url(
-            self._environ, script_name=self._url_base,
-            path_info=unicode('/'.join(args)).encode('utf-8'),
-            querystring=qs)
+        path_info = urllib.quote(
+            unicode('/'.join(args)).encode('utf-8'), safe='/~:')
+        if qs:
+            path_info += '?' + qs
+        return self._url_base + path_info
+
+    def absolute_url(self, *args, **kw):
+        rel_url = self.url(*args, **kw)
+        return request.resolve_relative_url(rel_url, self._environ)
 
     def context_url(self, *args, **kw):
         kw = util.get_context(**kw)
@@ -77,15 +107,25 @@
     def static_url(self, path):
         return self._static_url_base + path
 
+    def yui_url(self, path):
+        if self.use_cdn:
+            base = 'http://yui.yahooapis.com/3.0.0pr2/build/'
+        else:
+            base = self.static_url('/static/javascript/yui/build/')
+        return base + path
+
     controllers_dict = {
+        '+filediff': FileDiffUI,
+        '+revlog': RevLogUI,
         'annotate': AnnotateUI,
+        'atom': AtomUI,
         'changes': ChangeLogUI,
+        'diff': DiffUI,
+        'download': DownloadUI,
         'files': InventoryUI,
         'revision': RevisionUI,
-        'download': DownloadUI,
-        'atom': AtomUI,
         'search': SearchUI,
-        'diff': DiffUI,
+        'view': ViewUI,
         }
 
     def last_updated(self):
@@ -93,21 +133,35 @@
         change = h.get_changes([h.last_revid])[0]
         return change.date
 
-    def branch_url(self):
+    def public_branch_url(self):
         return self.branch.get_config().get_user_option('public_branch')
 
     def app(self, environ, start_response):
+        # Check again if the branch is blocked from being served, this is
+        # mostly for tests. It's already checked in apps/transport.py
+        if self.branch.get_config().get_user_option('http_serve') == 'False':
+            raise httpexceptions.HTTPNotFound()
         self._url_base = environ['SCRIPT_NAME']
         self._static_url_base = environ.get('loggerhead.static.url')
         if self._static_url_base is None:
             self._static_url_base = self._url_base
         self._environ = environ
         if self.served_url is _DEFAULT:
-            self.served_url = self.url([])
+            public_branch = self.public_branch_url()
+            if public_branch is not None:
+                self.served_url = public_branch
+            else:
+                # Loggerhead only supports serving .bzr/ on local branches, so
+                # we shouldn't suggest something that won't work.
+                try:
+                    util.local_path_from_url(self.branch.base)
+                    self.served_url = self.url([])
+                except bzrlib.errors.InvalidURL:
+                    self.served_url = None
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
-                self._url_base + '/changes')
+                self.absolute_url('/changes'))
         if path == 'static':
             return static_app(environ, start_response)
         cls = self.controllers_dict.get(path)

=== removed file 'loggerhead/apps/config.py'
--- loggerhead/apps/config.py	2008-12-15 21:23:24 +0000
+++ loggerhead/apps/config.py	1970-01-01 00:00:00 +0000
@@ -1,229 +0,0 @@
-"""A server that uses a loggerhead.conf file.
-
-We recreate the branch discovery and url scheme of the old branchview
-code.  It's all a bit horrible really.
-"""
-
-import logging
-import os
-import posixpath
-
-import bzrlib.lru_cache
-
-from bzrlib.util.configobj.configobj import ConfigObj
-
-from paste.request import path_info_pop
-from paste import httpexceptions
-from paste.wsgiwrappers import WSGIResponse
-
-from loggerhead.apps.branch import BranchWSGIApp
-from loggerhead.apps import favicon_app, static_app, robots_app
-from loggerhead.templatefunctions import templatefunctions
-from loggerhead.zptsupport import load_template
-from loggerhead import util
-
-log = logging.getLogger("loggerhead.controllers")
-
-from loggerhead.history import is_branch
-
-
-class Project(object):
-    """A project contains the branches.
-
-    There is some complication because we don't want to hold on to the
-    branches when they are not being browsed."""
-
-    def __init__(self, name, config, root_config, graph_cache):
-        self.name = name
-        self.friendly_name = config.get('name', name)
-        self.description = config.get('description', '')
-        self.long_description = config.get('long_description', '')
-        self._config = config
-        self._root_config = root_config
-        self.graph_cache = graph_cache
-
-        self.view_names = []
-        self.view_data_by_name = {}
-        for view_name in config.sections:
-            log.debug('Configuring (project %s) branch %s...', name, view_name)
-            self._add_view(
-                view_name, config[view_name], config[view_name].get('folder'))
-
-        self._auto_folder = config.get('auto_publish_folder', None)
-        self._auto_list = []
-        if self._auto_folder is not None:
-            self._recheck_auto_folders()
-
-    def _recheck_auto_folders(self):
-        if self._auto_folder is None:
-            return
-        auto_list = []
-        # scan a folder for bazaar branches, and add them automatically
-        for path, folders, filenames in os.walk(self._auto_folder):
-            for folder in folders:
-                folder = os.path.join(path, folder)
-                if is_branch(folder):
-                    auto_list.append(folder)
-        auto_list.sort()
-        if auto_list == self._auto_list:
-            # nothing has changed; do nothing.
-            return
-
-        # rebuild views:
-        self.view_names = []
-        log.debug('Rescanning auto-folder for project %s ...', self.name)
-        for folder in auto_list:
-            view_name = os.path.basename(folder)
-            log.debug('Auto-configuring (project %s) branch %s...',
-                      self.name,
-                      view_name)
-            self._add_view(view_name, ConfigObj(), folder)
-        self._auto_list = auto_list
-
-    def _get_branch_url(self, view, view_config, folder):
-        url = view_config.get('url', None)
-        if url is not None:
-            return url
-        url = self._config.get('url_prefix', None)
-        if url is not None:
-            return posixpath.join(url, folder) + '/'
-        return None
-
-    def _get_description(self, view, view_config, history):
-        description = view_config.get('description', None)
-        if description is not None:
-            return description
-        description = history._branch.get_config().get_user_option(
-                          'description')
-        return description
-
-    def _add_view(self, view_name, view_config, folder):
-        b = bzrlib.branch.Branch.open(folder)
-        view = BranchWSGIApp(b, view_name, view_config, self.graph_cache)
-        b.lock_read()
-        try:
-            history = view.get_history()
-            friendly_name = view_config.get('branch_name', None)
-            if friendly_name is None:
-                friendly_name = history.get_config().get_nickname()
-                if friendly_name is None:
-                    friendly_name = view_name
-            self.view_data_by_name[view_name] = {
-                'branch_path': folder,
-                'args': (view_name, view_config, self.graph_cache),
-                'description': self._get_description(view,
-                                                     view_config,
-                                                     history),
-                '_src_folder': folder,
-                '_view_config': view_config,
-                'friendly_name': friendly_name,
-                'name': view_name,
-                }
-            branch_url = self._get_branch_url(view, view_config, view_name)
-            if branch_url is not None:
-                self.view_data_by_name[view_name]['branch_url'] = branch_url
-            self.view_names.append(view_name)
-        finally:
-            b.unlock()
-
-    def view_named(self, name):
-        view_data = self.view_data_by_name.get(name)
-        if view_data is None:
-            return None
-        view_data = view_data.copy()
-        branch_path = view_data.pop('branch_path')
-        args = view_data.pop('args')
-        b = bzrlib.branch.Branch.open(branch_path)
-        b.lock_read()
-        view = BranchWSGIApp(b, *args)
-        for k in view_data:
-            setattr(view, k, view_data[k])
-        return view
-
-    def call(self, environ, start_response):
-        segment = path_info_pop(environ)
-        if not segment:
-            raise httpexceptions.HTTPNotFound()
-        else:
-            view = self.view_named(segment)
-            if view is None:
-                raise httpexceptions.HTTPNotFound()
-            try:
-                return view.app(environ, start_response)
-            finally:
-                view.branch.unlock()
-
-
-class Root(object):
-    """The root of the server -- renders as the browse view,
-    dispatches to Project above for each 'project'."""
-
-    def __init__(self, config):
-        self.projects = []
-        self.config = config
-        self.projects_by_name = {}
-        graph_cache = bzrlib.lru_cache.LRUCache()
-        for project_name in self.config.sections:
-            project = Project(project_name, self.config[project_name],
-                              self.config, graph_cache)
-            self.projects.append(project)
-            self.projects_by_name[project_name] = project
-
-    def browse(self, response):
-        # This is insanely complicated because we want to open and
-        # lock all the branches, render the view and then unlock the
-        # branches again.
-        for p in self.projects:
-            p._recheck_auto_folders()
-
-        class branch(object):
-
-            @staticmethod
-            def static_url(path):
-                return self._static_url_base + path
-        views_by_project = {}
-        all_views = []
-        try:
-            for p in self.projects:
-                views_by_project[p] = []
-                for vn in p.view_names:
-                    v = p.view_named(vn)
-                    all_views.append(v)
-                    views_by_project[p].append(v)
-            vals = {
-                'projects': self.projects,
-                'util': util,
-                'title': self.config.get('title', None),
-                'branch': branch,
-                'views_by_project': views_by_project,
-            }
-            vals.update(templatefunctions)
-            response.headers['Content-Type'] = 'text/html'
-            template = load_template('loggerhead.templates.browse')
-            template.expand_into(response, **vals)
-        finally:
-            for v in all_views:
-                v.branch.unlock()
-
-    def __call__(self, environ, start_response):
-        self._static_url_base = environ['loggerhead.static.url'] = \
-                                environ['SCRIPT_NAME']
-        segment = path_info_pop(environ)
-        if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
-        elif segment == '':
-            response = WSGIResponse()
-            self.browse(response)
-            return response(environ, start_response)
-        elif segment == 'robots.txt':
-            return robots_app(environ, start_response)
-        elif segment == 'static':
-            return static_app(environ, start_response)
-        elif segment == 'favicon.ico':
-            return favicon_app(environ, start_response)
-        else:
-            project = self.projects_by_name.get(segment)
-            if project is None:
-                raise httpexceptions.HTTPNotFound()
-            return project.call(environ, start_response)

=== modified file 'loggerhead/apps/error.py'
--- loggerhead/apps/error.py	2008-10-24 02:26:05 +0000
+++ loggerhead/apps/error.py	2009-10-17 08:47:38 +0000
@@ -53,6 +53,6 @@
 
 
 def errapp(environ, start_response):
-    '''Default (and trivial) error handling WSGI application.'''
+    """Default (and trivial) error handling WSGI application."""
     c = ErrorUI(environ['branch'], environ['exc_info'])
     return c(environ, start_response)

=== added file 'loggerhead/apps/http_head.py'
--- loggerhead/apps/http_head.py	1970-01-01 00:00:00 +0000
+++ loggerhead/apps/http_head.py	2011-02-10 02:33:15 +0000
@@ -0,0 +1,62 @@
+# Copyright (C) 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+"""WSGI apps tend to return body content as part of a HEAD request.
+
+We should definitely not do that.
+"""
+
+class HeadMiddleware(object):
+    """When we get a HEAD request, we should not return body content.
+
+    WSGI defaults to just generating everything, and not paying attention to
+    whether it is a GET or a HEAD request. It does that because of potential
+    issues getting the Headers correct.
+
+    This middleware works by just omitting the body if the request method is
+    HEAD.
+    """
+
+    def __init__(self, app):
+        self._wrapped_app = app
+        self._real_environ = None
+        self._real_start_response = None
+        self._real_writer = None
+
+    def noop_write(self, chunk):
+        """We intentionally ignore all body content that is returned."""
+        pass
+
+    def start_response(self, status, response_headers, exc_info=None):
+        if exc_info is None:
+            self._real_writer = self._real_start_response(status,
+                response_headers)
+        else:
+            self._real_writer = self._real_start_response(status,
+                response_headers, exc_info)
+        return self.noop_write
+
+    def __call__(self, environ, start_response):
+        self._real_environ = environ
+        self._real_start_response = start_response
+        if environ.get('REQUEST_METHOD', 'GET') == 'HEAD':
+            result = self._wrapped_app(environ, self.start_response)
+            for chunk in result:
+                pass
+        else:
+            result = self._wrapped_app(environ, start_response)
+            for chunk in result:
+                yield chunk

=== renamed file 'loggerhead/apps/filesystem.py' => 'loggerhead/apps/transport.py'
--- loggerhead/apps/filesystem.py	2009-02-19 18:04:11 +0000
+++ loggerhead/apps/transport.py	2011-03-16 14:43:36 +0000
@@ -1,118 +1,199 @@
-"""Serve branches at urls that mimic the file system layout."""
-
-import os
-import tempfile
-
-from bzrlib import branch, errors, lru_cache
+# Copyright (C) 2008-2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+"""Serve branches at urls that mimic a transport's file system layout."""
+
+import threading
+
+from bzrlib import branch, errors, lru_cache, urlutils
+from bzrlib.config import LocationConfig
+from bzrlib.smart import request
+from bzrlib.transport import get_transport
+from bzrlib.transport.http import wsgi
 
 from paste.request import path_info_pop
 from paste import httpexceptions
 from paste import urlparser
 
+from loggerhead import util
 from loggerhead.apps.branch import BranchWSGIApp
-from loggerhead.apps import favicon_app, static_app
+from loggerhead.apps import favicon_app, robots_app, static_app
 from loggerhead.controllers.directory_ui import DirectoryUI
 
-sql_dir = tempfile.mkdtemp(prefix='loggerhead-cache-')
-
-
-class BranchesFromFileSystemServer(object):
-
-    def __init__(self, path, root, name=None):
-        self.path = path
+# TODO: Use bzrlib.ui.bool_from_string(), added in bzr 1.18
+_bools = {
+    'yes': True, 'no': False,
+    'on': True, 'off': False,
+    '1': True, '0': False,
+    'true': True, 'false': False,
+    }
+
+class BranchesFromTransportServer(object):
+
+    def __init__(self, transport, root, name=None):
+        self.transport = transport
         self.root = root
         self.name = name
+        self._config = root._config
 
     def app_for_branch(self, branch):
         if not self.name:
-            name = branch.get_config().get_nickname()
+            name = branch._get_nick(local=True)
             is_root = True
         else:
             name = self.name
             is_root = False
         branch_app = BranchWSGIApp(
-            branch, name, {'cachepath': sql_dir}, self.root.graph_cache,
-            is_root=is_root)
+            branch,
+            name,
+            {'cachepath': self._config.SQL_DIR},
+            self.root.graph_cache,
+            is_root=is_root,
+            use_cdn=self._config.get_option('use_cdn'),
+            )
         return branch_app.app
 
     def app_for_non_branch(self, environ):
         segment = path_info_pop(environ)
         if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
+            raise httpexceptions.HTTPMovedPermanently.relative_redirect(
+                environ['SCRIPT_NAME'] + '/', environ)
         elif segment == '':
             if self.name:
                 name = self.name
             else:
                 name = '/'
-            return DirectoryUI(environ['loggerhead.static.url'],
-                               self.path,
-                               name)
+            return DirectoryUI(
+                environ['loggerhead.static.url'], self.transport, name)
         else:
-            new_path = os.path.join(self.path, segment)
+            new_transport = self.transport.clone(segment)
             if self.name:
-                new_name = os.path.join(self.name, segment)
+                new_name = urlutils.join(self.name, segment)
             else:
                 new_name = '/' + segment
-            return BranchesFromFileSystemServer(new_path, self.root, new_name)
-
-    def __call__(self, environ, start_response):
-        if not os.path.isdir(self.path):
+            return BranchesFromTransportServer(new_transport, self.root, new_name)
+
+    def app_for_bazaar_data(self, relpath):
+        if relpath == '/.bzr/smart':
+            root_transport = get_transport_for_thread(self.root.base)
+            wsgi_app = wsgi.SmartWSGIApp(root_transport)
+            return wsgi.RelpathSetter(wsgi_app, '', 'loggerhead.path_info')
+        else:
+            # TODO: Use something here that uses the transport API
+            # rather than relying on the local filesystem API.
+            base = self.transport.base
+            try:
+                path = util.local_path_from_url(base)
+            except errors.InvalidURL:
+                raise httpexceptions.HTTPNotFound()
+            else:
+                return urlparser.make_static(None, path)
+
+    def check_serveable(self, config):
+        value = config.get_user_option('http_serve')
+        if value is None:
+            return
+        elif not _bools.get(value.lower(), True):
             raise httpexceptions.HTTPNotFound()
+
+    def __call__(self, environ, start_response):
+        path = environ['PATH_INFO']
         try:
-            b = branch.Branch.open(self.path)
+            b = branch.Branch.open_from_transport(self.transport)
         except errors.NotBranchError:
+            if path.startswith('/.bzr'):
+                self.check_serveable(LocationConfig(self.transport.base))
+                return self.app_for_bazaar_data(path)(environ, start_response)
+            if not self.transport.listable() or not self.transport.has('.'):
+                raise httpexceptions.HTTPNotFound()
             return self.app_for_non_branch(environ)(environ, start_response)
         else:
-            return self.app_for_branch(b)(environ, start_response)
-
-
-class BranchesFromFileSystemRoot(object):
-
-    def __init__(self, folder):
-        self.graph_cache = lru_cache.LRUCache()
-        self.folder = folder
+            self.check_serveable(b.get_config())
+            if path.startswith('/.bzr'):
+                return self.app_for_bazaar_data(path)(environ, start_response)
+            else:
+                return self.app_for_branch(b)(environ, start_response)
+
+
+_transport_store = threading.local()
+
+def get_transport_for_thread(base):
+    thread_transports = getattr(_transport_store, 'transports', None)
+    if thread_transports is None:
+        thread_transports = _transport_store.transports = {}
+    if base in thread_transports:
+        return thread_transports[base]
+    transport = get_transport(base)
+    thread_transports[base] = transport
+    return transport
+
+
+class BranchesFromTransportRoot(object):
+
+    def __init__(self, base, config):
+        self.graph_cache = lru_cache.LRUCache(10)
+        self.base = base
+        self._config = config
 
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
+        environ['loggerhead.path_info'] = environ['PATH_INFO']
         if environ['PATH_INFO'].startswith('/static/'):
             segment = path_info_pop(environ)
             assert segment == 'static'
             return static_app(environ, start_response)
         elif environ['PATH_INFO'] == '/favicon.ico':
             return favicon_app(environ, start_response)
-        elif '/.bzr/' in environ['PATH_INFO']:
-            app = urlparser.make_static(None, self.folder)
-            return app(environ, start_response)
+        elif environ['PATH_INFO'] == '/robots.txt':
+            return robots_app(environ, start_response)
         else:
-            return BranchesFromFileSystemServer(
-                self.folder, self)(environ, start_response)
-
-
-class UserBranchesFromFileSystemRoot(object):
-
-    def __init__(self, folder, trunk_dir):
-        self.graph_cache = lru_cache.LRUCache()
-        self.folder = folder
-        self.trunk_dir = trunk_dir
+            transport = get_transport_for_thread(self.base)
+            return BranchesFromTransportServer(
+                transport, self)(environ, start_response)
+
+
+class UserBranchesFromTransportRoot(object):
+
+    def __init__(self, base, config):
+        self.graph_cache = lru_cache.LRUCache(10)
+        self.base = base
+        self._config = config
+        self.trunk_dir = config.get_option('trunk_dir')
 
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
-        path_info= environ['PATH_INFO']
+        environ['loggerhead.path_info'] = environ['PATH_INFO']
+        path_info = environ['PATH_INFO']
         if path_info.startswith('/static/'):
             segment = path_info_pop(environ)
             assert segment == 'static'
             return static_app(environ, start_response)
         elif path_info == '/favicon.ico':
             return favicon_app(environ, start_response)
+        elif environ['PATH_INFO'] == '/robots.txt':
+            return robots_app(environ, start_response)
         else:
+            transport = get_transport_for_thread(self.base)
             # segments starting with ~ are user branches
             if path_info.startswith('/~'):
                 segment = path_info_pop(environ)
-                new_path = os.path.join(self.folder, segment[1:])
-                return BranchesFromFileSystemServer(
-                    new_path, self, segment)(environ, start_response)
+                return BranchesFromTransportServer(
+                    transport.clone(segment[1:]), self, segment)(
+                    environ, start_response)
             else:
-                new_path = os.path.join(self.folder, self.trunk_dir)
-                return BranchesFromFileSystemServer(
-                    new_path, self)(environ, start_response)
+                return BranchesFromTransportServer(
+                    transport.clone(self.trunk_dir), self)(
+                    environ, start_response)

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2008-10-24 02:26:05 +0000
+++ loggerhead/changecache.py	2009-05-02 14:01:05 +0000
@@ -17,46 +17,63 @@
 #
 
 """
-a cache for chewed-up "change" data structures, which are basically just a
-different way of storing a revision.  the cache improves lookup times 10x
-over bazaar's xml revision structure, though, so currently still worth doing.
+a cache for chewed-up 'file change' data structures, which are basically just
+a different way of storing a revision delta.  the cache improves lookup times
+10x over bazaar's xml revision structure, though, so currently still worth
+doing.
 
 once a revision is committed in bazaar, it never changes, so once we have
 cached a change, it's good forever.
 """
 
 import cPickle
+import marshal
 import os
-
-from loggerhead import util
-from loggerhead.lockfile import LockFile
-
-with_lock = util.with_lock('_lock', 'ChangeCache')
+import tempfile
+import zlib
 
 try:
     from sqlite3 import dbapi2
 except ImportError:
     from pysqlite2 import dbapi2
 
+# We take an optimistic approach to concurrency here: we might do work twice
+# in the case of races, but not crash or corrupt data.
+
+def safe_init_db(filename, init_sql):
+    # To avoid races around creating the database, we create the db in
+    # a temporary file and rename it into the ultimate location.
+    fd, temp_path = tempfile.mkstemp(dir=os.path.dirname(filename))
+    os.close(fd)
+    con = dbapi2.connect(temp_path)
+    cur = con.cursor()
+    cur.execute(init_sql)
+    con.commit()
+    con.close()
+    os.rename(temp_path, filename)
 
 class FakeShelf(object):
 
     def __init__(self, filename):
         create_table = not os.path.exists(filename)
+        if create_table:
+            safe_init_db(
+                filename, "create table RevisionData "
+                "(revid binary primary key, data binary)")
         self.connection = dbapi2.connect(filename)
         self.cursor = self.connection.cursor()
-        if create_table:
-            self._create_table()
 
-    def _create_table(self):
-        self.cursor.execute(
+    def _create_table(self, filename):
+        con = dbapi2.connect(filename)
+        cur = con.cursor()
+        cur.execute(
             "create table RevisionData "
             "(revid binary primary key, data binary)")
-        self.connection.commit()
+        con.commit()
+        con.close()
 
     def _serialize(self, obj):
-        r = dbapi2.Binary(cPickle.dumps(obj, protocol=2))
-        return r
+        return dbapi2.Binary(cPickle.dumps(obj, protocol=2))
 
     def _unserialize(self, data):
         return cPickle.loads(str(data))
@@ -70,49 +87,72 @@
         else:
             return self._unserialize(filechange[0])
 
-    def add(self, revid_obj_pairs):
-        for (r, d) in revid_obj_pairs:
+    def add(self, revid, object):
+        try:
             self.cursor.execute(
                 "insert into revisiondata (revid, data) values (?, ?)",
-                (r, self._serialize(d)))
-        self.connection.commit()
+                (revid, self._serialize(object)))
+            self.connection.commit()
+        except dbapi2.IntegrityError:
+            # If another thread or process attempted to set the same key, we
+            # assume it set it to the same value and carry on with our day.
+            pass
 
 
 class FileChangeCache(object):
 
-    def __init__(self, history, cache_path):
-        self.history = history
+    def __init__(self, cache_path):
 
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
 
         self._changes_filename = os.path.join(cache_path, 'filechanges.sql')
 
-        # use a lockfile since the cache folder could be shared across
-        # different processes.
-        self._lock = LockFile(os.path.join(cache_path, 'filechange-lock'))
-
-    @with_lock
-    def get_file_changes(self, entries):
-        out = []
-        missing_entries = []
-        missing_entry_indices = []
+    def get_file_changes(self, entry):
         cache = FakeShelf(self._changes_filename)
-        for entry in entries:
-            changes = cache.get(entry.revid)
-            if changes is not None:
-                out.append(changes)
-            else:
-                missing_entries.append(entry)
-                missing_entry_indices.append(len(out))
-                out.append(None)
-        if missing_entries:
-            missing_changes = self.history.get_file_changes_uncached(
-                                  missing_entries)
-            revid_changes_pairs = []
-            for i, entry, changes in zip(
-                missing_entry_indices, missing_entries, missing_changes):
-                revid_changes_pairs.append((entry.revid, changes))
-                out[i] = changes
-            cache.add(revid_changes_pairs)
-        return out
+        changes = cache.get(entry.revid)
+        if changes is None:
+            changes = self.history.get_file_changes_uncached(entry)
+            cache.add(entry.revid, changes)
+        return changes
+
+
+class RevInfoDiskCache(object):
+    """Like `RevInfoMemoryCache` but backed in a sqlite DB."""
+
+    def __init__(self, cache_path):
+        if not os.path.exists(cache_path):
+            os.mkdir(cache_path)
+        filename = os.path.join(cache_path, 'revinfo.sql')
+        create_table = not os.path.exists(filename)
+        if create_table:
+            safe_init_db(
+                filename, "create table Data "
+                "(key binary primary key, revid binary, data binary)")
+        self.connection = dbapi2.connect(filename)
+        self.cursor = self.connection.cursor()
+
+    def get(self, key, revid):
+        self.cursor.execute(
+            "select revid, data from data where key = ?", (dbapi2.Binary(key),))
+        row = self.cursor.fetchone()
+        if row is None:
+            return None
+        elif str(row[0]) != revid:
+            return None
+        else:
+            return marshal.loads(zlib.decompress(row[1]))
+
+    def set(self, key, revid, data):
+        try:
+            self.cursor.execute(
+                'delete from data where key = ?', (dbapi2.Binary(key), ))
+            blob = zlib.compress(marshal.dumps(data))
+            self.cursor.execute(
+                "insert into data (key, revid, data) values (?, ?, ?)",
+                map(dbapi2.Binary, [key, revid, blob]))
+            self.connection.commit()
+        except dbapi2.IntegrityError:
+            # If another thread or process attempted to set the same key, we
+            # don't care too much -- it's only a cache after all!
+            pass

=== added file 'loggerhead/config.py'
--- loggerhead/config.py	1970-01-01 00:00:00 +0000
+++ loggerhead/config.py	2010-05-12 14:38:05 +0000
@@ -0,0 +1,146 @@
+#
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+"""Configuration tools for Loggerhead."""
+
+from optparse import OptionParser
+import sys
+import tempfile
+
+from bzrlib import config
+
+_temporary_sql_dir = None
+
+def _get_temporary_sql_dir():
+    global _temporary_sql_dir
+    if _temporary_sql_dir is None:
+        _temporary_sql_dir = tempfile.mkdtemp(prefix='loggerhead-cache-')
+    return _temporary_sql_dir
+
+def command_line_parser():
+    parser = OptionParser("%prog [options] <path>")
+    parser.set_defaults(
+        user_dirs=False,
+        show_version=False,
+        log_folder=None,
+        use_cdn=False,
+        sql_dir=None,
+        allow_writes=False,
+        )
+    parser.add_option("--user-dirs", action="store_true",
+                      help="Serve user directories as ~user.")
+    parser.add_option("--trunk-dir", metavar="DIR",
+                      help="The directory that contains the trunk branches.")
+    parser.add_option("--port", dest="user_port",
+                      help=("Port Loggerhead should listen on "
+                            "(defaults to 8080)."))
+    parser.add_option("--host", dest="user_host",
+                      help="Host Loggerhead should listen on.")
+    parser.add_option("--protocol", dest="protocol",
+                      help=("Protocol to use: http, scgi, fcgi, ajp"
+                           "(defaults to http)."))
+    parser.add_option("--log-level", default=None, action='callback',
+                      callback=_optparse_level_to_int_level,
+                      type="string",
+                      help="Set the verbosity of logging. Can either"
+                           " be set to a numeric or string"
+                           " (eg, 10=debug, 30=warning)")
+    parser.add_option("--memory-profile", action="store_true",
+                      help="Profile the memory usage using Dozer.")
+    parser.add_option("--prefix", dest="user_prefix",
+                      help="Specify host prefix.")
+    parser.add_option("--profile", action="store_true",
+                      help="Generate callgrind profile data to "
+                        "%d-stats.callgrind on each request.")
+    parser.add_option("--reload", action="store_true",
+                      help="Restarts the application when changing python"
+                           " files. Only used for development purposes.")
+    parser.add_option("--log-folder",
+                      help="The directory to place log files in.")
+    parser.add_option("--version", action="store_true", dest="show_version",
+                      help="Print the software version and exit")
+    parser.add_option("--use-cdn", action="store_true", dest="use_cdn",
+                      help="Serve YUI from Yahoo!'s CDN")
+    parser.add_option("--cache-dir", dest="sql_dir",
+                      help="The directory to place the SQL cache in")
+    parser.add_option("--allow-writes", action="store_true",
+                      help="Allow writing to the Bazaar server.")
+    return parser
+
+
+_log_levels = {
+    'debug': 10,
+    'info': 20,
+    'warning': 30,
+    'error': 40,
+    'critical': 50,
+}
+
+def _optparse_level_to_int_level(option, opt_str, value, parser):
+    parser.values.log_level = _level_to_int_level(value)
+
+
+def _level_to_int_level(value):
+    """Convert a string level to an integer value."""
+    if value is None:
+        return None
+    try:
+        return int(value)
+    except ValueError:
+        pass
+    return _log_levels[value.lower()]
+
+
+class LoggerheadConfig(object):
+    """A configuration object."""
+
+    def __init__(self, argv=None):
+        if argv is None:
+            argv = sys.argv[1:]
+        self._parser = command_line_parser()
+        self._options, self._args = self._parser.parse_args(argv)
+
+        sql_dir = self.get_option('sql_dir')
+        if sql_dir is None:
+            sql_dir = _get_temporary_sql_dir()
+        self.SQL_DIR = sql_dir
+
+    def get_option(self, option):
+        """Get the value for the config option, either
+        from ~/.bazaar/bazaar.conf or from the command line.
+        All loggerhead-specific settings start with 'http_'
+        """
+        global_config = config.GlobalConfig().get_user_option('http_'+option)
+        cmd_config = getattr(self._options, option)
+        if global_config is not None and (
+            cmd_config is None or cmd_config is False):
+            return global_config
+        else:
+            return cmd_config
+
+    def get_log_level(self):
+        opt = self.get_option('log_level')
+        return _level_to_int_level(opt)
+
+    def get_arg(self, index):
+        """Get an arg from the arg list."""
+        return self._args[index]
+
+    def print_help(self):
+        """Wrapper around OptionParser.print_help."""
+        return self._parser.print_help()
+
+    @property
+    def arg_count(self):
+        """Return the number of args from the option parser."""
+        return len(self._args)

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2009-02-11 03:23:45 +0000
+++ loggerhead/controllers/__init__.py	2011-03-16 11:40:05 +0000
@@ -17,9 +17,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import re
+import bzrlib.errors
 import time
 
+from paste.httpexceptions import HTTPNotFound
 from paste.request import path_info_pop, parse_querystring
 
 from loggerhead import util
@@ -34,15 +35,10 @@
         self.buf = []
         self.buflen = 0
         self.writefunc = writefunc
-        self.bytes_saved = 0
         self.buf_limit = buf_limit
 
     def flush(self):
-        chunk = ''.join(self.buf)
-        chunk = re.sub(r'\s*\n\s*', '\n', chunk)
-        chunk = re.sub(r'[ \t]+', ' ', chunk)
-        self.bytes_saved += self.buflen - len(chunk)
-        self.writefunc(chunk)
+        self.writefunc(''.join(self.buf))
         self.buf = []
         self.buflen = 0
 
@@ -76,7 +72,7 @@
         kwargs = dict(parse_querystring(environ))
         util.set_context(kwargs)
         args = []
-        while 1:
+        while True:
             arg = path_info_pop(environ)
             if arg is None:
                 break
@@ -98,23 +94,22 @@
 
         vals.update(self.get_values(path, kwargs, headers))
 
-        self.log.info('Getting information for %s: %r secs' % (
+        self.log.info('Getting information for %s: %.3f secs' % (
             self.__class__.__name__, time.time() - z))
         if 'Content-Type' not in headers:
             headers['Content-Type'] = 'text/html'
         writer = start_response("200 OK", headers.items())
+        if environ.get('REQUEST_METHOD') == 'HEAD':
+            # No content for a HEAD request
+            return []
         template = load_template(self.template_path)
         z = time.time()
         w = BufferingWriter(writer, 8192)
         template.expand_into(w, **vals)
         w.flush()
         self.log.info(
-            'Rendering %s: %r secs, %s bytes, %s (%2.1f%%) bytes saved' % (
-                self.__class__.__name__,
-                time.time() - z,
-                w.bytes,
-                w.bytes_saved,
-                100.0*w.bytes_saved/w.bytes))
+            'Rendering %s: %.3f secs, %s bytes' % (
+                self.__class__.__name__, time.time() - z, w.bytes))
         return []
 
     def get_revid(self):
@@ -122,6 +117,10 @@
         if h is None:
             return None
         if len(self.args) > 0 and self.args != ['']:
-            return h.fix_revid(self.args[0])
+            try:
+                revid = h.fix_revid(self.args[0])
+            except bzrlib.errors.NoSuchRevision:
+                raise HTTPNotFound;
         else:
-            return h.last_revid
+            revid = h.last_revid
+        return revid

=== added file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/annotate_ui.py	2010-12-01 08:30:02 +0000
@@ -0,0 +1,54 @@
+#
+# Copyright (C) 2010 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+from loggerhead.controllers.view_ui import ViewUI
+from loggerhead import util
+
+class AnnotateUI(ViewUI):
+
+    def annotate_file(self, info):
+        file_id = info['file_id']
+        revid = info['change'].revid
+        
+        tree = self.tree_for(file_id, revid)
+        
+        change_cache = {}
+        last_line_revid = None
+        parity = 1
+        for line_revid, text in tree.annotate_iter(file_id):
+            if line_revid == last_line_revid:
+                # remember which lines have a new revno and which don't
+                new_rev = False
+            else:
+                new_rev = True
+                parity ^= 1
+                last_line_revid = line_revid
+                if line_revid in change_cache:
+                    change = change_cache[line_revid]
+                else:
+                    change = self._history.get_changes([line_revid])[0]
+                    change_cache[line_revid] = change
+
+            yield util.Container(
+                parity=parity, new_rev=new_rev, change=change)
+            
+    def get_values(self, path, kwargs, headers):
+        values = super(AnnotateUI, self).get_values(path, kwargs, headers)
+        values['annotated'] = self.annotate_file(values)
+        
+        return values

=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/atom_ui.py	2009-07-07 23:52:24 +0000
@@ -26,7 +26,6 @@
 
     def get_values(self, path, kwargs, headers):
         history = self._history
-        revid = self.get_revid()
         pagesize = int(20)#self._branch.config.get('pagesize', '20'))
 
         revid_list = history.get_file_view(history.last_revid, None)
@@ -35,6 +34,6 @@
         headers['Content-Type'] = 'application/atom+xml'
         return {
             'changes': entries,
-            'updated': entries[0].date.isoformat() + 'Z',
+            'updated': entries[0].utc_date.isoformat(),
             'history': self._history,
         }

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/changelog_ui.py	2011-03-12 13:25:52 +0000
@@ -1,4 +1,5 @@
 #
+# Copyright (C) 2008, 2009 Canonical Ltd.
 # Copyright (C) 2006  Robey Pointer <robey@lag.net>
 # Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
 #
@@ -17,6 +18,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
+import urllib
+
+import simplejson
+
 from paste.httpexceptions import HTTPServerError
 
 from loggerhead import util
@@ -42,7 +47,8 @@
 
         try:
             revid, start_revid, revid_list = history.get_view(
-                revid, start_revid, filter_file_id, query)
+                revid, start_revid, filter_file_id, query,
+                extra_rev_count=pagesize+1)
             util.set_context(kwargs)
 
             if (query is not None) and (len(revid_list) == 0):
@@ -58,7 +64,10 @@
                 scan_list = revid_list[i:]
             change_list = scan_list[:pagesize]
             changes = list(history.get_changes(change_list))
-            history.add_changes(changes)
+            data = {}
+            for i, c in enumerate(changes):
+                c.index = i
+                data[str(i)] = urllib.quote(urllib.quote(c.revid, safe=''))
         except:
             self.log.exception('Exception fetching changes')
             raise HTTPServerError('Could not fetch changes')
@@ -71,9 +80,6 @@
             navigation.query = query
         util.fill_in_navigation(navigation)
 
-        # add parent & merge-point branch-nick info, in case it's useful
-        history.get_branch_nicks(changes)
-
         # Directory Breadcrumbs
         directory_breadcrumbs = (
             util.directory_breadcrumbs(
@@ -81,16 +87,24 @@
                 self._branch.is_root,
                 'changes'))
 
+        show_tag_col = False
+        for change in changes:
+            if change.tags is not None:
+                show_tag_col = True
+                break
+
         return {
             'branch': self._branch,
             'changes': changes,
+            'show_tag_col': show_tag_col,
+            'data': simplejson.dumps(data),
             'util': util,
             'history': history,
             'revid': revid,
             'navigation': navigation,
             'filter_file_id': filter_file_id,
             'start_revid': start_revid,
-            'viewing_from': (orig_start_revid is not None) and 
+            'viewing_from': (orig_start_revid is not None) and
                             (orig_start_revid != history.last_revid),
             'query': query,
             'search_failed': search_failed,

=== modified file 'loggerhead/controllers/diff_ui.py'
--- loggerhead/controllers/diff_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/diff_ui.py	2011-03-18 09:39:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008  Canonical Ltd.
+# Copyright (C) 2008-2011 Canonical Ltd.
 #                     (Authored by Martin Albisetti <argentina@gmail.com>)
 #
 # This program is free software; you can redistribute it and/or modify
@@ -26,8 +26,6 @@
 
 from loggerhead.controllers import TemplatedBranchView
 
-log = logging.getLogger("loggerhead.controllers")
-
 
 class DiffUI(TemplatedBranchView):
     """Class to output a diff for a single file or revisions."""
@@ -38,7 +36,7 @@
         z = time.time()
 
         args = []
-        while 1:
+        while True:
             arg = path_info_pop(environ)
             if arg is None:
                 break
@@ -72,8 +70,8 @@
         filename = '%s_%s.diff' % (revno1, revno2)
         headers = [
             ('Content-Type', 'application/octet-stream'),
-            ('Content-Length', len(content)),
-            ('Content-Disposition', 'attachment; filename=%s' % filename),
+            ('Content-Length', str(len(content))),
+            ('Content-Disposition', 'attachment; filename=%s' % (filename,)),
             ]
         start_response('200 OK', headers)
         return [content]

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/directory_ui.py	2009-10-17 06:35:33 +0000
@@ -18,9 +18,9 @@
 
 import datetime
 import logging
-import os
+import stat
 
-from bzrlib import branch
+from bzrlib import branch, errors
 
 from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
@@ -48,7 +48,7 @@
 
     template_path = 'loggerhead.templates.directory'
 
-    def __init__(self, static_url_base, path, name):
+    def __init__(self, static_url_base, transport, name):
 
         class _branch(object):
             context_url = 1
@@ -57,24 +57,29 @@
             def static_url(path):
                 return self._static_url_base + path
         self._branch = _branch
-        self._history_callable = lambda:None
-        self._path = path
+        self._history_callable = lambda: None
         self._name = name
         self._static_url_base = static_url_base
+        self.transport = transport
         self.log = logging.getLogger('')
 
     def get_values(self, path, kwargs, response):
-        listing = [d for d in os.listdir(self._path)
-                   if not d.startswith('.')
-                   and os.path.isdir(os.path.join(self._path, d))]
+        listing = [d for d in self.transport.list_dir('.')
+                   if not d.startswith('.')]
         listing.sort(key=lambda x: x.lower())
         dirs = []
         parity = 0
         for d in listing:
-            p = os.path.join(self._path, d)
             try:
-                b = branch.Branch.open(p)
+                b = branch.Branch.open_from_transport(self.transport.clone(d))
+                if b.get_config().get_user_option('http_serve') == 'False':
+                    continue
             except:
+                try:
+                    if not stat.S_ISDIR(self.transport.stat(d).st_mode):
+                        continue
+                except errors.NoSuchFile:
+                    continue
                 b = None
             dirs.append(DirEntry(d, parity, b))
             parity = 1 - parity

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2009-02-06 05:53:36 +0000
+++ loggerhead/controllers/download_ui.py	2010-05-05 19:03:40 +0000
@@ -19,7 +19,6 @@
 
 import logging
 import mimetypes
-import time
 import urllib
 
 from paste import httpexceptions
@@ -34,20 +33,19 @@
 
     def __call__(self, environ, start_response):
         # /download/<rev_id>/<file_id>/[filename]
-        z = time.time()
 
         h = self._history
 
         args = []
-        while 1:
+        while True:
             arg = path_info_pop(environ)
             if arg is None:
                 break
             args.append(arg)
 
         if len(args) < 2:
-            raise httpexceptions.HTTPMovedPermanently(self._branch.url(
-                      '../changes'))
+            raise httpexceptions.HTTPMovedPermanently(
+                self._branch.absolute_url('/changes'))
 
         revid = h.fix_revid(args[0])
         file_id = args[1]
@@ -63,9 +61,9 @@
         encoded_filename = urllib.quote(filename.encode('utf-8'))
         headers = [
             ('Content-Type', mime_type),
-            ('Content-Length', len(content)),
+            ('Content-Length', str(len(content))),
             ('Content-Disposition',
-             'attachment; filename*=utf-8\'\'%s' % encoded_filename),
+             "attachment; filename*=utf-8''%s" % (encoded_filename,)),
             ]
         start_response('200 OK', headers)
         return [content]

=== modified file 'loggerhead/controllers/error_ui.py'
--- loggerhead/controllers/error_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/error_ui.py	2009-10-17 08:47:38 +0000
@@ -28,22 +28,19 @@
     template_path = 'loggerhead.templates.error'
 
     def __init__(self, branch, exc_info):
-        super(ErrorUI, self).__init__(branch, lambda:None)
+        super(ErrorUI, self).__init__(branch, lambda: None)
         self.exc_info = exc_info
 
     def get_values(self, path, kwargs, response):
         exc_type, exc_object, exc_tb = self.exc_info
         description = StringIO()
         traceback.print_exception(exc_type, exc_object, None, file=description)
-        directory_breadcrumbs = (
-            util.directory_breadcrumbs(
-                self._branch.friendly_name,
-                self._branch.is_root,
-                'changes'))
+        directory_breadcrumbs = util.directory_breadcrumbs(
+            self._branch.friendly_name, self._branch.is_root, 'changes')
         return {
             'branch': self._branch,
             'error_title': ('An unexpected error occurred while'
-                            'proccesing the request:'),
+                            'processing the request:'),
             'error_description': description.getvalue(),
             'directory_breadcrumbs': directory_breadcrumbs,
         }

=== added file 'loggerhead/controllers/filediff_ui.py'
--- loggerhead/controllers/filediff_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/filediff_ui.py	2009-07-07 23:52:24 +0000
@@ -0,0 +1,92 @@
+from StringIO import StringIO
+import urllib
+
+from bzrlib import diff
+from bzrlib import errors
+from bzrlib import osutils
+
+from loggerhead import util
+from loggerhead.controllers import TemplatedBranchView
+
+def _process_diff(difftext):
+    chunks = []
+    chunk = None
+    for line in difftext.splitlines():
+        if len(line) == 0:
+            continue
+        if line.startswith('+++ ') or line.startswith('--- '):
+            continue
+        if line.startswith('@@ '):
+            # new chunk
+            if chunk is not None:
+                chunks.append(chunk)
+            chunk = util.Container()
+            chunk.diff = []
+            split_lines = line.split(' ')[1:3]
+            lines = [int(x.split(',')[0][1:]) for x in split_lines]
+            old_lineno = lines[0]
+            new_lineno = lines[1]
+        elif line.startswith(' '):
+            chunk.diff.append(util.Container(old_lineno=old_lineno,
+                                             new_lineno=new_lineno,
+                                             type='context',
+                                             line=line[1:]))
+            old_lineno += 1
+            new_lineno += 1
+        elif line.startswith('+'):
+            chunk.diff.append(util.Container(old_lineno=None,
+                                             new_lineno=new_lineno,
+                                             type='insert', line=line[1:]))
+            new_lineno += 1
+        elif line.startswith('-'):
+            chunk.diff.append(util.Container(old_lineno=old_lineno,
+                                             new_lineno=None,
+                                             type='delete', line=line[1:]))
+            old_lineno += 1
+        else:
+            chunk.diff.append(util.Container(old_lineno=None,
+                                             new_lineno=None,
+                                             type='unknown',
+                                             line=repr(line)))
+    if chunk is not None:
+        chunks.append(chunk)
+    return chunks
+
+
+def diff_chunks_for_file(repository, file_id, compare_revid, revid):
+    lines = {}
+    args = []
+    for r in (compare_revid, revid):
+        if r == 'null:':
+            lines[r] = []
+        else:
+            args.append((file_id, r, r))
+    for r, bytes_iter in repository.iter_files_bytes(args):
+        lines[r] = osutils.split_lines(''.join(bytes_iter))
+    buffer = StringIO()
+    try:
+        diff.internal_diff('', lines[compare_revid], '', lines[revid], buffer)
+    except errors.BinaryFile:
+        difftext = ''
+    else:
+        difftext = buffer.getvalue()
+
+    return _process_diff(difftext)
+
+
+class FileDiffUI(TemplatedBranchView):
+
+    template_path = 'loggerhead.templates.filediff'
+
+    def get_values(self, path, kwargs, headers):
+        revid = urllib.unquote(self.args[0])
+        compare_revid = urllib.unquote(self.args[1])
+        file_id = urllib.unquote(self.args[2])
+
+        chunks = diff_chunks_for_file(
+            self._history._branch.repository, file_id, compare_revid, revid)
+
+        return {
+            'util': util,
+            'chunks': chunks,
+        }

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2009-02-11 03:25:46 +0000
+++ loggerhead/controllers/inventory_ui.py	2010-07-12 15:12:04 +0000
@@ -22,7 +22,7 @@
 import posixpath
 import urllib
 
-from paste.httpexceptions import HTTPNotFound
+from paste.httpexceptions import HTTPNotFound, HTTPMovedPermanently
 
 from bzrlib import errors
 from bzrlib.revision import is_null as is_null_rev
@@ -31,8 +31,6 @@
 from loggerhead.controllers import TemplatedBranchView
 
 
-log = logging.getLogger("loggerhead.controllers")
-
 
 def dirname(path):
     if path is not None:
@@ -45,7 +43,7 @@
 
     template_path = 'loggerhead.templates.inventory'
 
-    def get_filelist(self, inv, path, sort_type='filename'):
+    def get_filelist(self, inv, path, sort_type, revno_url):
         """
         return the list of all files (and their attributes) within a given
         path subtree.
@@ -58,6 +56,9 @@
         dir_ie = inv[file_id]
         file_list = []
 
+        if dir_ie.kind != 'directory':
+            raise HTTPMovedPermanently(self._branch.context_url(['/view', revno_url, path]))
+
         revid_set = set()
 
         for filename, entry in dir_ie.children.iteritems():
@@ -74,7 +75,7 @@
             if path == '':
                 absolutepath = pathname
             else:
-                absolutepath = urllib.quote(path + '/' + pathname)
+                absolutepath = path + '/' + pathname
             revid = entry.revision
 
             file = util.Container(
@@ -107,12 +108,13 @@
 
         file_id = kwargs.get('file_id', None)
         start_revid = kwargs.get('start_revid', None)
-        sort_type = kwargs.get('sort', None)
+        sort_type = kwargs.get('sort', 'filename')
 
         # no navbar for revisions
         navigation = util.Container()
 
         if path is not None:
+            path = path.rstrip('/')
             file_id = rev_tree.path2id(path)
             if file_id is None:
                 raise HTTPNotFound()
@@ -145,12 +147,11 @@
                 revno_url = 'head:'
             else:
                 revno_url = history.get_revno(revid)
-            # add parent & merge-point branch-nick info, in case it's useful
-            history.get_branch_nicks([ change ])
+            history.add_branch_nicks(change)
 
             # Create breadcrumb trail for the path within the branch
             branch_breadcrumbs = util.branch_breadcrumbs(path, rev_tree, 'files')
-            filelist = self.get_filelist(rev_tree.inventory, path, sort_type)
+            filelist = self.get_filelist(rev_tree.inventory, path, sort_type, revno_url)
         else:
             start_revid = None
             change = None

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2009-02-13 02:47:13 +0000
+++ loggerhead/controllers/revision_ui.py	2011-03-02 14:07:21 +0000
@@ -17,205 +17,26 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import re
-from StringIO import StringIO
-
-import bzrlib.diff
+import simplejson
+import urllib
 
 from paste.httpexceptions import HTTPServerError
 
 from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
-from loggerhead.history import rich_filename
+from loggerhead.controllers.filediff_ui import diff_chunks_for_file
 
 
 DEFAULT_LINE_COUNT_LIMIT = 3000
 
-def _process_side_by_side_buffers(line_list, delete_list, insert_list):
-    while len(delete_list) < len(insert_list):
-        delete_list.append((None, '', 'context'))
-    while len(insert_list) < len(delete_list):
-        insert_list.append((None, '', 'context'))
-    while len(delete_list) > 0:
-        d = delete_list.pop(0)
-        i = insert_list.pop(0)
-        line_list.append(util.Container(old_lineno=d[0], new_lineno=i[0],
-                                        old_line=d[1], new_line=i[1],
-                                        old_type=d[2], new_type=i[2]))
-
-
-def _make_side_by_side(chunk_list):
-    """
-    turn a normal unified-style diff (post-processed by parse_delta) into a
-    side-by-side diff structure.  the new structure is::
-
-        chunks: list(
-            diff: list(
-                old_lineno: int,
-                new_lineno: int,
-                old_line: str,
-                new_line: str,
-                type: str('context' or 'changed'),
-            )
-        )
-    """
-    out_chunk_list = []
-    for chunk in chunk_list:
-        line_list = []
-        wrap_char = '<wbr/>'
-        delete_list, insert_list = [], []
-        for line in chunk.diff:
-            # Add <wbr/> every X characters so we can wrap properly
-            wrap_line = re.findall(r'.{%d}|.+$' % 78, line.line)
-            wrap_lines = [util.html_clean(_line) for _line in wrap_line]
-            wrapped_line = wrap_char.join(wrap_lines)
-
-            if line.type == 'context':
-                if len(delete_list) or len(insert_list):
-                    _process_side_by_side_buffers(line_list, delete_list,
-                                                  insert_list)
-                    delete_list, insert_list = [], []
-                line_list.append(util.Container(old_lineno=line.old_lineno,
-                                                new_lineno=line.new_lineno,
-                                                old_line=wrapped_line,
-                                                new_line=wrapped_line,
-                                                old_type=line.type,
-                                                new_type=line.type))
-            elif line.type == 'delete':
-                delete_list.append((line.old_lineno, wrapped_line, line.type))
-            elif line.type == 'insert':
-                insert_list.append((line.new_lineno, wrapped_line, line.type))
-        if len(delete_list) or len(insert_list):
-            _process_side_by_side_buffers(line_list, delete_list, insert_list)
-        out_chunk_list.append(util.Container(diff=line_list))
-    return out_chunk_list
+def dq(p):
+    return urllib.quote(urllib.quote(p, safe=''))
+
 
 class RevisionUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.revision'
 
-    def _process_diff(self, diff):
-        # doesn't really need to be a method; could be static.
-        chunks = []
-        chunk = None
-        for line in diff.splitlines():
-            if len(line) == 0:
-                continue
-            if line.startswith('+++ ') or line.startswith('--- '):
-                continue
-            if line.startswith('@@ '):
-                # new chunk
-                if chunk is not None:
-                    chunks.append(chunk)
-                chunk = util.Container()
-                chunk.diff = []
-                split_lines = line.split(' ')[1:3]
-                lines = [int(x.split(',')[0][1:]) for x in split_lines]
-                old_lineno = lines[0]
-                new_lineno = lines[1]
-            elif line.startswith(' '):
-                chunk.diff.append(util.Container(old_lineno=old_lineno,
-                                                 new_lineno=new_lineno,
-                                                 type='context',
-                                                 line=line[1:]))
-                old_lineno += 1
-                new_lineno += 1
-            elif line.startswith('+'):
-                chunk.diff.append(util.Container(old_lineno=None,
-                                                 new_lineno=new_lineno,
-                                                 type='insert', line=line[1:]))
-                new_lineno += 1
-            elif line.startswith('-'):
-                chunk.diff.append(util.Container(old_lineno=old_lineno,
-                                                 new_lineno=None,
-                                                 type='delete', line=line[1:]))
-                old_lineno += 1
-            else:
-                chunk.diff.append(util.Container(old_lineno=None,
-                                                 new_lineno=None,
-                                                 type='unknown',
-                                                 line=repr(line)))
-        if chunk is not None:
-            chunks.append(chunk)
-        return chunks
-
-    def _parse_diffs(self, old_tree, new_tree, delta):
-        """
-        Return a list of processed diffs, in the format::
-
-            list(
-                filename: str,
-                file_id: str,
-                chunks: list(
-                    diff: list(
-                        old_lineno: int,
-                        new_lineno: int,
-                        type: str('context', 'delete', or 'insert'),
-                        line: str,
-                    ),
-                ),
-            )
-        """
-        process = []
-        out = []
-
-        for old_path, new_path, fid, \
-            kind, text_modified, meta_modified in delta.renamed:
-            if text_modified:
-                process.append((old_path, new_path, fid, kind))
-        for path, fid, kind, text_modified, meta_modified in delta.modified:
-            process.append((path, path, fid, kind))
-
-        for old_path, new_path, fid, kind in process:
-            old_lines = old_tree.get_file_lines(fid)
-            new_lines = new_tree.get_file_lines(fid)
-            buffer = StringIO()
-            if old_lines != new_lines:
-                try:
-                    bzrlib.diff.internal_diff(old_path, old_lines,
-                                              new_path, new_lines, buffer)
-                except bzrlib.errors.BinaryFile:
-                    diff = ''
-                else:
-                    diff = buffer.getvalue()
-            else:
-                diff = ''
-            out.append(util.Container(
-                          filename=rich_filename(new_path, kind),
-                          file_id=fid,
-                          chunks=self._process_diff(diff),
-                          raw_diff=diff))
-
-        return out
-
-    def get_change_with_diff(self, revid, compare_revid):
-        h = self._history
-        change = h.get_changes([revid])[0]
-
-        if compare_revid is None:
-            if change.parents:
-                compare_revid = change.parents[0].revid
-            else:
-                compare_revid = 'null:'
-
-        rev_tree1 = h._branch.repository.revision_tree(compare_revid)
-        rev_tree2 = h._branch.repository.revision_tree(revid)
-        delta = rev_tree2.changes_from(rev_tree1)
-
-        change.changes = h.parse_delta(delta)
-        change.changes.modified = self._parse_diffs(rev_tree1,
-                                                    rev_tree2,
-                                                    delta)
-
-        return change
-
-    @staticmethod
-    def add_side_by_side(changes):
-        # FIXME: this is a rotten API.
-        for change in changes:
-            for m in change.changes.modified:
-                m.sbs_chunks = _make_side_by_side(m.chunks)
-
     def get_values(self, path, kwargs, headers):
         h = self._history
         revid = self.get_revid()
@@ -243,21 +64,45 @@
             navigation.query = query
         util.fill_in_navigation(navigation)
 
-        change = self.get_change_with_diff(revid, compare_revid)
-        # add parent & merge-point branch-nick info, in case it's useful
-        h.get_branch_nicks([change])
-
-        line_count_limit = DEFAULT_LINE_COUNT_LIMIT
-        line_count = 0
-        for file in change.changes.modified:
-            for chunk in file.chunks:
-                line_count += len(chunk.diff)
-
-        # let's make side-by-side diff be the default
-        # FIXME: not currently in use. Should be
-        side_by_side = not kwargs.get('unified', False)
-        if side_by_side:
-            self.add_side_by_side([change])
+        change = h.get_changes([revid])[0]
+
+        if compare_revid is None:
+            file_changes = h.get_file_changes(change)
+        else:
+            file_changes = h.file_changes_for_revision_ids(
+                compare_revid, change.revid)
+
+        if path in ('', '/'):
+            path = None
+
+        link_data = {}
+        path_to_id = {}
+        if path:
+            item = [x for x in file_changes.text_changes if x.filename == path][0]
+            diff_chunks = diff_chunks_for_file(
+                self._history._branch.repository, item.file_id,
+                item.old_revision, item.new_revision)
+        else:
+            diff_chunks = None
+            for i, item in enumerate(file_changes.text_changes):
+                item.index = i
+                link_data['diff-' + str(i)] = '%s/%s/%s' % (
+                    dq(item.new_revision), dq(item.old_revision), dq(item.file_id))
+                path_to_id[item.filename] = 'diff-' + str(i)
+
+        h.add_branch_nicks(change)
+
+        if '.' in change.revno:
+            # Walk "up" though the merge-sorted graph until we find a
+            # revision with merge depth 0: this is the revision that merged
+            # this one to mainline.
+            ri = self._history._rev_info
+            i = self._history._rev_indices[change.revid]
+            while ri[i][0][2] > 0:
+                i -= 1
+            merged_in = ri[i][0][3]
+        else:
+            merged_in = None
 
         # Directory Breadcrumbs
         directory_breadcrumbs = (
@@ -270,18 +115,21 @@
             'branch': self._branch,
             'revid': revid,
             'change': change,
+            'file_changes': file_changes,
+            'diff_chunks': diff_chunks,
+            'link_data': simplejson.dumps(link_data),
+            'specific_path': path,
+            'json_specific_path': simplejson.dumps(path),
+            'path_to_id': simplejson.dumps(path_to_id),
             'start_revid': start_revid,
             'filter_file_id': filter_file_id,
             'util': util,
             'history': h,
+            'merged_in': merged_in,
             'navigation': navigation,
             'query': query,
             'remember': remember,
             'compare_revid': compare_revid,
-            'side_by_side': side_by_side,
             'url': self._branch.context_url,
-            'line_count': line_count,
-            'line_count_limit': line_count_limit,
-            'show_plain_diffs': line_count > line_count_limit,
             'directory_breadcrumbs': directory_breadcrumbs,
         }

=== added file 'loggerhead/controllers/revlog_ui.py'
--- loggerhead/controllers/revlog_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/revlog_ui.py	2009-03-19 00:44:05 +0000
@@ -0,0 +1,27 @@
+import urllib
+
+from loggerhead import util
+from loggerhead.controllers import TemplatedBranchView
+
+
+class RevLogUI(TemplatedBranchView):
+
+    template_path = 'loggerhead.templates.revlog'
+
+    def get_values(self, path, kwargs, headers):
+        history = self._history
+
+        revid = urllib.unquote(self.args[0])
+
+        change = history.get_changes([revid])[0]
+        file_changes = history.get_file_changes(change)
+        history.add_branch_nicks(change)
+
+        return {
+            'branch': self._branch,
+            'entry': change,
+            'file_changes': file_changes,
+            'util': util,
+            'revid': revid,
+            'url': self._branch.context_url,
+        }

=== renamed file 'loggerhead/controllers/annotate_ui.py' => 'loggerhead/controllers/view_ui.py'
--- loggerhead/controllers/annotate_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/view_ui.py	2011-03-23 02:37:07 +0000
@@ -18,33 +18,74 @@
 #
 
 import os
-import posixpath
-
-from paste.httpexceptions import HTTPBadRequest, HTTPServerError
+import time
+
+import bzrlib.errors
+import bzrlib.textfile
+import bzrlib.osutils
+
+from paste.httpexceptions import HTTPBadRequest, HTTPServerError, HTTPMovedPermanently
 
 from loggerhead.controllers import TemplatedBranchView
+try:
+    from loggerhead.highlight import highlight
+except ImportError:
+    highlight = None
 from loggerhead import util
 
 
-def dirname(path):
-    while path.endswith('/'):
-        path = path[:-1]
-    path = posixpath.dirname(path)
-    return path
-
-
-class AnnotateUI (TemplatedBranchView):
-
-    template_path = 'loggerhead.templates.annotate'
+class ViewUI(TemplatedBranchView):
+
+    template_path = 'loggerhead.templates.view'
+    
+    def tree_for(self, file_id, revid):
+        file_revid = self._history.get_inventory(revid)[file_id].revision
+        return self._history._branch.repository.revision_tree(file_revid)
+
+    def text_lines(self, file_id, revid):
+        file_name = os.path.basename(self._history.get_path(revid, file_id))
+        
+        tree = self.tree_for(file_id, revid)
+        file_text = tree.get_file_text(file_id)
+        encoding = 'utf-8'
+        try:
+            file_text = file_text.decode(encoding)
+        except UnicodeDecodeError:
+            encoding = 'iso-8859-15'
+            file_text = file_text.decode(encoding)
+
+        file_lines = bzrlib.osutils.split_lines(file_text)
+        # This can throw bzrlib.errors.BinaryFile (which our caller catches).
+        bzrlib.textfile.check_text_lines(file_lines)
+        
+        if highlight is not None:
+            hl_lines = highlight(file_name, file_text, encoding)
+            # highlight strips off extra newlines at the end of the file.
+            extra_lines = len(file_lines) - len(hl_lines)
+            hl_lines.extend([u''] * extra_lines)
+        else:
+            hl_lines = map(util.html_escape, file_lines)
+        
+        return hl_lines;
+
+    def file_contents(self, file_id, revid):
+        try:
+            file_lines = self.text_lines(file_id, revid)
+        except bzrlib.errors.BinaryFile:
+            # bail out; this isn't displayable text
+            return ['(This is a binary file.)']
+
+        return file_lines
 
     def get_values(self, path, kwargs, headers):
         history = self._history
+        branch = history._branch
         revid = self.get_revid()
         revid = history.fix_revid(revid)
         file_id = kwargs.get('file_id', None)
         if (file_id is None) and (path is None):
             raise HTTPBadRequest('No file_id or filename '
-                                 'provided to annotate')
+                                 'provided to view')
 
         if file_id is None:
             file_id = history.get_file_id(revid, path)
@@ -58,7 +99,7 @@
 
         change = history.get_changes([ revid ])[0]
         # If we're looking at the tip, use head: in the URL instead
-        if revid == history.last_revid:
+        if revid == branch.last_revision():
             revno_url = 'head:'
         else:
             revno_url = history.get_revno(revid)
@@ -78,17 +119,22 @@
             raise HTTPServerError('Could not fetch changes')
         branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
 
+        if inv[file_id].kind == "directory":
+            raise HTTPMovedPermanently(self._branch.context_url(['/files', revno_url, path]))
+
         return {
-            'revid': revid,
+            # In AnnotateUI, "annotated" is a generator giving revision
+            # numbers per lines, but the template checks if "annotated" is
+            # true or not before using it, so we have to define it here also.
+            'annotated': False,
             'revno_url': revno_url,
             'file_id': file_id,
-            'path': path,
+            'file_path': path,
             'filename': filename,
             'navigation': navigation,
             'change': change,
-            'contents': list(history.annotate_file(file_id, revid)),
+            'contents':  self.file_contents(file_id, revid),
             'fileview_active': True,
             'directory_breadcrumbs': directory_breadcrumbs,
             'branch_breadcrumbs': branch_breadcrumbs,
-            'history': self._history,
         }

=== modified file 'loggerhead/daemon.py'
--- loggerhead/daemon.py	2008-12-07 15:24:05 +0000
+++ loggerhead/daemon.py	2009-05-13 14:03:48 +0000
@@ -53,7 +53,7 @@
             pass
 
     f = open(pidfile, 'w')
-    f.write('%d\n' % os.getpid())
+    f.write('%d\n' % (os.getpid(),))
     f.close()
 
 

=== added file 'loggerhead/highlight.py'
--- loggerhead/highlight.py	1970-01-01 00:00:00 +0000
+++ loggerhead/highlight.py	2010-12-01 08:30:02 +0000
@@ -0,0 +1,55 @@
+#
+# Copyright (C) 2009  Peter Bui <pnutzh4x0r@gmail.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+import bzrlib.osutils
+import cgi
+
+from pygments import highlight as _highlight_func
+from pygments.lexers import guess_lexer, guess_lexer_for_filename, TextLexer
+from pygments.formatters import HtmlFormatter
+from pygments.util import ClassNotFound
+
+DEFAULT_PYGMENT_STYLE = 'colorful'
+
+# Trying to highlight very large files using pygments was killing
+# loggerhead on launchpad.net, because pygments isn't very fast.
+# So we only highlight files if they're 512K or smaller.
+MAX_HIGHLIGHT_SIZE = 512000;
+
+def highlight(path, text, encoding, style=DEFAULT_PYGMENT_STYLE):
+    """
+    Returns a list of highlighted (i.e. HTML formatted) strings.
+    """
+
+    if len(text) > MAX_HIGHLIGHT_SIZE:
+        return map(cgi.escape,  bzrlib.osutils.split_lines(text))
+
+    formatter = HtmlFormatter(style=style, nowrap=True, classprefix='pyg-')
+
+    try:
+        lexer = guess_lexer_for_filename(path, text[:1024], encoding=encoding)
+    except (ClassNotFound, ValueError):
+        try:
+            lexer = guess_lexer(text[:1024], encoding=encoding)
+        except (ClassNotFound, ValueError):
+            lexer = TextLexer(encoding=encoding)
+
+    hl_lines = _highlight_func(text, lexer, formatter)
+    hl_lines = bzrlib.osutils.split_lines(hl_lines)
+
+    return hl_lines

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-02-13 02:47:13 +0000
+++ loggerhead/history.py	2011-03-16 14:43:36 +0000
@@ -1,5 +1,4 @@
-#
-# Copyright (C) 2008  Canonical Ltd.
+# Copyright (C) 2006-2011 Canonical Ltd.
 #                     (Authored by Martin Albisetti <argentina@gmail.com>)
 # Copyright (C) 2006  Robey Pointer <robey@lag.net>
 # Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
@@ -34,38 +33,18 @@
 import re
 import textwrap
 import threading
-import time
-import urllib
-from StringIO import StringIO
+
+from bzrlib import tag
+import bzrlib.branch
+import bzrlib.delta
+import bzrlib.errors
+import bzrlib.foreign
+import bzrlib.revision
 
 from loggerhead import search
 from loggerhead import util
 from loggerhead.wholehistory import compute_whole_history_data
 
-import bzrlib
-import bzrlib.branch
-import bzrlib.diff
-import bzrlib.errors
-import bzrlib.progress
-import bzrlib.revision
-import bzrlib.textfile
-import bzrlib.tsort
-import bzrlib.ui
-
-# bzrlib's UIFactory is not thread-safe
-uihack = threading.local()
-
-
-class ThreadSafeUIFactory (bzrlib.ui.SilentUIFactory):
-
-    def nested_progress_bar(self):
-        if getattr(uihack, '_progress_bar_stack', None) is None:
-            pbs = bzrlib.progress.ProgressBarStack(
-                      klass=bzrlib.progress.DummyProgress)
-            uihack._progress_bar_stack = pbs
-        return uihack._progress_bar_stack.get_nested()
-
-bzrlib.ui.ui_factory = ThreadSafeUIFactory()
 
 def is_branch(folder):
     try:
@@ -82,7 +61,7 @@
     module (Robey, the original author of this code, apparently favored this
     style of message).
     """
-    message = message.splitlines()
+    message = message.lstrip().splitlines()
 
     if len(message) == 1:
         message = textwrap.wrap(message[0])
@@ -109,9 +88,6 @@
     return path
 
 
-# from bzrlib
-
-
 class _RevListToTimestamps(object):
     """This takes a list of revisions, and allows you to bisect by date"""
 
@@ -129,8 +105,109 @@
     def __len__(self):
         return len(self.revid_list)
 
-
-class History (object):
+class FileChangeReporter(object):
+
+    def __init__(self, old_inv, new_inv):
+        self.added = []
+        self.modified = []
+        self.renamed = []
+        self.removed = []
+        self.text_changes = []
+        self.old_inv = old_inv
+        self.new_inv = new_inv
+
+    def revid(self, inv, file_id):
+        try:
+            return inv[file_id].revision
+        except bzrlib.errors.NoSuchId:
+            return 'null:'
+
+    def report(self, file_id, paths, versioned, renamed, modified,
+               exe_change, kind):
+        if modified not in ('unchanged', 'kind changed'):
+            if versioned == 'removed':
+                filename = rich_filename(paths[0], kind[0])
+            else:
+                filename = rich_filename(paths[1], kind[1])
+            self.text_changes.append(util.Container(
+                filename=filename, file_id=file_id,
+                old_revision=self.revid(self.old_inv, file_id),
+                new_revision=self.revid(self.new_inv, file_id)))
+        if versioned == 'added':
+            self.added.append(util.Container(
+                filename=rich_filename(paths[1], kind),
+                file_id=file_id, kind=kind[1]))
+        elif versioned == 'removed':
+            self.removed.append(util.Container(
+                filename=rich_filename(paths[0], kind),
+                file_id=file_id, kind=kind[0]))
+        elif renamed:
+            self.renamed.append(util.Container(
+                old_filename=rich_filename(paths[0], kind[0]),
+                new_filename=rich_filename(paths[1], kind[1]),
+                file_id=file_id,
+                text_modified=modified == 'modified'))
+        else:
+            self.modified.append(util.Container(
+                filename=rich_filename(paths[1], kind),
+                file_id=file_id))
+
+# The lru_cache is not thread-safe, so we need a lock around it for
+# all threads.
+rev_info_memory_cache_lock = threading.RLock()
+
+class RevInfoMemoryCache(object):
+    """A store that validates values against the revids they were stored with.
+
+    We use a unique key for each branch.
+
+    The reason for not just using the revid as the key is so that when a new
+    value is provided for a branch, we replace the old value used for the
+    branch.
+
+    There is another implementation of the same interface in
+    loggerhead.changecache.RevInfoDiskCache.
+    """
+
+    def __init__(self, cache):
+        self._cache = cache
+
+    def get(self, key, revid):
+        """Return the data associated with `key`, subject to a revid check.
+
+        If a value was stored under `key`, with the same revid, return it.
+        Otherwise return None.
+        """
+        rev_info_memory_cache_lock.acquire()
+        try:
+            cached = self._cache.get(key)
+        finally:
+            rev_info_memory_cache_lock.release()
+        if cached is None:
+            return None
+        stored_revid, data = cached
+        if revid == stored_revid:
+            return data
+        else:
+            return None
+
+    def set(self, key, revid, data):
+        """Store `data` under `key`, to be checked against `revid` on get().
+        """
+        rev_info_memory_cache_lock.acquire()
+        try:
+            self._cache[key] = (revid, data)
+        finally:
+            rev_info_memory_cache_lock.release()
+
+# Used to store locks that prevent multiple threads from building a 
+# revision graph for the same branch at the same time, because that can
+# cause severe performance issues that are so bad that the system seems
+# to hang.
+revision_graph_locks = {}
+revision_graph_check_lock = threading.Lock()
+
+class History(object):
     """Decorate a branch to provide information for rendering.
 
     History objects are expected to be short lived -- when serving a request
@@ -138,31 +215,96 @@
     around it, serve the request, throw the History object away, unlock the
     branch and throw it away.
 
-    :ivar _file_change_cache: xx
+    :ivar _file_change_cache: An object that caches information about the
+        files that changed between two revisions.
+    :ivar _rev_info: A list of information about revisions.  This is by far
+        the most cryptic data structure in loggerhead.  At the top level, it
+        is a list of 3-tuples [(merge-info, where-merged, parents)].
+        `merge-info` is (seq, revid, merge_depth, revno_str, end_of_merge) --
+        like a merged sorted list, but the revno is stringified.
+        `where-merged` is a tuple of revisions that have this revision as a
+        non-lefthand parent.  Finally, `parents` is just the usual list of
+        parents of this revision.
+    :ivar _rev_indices: A dictionary mapping each revision id to the index of
+        the information about it in _rev_info.
+    :ivar _revno_revid: A dictionary mapping stringified revnos to revision
+        ids.
     """
 
-    def __init__(self, branch, whole_history_data_cache):
+    def _load_whole_history_data(self, caches, cache_key):
+        """Set the attributes relating to the whole history of the branch.
+
+        :param caches: a list of caches with interfaces like
+            `RevInfoMemoryCache` and be ordered from fastest to slowest.
+        :param cache_key: the key to use with the caches.
+        """
+        self._rev_indices = None
+        self._rev_info = None
+
+        missed_caches = []
+        def update_missed_caches():
+            for cache in missed_caches:
+                cache.set(cache_key, self.last_revid, self._rev_info)
+
+        # Theoretically, it's possible for two threads to race in creating
+        # the Lock() object for their branch, so we put a lock around
+        # creating the per-branch Lock().
+        revision_graph_check_lock.acquire()
+        try:
+            if cache_key not in revision_graph_locks:
+                revision_graph_locks[cache_key] = threading.Lock()
+        finally:
+            revision_graph_check_lock.release()
+
+        revision_graph_locks[cache_key].acquire()
+        try:
+            for cache in caches:
+                data = cache.get(cache_key, self.last_revid)
+                if data is not None:
+                    self._rev_info = data
+                    update_missed_caches()
+                    break
+                else:
+                    missed_caches.append(cache)
+            else:
+                whole_history_data = compute_whole_history_data(self._branch)
+                self._rev_info, self._rev_indices = whole_history_data
+                update_missed_caches()
+        finally:
+            revision_graph_locks[cache_key].release()
+
+        if self._rev_indices is not None:
+            self._revno_revid = {}
+            for ((_, revid, _, revno_str, _), _, _) in self._rev_info:
+                self._revno_revid[revno_str] = revid
+        else:
+            self._revno_revid = {}
+            self._rev_indices = {}
+            for ((seq, revid, _, revno_str, _), _, _) in self._rev_info:
+                self._rev_indices[revid] = seq
+                self._revno_revid[revno_str] = revid
+
+    def __init__(self, branch, whole_history_data_cache, file_cache=None,
+                 revinfo_disk_cache=None, cache_key=None):
         assert branch.is_locked(), (
             "Can only construct a History object with a read-locked branch.")
-        self._file_change_cache = None
+        if file_cache is not None:
+            self._file_change_cache = file_cache
+            file_cache.history = self
+        else:
+            self._file_change_cache = None
         self._branch = branch
+        self._branch_tags = None
         self._inventory_cache = {}
         self._branch_nick = self._branch.get_config().get_nickname()
-        self.log = logging.getLogger('loggerhead.%s' % self._branch_nick)
+        self.log = logging.getLogger('loggerhead.%s' % (self._branch_nick,))
 
         self.last_revid = branch.last_revision()
 
-        whole_history_data = whole_history_data_cache.get(self.last_revid)
-        if whole_history_data is None:
-            whole_history_data = compute_whole_history_data(branch)
-            whole_history_data_cache[self.last_revid] = whole_history_data
-
-        (self._revision_graph, self._full_history, self._revision_info,
-         self._revno_revid, self._merge_sort, self._where_merged,
-         ) = whole_history_data
-
-    def use_file_cache(self, cache):
-        self._file_change_cache = cache
+        caches = [RevInfoMemoryCache(whole_history_data_cache)]
+        if revinfo_disk_cache:
+            caches.append(revinfo_disk_cache)
+        self._load_whole_history_data(caches, cache_key)
 
     @property
     def has_revisions(self):
@@ -172,12 +314,12 @@
         return self._branch.get_config()
 
     def get_revno(self, revid):
-        if revid not in self._revision_info:
+        if revid not in self._rev_indices:
             # ghost parent?
             return 'unknown'
-        (seq, revid, merge_depth,
-         revno_str, end_of_merge) = self._revision_info[revid]
-        return revno_str
+        seq = self._rev_indices[revid]
+        revno = self._rev_info[seq][0][3]
+        return revno
 
     def get_revids_from(self, revid_list, start_revid):
         """
@@ -185,24 +327,38 @@
         revid in revid_list.
         """
         if revid_list is None:
-            revid_list = self._full_history
+            # Just yield the mainline, starting at start_revid
+            revid = start_revid
+            is_null = bzrlib.revision.is_null
+            while not is_null(revid):
+                yield revid
+                parents = self._rev_info[self._rev_indices[revid]][2]
+                if not parents:
+                    return
+                revid = parents[0]
+            return
         revid_set = set(revid_list)
         revid = start_revid
 
         def introduced_revisions(revid):
             r = set([revid])
-            seq, revid, md, revno, end_of_merge = self._revision_info[revid]
+            seq = self._rev_indices[revid]
+            md = self._rev_info[seq][0][2]
             i = seq + 1
-            while i < len(self._merge_sort) and self._merge_sort[i][2] > md:
-                r.add(self._merge_sort[i][1])
+            while i < len(self._rev_info) and self._rev_info[i][0][2] > md:
+                r.add(self._rev_info[i][0][1])
                 i += 1
             return r
-        while 1:
+        while revid_set:
             if bzrlib.revision.is_null(revid):
                 return
-            if introduced_revisions(revid) & revid_set:
+            rev_introduced = introduced_revisions(revid)
+            matching = rev_introduced.intersection(revid_set)
+            if matching:
+                # We don't need to look for these anymore.
+                revid_set.difference_update(matching)
                 yield revid
-            parents = self._revision_graph[revid]
+            parents = self._rev_info[self._rev_indices[revid]][2]
             if len(parents) == 0:
                 return
             revid = parents[0]
@@ -210,25 +366,17 @@
     def get_short_revision_history_by_fileid(self, file_id):
         # FIXME: would be awesome if we could get, for a folder, the list of
         # revisions where items within that folder changed.i
-        try:
-            # FIXME: Workaround for bzr versions prior to 1.6b3.
-            # Remove me eventually pretty please  :)
-            w = self._branch.repository.weave_store.get_weave(
-                    file_id, self._branch.repository.get_transaction())
-            w_revids = w.versions()
-            revids = [r for r in self._full_history if r in w_revids]
-        except AttributeError:
-            possible_keys = [(file_id, revid) for revid in self._full_history]
-            get_parent_map = self._branch.repository.texts.get_parent_map
-            # We chunk the requests as this works better with GraphIndex.
-            # See _filter_revisions_touching_file_id in bzrlib/log.py
-            # for more information.
-            revids = []
-            chunk_size = 1000
-            for start in xrange(0, len(possible_keys), chunk_size):
-                next_keys = possible_keys[start:start + chunk_size]
-                revids += [k[1] for k in get_parent_map(next_keys)]
-            del possible_keys, next_keys
+        possible_keys = [(file_id, revid) for revid in self._rev_indices]
+        get_parent_map = self._branch.repository.texts.get_parent_map
+        # We chunk the requests as this works better with GraphIndex.
+        # See _filter_revisions_touching_file_id in bzrlib/log.py
+        # for more information.
+        revids = []
+        chunk_size = 1000
+        for start in xrange(0, len(possible_keys), chunk_size):
+            next_keys = possible_keys[start:start + chunk_size]
+            revids += [k[1] for k in get_parent_map(next_keys)]
+        del possible_keys, next_keys
         return revids
 
     def get_revision_history_since(self, revid_list, date):
@@ -329,15 +477,41 @@
         if revid is None:
             revid = self.last_revid
         if file_id is not None:
-            # since revid is 'start_revid', possibly should start the path
-            # tracing from revid... FIXME
-            revlist = list(self.get_short_revision_history_by_fileid(file_id))
-            revlist = list(self.get_revids_from(revlist, revid))
+            revlist = list(
+                self.get_short_revision_history_by_fileid(file_id))
+            revlist = self.get_revids_from(revlist, revid)
         else:
-            revlist = list(self.get_revids_from(None, revid))
+            revlist = self.get_revids_from(None, revid)
         return revlist
 
-    def get_view(self, revid, start_revid, file_id, query=None):
+    @staticmethod
+    def _iterate_sufficiently(iterable, stop_at, extra_rev_count):
+        """Return a list of iterable.
+
+        If extra_rev_count is None, fully consume iterable.
+        Otherwise, stop at 'stop_at' + extra_rev_count.
+
+        Example:
+          iterate until you find stop_at, then iterate 10 more times.
+        """
+        if extra_rev_count is None:
+            return list(iterable)
+        result = []
+        found = False
+        for n in iterable:
+            result.append(n)
+            if n == stop_at:
+                found = True
+                break
+        if found:
+            for count, n in enumerate(iterable):
+                if count >= extra_rev_count:
+                    break
+                result.append(n)
+        return result
+
+    def get_view(self, revid, start_revid, file_id, query=None,
+                 extra_rev_count=None):
         """
         use the URL parameters (revid, start_revid, file_id, and query) to
         determine the revision list we're viewing (start_revid, file_id, query)
@@ -348,6 +522,10 @@
               file.
             - if a start_revid is given, we're viewing the branch from a
               specific revision up the tree.
+            - if extra_rev_count is given, find the view from start_revid =>
+              revid, and continue an additional 'extra_rev_count'. If not
+              given, then revid_list will contain the full history of
+              start_revid
 
         these may be combined to view revisions for a specific file, from
         a specific revision, with a specific search query.
@@ -366,12 +544,16 @@
 
         if query is None:
             revid_list = self.get_file_view(start_revid, file_id)
+            revid_list = self._iterate_sufficiently(revid_list, revid,
+                                                    extra_rev_count)
             if revid is None:
                 revid = start_revid
             if revid not in revid_list:
                 # if the given revid is not in the revlist, use a revlist that
                 # starts at the given revid.
                 revid_list = self.get_file_view(revid, file_id)
+                revid_list = self._iterate_sufficiently(revid_list, revid,
+                                                        extra_rev_count)
                 start_revid = revid
             return revid, start_revid, revid_list
 
@@ -394,7 +576,7 @@
     def get_inventory(self, revid):
         if revid not in self._inventory_cache:
             self._inventory_cache[revid] = (
-                self._branch.repository.get_revision_inventory(revid))
+                self._branch.repository.get_inventory(revid))
         return self._inventory_cache[revid]
 
     def get_path(self, revid, file_id):
@@ -419,10 +601,10 @@
 
         merge_point = []
         while True:
-            children = self._where_merged.get(revid, [])
+            children = self._rev_info[self._rev_indices[revid]][1]
             nexts = []
             for child in children:
-                child_parents = self._revision_graph[child]
+                child_parents = self._rev_info[self._rev_indices[child]][2]
                 if child_parents[0] == revid:
                     nexts.append(child)
                 else:
@@ -448,40 +630,37 @@
             revnol = revno.split(".")
             revnos = ".".join(revnol[:-2])
             revnolast = int(revnol[-1])
-            if revnos in d.keys():
+            if revnos in d:
                 m = d[revnos][0]
                 if revnolast < m:
                     d[revnos] = (revnolast, revid)
             else:
                 d[revnos] = (revnolast, revid)
 
-        return [d[revnos][1] for revnos in d.keys()]
+        return [revid for (_, revid) in d.itervalues()]
 
-    def get_branch_nicks(self, changes):
+    def add_branch_nicks(self, change):
         """
-        given a list of changes from L{get_changes}, fill in the branch nicks
-        on all parents and merge points.
+        given a 'change', fill in the branch nicks on all parents and merge
+        points.
         """
         fetch_set = set()
-        for change in changes:
-            for p in change.parents:
-                fetch_set.add(p.revid)
-            for p in change.merge_points:
-                fetch_set.add(p.revid)
+        for p in change.parents:
+            fetch_set.add(p.revid)
+        for p in change.merge_points:
+            fetch_set.add(p.revid)
         p_changes = self.get_changes(list(fetch_set))
         p_change_dict = dict([(c.revid, c) for c in p_changes])
-        for change in changes:
-            # arch-converted branches may not have merged branch info :(
-            for p in change.parents:
-                if p.revid in p_change_dict:
-                    p.branch_nick = p_change_dict[p.revid].branch_nick
-                else:
-                    p.branch_nick = '(missing)'
-            for p in change.merge_points:
-                if p.revid in p_change_dict:
-                    p.branch_nick = p_change_dict[p.revid].branch_nick
-                else:
-                    p.branch_nick = '(missing)'
+        for p in change.parents:
+            if p.revid in p_change_dict:
+                p.branch_nick = p_change_dict[p.revid].branch_nick
+            else:
+                p.branch_nick = '(missing)'
+        for p in change.merge_points:
+            if p.revid in p_change_dict:
+                p.branch_nick = p_change_dict[p.revid].branch_nick
+            else:
+                p.branch_nick = '(missing)'
 
     def get_changes(self, revid_list):
         """Return a list of changes objects for the given revids.
@@ -526,75 +705,74 @@
 
         return [self._change_from_revision(rev) for rev in rev_list]
 
-    def _get_deltas_for_revisions_with_trees(self, revisions):
-        """Produce a list of revision deltas.
-
-        Note that the input is a sequence of REVISIONS, not revision_ids.
-        Trees will be held in memory until the generator exits.
-        Each delta is relative to the revision's lefthand predecessor.
-        (This is copied from bzrlib.)
-        """
-        required_trees = set()
-        for revision in revisions:
-            required_trees.add(revision.revid)
-            required_trees.update([p.revid for p in revision.parents[:1]])
-        trees = dict((t.get_revision_id(), t) for
-                     t in self._branch.repository.revision_trees(
-                         required_trees))
-        ret = []
-        for revision in revisions:
-            if not revision.parents:
-                old_tree = self._branch.repository.revision_tree(
-                    bzrlib.revision.NULL_REVISION)
-            else:
-                old_tree = trees[revision.parents[0].revid]
-            tree = trees[revision.revid]
-            ret.append(tree.changes_from(old_tree))
-        return ret
-
     def _change_from_revision(self, revision):
         """
         Given a bzrlib Revision, return a processed "change" for use in
         templates.
         """
-        commit_time = datetime.datetime.fromtimestamp(revision.timestamp)
-
-        parents = [util.Container(revid=r,
-                   revno=self.get_revno(r)) for r in revision.parent_ids]
-
         message, short_message = clean_message(revision.message)
 
+        if self._branch_tags is None:
+            self._branch_tags = self._branch.tags.get_reverse_tag_dict()
+
+        revtags = None
+        if revision.revision_id in self._branch_tags:
+          # tag.sort_* functions expect (tag, data) pairs, so we generate them,
+          # and then strip them
+          tags = [(t, None) for t in self._branch_tags[revision.revision_id]]
+          tag.sort_natural(self._branch, tags)
+          revtags = u', '.join([t[0] for t in tags])
+
         entry = {
             'revid': revision.revision_id,
-            'date': commit_time,
-            'author': revision.get_apparent_author(),
+            'date': datetime.datetime.fromtimestamp(revision.timestamp),
+            'utc_date': datetime.datetime.utcfromtimestamp(revision.timestamp),
+            'committer': revision.committer,
+            'authors': revision.get_apparent_authors(),
             'branch_nick': revision.properties.get('branch-nick', None),
             'short_comment': short_message,
             'comment': revision.message,
             'comment_clean': [util.html_clean(s) for s in message],
             'parents': revision.parent_ids,
+            'bugs': [bug.split()[0] for bug in revision.properties.get('bugs', '').splitlines()],
+            'tags': revtags,
         }
+        if isinstance(revision, bzrlib.foreign.ForeignRevision):
+            foreign_revid, mapping = (rev.foreign_revid, rev.mapping)
+        elif ":" in revision.revision_id:
+            try:
+                foreign_revid, mapping = \
+                    bzrlib.foreign.foreign_vcs_registry.parse_revision_id(
+                        revision.revision_id)
+            except bzrlib.errors.InvalidRevisionId:
+                foreign_revid = None
+                mapping = None
+        else:
+            foreign_revid = None
+        if foreign_revid is not None:
+            entry["foreign_vcs"] = mapping.vcs.abbreviation
+            entry["foreign_revid"] = mapping.vcs.show_foreign_revid(foreign_revid)
         return util.Container(entry)
 
-    def get_file_changes_uncached(self, entries):
-        delta_list = self._get_deltas_for_revisions_with_trees(entries)
-
-        return [self.parse_delta(delta) for delta in delta_list]
-
-    def get_file_changes(self, entries):
+    def get_file_changes_uncached(self, entry):
+        if entry.parents:
+            old_revid = entry.parents[0].revid
+        else:
+            old_revid = bzrlib.revision.NULL_REVISION
+        return self.file_changes_for_revision_ids(old_revid, entry.revid)
+
+    def get_file_changes(self, entry):
         if self._file_change_cache is None:
-            return self.get_file_changes_uncached(entries)
+            return self.get_file_changes_uncached(entry)
         else:
-            return self._file_change_cache.get_file_changes(entries)
-
-    def add_changes(self, entries):
-        changes_list = self.get_file_changes(entries)
-
-        for entry, changes in zip(entries, changes_list):
-            entry.changes = changes
+            return self._file_change_cache.get_file_changes(entry)
+
+    def add_changes(self, entry):
+        changes = self.get_file_changes(entry)
+        entry.changes = changes
 
     def get_file(self, file_id, revid):
-        "returns (path, filename, data)"
+        """Returns (path, filename, file contents)"""
         inv = self.get_inventory(revid)
         inv_entry = inv[file_id]
         rev_tree = self._branch.repository.revision_tree(inv_entry.revision)
@@ -603,7 +781,7 @@
             path = '/' + path
         return path, inv_entry.name, rev_tree.get_file_text(file_id)
 
-    def parse_delta(self, delta):
+    def file_changes_for_revision_ids(self, old_revid, new_revid):
         """
         Return a nested data structure containing the changes in a delta::
 
@@ -613,74 +791,24 @@
             modified: list(
                 filename: str,
                 file_id: str,
-            )
+            ),
+            text_changes: list((filename, file_id)),
         """
-        added = []
-        modified = []
-        renamed = []
-        removed = []
-
-        for path, fid, kind in delta.added:
-            added.append((rich_filename(path, kind), fid))
-
-        for path, fid, kind, text_modified, meta_modified in delta.modified:
-            modified.append(util.Container(filename=rich_filename(path, kind),
-                                           file_id=fid))
-
-        for old_path, new_path, fid, kind, text_modified, meta_modified in \
-delta.renamed:
-            renamed.append((rich_filename(old_path, kind),
-                            rich_filename(new_path, kind), fid))
-            if meta_modified or text_modified:
-                modified.append(util.Container(
-                    filename=rich_filename(new_path, kind), file_id=fid))
-
-        for path, fid, kind in delta.removed:
-            removed.append((rich_filename(path, kind), fid))
-
-        return util.Container(added=added, renamed=renamed,
-                              removed=removed, modified=modified)
-
-    def annotate_file(self, file_id, revid):
-        z = time.time()
-        lineno = 1
-        parity = 0
-
-        file_revid = self.get_inventory(revid)[file_id].revision
-        oldvalues = None
-        tree = self._branch.repository.revision_tree(file_revid)
-        revid_set = set()
-
-        try:
-            bzrlib.textfile.check_text_lines(tree.get_file_lines(file_id))
-        except bzrlib.errors.BinaryFile:
-                # bail out; this isn't displayable text
-                yield util.Container(parity=0, lineno=1, status='same',
-                                     text='(This is a binary file.)',
-                                     change=util.Container())
+        repo = self._branch.repository
+        if (bzrlib.revision.is_null(old_revid) or
+            bzrlib.revision.is_null(new_revid)):
+            old_tree, new_tree = map(
+                repo.revision_tree, [old_revid, new_revid])
         else:
-            for line_revid, text in tree.annotate_iter(file_id):
-                revid_set.add(line_revid)
-
-            change_cache = dict([(c.revid, c) \
-                    for c in self.get_changes(list(revid_set))])
-
-            last_line_revid = None
-            for line_revid, text in tree.annotate_iter(file_id):
-                if line_revid == last_line_revid:
-                    # remember which lines have a new revno and which don't
-                    status = 'same'
-                else:
-                    status = 'changed'
-                    parity ^= 1
-                    last_line_revid = line_revid
-                    change = change_cache[line_revid]
-                    trunc_revno = change.revno
-                    if len(trunc_revno) > 10:
-                        trunc_revno = trunc_revno[:9] + '...'
-
-                yield util.Container(parity=parity, lineno=lineno, status=status,
-                                     change=change, text=util.fixed_width(text))
-                lineno += 1
-
-        self.log.debug('annotate: %r secs' % (time.time() - z))
+            old_tree, new_tree = repo.revision_trees([old_revid, new_revid])
+
+        reporter = FileChangeReporter(old_tree.inventory, new_tree.inventory)
+
+        bzrlib.delta.report_changes(new_tree.iter_changes(old_tree), reporter)
+
+        return util.Container(
+            added=sorted(reporter.added, key=lambda x: x.filename),
+            renamed=sorted(reporter.renamed, key=lambda x: x.new_filename),
+            removed=sorted(reporter.removed, key=lambda x: x.filename),
+            modified=sorted(reporter.modified, key=lambda x: x.filename),
+            text_changes=sorted(reporter.text_changes, key=lambda x: x.filename))

=== added file 'loggerhead/load_test.py'
--- loggerhead/load_test.py	1970-01-01 00:00:00 +0000
+++ loggerhead/load_test.py	2011-03-02 14:07:21 +0000
@@ -0,0 +1,230 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Code to do some load testing against a loggerhead instance.
+
+This is basically meant to take a list of actions to take, and run it against a
+real host, and see how the results respond.::
+
+    {"parameters": {
+         "base_url": "http://localhost:8080",
+     },
+     "requests": [
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"}
+     ],
+    }
+
+All threads have a Queue length of 1. When a third request for a given thread
+is seen, no more requests are queued until that thread finishes its current
+job. So this results in all requests being issued sequentially::
+
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"}
+
+While this would cause all requests to be sent in parallel:
+
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "3", "relpath": "/changes"},
+        {"thread": "4", "relpath": "/changes"}
+
+This should keep 2 threads pipelined with activity, as long as they finish in
+approximately the same speed. We'll start the first thread running, and the
+second thread, and queue up both with a second request once the first finishes.
+When we get to the third request for thread "1", we block on queuing up more
+work until the first thread 1 request has finished.
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"},
+        {"thread": "1", "relpath": "/changes"},
+        {"thread": "2", "relpath": "/changes"}
+
+There is not currently a way to say "run all these requests keeping exactly 2
+threads active". Though if you know the load pattern, you could approximate
+this.
+"""
+
+import threading
+import time
+import Queue
+
+import simplejson
+
+from bzrlib import (
+    errors,
+    transport,
+    urlutils,
+    )
+
+# This code will be doing multi-threaded requests against bzrlib.transport
+# code. We want to make sure to load everything ahead of time, so we don't get
+# lazy-import failures
+_ = transport.get_transport('http://example.com')
+
+
+class RequestDescription(object):
+    """Describes info about a request."""
+
+    def __init__(self, descrip_dict):
+        self.thread = descrip_dict.get('thread', '1')
+        self.relpath = descrip_dict['relpath']
+
+
+class RequestWorker(object):
+    """Process requests in a worker thread."""
+
+    _timer = time.time
+
+    def __init__(self, identifier, blocking_time=1.0, _queue_size=1):
+        self.identifier = identifier
+        self.queue = Queue.Queue(_queue_size)
+        self.start_time = self.end_time = None
+        self.stats = []
+        self.blocking_time = blocking_time
+
+    def step_next(self):
+        url = self.queue.get(True, self.blocking_time)
+        if url == '<noop>':
+            # This is usually an indicator that we want to stop, so just skip
+            # this one.
+            self.queue.task_done()
+            return
+        self.start_time = self._timer()
+        success = self.process(url)
+        self.end_time = self._timer()
+        self.update_stats(url, success)
+        self.queue.task_done()
+
+    def run(self, stop_event):
+        while not stop_event.isSet():
+            try:
+                self.step_next()
+            except Queue.Empty:
+                pass
+
+    def process(self, url):
+        base, path = urlutils.split(url)
+        t = transport.get_transport(base)
+        try:
+            # TODO: We should probably look into using some part of
+            #       blocking_timeout to decide when to stop trying to read
+            #       content
+            content = t.get_bytes(path)
+        except (errors.TransportError, errors.NoSuchFile):
+            return False
+        return True
+
+    def update_stats(self, url, success):
+        self.stats.append((url, success, self.end_time - self.start_time))
+
+
+class ActionScript(object):
+    """This tracks the actions that we want to perform."""
+
+    _worker_class = RequestWorker
+    _default_base_url = 'http://localhost:8080'
+    _default_blocking_timeout = 60.0
+
+    def __init__(self):
+        self.base_url = self._default_base_url
+        self.blocking_timeout = self._default_blocking_timeout
+        self._requests = []
+        self._threads = {}
+        self.stop_event = threading.Event()
+
+    @classmethod
+    def parse(cls, content):
+        script = cls()
+        json_dict = simplejson.loads(content)
+        if 'parameters' not in json_dict:
+            raise ValueError('Missing "parameters" section')
+        if 'requests' not in json_dict:
+            raise ValueError('Missing "requests" section')
+        param_dict = json_dict['parameters']
+        request_list = json_dict['requests']
+        base_url = param_dict.get('base_url', None)
+        if base_url is not None:
+            script.base_url = base_url
+        blocking_timeout = param_dict.get('blocking_timeout', None)
+        if blocking_timeout is not None:
+            script.blocking_timeout = blocking_timeout
+        for request_dict in request_list:
+            script.add_request(request_dict)
+        return script
+
+    def add_request(self, request_dict):
+        request = RequestDescription(request_dict)
+        self._requests.append(request)
+
+    def _get_worker(self, thread_id):
+        if thread_id in self._threads:
+            return self._threads[thread_id][0]
+        handler = self._worker_class(thread_id,
+                                     blocking_time=self.blocking_timeout/10.0)
+
+        t = threading.Thread(target=handler.run, args=(self.stop_event,),
+                             name='Thread-%s' % (thread_id,))
+        self._threads[thread_id] = (handler, t)
+        t.start()
+        return handler
+
+    def finish_queues(self):
+        """Wait for all queues of all children to finish."""
+        for h, t in self._threads.itervalues():
+            h.queue.join()
+
+    def stop_and_join(self):
+        """Stop all running workers, and return.
+
+        This will stop even if workers still have work items.
+        """
+        self.stop_event.set()
+        for h, t in self._threads.itervalues():
+            # Signal the queue that it should stop blocking, we don't have to
+            # wait for the queue to empty, because we may see stop_event before
+            # we see the <noop>
+            h.queue.put('<noop>')
+            # And join the controlling thread
+            for i in range(10):
+                t.join(self.blocking_timeout / 10.0)
+                if not t.isAlive():
+                    break
+
+    def _full_url(self, relpath):
+        return self.base_url + relpath
+
+    def run(self):
+        self.stop_event.clear()
+        for request in self._requests:
+            full_url = self._full_url(request.relpath)
+            worker = self._get_worker(request.thread)
+            worker.queue.put(full_url, True, self.blocking_timeout)
+        self.finish_queues()
+        self.stop_and_join()
+
+
+def run_script(filename):
+    with open(filename, 'rb') as f:
+        content = f.read()
+    script = ActionScript.parse(content)
+    script.run()
+    return script

=== removed file 'loggerhead/lockfile.py'
--- loggerhead/lockfile.py	2008-10-24 02:26:05 +0000
+++ loggerhead/lockfile.py	1970-01-01 00:00:00 +0000
@@ -1,84 +0,0 @@
-#
-# Copyright (C) 2006  Robey Pointer <robey@lag.net>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
-
-import os
-import threading
-import time
-
-from loggerhead import util
-
-
-with_lock = util.with_lock('_tlock', 'LockFile')
-
-MAX_STALE_TIME = 5 * 60
-
-
-class LockFile (object):
-    """
-    simple lockfile implementation that mimics the API of threading.Lock, so
-    it can be used interchangably.  it's actually a reentrant lock, so the
-    lock may be acquired multiple times by the same thread, as long as it's
-    released an equal number of times.  unlike threading.Lock, this lock can
-    be used across processes.
-
-    this uses os.open(O_CREAT|O_EXCL), which apparently works even on windows,
-    but will not work over NFS, if anyone still uses that.  so don't put the
-    cache folder on an NFS server...
-    """
-
-    def __init__(self, filename):
-        self._filename = filename
-        # thread lock to maintain internal consistency
-        self._tlock = threading.Lock()
-        self._count = 0
-        if os.path.exists(filename):
-            # remove stale locks left over from a previous run
-            if time.time() - os.stat(filename).st_mtime > MAX_STALE_TIME:
-                os.remove(filename)
-
-    @with_lock
-    def _try_acquire(self):
-        if self._count > 0:
-            self._count += 1
-            return True
-        try:
-            fd = os.open(self._filename,
-                         os.O_WRONLY | os.O_CREAT | os.O_EXCL, 0600)
-            os.close(fd)
-            self._count += 1
-            return True
-        except OSError:
-            return False
-
-    def acquire(self):
-        # try over and over, sleeping on exponential backoff with
-        # an upper limit of about 5 seconds
-        pause = 0.1
-        #max_pause = 5.0
-        max_pause = 0.1
-        while True:
-            if self._try_acquire():
-                return
-            time.sleep(pause)
-            pause = min(pause * 2.0, max_pause)
-
-    @with_lock
-    def release(self):
-        self._count -= 1
-        if self._count == 0:
-            os.remove(self._filename)

=== modified file 'loggerhead/lsprof.py'
--- loggerhead/lsprof.py	2008-10-24 02:26:05 +0000
+++ loggerhead/lsprof.py	2009-07-07 23:52:24 +0000
@@ -73,7 +73,6 @@
             d = d[:top]
         cols = "% 12s %12s %11.4f %11.4f   %s\n"
         hcols = "% 12s %12s %12s %12s %s\n"
-        cols2 = "+%12s %12s %11.4f %11.4f +  %s\n"
         file.write(hcols % ("CallCount", "Recursive", "Total(ms)",
                             "Inline(ms)", "module:lineno(function)"))
         for e in d:

=== renamed file 'serve-branches' => 'loggerhead/main.py' (properties changed: +x to -x)
--- serve-branches	2009-02-11 03:04:17 +0000
+++ loggerhead/main.py	2011-03-16 11:39:27 +0000
@@ -1,4 +1,6 @@
-#!/usr/bin/env python
+#
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -19,112 +21,111 @@
 import os
 import sys
 
-from optparse import OptionParser
+from bzrlib.plugin import load_plugins
 
 from paste import httpserver
 from paste.httpexceptions import HTTPExceptionHandler, HTTPInternalServerError
 from paste.translogger import TransLogger
 
 from loggerhead import __version__
-from loggerhead.apps.filesystem import (
-    BranchesFromFileSystemRoot, UserBranchesFromFileSystemRoot)
+from loggerhead.apps.transport import (
+    BranchesFromTransportRoot, UserBranchesFromTransportRoot)
+from loggerhead.config import LoggerheadConfig
 from loggerhead.util import Reloader
 from loggerhead.apps.error import ErrorHandlerApp
 
 
-def command_line_parser():
-    parser = OptionParser("%prog [options] <path>")
-    parser.set_defaults(
-        user_dirs=False,
-        show_version=False,
-        log_folder=None,
-        )
-    parser.add_option("--user-dirs", action="store_true", dest="user_dirs",
-                      help="Serve user directories as ~user.")
-    parser.add_option("--trunk-dir", metavar="DIR",
-                      help="The directory that contains the trunk branches.")
-    parser.add_option("--port", dest="user_port",
-                      help=("Port Loggerhead should listen on "
-                            "(defaults to 8080)."))
-    parser.add_option("--host", dest="user_host",
-                      help="Host Loggerhead should listen on.")
-    parser.add_option("--prefix", dest="user_prefix",
-                      help="Specify host prefix.")
-    parser.add_option("--profile", action="store_true", dest="profile",
-                      help="Generate callgrind profile data to "
-                        "%d-stats.callgrind on each request.")
-    parser.add_option("--reload", action="store_true", dest="reload",
-                      help="Restarts the application when changing python"
-                           " files. Only used for development purposes.")
-    parser.add_option('--log-folder', dest="log_folder",
-                      type=str, help="The directory to place log files in.")
-    parser.add_option("--version", action="store_true", dest="show_version",
-                      help="Print the software version and exit")
-    return parser
-
-
-def main(args):
-    parser = command_line_parser()
-    (options, args) = parser.parse_args(sys.argv[1:])
-
-    if options.show_version:
-        print "loggerhead %s" % __version__
+def get_config_and_path(args):
+    config = LoggerheadConfig(args)
+
+    if config.get_option('show_version'):
+        print "loggerhead %s" % (__version__,)
         sys.exit(0)
 
-    if len(args) > 1:
-        parser.print_help()
-        sys.exit(1)
-    elif len(args) == 1:
-        [path] = args
-    else:
-        path = '.'
-
-    if not os.path.isdir(path):
-        print "%s is not a directory" % path
-        sys.exit(1)
-
-    if options.trunk_dir and not options.user_dirs:
+    if config.arg_count > 1:
+        config.print_help()
+        sys.exit(1)
+    elif config.arg_count == 1:
+        base = config.get_arg(0)
+    else:
+        base = '.'
+
+    if not config.get_option('allow_writes'):
+        base = 'readonly+' + base
+
+    return config, base
+
+
+def setup_logging(config, init_logging=True, log_file=None):
+    log_level = config.get_log_level()
+    if init_logging:
+        logging.basicConfig()
+        if log_level is not None:
+            logging.getLogger('').setLevel(log_level)
+    logger = logging.getLogger('loggerhead')
+    if log_level is not None:
+        logger.setLevel(log_level)
+    if log_file is not None:
+        handler = logging.StreamHandler(log_file)
+    else:
+        if config.get_option('log_folder'):
+            logfile_path = os.path.join(
+                config.get_option('log_folder'), 'serve-branches.log')
+        else:
+            logfile_path = 'serve-branches.log'
+        handler = logging.FileHandler(logfile_path, 'a')
+        formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(name)s:'
+                                      ' %(message)s')
+        handler.setFormatter(formatter)
+    # We set the handler to accept all messages, the *logger* won't emit them
+    # if it is configured to suppress it
+    handler.setLevel(logging.DEBUG)
+    logger.addHandler(handler)
+    def _restrict_logging(logger_name):
+        logger = logging.getLogger(logger_name)
+        if logger.getEffectiveLevel() < logging.INFO:
+            logger.setLevel(logging.INFO)
+    # simpleTAL is *very* verbose in DEBUG mode, which is otherwise the
+    # default. So quiet it up a bit.
+    _restrict_logging('simpleTAL')
+    _restrict_logging('simpleTALES')
+    return logger
+
+
+def make_app_for_config_and_path(config, base):
+    if config.get_option('trunk_dir') and not config.get_option('user_dirs'):
         print "--trunk-dir is only valid with --user-dirs"
         sys.exit(1)
 
-    if options.reload:
+    if config.get_option('reload'):
         if Reloader.is_installed():
             Reloader.install()
         else:
             return Reloader.restart_with_reloader()
 
-    if options.user_dirs:
-        if not options.trunk_dir:
+    if config.get_option('user_dirs'):
+        if not config.get_option('trunk_dir'):
             print "You didn't specify a directory for the trunk directories."
             sys.exit(1)
-        app = UserBranchesFromFileSystemRoot(path, options.trunk_dir)
-    else:
-        app = BranchesFromFileSystemRoot(path)
-
-    # setup_logging()
-    logging.basicConfig()
-    logging.getLogger('').setLevel(logging.DEBUG)
-    logger = getattr(app, 'log', logging.getLogger('loggerhead'))
-    if options.log_folder:
-        logfile_path = os.path.join(options.log_folder, 'serve-branches.log')
-    else:
-        logfile_path = 'serve-branches.log'
-    logfile = logging.FileHandler(logfile_path, 'a')
-    formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(name)s:'
-                                  ' %(message)s')
-    logfile.setFormatter(formatter)
-    logfile.setLevel(logging.DEBUG)
-    logger.addHandler(logfile)
-    # setup_logging() #end
-    app = TransLogger(app, logger=logger)
-    if options.profile:
+        app = UserBranchesFromTransportRoot(base, config)
+    else:
+        app = BranchesFromTransportRoot(base, config)
+
+    setup_logging(config)
+
+    if config.get_option('profile'):
         from loggerhead.middleware.profile import LSProfMiddleware
         app = LSProfMiddleware(app)
+    if config.get_option('memory_profile'):
+        from dozer import Dozer
+        app = Dozer(app)
 
-    if not options.user_prefix:
+    if not config.get_option('user_prefix'):
         prefix = '/'
     else:
-        prefix = options.user_prefix
+        prefix = config.get_option('user_prefix')
+        if not prefix.startswith('/'):
+            prefix = '/' + prefix
 
     try:
         from paste.deploy.config import PrefixMiddleware
@@ -146,19 +147,43 @@
 
     app = HTTPExceptionHandler(app)
     app = ErrorHandlerApp(app)
-
-    if not options.user_port:
+    app = TransLogger(app, logger=logging.getLogger('loggerhead'))
+
+    return app
+
+
+def main(args):
+    load_plugins()
+
+    config, path = get_config_and_path(args)
+
+    app = make_app_for_config_and_path(config, path)
+
+    if not config.get_option('user_port'):
         port = '8080'
     else:
-        port = options.user_port
+        port = config.get_option('user_port')
 
-    if not options.user_host:
+    if not config.get_option('user_host'):
         host = '0.0.0.0'
     else:
-        host = options.user_host
-
-    httpserver.serve(app, host=host, port=port)
-
-
-if __name__ == "__main__":
-    main(sys.argv)
+        host = config.get_option('user_host')
+
+    if not config.get_option('protocol'):
+        protocol = 'http'
+    else:
+        protocol = config.get_option('protocol')
+
+    if protocol == 'http':
+        httpserver.serve(app, host=host, port=port)
+    else:
+        if protocol == 'fcgi':
+            from flup.server.fcgi import WSGIServer
+        elif protocol == 'scgi':
+            from flup.server.scgi import WSGIServer
+        elif protocol == 'ajp':
+            from flup.server.ajp import WSGIServer
+        else:
+            print 'Unknown protocol: %s.' % (protocol)
+            sys.exit(1)
+        WSGIServer(app, bindAddress=(host, int(port))).run()

=== modified file 'loggerhead/middleware/profile.py'
--- loggerhead/middleware/profile.py	2008-11-14 06:48:20 +0000
+++ loggerhead/middleware/profile.py	2009-10-17 08:47:38 +0000
@@ -1,11 +1,12 @@
-'''Profiling middleware for paste.'''
-import cgi
+"""Profiling middleware for Paste."""
+
 import threading
 
 from bzrlib.lsprof import profile
 
+
 class LSProfMiddleware(object):
-    '''Paste middleware for profiling with lsprof.'''
+    """Paste middleware for profiling with lsprof."""
 
     def __init__(self, app, global_conf=None):
         self.app = app
@@ -26,8 +27,7 @@
         try:
             ret, stats = profile(self.__run_app, environ, start_response)
             self.__count += 1
-            stats.save("%d-stats.callgrind" % self.__count, format="callgrind")
+            stats.save("%d-stats.callgrind" % (self.__count,), format="callgrind")
             return ret
         finally:
             self.lock.release()
-

=== modified file 'loggerhead/search.py'
--- loggerhead/search.py	2008-11-05 00:50:06 +0000
+++ loggerhead/search.py	2009-04-17 03:38:04 +0000
@@ -17,13 +17,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-import sets
-try:
-    from bzrlib.plugins.search import errors
-    from bzrlib.plugins.search import index as _mod_index
-    from bzrlib.plugins.search.index import FileTextHit, RevisionHit
-except ImportError:
-    _mod_index = None
+
+_mod_index = None
+def import_search():
+    global errors, _mod_index, FileTextHit, RevisionHit
+    if _mod_index is not None:
+        return
+    try:
+        from bzrlib.plugins.search import errors
+        from bzrlib.plugins.search import index as _mod_index
+        from bzrlib.plugins.search.index import FileTextHit, RevisionHit
+    except ImportError:
+        _mod_index = None
 
 
 def search_revisions(branch, query_list, suggest=False):
@@ -36,6 +41,7 @@
     param suggest: Optional flag to request suggestions instead of results
     return: A list for results, either revision ids or terms
     """
+    import_search()
     if _mod_index is None:
         return None # None indicates could-not-search
     try:
@@ -59,6 +65,6 @@
                     revid_list.append(result.text_key[1])
                 elif isinstance(result, RevisionHit):
                     revid_list.append(result.revision_key[0])
-            return list(sets.Set(revid_list))
+            return list(set(revid_list))
     finally:
         index._branch.unlock()

=== modified file 'loggerhead/static/css/diff.css'
--- loggerhead/static/css/diff.css	2009-02-17 02:19:38 +0000
+++ loggerhead/static/css/diff.css	2011-02-18 05:41:17 +0000
@@ -1,145 +1,113 @@
 /*Info Box*/
 .infoContainer {
-	width:970px;
-	padding:10px 10px 0px 10px;
-	margin:0px 0px 10px 0px;
-	color:#666;
-	background:#fcfcfc url(../images/bg_infobox.gif) repeat-y top right;
-	border:1px solid #b7b7b7;
-}
+    width: 970px;
+    padding: 10px 10px 0 10px;
+    margin: 0 0 10px 0;
+    color: #666;
+    background: #fcfcfc url(../images/bg_infobox.gif) repeat-y top right;
+    border: 1px solid #b7b7b7;
+    }
 ul#list-files {
-	float:right;
-	width:460px;
-	list-style:none;
-	margin:0px;
-	padding:0px;
-	font:normal 11px/normal sans-serif;
-	color:#000;
-}
+    float: right;
+    width: 460px;
+    font-size: 77%;
+    color: #000;
+    }
 ul#list-files li {
-	width:440px;
-	padding:2px 0px 2px 20px;
-	margin:0px 0px 2px 8px;
-	background:url(../images/ico_file.gif) 2px 0px no-repeat;
-}
+    width: 440px;
+    padding: 2px 0 2px 20px;
+    margin: 0 0 2px 8px;
+    background: url(../images/ico_file.gif) 2px 0 no-repeat;
+    }
 ul#list-files li.desc {
     margin-left: 0;
-	background:url(../images/ico_description.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_description.gif) 0 0 no-repeat;
+    }
 #infTxt {
-	float:left;
-	width:493px;
-	margin:0px 0px 5px 0px;
-}
-.information, .committer, .timer, .mfrom, .mto, .revid {
-	clear:both;
-	margin:0px;
-	padding:0px 0px 2px 18px;
-	font:normal 11px/1.3 sans-serif;
-	color:#666;
-	background:url(../images/ico_description.gif) no-repeat top left;
-}
-.revid {
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-}
-.committer, .mfrom, .mto {
-	color:#000;
-	padding:1px 0px 0px 18px;
-	margin:0px 10px 5px 0px;
-	float:left;
-	display:inline;
-	background:url(../images/ico_committer.gif) no-repeat 2px 0px;
-}
-.timer {
-	margin:0px 10px 5px 0px;
-	padding:1px 0px 0px 18px;
-	float:left;
-	clear:none;
-	display:inline;
-	background:url(../images/ico_time.gif) no-repeat top left;
-}
-.mfrom {
-	background:url(../images/ico_mergefrom.gif) no-repeat top left;
-}
-.mto {
-	background:url(../images/ico_mergeto.gif) no-repeat top left;
-}
-/**/
-#loggerheadCont p.expand, #loggerheadCont p.expand a, #loggerheadCont p.expand a:link, #loggerheadCont p.expand a:visited, #loggerheadCont p.expand a:hover {
-	color:#0f6b97;
-	text-decoration:none;
-	margin-right:10px;
-	float:left;	
-}
-#loggerheadCont p.codin, #loggerheadCont p.codin a, #loggerheadCont p.codin a:link, #loggerheadCont p.codin a:visited, #loggerheadCont p.codin a:hover {
-	color:#000;
-	font-size:12px;
-	text-decoration:none;
-	margin-right:10px;
-	float:left;	
-}
+    float: left;
+    width: 493px;
+    margin: 0 0 5px 0;
+    font-size:  77%;
+    color: #666;
+    }
 /*Diff Boxes*/
 .diffBox {
-	width:100%;
-	padding:6px 0%;
-	background:#eef6fa;
-	border:1px solid #d2d2d2;
-	margin:10px 0px 0px 0px;
-}
+    width: 100%;
+    padding: 6px 0;
+    background: #eef6fa;
+    border: 1px solid #d2d2d2;
+    margin: 10px 0 0 0;
+    }
 .diffBox img {
-	vertical-align:middle;
-}
-.diffBox a, .diffBox a:link, .diffBox a:visited, .diffBox a:hover {
-	text-decoration:none;
-}
+    vertical-align: middle;
+    }
+.diffBox a {
+    text-decoration: none;
+    }
 .diff {
-    margin: 2em 0 2em 0;
-}
-.lineNumber {
-	float:left;
-	width:2%;
-	padding:0px 1%; 
-	border-right:1px solid #d5d5d5;
-	border-left:1px solid #d5d5d5;
-	color:#000;
-	background:#f8fbfd;
-	font:normal 12px/1.8 monospace;
-	text-align:right;
-}
-.lineCode, .removecode, .addcode {
-	float:left;
-	width:43%;
-	padding:0px 1%; 
-	color:#000;
-	background:#fff;
-	font:normal 12px/1.8 monospace;
-}
-.lineCode-u {
-	float:left;
-	width:88%;
-	padding:0px 1%; 
-	color:#000;
-	background:#fff;
-	font:normal 12px/1.8 monospace;
-}
-.delete, .insert {
-	display:block;
-	background:#ff7f7f;
-}
-.insert {
-	background:#92ed92;
-}
+    margin: 2em 0;
+    }
+.diffinfo {
+    overflow: hidden;
+    }
+.lineNumber {
+    width: 2%;
+    border-width: 0 1px;
+    border-style: solid;
+    border-color: #d5d5d5;
+    background: #f8fbfd;
+    text-align: right;
+    }
+.unified .lineNumber.second {
+    border-left: 0;
+    }
+.code,
+.lineNumber {
+    float: left;
+    padding: 0 1%;
+    color: #000;
+    font-size: 85%;
+    font-family: 'UbuntuBeta Mono', 'Ubuntu Mono', monospace;
+    line-height: 180%;
+    }
+.blank {
+    float: left;
+    padding: 0 1%;
+    }
+.unified .code {
+    width: 89%;
+    }
+.sbs .code,
+.blank {
+    width: 43%;
+    overflow: hidden;
+    }
+.unified .insert-row .lineNumber.first {
+    background: #c9ffc9;
+    }
+.unified .delete-row .lineNumber.second {
+    background: #ffbfbf;
+    }
+.code.insert,
+.insert-row .lineNumber.second,
+.both-row .lineNumber.second {
+    background: #92ed92;
+    }
+.code.delete,
+.delete-row .lineNumber.first,
+.both-row .lineNumber.first {
+    background: #ff7f7f;
+    }
 .separate {
     height: 2em;
     background: #e3e3e3;
-}
+    }
 ul#pages {
-	margin-top:10px;
-}
+    margin-top: 10px;
+    }
 /*Foot Links*/
 #loggerheadCont p.lpandbzr {
-	float:left;
-	padding:3px 0px 0px 0px;
-	margin-top:10px;
-}
+    float: left;
+    padding: 3px 0 0 0;
+    margin-top: 10px;
+    }

=== modified file 'loggerhead/static/css/files.css'
--- loggerhead/static/css/files.css	2009-01-23 20:53:40 +0000
+++ loggerhead/static/css/files.css	2011-02-18 05:41:17 +0000
@@ -1,41 +1,5 @@
-/*Info Box*/
-.information, .committer, .timer, .mfrom, .mto, .revid {
-	float:left;
-	margin:0px 10px 5px 0px;
-	padding:0px 0px 2px 18px;
-	font:normal 11px/1.3 sans-serif;
-	color:#666;
-	background:url(../images/ico_description.gif) no-repeat top left;
-}
-.revid {
-    float:left;
-    padding:1px 0px 0px 18px;
-    margin:0px 10px 5px 0px;
-}
-.comiter, .mfrom, .mto {
-	color:#000;
-	padding:1px 0px 0px 18px;
-	margin:0px 10px 5px 0px;
-	float:left;
-	display:inline;
-	background:url(../images/ico_committer.gif) no-repeat 2px 0px;
-}
-.timer {
-	margin:0px 10px 5px 0px;
-	padding:1px 0px 0px 18px;
-	float:left;
-	clear:none;
-	display:inline;
-	background:url(../images/ico_time.gif) no-repeat top left;
-}
-.mfrom {
-	background:url(../images/ico_mergefrom.gif) no-repeat top left;
-}
-.mto {
-	background:url(../images/ico_mergetop.gif) no-repeat top left;
-}
-.information {
-    clear:both;
-    width:100%;
-    position:relative;
-}
+#infTxt {
+    font-size: 77%;
+    color: #666;
+    margin-bottom: 10px;
+    }

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2009-01-23 20:53:40 +0000
+++ loggerhead/static/css/global.css	2011-02-24 00:44:09 +0000
@@ -1,339 +1,345 @@
+/*
+    CSS Structure
+    - Reset and Generic Styles
+    - Header & Navigation
+        - Loggerhead Menu Tabs
+        - Sub-menu Tabs
+        - Breadcrumbs
+    - Search & RSS 
+    - General Content
+    - Log Table
+    - Revision List
+    - Pagination
+    - Footer
+*/
+
+
+/* =========================
+   Reset and Generic Styles
+*/
+body, img, form, tr, ul, li, h1, h2, h3, h4, p, table {
+    padding: 0;
+    margin: 0;
+    }
+body, html, table {
+    font-size: 93%;
+    }
 body {
-	padding:0px;
-	margin:0px;
-	font:normal 12px/normal sans-serif;
-	color:#666;
-	background:#fff;
-}
-a, a:link, a:visited, a:hover {
-	color:#0f597f;
-	text-decoration:none;
-	font-weight:normal;
-	outline:none;
-}
+    font-weight: normal;
+    font-family: UbuntuBeta, Ubuntu, 'Bitstream Vera Sans', 'DejaVu Sans', Tahoma, sans-serif;
+    color: #000;
+    background: #fff;
+    }
+a {
+    color: #03A;
+    text-decoration: none;
+    }
+a:hover {
+    text-decoration: underline;
+    }
 img {
-	padding:0px;
-	margin:0px;
-	border:0px;
-	outline:none;
-}
-form {
-  margin: 0;
-  padding: 0;
-}
-tr td {
-  vertical-align: top;
-}
+    border: 0;
+    outline: none;
+    }
 .clear {
-	clear:both;
-	font-size:0px;
-	height:0px;
-}
-.fr {float:right;}
-.fl {float:left;}
-
+    clear: both;
+    font-size: 0;
+    height: 0;
+    }
+.fr {
+    float: right;
+    }
+.fl {
+    float: left;
+    }
 ul {
-    list-style:none;
+    list-style: none;
+    }
+table {
+    caption-side: top;
+    width: 100%;
+    border: none;
+    border-spacing: 0;
+    border-collapse: collapse;
+    }
+tr td {
+    vertical-align: top;
+    }
+th, td {
+    border-color: #D2D2D2;
+    }
+th {
+    text-align: left;
+    color: #5A5A5A;
+    padding: 4px 8px;
+    border-width: 1px;
+    border-style: solid;
+    }
+td {
+    padding: 4px 4px;
+    border-width: 0 0 1px 0;
+    border-style: dotted;
+    }
+h1 {
+    font-size:138.5%;
+    padding: 10px 0 5px 0;
+    }
+h2 {
+    font-size: 123.1%;
+    padding: 10px 0 5px 0;
+    }
+p {
+    padding: 5px 0;
+    }
+p img {
+    vertical-align: middle;
+    }
+
+
+/* =========================
+   Header & Navigation
+*/
+
+.black-link {
+    border-bottom: 1px solid #D2D2D2;
+    padding: 6px 1% 8px 1%;
+    font-size: 85%;
 }
 
-/* Loggerhead Menu tabs */
+/* Branch Title */
+h1.branch-name {
+    font-size: 131%;
+    padding: 0;
+    margin: 20px 0 0 1%;
+    }
+
+/* Loggerhead Menu Tabs */
 ul#menuTabs {
-	width:98%;
-	height:33px;
-	list-style:none;
-	color:#fff;
-	background:#fff url(../images/bg_menuTabs.gif) bottom repeat-x;
-	margin:0px;
-	padding:12px 0px 0px 2%;
-}
-ul#menuTabs li a:link, ul#menuTabs li a:visited {
-	float:left;
-	width:93px;
-	padding:10px 0px 0px 0px;
-	height:23px;
-	font:normal 14px/1.5 sans-serif;
-	text-align:center;
-	color:#fff;
-	text-decoration:none;
-	background:#ad2001 url(../images/bg_Tabs.gif) bottom center;
-}
+    height: 33px;
+    color: #fff;
+    margin: 0;
+    padding: 15px 20px 0 1%;
+    }
 ul#menuTabs li {
-	float:left;
-	color:#fff;
-	background:#f00 url(../images/bg_Tabs.gif) bottom center;
-}
+    float: left;
+    padding: 0 7px 0 0;
+    }
+ul#menuTabs li a {
+    padding: 3px 6px;
+    color: #000;
+    }
 ul#menuTabs li a:hover {
-	float:left;
-	color:#000;
-	background:#666 url(../images/bg_Tabs.gif) top center;
-	text-decoration:none;
-}
-ul#menuTabs li a#on:link, ul#menuTabs li a#on:visited, ul#menuTabs a#on:hover {
-	float:left;
-	color:#000;
-	background:#666 url(../images/bg_Tabs.gif) top center;
-	text-decoration:none;
-}
-/*Sub menu tabs*/
+    background-color: #f3f3f3;
+    text-decoration: none;
+    }
+ul#menuTabs li a#on {
+    font-weight: bold;
+    color: #fff;
+    background-color: #747474;
+    }
+
+/* Sub-menu Tabs */
 ul#submenuTabs {
-	width:100%;
-	height:24px;
-	list-style:none;
-	color:#fff;
-	background:#fff;
-	margin:0px;
-	padding:10px 0%;
-}
-ul#submenuTabs li a:link, ul#submenuTabs li a:visited {
-	float:left;
-	display:inline;
-	height:22px;
-	padding:2px 15px 0px 15px;
-	font:normal 14px/1.5 sans-serif;
-	text-align:center;
-	color:#fff;
-	text-decoration:none;
-	background:#2f40a5;
-}
+    height: 24px;
+    margin-bottom: 10px;
+    }
 ul#submenuTabs li {
-	float:left;
-	display:inline;
-	margin-left:1px;
-	color:#fff;
-	background:#2f40a5;
-}
-ul#submenuTabs li a:hover {
-	float:left;
-	display:inline;
-	color:#fff;
-	background:#2f40a5;
-	text-decoration:underline;
-}
-ul#submenuTabs li#first a, ul#submenuTabs li#first a:link, ul#submenuTabs li#first a:visited, ul#submenuTabs li#first a:hover {
-	background:#2f40a5 url(../images/bg_submenuTabs.gif) top left no-repeat;
-}
-ul#submenuTabs li#last a, ul#submenuTabs li#last a:link, ul#submenuTabs li#last a:visited, ul#submenuTabs li#last a:hover {
-	background:#2f40a5 url(../images/bg_submenuTabs.gif) bottom right no-repeat;
-}
-
-/* Search & RSS */
-#finderBox {
-	width:300px;
-	float:right;
-	padding: 10px 0px 0px 0px;
-}
-#finderBox label {
-	float:left;
-	display:inline;
-	width:100px;
-	font:normal 12px/normal sans-serif;
-	text-align:right;
-	color:#000;
-	padding:5px 5px 0px 0px;
-}
-#finderBox input {
-	float:left;
-	font:normal 12px/normal sans-serif;
-	color:#666;
-	display:inline;
-	width:130px;
-	padding:3px;
-	background:#fff url(../images/bg_search_input.gif) 110px 3px no-repeat;
-	border:1px solid #e9e9e9;
-}
-#search_terms {
-    display:none;
-    border:solid 1px black;
-    width:180px;
-    overflow:auto;
-    background: #fff;
-}
-.rssfeed {
-	float:left;
-	display:inline;
-	padding:0px 0px 0px 10px;
-}
-
-/* General Content */
+    float: left;
+    display: inline;
+    margin-right: 20px;
+    }
+ul#submenuTabs li a {
+    float: left;
+    display: inline;
+    height: 22px;
+    padding: 0 0 0 18px;
+    background: transparent url(../images/ico_description.gif) no-repeat center left;
+    }
+
+/* Breadcrumbs */
+#breadcrumbs {
+    font-size: 77%;
+    margin-bottom: 10px;
+    }
+
+#branch-info {
+    width: 900px;
+    padding: 10px;
+    margin: 0 0 10px 0;
+    color: #666;
+    background: #fcfcfc;
+    border: 1px solid #b7b7b7;
+    }
+
+
+/* =========================
+   General Content
+*/
 #loggerheadCont {
-	width:98%;
-	padding:10px 1%;
-	color:#242424;
-	background:#fff;
-}
-#loggerheadCont h1 {
-	font-size:18px;
-	color:#666;
-	margin:0px;
-	padding:10px 0px 5px 0px;
-}
-#loggerheadCont h1 span {
-    color:#889ea9;
-}
-#loggerheadCont p {
-	color:#000;
-	margin:0px;
-	padding:5px 0px;
-}
-#loggerheadCont p.expand, #loggerheadCont p.expand a, #loggerheadCont p.expand a:link, #loggerheadCont p.expand a:visited, #loggerheadCont p.expand a:hover {
-	color:#000;
-	text-decoration:none;	
-}
-#loggerheadCont p img {
-	vertical-align:middle;
-}
-.link {
-	padding:0px 11px 0px 0px;
-	margin:0px 2px 0px 0px;
-	font-weight:bold;
-}
-.revnolink a, .revnolink a:visited, .revnolink a:link, .revnolink a:hover {
-	color:#CC6600;
-	text-decoration:none;		
-}
-
-/*Log Table*/
+    width: 98%;
+    padding: 0 1% 10px 1%;
+    }
+
+
+/* =========================
+   Log Table
+*/
 #logentries {
-	caption-side:top;
-	width:99%;
-	margin:0px auto 10px auto;
-	padding:0px;
-	background:#fff;
-	border:none;
-	border-spacing:0px;
-	border-collapse:collapse;
-}
-.logheader {
-	background:#fff;
-	font:bold 14px/normal sans-serif;
-	padding:0px;
-	margin:0px;
-}
-.blueRow0, .blueRow1 {
-	background:#eef6fa;
-	font:normal 14px/normal sans-serif;	
-	padding:0px;
-	margin:0px;
-}
-.blueRow1 {
-	background:#f8fbfd;
-}
-.revisionnumber, .expandcell, .diffcell, .downloadcell, .revnro, .expcell, .diffr, .downr, .allicones {
-	padding:4px 0px;
-	width:42px;
-	text-align:center;
-	border:1px solid #d2d2d2;
-}
-.expandcell, .expcell {
-	width:28px;	
-}
-.summarycell, .authorcell, .datecell, .timedate{
-  padding: 4px 8px;
-  border: 1px solid #d2d2d2;
-}
-.icocell {
-  width: 18px;
-}
-.datecell, .date {
-  width: 180px;
-}
-.timedate, .timedate2 {
-	width:80px;
-}
-.downloadcell, .downr {
-	width:47px;
-}
-.revnro, .expcell, .diffr, .summcell, .autcell, .icocell, .date, .downr, .allicones, .timedate2 {
-  border: 0;
-  padding: 4px 4px;
-  border-bottom:1px solid #d2d2d2;
-}
+    margin-bottom: 10px;
+    clear: left;
+    }
+p.expand, p.codin {
+    margin-right: 10px;
+    float: left;
+    }
+p.expand a, p.codin a {
+    color: #000;
+    text-decoration: none;
+    }
+.show_if_js {
+    display: none;
+    }
+
+/* th and td styles */
+th.revisionnumber,
+th.expandcell,
+th.diffcell,
+th.downloadcell {
+    padding-left: 0;
+    padding-right: 0;
+    text-align: center;
+    }
+th.diffcell,
+th.revisionnumber {
+    width: 42px;
+    }
+th.expandcell,
+td.expcell {
+    width: 10px;
+    }
+td.icocell {
+    width: 18px;
+    }
+th.authorcell,
+td.autcell,
+th.tagscell,
+td.tagcell {
+    width: 240px;
+    }
+th.datecell,
+td.date {
+    width: 180px;
+    }
+th.timedate,
+td.timedate2 {
+    width: 80px;
+    }
+th.downloadcell,
+td.downr {
+    width: 47px;
+    }
+
+
+/* =========================
+   Revision List
+*/
+.revision_log {
+    overflow: hidden;
+    }
 .revisioninfo {
-    margin-top:4px;
-}
+    margin-top: 4px;
+    }
 ul.expandrev {
-	list-style:none;
-	font:normal 11px/normal sans-serif;
-	color:#000;
-}
+    font-size: 85%;
+    }
 ul.expandrev li {
-	padding:2px 0px 2px 20px;
-	margin:0px 0px 2px 0px;
-}
+    padding: 2px 0 2px 20px;
+    margin: 0 0 2px 0;
+    }
 ul.expandrev li.mfrom {
-	background:url(../images/ico_mergefrom.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_mergefrom.gif) 0 0 no-repeat;
+    }
 ul.expandrev li.mto {
-	background:url(../images/ico_mergeto.gif) 0px 0px no-repeat;
-}
+    background: url(../images/ico_mergeto.gif) 0 0 no-repeat;
+    }
 ul.expandrev li.committerli {
-	background:url(../images/ico_committer.gif) 2px 0px no-repeat;
-}
+    background: url(../images/ico_committer.gif) 2px 0 no-repeat;
+    }
 ul.expandrev li.desc {
-	background:url(../images/ico_description.gif) 0px 0px no-repeat;
-	color:#666;
-}
+    background: url(../images/ico_description.gif) 0 0 no-repeat;
+    color: #666;
+    }
 ul.expandrev li.files {
-	background:url(../images/ico_file.gif) 2px 0px no-repeat;
+    background: url(../images/ico_file.gif) 2px 0 no-repeat;
     margin-left: 8px;
-}
-
-/* Navigation */
-ul {border:0; margin:0; padding:0;}
-
+    }
+.revnolink a {
+    color: #000;
+    }
+
+
+/* =========================
+   Pagination
+*/
 ul#pages {
-	float:right;
-}
+    float: right;
+    }
 #pages li {
-	border:0; margin:0; padding:0;
-	font-size:12px;
-	list-style:none;
-	float:left;
-	display:inline;
-}
-#pages a {
-	border:solid 1px #DDDDDD;
-	margin-right:2px;
-}
-#pages .next a, #pages .previous a {
-	font-weight:bold;
-	border:solid 1px #fff;
-}
-#pages .active {
-	color:#f37b06;
-	font-weight:bold;
-	display:block;
-	float:left;
-	padding:4px 6px;
-}
-#pages a:link, #pages a:visited {
-	color:#f37b06;
-	display:block;
-	float:left;
-	padding:3px 6px;
-	text-decoration:none;
-}
-#pages .next a {
-	color:#0f6b97;
-}
-#pages a:hover {
-	border:solid 1px #666666;
-}
-.breadcrumb a {
-	color:#889ea9;
-	font-weight:bold;
-	font-size:18px;
-}
-.breadcrumb a:hover{
-	color:#666;
-	font-weight:bold;
-	font-size:18px;
-}
-#branch-info {
-	width:900px;
-	padding:10px 10px 10px 10px;
-	margin:0px 0px 10px 0px;
-	color:#666;
-	background:#fcfcfc;
-	border:1px solid #b7b7b7;
-}
+    font-size: 77%;
+    font-weight: bold;
+    float: left;
+    display: inline;
+    padding: 3px 6px;
+    margin-right: 2px;
+    }
+
+
+/* =========================
+   Search & RSS
+*/
+#finderBox {
+    width: 300px;
+    float: right;
+    padding: 10px 0 0 0;
+    }
+#finderBox label,
+#finderBox input {
+    float: left;
+    font-weight: normal;
+    font-size: 12px;
+    }
+#finderBox label {
+    display: inline;
+    width: 100px;
+    text-align: right;
+    color: #000;
+    padding: 5px 5px 0 0;
+    }
+#finderBox input {
+    color: #666;
+    display: inline;
+    width: 130px;
+    padding: 3px;
+    background: #fff url(../images/bg_search_input.gif) 110px 3px no-repeat;
+    border: 1px solid #e9e9e9;
+    }
+#search_terms {
+    display: none;
+    border: solid 1px black;
+    width: 180px;
+    overflow: auto;
+    background: #fff;
+    }
+.rssfeed {
+    float: left;
+    display: inline;
+    padding: 0 0 0 10px;
+    }
+
+
+/* =========================
+   Footer
+*/
+#footer {
+    font-size: 77%;
+    }

=== added file 'loggerhead/static/css/highlight.css'
--- loggerhead/static/css/highlight.css	1970-01-01 00:00:00 +0000
+++ loggerhead/static/css/highlight.css	2009-03-01 20:16:18 +0000
@@ -0,0 +1,59 @@
+.pyg-c { color: #808080 } /* Comment */
+.pyg-err { color: #F00000; background-color: #F0A0A0 } /* Error */
+.pyg-k { color: #008000; font-weight: bold } /* Keyword */
+.pyg-o { color: #303030 } /* Operator */
+.pyg-cm { color: #808080 } /* Comment.Multiline */
+.pyg-cp { color: #507090 } /* Comment.Preproc */
+.pyg-c1 { color: #808080 } /* Comment.Single */
+.pyg-cs { color: #cc0000; font-weight: bold } /* Comment.Special */
+.pyg-gd { color: #A00000 } /* Generic.Deleted */
+.pyg-ge { font-style: italic } /* Generic.Emph */
+.pyg-gr { color: #FF0000 } /* Generic.Error */
+.pyg-gh { color: #000080; font-weight: bold } /* Generic.Heading */
+.pyg-gi { color: #00A000 } /* Generic.Inserted */
+.pyg-go { color: #808080 } /* Generic.Output */
+.pyg-gp { color: #c65d09; font-weight: bold } /* Generic.Prompt */
+.pyg-gs { font-weight: bold } /* Generic.Strong */
+.pyg-gu { color: #800080; font-weight: bold } /* Generic.Subheading */
+.pyg-gt { color: #0040D0 } /* Generic.Traceback */
+.pyg-kc { color: #008000; font-weight: bold } /* Keyword.Constant */
+.pyg-kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
+.pyg-kp { color: #003080; font-weight: bold } /* Keyword.Pseudo */
+.pyg-kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
+.pyg-kt { color: #303090; font-weight: bold } /* Keyword.Type */
+.pyg-m { color: #6000E0; font-weight: bold } /* Literal.Number */
+.pyg-s { background-color: #fff0f0 } /* Literal.String */
+.pyg-na { color: #0000C0 } /* Name.Attribute */
+.pyg-nb { color: #007020 } /* Name.Builtin */
+.pyg-nc { color: #B00060; font-weight: bold } /* Name.Class */
+.pyg-no { color: #003060; font-weight: bold } /* Name.Constant */
+.pyg-nd { color: #505050; font-weight: bold } /* Name.Decorator */
+.pyg-ni { color: #800000; font-weight: bold } /* Name.Entity */
+.pyg-ne { color: #F00000; font-weight: bold } /* Name.Exception */
+.pyg-nf { color: #0060B0; font-weight: bold } /* Name.Function */
+.pyg-nl { color: #907000; font-weight: bold } /* Name.Label */
+.pyg-nn { color: #0e84b5; font-weight: bold } /* Name.Namespace */
+.pyg-nt { color: #007000 } /* Name.Tag */
+.pyg-nv { color: #906030 } /* Name.Variable */
+.pyg-ow { color: #000000; font-weight: bold } /* Operator.Word */
+.pyg-w { color: #bbbbbb } /* Text.Whitespace */
+.pyg-mf { color: #6000E0; font-weight: bold } /* Literal.Number.Float */
+.pyg-mh { color: #005080; font-weight: bold } /* Literal.Number.Hex */
+.pyg-mi { color: #0000D0; font-weight: bold } /* Literal.Number.Integer */
+.pyg-mo { color: #4000E0; font-weight: bold } /* Literal.Number.Oct */
+.pyg-sb { background-color: #fff0f0 } /* Literal.String.Backtick */
+.pyg-sc { color: #0040D0 } /* Literal.String.Char */
+.pyg-sd { color: #D04020 } /* Literal.String.Doc */
+.pyg-s2 { background-color: #fff0f0 } /* Literal.String.Double */
+.pyg-se { color: #606060; font-weight: bold; background-color: #fff0f0 } /* Literal.String.Escape */
+.pyg-sh { background-color: #fff0f0 } /* Literal.String.Heredoc */
+.pyg-si { background-color: #e0e0e0 } /* Literal.String.Interpol */
+.pyg-sx { color: #D02000; background-color: #fff0f0 } /* Literal.String.Other */
+.pyg-sr { color: #000000; background-color: #fff0ff } /* Literal.String.Regex */
+.pyg-s1 { background-color: #fff0f0 } /* Literal.String.Single */
+.pyg-ss { color: #A06000 } /* Literal.String.Symbol */
+.pyg-bp { color: #007020 } /* Name.Builtin.Pseudo */
+.pyg-vc { color: #306090 } /* Name.Variable.Class */
+.pyg-vg { color: #d07000; font-weight: bold } /* Name.Variable.Global */
+.pyg-vi { color: #3030B0 } /* Name.Variable.Instance */
+.pyg-il { color: #0000D0; font-weight: bold } /* Literal.Number.Integer.Long */

=== renamed file 'loggerhead/static/css/annotate.css' => 'loggerhead/static/css/view.css'
--- loggerhead/static/css/annotate.css	2008-08-05 19:44:50 +0000
+++ loggerhead/static/css/view.css	2011-02-18 05:41:17 +0000
@@ -1,27 +1,43 @@
 /*table*/
-.annoLineTit, .annoLine, .annoRevTit, .annoRev, .annoComm, .annoCommTit, .annoContTit, .annoCont {
-	width:45px;
-	border:1px solid #d2d2d2;
-}
-.annoLine, .annoRev, .annoComm, .annoCont {
-	border:none;	
-}
-.annoLine {
-	width:37px;
-}
-.annoContTit, .annoCont {
-	width:auto;
-}
-.annoRevTit, .annoRev {
-	width:70px;
-	text-align:center;
-}
-.annoComm, .annoCommTit {
-	width:200px;
-}
-.annoLine, .annoCont {
-	font:normal 12px/normal monospace;
-}
-.annoLine {
-	text-align:right;
-}
+.viewLineTit,
+.viewRevTit,
+.viewContTit {
+    border: 1px solid #d2d2d2;
+    }
+.viewLine,
+.viewRev,
+.viewComm,
+.viewCont {
+    border: none;
+    }
+.viewLine,
+.viewLineTit {
+    padding-right: .2em;
+    }
+.viewLine {
+    width: 37px;
+    }
+.viewContTit,
+.viewCont {
+    width: auto;
+    padding-left: .3em;
+    }
+.viewRevTit,
+.viewRev {
+    width: 70px;
+    text-align: center;
+    }
+.viewLine,
+.viewCont {
+    font-size: 116%;
+    font-family: 'UbuntuBeta Mono', 'Ubuntu Mono', monospace;
+    line-height: 108%;
+    whitespace: pre;
+    }
+.viewCont pre {
+    margin: 0;
+    }
+.viewLine {
+    text-align: right;
+    font-size: 93%;
+    }

=== modified file 'loggerhead/static/images/bg_Tabs.gif'
Binary files loggerhead/static/images/bg_Tabs.gif and loggerhead/static/images/bg_Tabs.gif differ
=== modified file 'loggerhead/static/images/bg_menuTabs.gif'
Binary files loggerhead/static/images/bg_menuTabs.gif and loggerhead/static/images/bg_menuTabs.gif differ
=== modified file 'loggerhead/static/images/bg_submenuTabs.gif'
Binary files loggerhead/static/images/bg_submenuTabs.gif and loggerhead/static/images/bg_submenuTabs.gif differ
=== added file 'loggerhead/static/images/favicon.png'
Binary files loggerhead/static/images/favicon.png and loggerhead/static/images/favicon.png differ
=== added file 'loggerhead/static/images/ico_bug.png'
Binary files loggerhead/static/images/ico_bug.png and loggerhead/static/images/ico_bug.png differ
=== added file 'loggerhead/static/images/ico_tag.gif'
Binary files loggerhead/static/images/ico_tag.gif and loggerhead/static/images/ico_tag.gif differ
=== added file 'loggerhead/static/images/spinner.gif'
Binary files loggerhead/static/images/spinner.gif and loggerhead/static/images/spinner.gif differ
=== added file 'loggerhead/static/javascript/changelog.js'
--- loggerhead/static/javascript/changelog.js	1970-01-01 00:00:00 +0000
+++ loggerhead/static/javascript/changelog.js	2009-03-31 21:11:38 +0000
@@ -0,0 +1,76 @@
+function toggle_expand_all(action)
+{
+  var revlogs = Y.all('.revision_log');
+  if (revlogs == null) return;
+  revlogs.each(
+    function(item, i)
+    {
+      var collapsable = item.collapsable;
+      if(action == 'close')
+      {
+        Y.get('#expand_all').setStyle('display','block');
+        Y.get('#collapse_all').setStyle('display','none');
+        collapsable.close();
+      }
+      else if(action == 'open')
+      {
+        Y.get('#expand_all').setStyle('display','none');
+        Y.get('#collapse_all').setStyle('display','block');
+        collapsable.open();
+      }
+    });
+}
+
+Y.on(
+  'click',
+  function (event) {
+    event.preventDefault();
+    toggle_expand_all('open');
+  },
+  '#expand_all a'
+);
+
+Y.on(
+  'click',
+  function (event) {
+    event.preventDefault();
+    toggle_expand_all('close');
+  },
+  '#collapse_all a'
+);
+
+Y.on(
+  "domready", function () {
+    Y.all(".show_if_js").removeClass("show_if_js");
+  });
+
+Y.on(
+  'domready',
+  function()
+  {
+    var revlogs = Y.all('.revision_log');
+    if (revlogs == null) return;
+    revlogs.each(
+      function(item, i)
+      {
+        var revid = revids[item.get('id').replace('log-', '')];
+        var collapsable = new Collapsable(
+          {
+            expand_icon: item.query('.expand_icon'),
+            open_node: item.query('.long_description'),
+            close_node: item.query('.short_description'),
+            source: global_path + '+revlog/' + revid,
+            loading: item.query('.loading'),
+            is_open: false
+          });
+
+        item.query('.expand_revisioninfo a').on(
+          'click',
+          function(e) {
+            e.preventDefault();
+            collapsable.toggle();
+          });
+        item.collapsable = collapsable;
+      });
+
+  });

=== modified file 'loggerhead/static/javascript/custom.js'
--- loggerhead/static/javascript/custom.js	2008-11-26 14:16:27 +0000
+++ loggerhead/static/javascript/custom.js	2009-04-01 15:28:21 +0000
@@ -1,248 +1,282 @@
+Y = YUI().use("node", "io-base", "anim");
+
 var global_timeout_id = null;
 var global_search_request = null;
 
-window.addEvent('domready', function() 
-{
-    var search_box = $('q');
-    if ($defined(search_box))
+Y.on(
+  'domready',
+  function()
+  {
+    var search_box = Y.get('#q');
+    if (!Y.Lang.isNull(search_box))
     {
-        search_box.removeEvents();
-        search_box.addEvents(
-        {
-            keyup: function()
-            {
-                if($('q').value == '')
-                {
-                    $('search_terms').setStyle('display','none');
-                }
-                else
-                {
-                    if (null != global_timeout_id)
-                    {
-                        clearTimeout(global_timeout_id);
-                    }
-                        global_timeout_id = setTimeout('$("q").fireEvent("search",$("q").value)',200);
-                }
-            },
-
-            search: function(query)
-            {
-                url = global_path + 'search?query=' + query;
-
-                if ($defined(global_search_request))
-                {
-                    global_search_request.cancel();
-                }
-                global_search_request = new Request({'url':url,'method':'get','onComplete':function(response)
-                  {
-                     cool_search(response,query);
-                 }});
-
-                global_search_request.send('');
-                var posicion = search_box.getPosition();
-                var size     = search_box.getSize();
-
-                $('search_terms').setStyle('position','absolute');
-                $('search_terms').setStyle('left',posicion.x);
-                $('search_terms').setStyle('top',posicion.y + size.y);
-                $('search_terms').setStyle('display','block');
-                $('search_terms').set('html','Loading...');
-
-                new Request({'url':url,'method':'get','onComplete':cool_search}).send('');
-
-
+      function get_suggestions() {
+        var query = search_box.get('value');
+        var url = global_path + 'search?query=' + query;
+
+        if (!Y.Lang.isNull(global_search_request))
+        {
+          global_search_request.abort();
+        }
+        global_search_request = Y.io(
+          url,
+          {
+            on: {complete: cool_search},
+            arguments: [query]
+          }
+        );
+
+        var region = search_box.get('region');
+        var current_query = search_box.get('value');
+
+        Y.get('#search_terms').setStyle('display', 'block');
+        Y.get('#search_terms').setStyle('position', 'absolute');
+        Y.get('#search_terms').setStyle('left', region.left);
+        Y.get('#search_terms').setStyle('top', region.bottom);
+        Y.get('#search_terms').set('innerHTML','Loading...');
+      }
+      search_box.on(
+        "keyup",
+        function(event)
+        {
+          if(search_box.get('value') == '')
+          {
+            Y.get('#search_terms').setStyle('display', 'none');
+          }
+          else
+          {
+            if (null != global_timeout_id)
+            {
+              clearTimeout(global_timeout_id);
             }
+            global_timeout_id = setTimeout(get_suggestions, 200);
+          }
         });
     }
-});
+  });
 
-function cool_search(response, query)
+function cool_search(tid, response, query)
 {
-	var posicion = $('q').getPosition();
-	var size     = $('q').getSize();
-    var current_query = $('q').get('value');
-    if (current_query == query)
-    {
-	    $('search_terms').set('html',response);
-	    $('search_terms').setStyle('display','block');
-	    $('search_terms').setStyle('position','absolute');
-	    $('search_terms').setStyle('left',posicion.x);
-	    $('search_terms').setStyle('top',posicion.y + size.y);
-    }
+  var q = Y.get('#q');
+  var region = q.get('region');
+  var current_query = q.get('value');
+  if (current_query == query)
+  {
+    Y.get('#search_terms').set('innerHTML', response.responseText);
+    Y.get('#search_terms').setStyle('display', 'block');
+    Y.get('#search_terms').setStyle('position', 'absolute');
+    Y.get('#search_terms').setStyle('left', region.left);
+    Y.get('#search_terms').setStyle('top', region.bottom);
+  }
 }
 
 function hide_search()
 {
-    hide_div = setTimeout("$('search_terms').setStyle('display','none')", 300);
-}
-var Colapsable = new Class({
-    initialize: function(item,expand_icon,open_content,close_content,is_open)
-    {
-        this.is_open = false;
-        if ($defined(is_open))
-        {
-            this.is_open = is_open;
-        }
-        this.item = item;
-        item.set('colapsable',this);
-        this.open_content  = open_content;
-        this.close_content = close_content;
-        this.expand_icon   = expand_icon;
-
-        var expander = new Fx.Slide(this.item, { duration: 200 } );
-        if (!this.is_open)
-        {
-            expander.hide();
-            if ($defined(this.expand_icon))
-            {
-                this.expand_icon.set('src',this.expand_icon.title);
-            }
-        }
-        else
-        {
-            if ($defined(this.expand_icon))
-            {
-                this.expand_icon.set('src',this.expand_icon.alt);
-            }
-        }
-    },
-
-    open: function()
-    {
-        this.item.setStyle('display', 'block');
-        var expander = this.item.get('slide');
-        expander.slideIn();
-        if ($defined(this.open_content))
-        {
-            for (var i=0;i<this.open_content.length;++i)
-            {
-                this.open_content[i].setStyle('display','block');
-            }
-		}
-		
-        if ($defined(this.close_content))
-        {
-            for (var i=0;i<this.close_content.length;++i)
-            {
-                this.close_content[i].setStyle('display','none');
-            }
-        }
-
-        if ($defined(this.expand_icon))
-        {
-            this.expand_icon.set('src',this.expand_icon.alt);
-        }
-        this.is_open = true;
-    },
-
-    close: function()
-    {
-        var expander = this.item.get('slide');
-        expander.slideOut();
-        if ($defined(this.open_content))
-        {
-            for (var i=0;i<this.open_content.length;++i)
-            {
-                this.open_content[i].setStyle('display','none');
-            }
-        }
-
-        if ($defined(this.close_content))
-        {
-            for (var i=0;i<this.close_content.length;++i)
-            {
-                this.close_content[i].setStyle('display','block');
-            }
-        }
-        if ($defined(this.expand_icon))
-        {
-            this.expand_icon.set('src',this.expand_icon.title);
-        }
-        this.is_open = false;
-    },
-
-    isOpen : function()
-    {
-        return this.is_open;
-    },
-
-    toggle: function()
-    {
-        if (this.isOpen())
-        {
-            this.close();
-        }
-        else
-        {
-            this.open();
-        }
-    }
-    });
-
-
-window.addEvent('domready', function()
-{
-    $$('.revision_log').each(function(item, i)
-    {
-        var item_slide = item.getElement('.revisioninfo');
-        var open_content  = new Array();
-        var close_content = new Array();
-        open_content.push(item.getElement('.long_description'));
-        close_content.push(item.getElement('.short_description'));
-        var expand_icon = item.getElement('.expand_icon');
-        var colapsable = new Colapsable(item_slide,expand_icon,open_content,close_content);
-        
-        item.getElement('.expand_revisioninfo').addEvent('click',function(){colapsable.toggle();});
-        item.colapsable = colapsable;
-    });
-
-    $$('.diffBox').each(function(item, i)
-    {
-        var item_slide = item.getNext('.diffinfo');
-        var expand_icon = item.getElement( '.expand_diff' );
-        var colapsable = new Colapsable(item_slide,expand_icon,null,null,true);
-        item.getElement( '.expand_diff' ).addEvent( 'click', function(){colapsable.toggle();});
-        item.colapsable=colapsable;
-    });
-});
-
-function toggle_expand_all(action)
-{
-    $$('.revision_log').each(function(item, i)
-    {
-    	var colapsable = item.colapsable;
-        if(action == 'close')
-        {
-            $('expand_all').setStyle('display','block');
-            $('collapse_all').setStyle('display','none');
-            colapsable.close();
-        }
-        else if(action == 'open')
-        {
-            $('expand_all').setStyle('display','none');
-            $('collapse_all').setStyle('display','block');
-            colapsable.open();
-        }
-    });
-}
-
-function toggle_expand_all_revisionview(action)
-{
-    $$('.diffBox').each(function(item, i)
-    {
-    	var colapsable = item.colapsable;
-        if(action == 'close')
-        {
-            $('expand_all').setStyle('display','block');
-            $('collapse_all').setStyle('display','none');
-            colapsable.close();
-        }
-        else if(action == 'open')
-        {
-            $('expand_all').setStyle('display','none');
-            $('collapse_all').setStyle('display','block');
-            colapsable.open();
-        }
-    });
-}
+  setTimeout("Y.get('#search_terms').setStyle('display','none')", 300);
+}
+
+function Collapsable(config)
+{
+  this.is_open = config.is_open;
+  this.open_node = config.open_node;
+  this.close_node = config.close_node;
+  this.expand_icon = config.expand_icon;
+  this.source = config.source;
+  this.loading = config.loading;
+  this.node_process = config.node_process;
+  this.container = null;
+  this.anim = null;
+  this._loading = false;
+}
+
+function get_height(node) {
+  node.setStyle('position', 'absolute');
+  node.setStyle('top', -1000000000);
+  node.setStyle('display', 'block');
+  var height = node.get('region').height;
+  node.setStyle('display', 'none');
+  node.setStyle('position', 'static');
+  node.setStyle('top', 'auto');
+  return height;
+}
+
+Collapsable.prototype._animate = function (callback)
+{
+  if (this.anim) this.anim.stop();
+
+  this.anim = new Y.Anim(
+    {
+      node: this.container,
+      from: {
+        marginBottom: this.container.getStyle('marginBottom')
+      },
+      to: {
+        marginBottom: 0
+      },
+      duration: 0.2
+    });
+
+  this.anim.run();
+  this.anim.on('end', this.animComplete, this, callback);
+}
+
+Collapsable.prototype._load_finished = function(tid, res, args)
+{
+  var l = res.responseText.split('\n');
+  l.splice(0, 1);
+  var newNode = Y.Node.create(l.join(''));
+  if (this.node_process)
+    this.node_process(newNode);
+  this.source = null;
+  newNode.setStyle('display', 'none');
+  this.loading.ancestor().insertBefore(newNode, this.loading);
+  var delta = this.loading.get('region').height - get_height(newNode);
+  newNode.setStyle('display', 'block');
+  this.container.setStyle('marginBottom', parseFloat(this.container.getStyle('marginBottom')) + delta);
+  this.loading.ancestor().removeChild(this.loading);
+  this._animate(args[0]);
+};
+
+Collapsable.prototype._ensure_container = function(callback)
+{
+  if (this.container == null) {
+    this.container = Y.Node.create('<div></div>');
+    if (this.closed_node) {
+      this.closed_node.ancestor().replaceChild(
+        this.container, this.closed_node);
+      this.container.appendChild(this.closed_node);
+      if (this.open_node) {
+        this.container.appendChild(this.open_node);
+      }
+    }
+    else {
+      this.open_node.ancestor().replaceChild(
+        this.container, this.open_node);
+      this.container.appendChild(this.open_node);
+    }
+    var outer = Y.Node.create('<div style="overflow:hidden;"></div>');
+    this.container.ancestor().replaceChild(outer, this.container);
+    outer.appendChild(this.container);
+  }
+}
+
+/* What happens when you click open.
+ *
+ * 1. The arrow flips to the expanded position.
+ *
+ * 2. If necessary, the div which will be running the animation is
+ * created and the open/closed content stuffed into it (and has height
+ * set to the height of the closed content).
+ *
+ * 3. The open content is shown and the closed content is closed.
+ *
+ * 4. The animation to expose all of the open content is started.
+ *
+ * 5. If we have to do ajax to load content, start the request.
+ *
+ * 6. When the request completes, parse the content into a node, run
+ * the node_process callback over it and replace the spinner (assumed
+ * to be appropriately contained in the open node) with the new node.
+ *
+ * 7. If the animation showing the open content has not completed,
+ * stop it.
+ *
+ * 8. Start a new animation to show the rest of the new content.
+ */
+
+Collapsable.prototype.open = function(callback)
+{
+  this.expand_icon.set('src', expanded_icon_path);
+
+  this._ensure_container();
+
+  var open_height = get_height(this.open_node);
+
+  var close_height;
+  if (this.close_node) {
+    close_height = this.close_node.get('region').height;
+  }
+  else {
+    close_height = 0;
+  }
+
+  this.container.setStyle('marginBottom', close_height - open_height);
+  if (this.close_node) {
+    this.close_node.setStyle('display', 'none');
+  }
+  this.open_node.setStyle('display', 'block');
+
+  this._animate(callback);
+
+  if (this.source) {
+    Y.io(
+      this.source,
+      {
+        on: {complete: this._load_finished},
+        arguments: [callback],
+        context: this
+      });
+    return;
+  }
+
+};
+
+Collapsable.prototype.animComplete = function(evt, callback)
+{
+  this.anim = null;
+  if (this._loading) return;
+  if (callback) callback();
+  this.is_open = true;
+};
+
+Collapsable.prototype.close = function()
+{
+  this._ensure_container();
+
+  var open_height = this.open_node.get('region').height;
+
+  var close_height;
+  if (this.close_node) {
+    close_height = get_height(this.close_node);
+  }
+  else {
+    close_height = 0;
+  }
+
+  var anim = new Y.Anim(
+    {
+      node: this.container,
+      from: {
+        marginBottom: 0
+      },
+      to: {
+        marginBottom: close_height - open_height
+      },
+      duration: 0.2
+    });
+  anim.on("end", this.closeComplete, this);
+  anim.run();
+};
+
+Collapsable.prototype.closeComplete = function () {
+  this.open_node.setStyle('display', 'none');
+  if (this.close_node) {
+    this.close_node.setStyle('display', 'block');
+  }
+  this.container.setStyle('marginBottom', 0);
+  this.expand_icon.set('src', collapsed_icon_path);
+  this.is_open = false;
+};
+
+Collapsable.prototype.toggle = function()
+{
+  if (this.is_open)
+  {
+    this.close();
+  }
+  else
+  {
+    this.open();
+  }
+};
 

=== added file 'loggerhead/static/javascript/diff.js'
--- loggerhead/static/javascript/diff.js	1970-01-01 00:00:00 +0000
+++ loggerhead/static/javascript/diff.js	2009-03-31 21:11:38 +0000
@@ -0,0 +1,204 @@
+var unified = true;
+
+function make_unified(chunk) {
+  var pending_added = [];
+  function flush_adds(before) {
+    for (var i = 0; i < pending_added.length; i++) {
+      before.ancestor().insertBefore(pending_added[i], before);
+    }
+    pending_added.length = 0;
+  }
+  chunk.get('children').filter(".pseudorow").each(
+    function (line) {
+      if (line.hasClass("context-row")) {
+        flush_adds(line);
+        line.removeChild(line.query('.code'));
+      }
+      else if (line.hasClass("both-row")) {
+        var added_line = line.create('<div class="pseudorow insert-row"><div class="lineNumber first">&nbsp;</div><div class="clear">&nbsp;</div></div>');
+        var clear = added_line.query('.clear');
+        added_line.insertBefore(line.query('.lineNumber.second'), clear);
+        added_line.insertBefore(line.query('.code.insert'), clear);
+        pending_added[pending_added.length] = added_line;
+        line.insertBefore(line.create('<div class="lineNumber second">&nbsp;</div>'), line.query('.code.delete'));
+        line.replaceClass("both-row", "delete-row");
+      }
+      else if (line.hasClass("insert-row")) {
+        flush_adds(line);
+        line.removeChild(line.query('.blank'));
+      }
+      else if (line.hasClass("delete-row")) {
+        line.removeChild(line.query('.blank'));
+        line.insertBefore(line.query('.lineNumber.second'), line.query('.code.delete'));
+      }
+    });
+  flush_adds(null);
+  chunk.replaceClass('sbs', 'unified');
+}
+
+function make_sbs(chunk) {
+  var added = [];
+  var removed = [];
+  function clear_bufs(before) {
+    if (!added.length && !removed.length) return;
+    var common = Math.min(added.length, removed.length);
+    for (var i = 0; i < common; i++) {
+      var a = added[i];
+      var r = removed[i];
+      a.ancestor().removeChild(a);
+      r.removeChild(r.query('.lineNumber.second'));
+      r.insertBefore(a.query('.lineNumber.second'), r.query('.clear'));
+      r.insertBefore(a.query('.code.insert'), r.query('.clear'));
+      r.replaceClass('removed-row', 'both-row');
+    }
+    if (added.length > removed.length) {
+      for (var j = common; j < added.length; j++) {
+        a = added[j];
+        a.insertBefore(a.create('<div class="blank">&nbsp;</div>'), a.query('.lineNumber.second'));
+      }
+    }
+    else if (added.length < removed.length) {
+      for (var j = common; j < removed.length; j++) {
+        r = removed[j];
+        r.insertBefore(r.query('.code.delete'), r.query('.lineNumber.second'));
+        r.insertBefore(r.create('<div class="blank">&nbsp;</div>'), r.query('.clear'));
+      }
+    }
+    added.length = 0;
+    removed.length = 0;
+  }
+  chunk.get('children').filter(".pseudorow").each(
+    function (line) {
+      if (line.hasClass("context-row")) {
+        clear_bufs(line);
+        line.insertBefore(line.query('.code').cloneNode(true), line.query(".second"));
+      }
+      else if (line.hasClass("insert-row")) {
+        added[added.length] = line;
+      }
+      else if (line.hasClass("delete-row")) {
+        removed[removed.length] = line;
+      }
+    });
+  clear_bufs(null);
+  chunk.replaceClass('unified', 'sbs');
+
+}
+
+function toggle_unified_sbs(event) {
+  event.preventDefault();
+  var pts = Y.all(".pseudotable");
+  if (unified) {
+    pts && pts.each(make_sbs);
+    unified = false;
+    Y.get("#toggle_unified_sbs").set('innerHTML', "Show unified diffs");
+  }
+  else {
+    pts && pts.each(make_unified);
+    unified = true;
+    Y.get("#toggle_unified_sbs").set('innerHTML', "Show diffs side-by-side");
+  }
+}
+
+Y.on("click", toggle_unified_sbs, '#toggle_unified_sbs');
+
+function toggle_expand_all_revisionview(action)
+{
+  var diffs = Y.all('.diff');
+  if (diffs == null) return;
+  diffs.each(
+    function(item, i)
+    {
+      var collapsable = item.collapsable;
+      if(action == 'close')
+      {
+        Y.get('#expand_all').setStyle('display','block');
+        Y.get('#collapse_all').setStyle('display','none');
+        collapsable.close();
+      }
+      else if(action == 'open')
+      {
+        Y.get('#expand_all').setStyle('display','none');
+        Y.get('#collapse_all').setStyle('display','block');
+        collapsable.open();
+      }
+    });
+}
+
+Y.on(
+  'click',
+  function (event) {
+    event.preventDefault();
+    toggle_expand_all_revisionview('open');
+  },
+  '#expand_all a'
+);
+
+Y.on(
+  'click',
+  function (event) {
+    event.preventDefault();
+    toggle_expand_all_revisionview('close');
+  },
+  '#collapse_all a'
+);
+
+function node_process(node) {
+  if (!unified) {
+    node.get('children').filter('.pseudotable').each(make_sbs);
+  }
+}
+
+function zoom_to_diff (path) {
+  var collapsable = Y.get('#' + path_to_id[path]).collapsable;
+  if (!collapsable.is_open) {
+    collapsable.open(
+      function () {
+        window.location.hash = '#' + path;
+      });
+  }
+}
+
+Y.on(
+  "domready", function () {
+    Y.all(".show_if_js").removeClass("show_if_js");
+    if (!specific_path) {
+      Y.all("#list-files a").on(
+        'click',
+        function (e) {
+          e.preventDefault();
+          var path = decodeURIComponent(e.target.get('href').split('#')[1]);
+          window.location.hash = '#' + path;
+          zoom_to_diff(path);
+        });
+    }
+    var diffs = Y.all('.diff');
+    if (diffs == null) return;
+    diffs.each(
+      function(item, i)
+      {
+        var source_url = null;
+        if (!specific_path)
+          source_url = global_path + '+filediff/' + link_data[item.get('id')];
+        item.query('.the-link').on(
+          'click',
+          function(e) {
+            e.preventDefault();
+            collapsable.toggle();
+          });
+        var collapsable = new Collapsable(
+          {
+            expand_icon: item.query('.expand_diff'),
+            open_node: item.query('.diffinfo'),
+            close_node: null,
+            source: source_url,
+            is_open: specific_path != null,
+            loading: item.query('.loading'),
+            node_process: node_process
+          });
+       item.collapsable=collapsable;
+       });
+    if (window.location.hash && !specific_path) {
+      zoom_to_diff(window.location.hash.substring(1));
+    }
+  });

=== removed file 'loggerhead/static/javascript/mootools-1.2-core.js'
--- loggerhead/static/javascript/mootools-1.2-core.js	2008-06-13 23:50:23 +0000
+++ loggerhead/static/javascript/mootools-1.2-core.js	1970-01-01 00:00:00 +0000
@@ -1,333 +0,0 @@
-//MooTools, <http://mootools.net>, My Object Oriented (JavaScript) Tools. Copyright (c) 2006-2008 Valerio Proietti, <http://mad4milk.net>, MIT Style License.
-
-var MooTools={version:"1.2.0",build:""};var Native=function(J){J=J||{};var F=J.afterImplement||function(){};var G=J.generics;G=(G!==false);var H=J.legacy;
-var E=J.initialize;var B=J.protect;var A=J.name;var C=E||H;C.constructor=Native;C.$family={name:"native"};if(H&&E){C.prototype=H.prototype;}C.prototype.constructor=C;
-if(A){var D=A.toLowerCase();C.prototype.$family={name:D};Native.typize(C,D);}var I=function(M,K,N,L){if(!B||L||!M.prototype[K]){M.prototype[K]=N;}if(G){Native.genericize(M,K,B);
-}F.call(M,K,N);return M;};C.implement=function(L,K,N){if(typeof L=="string"){return I(this,L,K,N);}for(var M in L){I(this,M,L[M],K);}return this;};C.alias=function(M,K,N){if(typeof M=="string"){M=this.prototype[M];
-if(M){I(this,K,M,N);}}else{for(var L in M){this.alias(L,M[L],K);}}return this;};return C;};Native.implement=function(D,C){for(var B=0,A=D.length;B<A;B++){D[B].implement(C);
-}};Native.genericize=function(B,C,A){if((!A||!B[C])&&typeof B.prototype[C]=="function"){B[C]=function(){var D=Array.prototype.slice.call(arguments);return B.prototype[C].apply(D.shift(),D);
-};}};Native.typize=function(A,B){if(!A.type){A.type=function(C){return($type(C)===B);};}};Native.alias=function(E,B,A,F){for(var D=0,C=E.length;D<C;D++){E[D].alias(B,A,F);
-}};(function(B){for(var A in B){Native.typize(B[A],A);}})({"boolean":Boolean,"native":Native,object:Object});(function(B){for(var A in B){new Native({name:A,initialize:B[A],protect:true});
-}})({String:String,Function:Function,Number:Number,Array:Array,RegExp:RegExp,Date:Date});(function(B,A){for(var C=A.length;C--;C){Native.genericize(B,A[C],true);
-}return arguments.callee;})(Array,["pop","push","reverse","shift","sort","splice","unshift","concat","join","slice","toString","valueOf","indexOf","lastIndexOf"])(String,["charAt","charCodeAt","concat","indexOf","lastIndexOf","match","replace","search","slice","split","substr","substring","toLowerCase","toUpperCase","valueOf"]);
-function $chk(A){return !!(A||A===0);}function $clear(A){clearTimeout(A);clearInterval(A);return null;}function $defined(A){return(A!=undefined);}function $empty(){}function $arguments(A){return function(){return arguments[A];
-};}function $lambda(A){return(typeof A=="function")?A:function(){return A;};}function $extend(C,A){for(var B in (A||{})){C[B]=A[B];}return C;}function $unlink(C){var B;
-switch($type(C)){case"object":B={};for(var E in C){B[E]=$unlink(C[E]);}break;case"hash":B=$unlink(C.getClean());break;case"array":B=[];for(var D=0,A=C.length;
-D<A;D++){B[D]=$unlink(C[D]);}break;default:return C;}return B;}function $merge(){var E={};for(var D=0,A=arguments.length;D<A;D++){var B=arguments[D];if($type(B)!="object"){continue;
-}for(var C in B){var G=B[C],F=E[C];E[C]=(F&&$type(G)=="object"&&$type(F)=="object")?$merge(F,G):$unlink(G);}}return E;}function $pick(){for(var B=0,A=arguments.length;
-B<A;B++){if(arguments[B]!=undefined){return arguments[B];}}return null;}function $random(B,A){return Math.floor(Math.random()*(A-B+1)+B);}function $splat(B){var A=$type(B);
-return(A)?((A!="array"&&A!="arguments")?[B]:B):[];}var $time=Date.now||function(){return new Date().getTime();};function $try(){for(var B=0,A=arguments.length;
-B<A;B++){try{return arguments[B]();}catch(C){}}return null;}function $type(A){if(A==undefined){return false;}if(A.$family){return(A.$family.name=="number"&&!isFinite(A))?false:A.$family.name;
-}if(A.nodeName){switch(A.nodeType){case 1:return"element";case 3:return(/\S/).test(A.nodeValue)?"textnode":"whitespace";}}else{if(typeof A.length=="number"){if(A.callee){return"arguments";
-}else{if(A.item){return"collection";}}}}return typeof A;}var Hash=new Native({name:"Hash",initialize:function(A){if($type(A)=="hash"){A=$unlink(A.getClean());
-}for(var B in A){this[B]=A[B];}return this;}});Hash.implement({getLength:function(){var B=0;for(var A in this){if(this.hasOwnProperty(A)){B++;}}return B;
-},forEach:function(B,C){for(var A in this){if(this.hasOwnProperty(A)){B.call(C,this[A],A,this);}}},getClean:function(){var B={};for(var A in this){if(this.hasOwnProperty(A)){B[A]=this[A];
-}}return B;}});Hash.alias("forEach","each");function $H(A){return new Hash(A);}Array.implement({forEach:function(C,D){for(var B=0,A=this.length;B<A;B++){C.call(D,this[B],B,this);
-}}});Array.alias("forEach","each");function $A(C){if(C.item){var D=[];for(var B=0,A=C.length;B<A;B++){D[B]=C[B];}return D;}return Array.prototype.slice.call(C);
-}function $each(C,B,D){var A=$type(C);((A=="arguments"||A=="collection"||A=="array")?Array:Hash).each(C,B,D);}var Browser=new Hash({Engine:{name:"unknown",version:""},Platform:{name:(navigator.platform.match(/mac|win|linux/i)||["other"])[0].toLowerCase()},Features:{xpath:!!(document.evaluate),air:!!(window.runtime)},Plugins:{}});
-if(window.opera){Browser.Engine={name:"presto",version:(document.getElementsByClassName)?950:925};}else{if(window.ActiveXObject){Browser.Engine={name:"trident",version:(window.XMLHttpRequest)?5:4};
-}else{if(!navigator.taintEnabled){Browser.Engine={name:"webkit",version:(Browser.Features.xpath)?420:419};}else{if(document.getBoxObjectFor!=null){Browser.Engine={name:"gecko",version:(document.getElementsByClassName)?19:18};
-}}}}Browser.Engine[Browser.Engine.name]=Browser.Engine[Browser.Engine.name+Browser.Engine.version]=true;if(window.orientation!=undefined){Browser.Platform.name="ipod";
-}Browser.Platform[Browser.Platform.name]=true;Browser.Request=function(){return $try(function(){return new XMLHttpRequest();},function(){return new ActiveXObject("MSXML2.XMLHTTP");
-});};Browser.Features.xhr=!!(Browser.Request());Browser.Plugins.Flash=(function(){var A=($try(function(){return navigator.plugins["Shockwave Flash"].description;
-},function(){return new ActiveXObject("ShockwaveFlash.ShockwaveFlash").GetVariable("$version");})||"0 r0").match(/\d+/g);return{version:parseInt(A[0]||0+"."+A[1]||0),build:parseInt(A[2]||0)};
-})();function $exec(B){if(!B){return B;}if(window.execScript){window.execScript(B);}else{var A=document.createElement("script");A.setAttribute("type","text/javascript");
-A.text=B;document.head.appendChild(A);document.head.removeChild(A);}return B;}Native.UID=1;var $uid=(Browser.Engine.trident)?function(A){return(A.uid||(A.uid=[Native.UID++]))[0];
-}:function(A){return A.uid||(A.uid=Native.UID++);};var Window=new Native({name:"Window",legacy:(Browser.Engine.trident)?null:window.Window,initialize:function(A){$uid(A);
-if(!A.Element){A.Element=$empty;if(Browser.Engine.webkit){A.document.createElement("iframe");}A.Element.prototype=(Browser.Engine.webkit)?window["[[DOMElement.prototype]]"]:{};
-}return $extend(A,Window.Prototype);},afterImplement:function(B,A){window[B]=Window.Prototype[B]=A;}});Window.Prototype={$family:{name:"window"}};new Window(window);
-var Document=new Native({name:"Document",legacy:(Browser.Engine.trident)?null:window.Document,initialize:function(A){$uid(A);A.head=A.getElementsByTagName("head")[0];
-A.html=A.getElementsByTagName("html")[0];A.window=A.defaultView||A.parentWindow;if(Browser.Engine.trident4){$try(function(){A.execCommand("BackgroundImageCache",false,true);
-});}return $extend(A,Document.Prototype);},afterImplement:function(B,A){document[B]=Document.Prototype[B]=A;}});Document.Prototype={$family:{name:"document"}};
-new Document(document);Array.implement({every:function(C,D){for(var B=0,A=this.length;B<A;B++){if(!C.call(D,this[B],B,this)){return false;}}return true;
-},filter:function(D,E){var C=[];for(var B=0,A=this.length;B<A;B++){if(D.call(E,this[B],B,this)){C.push(this[B]);}}return C;},clean:function(){return this.filter($defined);
-},indexOf:function(C,D){var A=this.length;for(var B=(D<0)?Math.max(0,A+D):D||0;B<A;B++){if(this[B]===C){return B;}}return -1;},map:function(D,E){var C=[];
-for(var B=0,A=this.length;B<A;B++){C[B]=D.call(E,this[B],B,this);}return C;},some:function(C,D){for(var B=0,A=this.length;B<A;B++){if(C.call(D,this[B],B,this)){return true;
-}}return false;},associate:function(C){var D={},B=Math.min(this.length,C.length);for(var A=0;A<B;A++){D[C[A]]=this[A];}return D;},link:function(C){var A={};
-for(var E=0,B=this.length;E<B;E++){for(var D in C){if(C[D](this[E])){A[D]=this[E];delete C[D];break;}}}return A;},contains:function(A,B){return this.indexOf(A,B)!=-1;
-},extend:function(C){for(var B=0,A=C.length;B<A;B++){this.push(C[B]);}return this;},getLast:function(){return(this.length)?this[this.length-1]:null;},getRandom:function(){return(this.length)?this[$random(0,this.length-1)]:null;
-},include:function(A){if(!this.contains(A)){this.push(A);}return this;},combine:function(C){for(var B=0,A=C.length;B<A;B++){this.include(C[B]);}return this;
-},erase:function(B){for(var A=this.length;A--;A){if(this[A]===B){this.splice(A,1);}}return this;},empty:function(){this.length=0;return this;},flatten:function(){var D=[];
-for(var B=0,A=this.length;B<A;B++){var C=$type(this[B]);if(!C){continue;}D=D.concat((C=="array"||C=="collection"||C=="arguments")?Array.flatten(this[B]):this[B]);
-}return D;},hexToRgb:function(B){if(this.length!=3){return null;}var A=this.map(function(C){if(C.length==1){C+=C;}return C.toInt(16);});return(B)?A:"rgb("+A+")";
-},rgbToHex:function(D){if(this.length<3){return null;}if(this.length==4&&this[3]==0&&!D){return"transparent";}var B=[];for(var A=0;A<3;A++){var C=(this[A]-0).toString(16);
-B.push((C.length==1)?"0"+C:C);}return(D)?B:"#"+B.join("");}});Function.implement({extend:function(A){for(var B in A){this[B]=A[B];}return this;},create:function(B){var A=this;
-B=B||{};return function(D){var C=B.arguments;C=(C!=undefined)?$splat(C):Array.slice(arguments,(B.event)?1:0);if(B.event){C=[D||window.event].extend(C);
-}var E=function(){return A.apply(B.bind||null,C);};if(B.delay){return setTimeout(E,B.delay);}if(B.periodical){return setInterval(E,B.periodical);}if(B.attempt){return $try(E);
-}return E();};},pass:function(A,B){return this.create({arguments:A,bind:B});},attempt:function(A,B){return this.create({arguments:A,bind:B,attempt:true})();
-},bind:function(B,A){return this.create({bind:B,arguments:A});},bindWithEvent:function(B,A){return this.create({bind:B,event:true,arguments:A});},delay:function(B,C,A){return this.create({delay:B,bind:C,arguments:A})();
-},periodical:function(A,C,B){return this.create({periodical:A,bind:C,arguments:B})();},run:function(A,B){return this.apply(B,$splat(A));}});Number.implement({limit:function(B,A){return Math.min(A,Math.max(B,this));
-},round:function(A){A=Math.pow(10,A||0);return Math.round(this*A)/A;},times:function(B,C){for(var A=0;A<this;A++){B.call(C,A,this);}},toFloat:function(){return parseFloat(this);
-},toInt:function(A){return parseInt(this,A||10);}});Number.alias("times","each");(function(B){var A={};B.each(function(C){if(!Number[C]){A[C]=function(){return Math[C].apply(null,[this].concat($A(arguments)));
-};}});Number.implement(A);})(["abs","acos","asin","atan","atan2","ceil","cos","exp","floor","log","max","min","pow","sin","sqrt","tan"]);String.implement({test:function(A,B){return((typeof A=="string")?new RegExp(A,B):A).test(this);
-},contains:function(A,B){return(B)?(B+this+B).indexOf(B+A+B)>-1:this.indexOf(A)>-1;},trim:function(){return this.replace(/^\s+|\s+$/g,"");},clean:function(){return this.replace(/\s+/g," ").trim();
-},camelCase:function(){return this.replace(/-\D/g,function(A){return A.charAt(1).toUpperCase();});},hyphenate:function(){return this.replace(/[A-Z]/g,function(A){return("-"+A.charAt(0).toLowerCase());
-});},capitalize:function(){return this.replace(/\b[a-z]/g,function(A){return A.toUpperCase();});},escapeRegExp:function(){return this.replace(/([-.*+?^${}()|[\]\/\\])/g,"\\$1");
-},toInt:function(A){return parseInt(this,A||10);},toFloat:function(){return parseFloat(this);},hexToRgb:function(B){var A=this.match(/^#?(\w{1,2})(\w{1,2})(\w{1,2})$/);
-return(A)?A.slice(1).hexToRgb(B):null;},rgbToHex:function(B){var A=this.match(/\d{1,3}/g);return(A)?A.rgbToHex(B):null;},stripScripts:function(B){var A="";
-var C=this.replace(/<script[^>]*>([\s\S]*?)<\/script>/gi,function(){A+=arguments[1]+"\n";return"";});if(B===true){$exec(A);}else{if($type(B)=="function"){B(A,C);
-}}return C;},substitute:function(A,B){return this.replace(B||(/\\?\{([^}]+)\}/g),function(D,C){if(D.charAt(0)=="\\"){return D.slice(1);}return(A[C]!=undefined)?A[C]:"";
-});}});Hash.implement({has:Object.prototype.hasOwnProperty,keyOf:function(B){for(var A in this){if(this.hasOwnProperty(A)&&this[A]===B){return A;}}return null;
-},hasValue:function(A){return(Hash.keyOf(this,A)!==null);},extend:function(A){Hash.each(A,function(C,B){Hash.set(this,B,C);},this);return this;},combine:function(A){Hash.each(A,function(C,B){Hash.include(this,B,C);
-},this);return this;},erase:function(A){if(this.hasOwnProperty(A)){delete this[A];}return this;},get:function(A){return(this.hasOwnProperty(A))?this[A]:null;
-},set:function(A,B){if(!this[A]||this.hasOwnProperty(A)){this[A]=B;}return this;},empty:function(){Hash.each(this,function(B,A){delete this[A];},this);
-return this;},include:function(B,C){var A=this[B];if(A==undefined){this[B]=C;}return this;},map:function(B,C){var A=new Hash;Hash.each(this,function(E,D){A.set(D,B.call(C,E,D,this));
-},this);return A;},filter:function(B,C){var A=new Hash;Hash.each(this,function(E,D){if(B.call(C,E,D,this)){A.set(D,E);}},this);return A;},every:function(B,C){for(var A in this){if(this.hasOwnProperty(A)&&!B.call(C,this[A],A)){return false;
-}}return true;},some:function(B,C){for(var A in this){if(this.hasOwnProperty(A)&&B.call(C,this[A],A)){return true;}}return false;},getKeys:function(){var A=[];
-Hash.each(this,function(C,B){A.push(B);});return A;},getValues:function(){var A=[];Hash.each(this,function(B){A.push(B);});return A;},toQueryString:function(A){var B=[];
-Hash.each(this,function(F,E){if(A){E=A+"["+E+"]";}var D;switch($type(F)){case"object":D=Hash.toQueryString(F,E);break;case"array":var C={};F.each(function(H,G){C[G]=H;
-});D=Hash.toQueryString(C,E);break;default:D=E+"="+encodeURIComponent(F);}if(F!=undefined){B.push(D);}});return B.join("&");}});Hash.alias({keyOf:"indexOf",hasValue:"contains"});
-var Event=new Native({name:"Event",initialize:function(A,F){F=F||window;var K=F.document;A=A||F.event;if(A.$extended){return A;}this.$extended=true;var J=A.type;
-var G=A.target||A.srcElement;while(G&&G.nodeType==3){G=G.parentNode;}if(J.test(/key/)){var B=A.which||A.keyCode;var M=Event.Keys.keyOf(B);if(J=="keydown"){var D=B-111;
-if(D>0&&D<13){M="f"+D;}}M=M||String.fromCharCode(B).toLowerCase();}else{if(J.match(/(click|mouse|menu)/i)){K=(!K.compatMode||K.compatMode=="CSS1Compat")?K.html:K.body;
-var I={x:A.pageX||A.clientX+K.scrollLeft,y:A.pageY||A.clientY+K.scrollTop};var C={x:(A.pageX)?A.pageX-F.pageXOffset:A.clientX,y:(A.pageY)?A.pageY-F.pageYOffset:A.clientY};
-if(J.match(/DOMMouseScroll|mousewheel/)){var H=(A.wheelDelta)?A.wheelDelta/120:-(A.detail||0)/3;}var E=(A.which==3)||(A.button==2);var L=null;if(J.match(/over|out/)){switch(J){case"mouseover":L=A.relatedTarget||A.fromElement;
-break;case"mouseout":L=A.relatedTarget||A.toElement;}if(!(function(){while(L&&L.nodeType==3){L=L.parentNode;}return true;}).create({attempt:Browser.Engine.gecko})()){L=false;
-}}}}return $extend(this,{event:A,type:J,page:I,client:C,rightClick:E,wheel:H,relatedTarget:L,target:G,code:B,key:M,shift:A.shiftKey,control:A.ctrlKey,alt:A.altKey,meta:A.metaKey});
-}});Event.Keys=new Hash({enter:13,up:38,down:40,left:37,right:39,esc:27,space:32,backspace:8,tab:9,"delete":46});Event.implement({stop:function(){return this.stopPropagation().preventDefault();
-},stopPropagation:function(){if(this.event.stopPropagation){this.event.stopPropagation();}else{this.event.cancelBubble=true;}return this;},preventDefault:function(){if(this.event.preventDefault){this.event.preventDefault();
-}else{this.event.returnValue=false;}return this;}});var Class=new Native({name:"Class",initialize:function(B){B=B||{};var A=function(E){for(var D in this){this[D]=$unlink(this[D]);
-}for(var F in Class.Mutators){if(!this[F]){continue;}Class.Mutators[F](this,this[F]);delete this[F];}this.constructor=A;if(E===$empty){return this;}var C=(this.initialize)?this.initialize.apply(this,arguments):this;
-if(this.options&&this.options.initialize){this.options.initialize.call(this);}return C;};$extend(A,this);A.constructor=Class;A.prototype=B;return A;}});
-Class.implement({implement:function(){Class.Mutators.Implements(this.prototype,Array.slice(arguments));return this;}});Class.Mutators={Implements:function(A,B){$splat(B).each(function(C){$extend(A,($type(C)=="class")?new C($empty):C);
-});},Extends:function(self,klass){var instance=new klass($empty);delete instance.parent;delete instance.parentOf;for(var key in instance){var current=self[key],previous=instance[key];
-if(current==undefined){self[key]=previous;continue;}var ctype=$type(current),ptype=$type(previous);if(ctype!=ptype){continue;}switch(ctype){case"function":if(!arguments.callee.caller){self[key]=eval("("+String(current).replace(/\bthis\.parent\(\s*(\))?/g,function(full,close){return"arguments.callee._parent_.call(this"+(close||", ");
-})+")");}self[key]._parent_=previous;break;case"object":self[key]=$merge(previous,current);}}self.parent=function(){return arguments.callee.caller._parent_.apply(this,arguments);
-};self.parentOf=function(descendant){return descendant._parent_.apply(this,Array.slice(arguments,1));};}};var Chain=new Class({chain:function(){this.$chain=(this.$chain||[]).extend(arguments);
-return this;},callChain:function(){return(this.$chain&&this.$chain.length)?this.$chain.shift().apply(this,arguments):false;},clearChain:function(){if(this.$chain){this.$chain.empty();
-}return this;}});var Events=new Class({addEvent:function(C,B,A){C=Events.removeOn(C);if(B!=$empty){this.$events=this.$events||{};this.$events[C]=this.$events[C]||[];
-this.$events[C].include(B);if(A){B.internal=true;}}return this;},addEvents:function(A){for(var B in A){this.addEvent(B,A[B]);}return this;},fireEvent:function(C,B,A){C=Events.removeOn(C);
-if(!this.$events||!this.$events[C]){return this;}this.$events[C].each(function(D){D.create({bind:this,delay:A,"arguments":B})();},this);return this;},removeEvent:function(B,A){B=Events.removeOn(B);
-if(!this.$events||!this.$events[B]){return this;}if(!A.internal){this.$events[B].erase(A);}return this;},removeEvents:function(C){for(var D in this.$events){if(C&&C!=D){continue;
-}var B=this.$events[D];for(var A=B.length;A--;A){this.removeEvent(D,B[A]);}}return this;}});Events.removeOn=function(A){return A.replace(/^on([A-Z])/,function(B,C){return C.toLowerCase();
-});};var Options=new Class({setOptions:function(){this.options=$merge.run([this.options].extend(arguments));if(!this.addEvent){return this;}for(var A in this.options){if($type(this.options[A])!="function"||!(/^on[A-Z]/).test(A)){continue;
-}this.addEvent(A,this.options[A]);delete this.options[A];}return this;}});Document.implement({newElement:function(A,B){if(Browser.Engine.trident&&B){["name","type","checked"].each(function(C){if(!B[C]){return ;
-}A+=" "+C+'="'+B[C]+'"';if(C!="checked"){delete B[C];}});A="<"+A+">";}return $.element(this.createElement(A)).set(B);},newTextNode:function(A){return this.createTextNode(A);
-},getDocument:function(){return this;},getWindow:function(){return this.defaultView||this.parentWindow;},purge:function(){var C=this.getElementsByTagName("*");
-for(var B=0,A=C.length;B<A;B++){Browser.freeMem(C[B]);}}});var Element=new Native({name:"Element",legacy:window.Element,initialize:function(A,B){var C=Element.Constructors.get(A);
-if(C){return C(B);}if(typeof A=="string"){return document.newElement(A,B);}return $(A).set(B);},afterImplement:function(A,B){if(!Array[A]){Elements.implement(A,Elements.multi(A));
-}Element.Prototype[A]=B;}});Element.Prototype={$family:{name:"element"}};Element.Constructors=new Hash;var IFrame=new Native({name:"IFrame",generics:false,initialize:function(){var E=Array.link(arguments,{properties:Object.type,iframe:$defined});
-var C=E.properties||{};var B=$(E.iframe)||false;var D=C.onload||$empty;delete C.onload;C.id=C.name=$pick(C.id,C.name,B.id,B.name,"IFrame_"+$time());B=new Element(B||"iframe",C);
-var A=function(){var F=$try(function(){return B.contentWindow.location.host;});if(F&&F==window.location.host){var H=new Window(B.contentWindow);var G=new Document(B.contentWindow.document);
-$extend(H.Element.prototype,Element.Prototype);}D.call(B.contentWindow,B.contentWindow.document);};(!window.frames[C.id])?B.addListener("load",A):A();return B;
-}});var Elements=new Native({initialize:function(F,B){B=$extend({ddup:true,cash:true},B);F=F||[];if(B.ddup||B.cash){var G={},E=[];for(var C=0,A=F.length;
-C<A;C++){var D=$.element(F[C],!B.cash);if(B.ddup){if(G[D.uid]){continue;}G[D.uid]=true;}E.push(D);}F=E;}return(B.cash)?$extend(F,this):F;}});Elements.implement({filter:function(A,B){if(!A){return this;
-}return new Elements(Array.filter(this,(typeof A=="string")?function(C){return C.match(A);}:A,B));}});Elements.multi=function(A){return function(){var B=[];
-var F=true;for(var D=0,C=this.length;D<C;D++){var E=this[D][A].apply(this[D],arguments);B.push(E);if(F){F=($type(E)=="element");}}return(F)?new Elements(B):B;
-};};Window.implement({$:function(B,C){if(B&&B.$family&&B.uid){return B;}var A=$type(B);return($[A])?$[A](B,C,this.document):null;},$$:function(A){if(arguments.length==1&&typeof A=="string"){return this.document.getElements(A);
-}var F=[];var C=Array.flatten(arguments);for(var D=0,B=C.length;D<B;D++){var E=C[D];switch($type(E)){case"element":E=[E];break;case"string":E=this.document.getElements(E,true);
-break;default:E=false;}if(E){F.extend(E);}}return new Elements(F);},getDocument:function(){return this.document;},getWindow:function(){return this;}});
-$.string=function(C,B,A){C=A.getElementById(C);return(C)?$.element(C,B):null;};$.element=function(A,D){$uid(A);if(!D&&!A.$family&&!(/^object|embed$/i).test(A.tagName)){var B=Element.Prototype;
-for(var C in B){A[C]=B[C];}}return A;};$.object=function(B,C,A){if(B.toElement){return $.element(B.toElement(A),C);}return null;};$.textnode=$.whitespace=$.window=$.document=$arguments(0);
-Native.implement([Element,Document],{getElement:function(A,B){return $(this.getElements(A,true)[0]||null,B);},getElements:function(A,D){A=A.split(",");
-var C=[];var B=(A.length>1);A.each(function(E){var F=this.getElementsByTagName(E.trim());(B)?C.extend(F):C=F;},this);return new Elements(C,{ddup:B,cash:!D});
-}});Element.Storage={get:function(A){return(this[A]||(this[A]={}));}};Element.Inserters=new Hash({before:function(B,A){if(A.parentNode){A.parentNode.insertBefore(B,A);
-}},after:function(B,A){if(!A.parentNode){return ;}var C=A.nextSibling;(C)?A.parentNode.insertBefore(B,C):A.parentNode.appendChild(B);},bottom:function(B,A){A.appendChild(B);
-},top:function(B,A){var C=A.firstChild;(C)?A.insertBefore(B,C):A.appendChild(B);}});Element.Inserters.inside=Element.Inserters.bottom;Element.Inserters.each(function(C,B){var A=B.capitalize();
-Element.implement("inject"+A,function(D){C(this,$(D,true));return this;});Element.implement("grab"+A,function(D){C($(D,true),this);return this;});});Element.implement({getDocument:function(){return this.ownerDocument;
-},getWindow:function(){return this.ownerDocument.getWindow();},getElementById:function(D,C){var B=this.ownerDocument.getElementById(D);if(!B){return null;
-}for(var A=B.parentNode;A!=this;A=A.parentNode){if(!A){return null;}}return $.element(B,C);},set:function(D,B){switch($type(D)){case"object":for(var C in D){this.set(C,D[C]);
-}break;case"string":var A=Element.Properties.get(D);(A&&A.set)?A.set.apply(this,Array.slice(arguments,1)):this.setProperty(D,B);}return this;},get:function(B){var A=Element.Properties.get(B);
-return(A&&A.get)?A.get.apply(this,Array.slice(arguments,1)):this.getProperty(B);},erase:function(B){var A=Element.Properties.get(B);(A&&A.erase)?A.erase.apply(this,Array.slice(arguments,1)):this.removeProperty(B);
-return this;},match:function(A){return(!A||Element.get(this,"tag")==A);},inject:function(B,A){Element.Inserters.get(A||"bottom")(this,$(B,true));return this;
-},wraps:function(B,A){B=$(B,true);return this.replaces(B).grab(B,A);},grab:function(B,A){Element.Inserters.get(A||"bottom")($(B,true),this);return this;
-},appendText:function(B,A){return this.grab(this.getDocument().newTextNode(B),A);},adopt:function(){Array.flatten(arguments).each(function(A){A=$(A,true);
-if(A){this.appendChild(A);}},this);return this;},dispose:function(){return(this.parentNode)?this.parentNode.removeChild(this):this;},clone:function(D,C){switch($type(this)){case"element":var H={};
-for(var G=0,E=this.attributes.length;G<E;G++){var B=this.attributes[G],L=B.nodeName.toLowerCase();if(Browser.Engine.trident&&(/input/i).test(this.tagName)&&(/width|height/).test(L)){continue;
-}var K=(L=="style"&&this.style)?this.style.cssText:B.nodeValue;if(!$chk(K)||L=="uid"||(L=="id"&&!C)){continue;}if(K!="inherit"&&["string","number"].contains($type(K))){H[L]=K;
-}}var J=new Element(this.nodeName.toLowerCase(),H);if(D!==false){for(var I=0,F=this.childNodes.length;I<F;I++){var A=Element.clone(this.childNodes[I],true,C);
-if(A){J.grab(A);}}}return J;case"textnode":return document.newTextNode(this.nodeValue);}return null;},replaces:function(A){A=$(A,true);A.parentNode.replaceChild(this,A);
-return this;},hasClass:function(A){return this.className.contains(A," ");},addClass:function(A){if(!this.hasClass(A)){this.className=(this.className+" "+A).clean();
-}return this;},removeClass:function(A){this.className=this.className.replace(new RegExp("(^|\\s)"+A+"(?:\\s|$)"),"$1").clean();return this;},toggleClass:function(A){return this.hasClass(A)?this.removeClass(A):this.addClass(A);
-},getComputedStyle:function(B){if(this.currentStyle){return this.currentStyle[B.camelCase()];}var A=this.getWindow().getComputedStyle(this,null);return(A)?A.getPropertyValue([B.hyphenate()]):null;
-},empty:function(){$A(this.childNodes).each(function(A){Browser.freeMem(A);Element.empty(A);Element.dispose(A);},this);return this;},destroy:function(){Browser.freeMem(this.empty().dispose());
-return null;},getSelected:function(){return new Elements($A(this.options).filter(function(A){return A.selected;}));},toQueryString:function(){var A=[];
-this.getElements("input, select, textarea").each(function(B){if(!B.name||B.disabled){return ;}var C=(B.tagName.toLowerCase()=="select")?Element.getSelected(B).map(function(D){return D.value;
-}):((B.type=="radio"||B.type=="checkbox")&&!B.checked)?null:B.value;$splat(C).each(function(D){if(D){A.push(B.name+"="+encodeURIComponent(D));}});});return A.join("&");
-},getProperty:function(C){var B=Element.Attributes,A=B.Props[C];var D=(A)?this[A]:this.getAttribute(C,2);return(B.Bools[C])?!!D:(A)?D:D||null;},getProperties:function(){var A=$A(arguments);
-return A.map(function(B){return this.getProperty(B);},this).associate(A);},setProperty:function(D,E){var C=Element.Attributes,B=C.Props[D],A=$defined(E);
-if(B&&C.Bools[D]){E=(E||!A)?true:false;}else{if(!A){return this.removeProperty(D);}}(B)?this[B]=E:this.setAttribute(D,E);return this;},setProperties:function(A){for(var B in A){this.setProperty(B,A[B]);
-}return this;},removeProperty:function(D){var C=Element.Attributes,B=C.Props[D],A=(B&&C.Bools[D]);(B)?this[B]=(A)?false:"":this.removeAttribute(D);return this;
-},removeProperties:function(){Array.each(arguments,this.removeProperty,this);return this;}});(function(){var A=function(D,B,I,C,F,H){var E=D[I||B];var G=[];
-while(E){if(E.nodeType==1&&(!C||Element.match(E,C))){G.push(E);if(!F){break;}}E=E[B];}return(F)?new Elements(G,{ddup:false,cash:!H}):$(G[0],H);};Element.implement({getPrevious:function(B,C){return A(this,"previousSibling",null,B,false,C);
-},getAllPrevious:function(B,C){return A(this,"previousSibling",null,B,true,C);},getNext:function(B,C){return A(this,"nextSibling",null,B,false,C);},getAllNext:function(B,C){return A(this,"nextSibling",null,B,true,C);
-},getFirst:function(B,C){return A(this,"nextSibling","firstChild",B,false,C);},getLast:function(B,C){return A(this,"previousSibling","lastChild",B,false,C);
-},getParent:function(B,C){return A(this,"parentNode",null,B,false,C);},getParents:function(B,C){return A(this,"parentNode",null,B,true,C);},getChildren:function(B,C){return A(this,"nextSibling","firstChild",B,true,C);
-},hasChild:function(B){B=$(B,true);return(!!B&&$A(this.getElementsByTagName(B.tagName)).contains(B));}});})();Element.Properties=new Hash;Element.Properties.style={set:function(A){this.style.cssText=A;
-},get:function(){return this.style.cssText;},erase:function(){this.style.cssText="";}};Element.Properties.tag={get:function(){return this.tagName.toLowerCase();
-}};Element.Properties.href={get:function(){return(!this.href)?null:this.href.replace(new RegExp("^"+document.location.protocol+"//"+document.location.host),"");
-}};Element.Properties.html={set:function(){return this.innerHTML=Array.flatten(arguments).join("");}};Native.implement([Element,Window,Document],{addListener:function(B,A){if(this.addEventListener){this.addEventListener(B,A,false);
-}else{this.attachEvent("on"+B,A);}return this;},removeListener:function(B,A){if(this.removeEventListener){this.removeEventListener(B,A,false);}else{this.detachEvent("on"+B,A);
-}return this;},retrieve:function(B,A){var D=Element.Storage.get(this.uid);var C=D[B];if($defined(A)&&!$defined(C)){C=D[B]=A;}return $pick(C);},store:function(B,A){var C=Element.Storage.get(this.uid);
-C[B]=A;return this;},eliminate:function(A){var B=Element.Storage.get(this.uid);delete B[A];return this;}});Element.Attributes=new Hash({Props:{html:"innerHTML","class":"className","for":"htmlFor",text:(Browser.Engine.trident)?"innerText":"textContent"},Bools:["compact","nowrap","ismap","declare","noshade","checked","disabled","readonly","multiple","selected","noresize","defer"],Camels:["value","accessKey","cellPadding","cellSpacing","colSpan","frameBorder","maxLength","readOnly","rowSpan","tabIndex","useMap"]});
-Browser.freeMem=function(A){if(!A){return ;}if(Browser.Engine.trident&&(/object/i).test(A.tagName)){for(var B in A){if(typeof A[B]=="function"){A[B]=$empty;
-}}Element.dispose(A);}if(A.uid&&A.removeEvents){A.removeEvents();}};(function(B){var C=B.Bools,A=B.Camels;B.Bools=C=C.associate(C);Hash.extend(Hash.combine(B.Props,C),A.associate(A.map(function(D){return D.toLowerCase();
-})));B.erase("Camels");})(Element.Attributes);window.addListener("unload",function(){window.removeListener("unload",arguments.callee);document.purge();
-if(Browser.Engine.trident){CollectGarbage();}});Element.Properties.events={set:function(A){this.addEvents(A);}};Native.implement([Element,Window,Document],{addEvent:function(E,G){var H=this.retrieve("events",{});
-H[E]=H[E]||{keys:[],values:[]};if(H[E].keys.contains(G)){return this;}H[E].keys.push(G);var F=E,A=Element.Events.get(E),C=G,I=this;if(A){if(A.onAdd){A.onAdd.call(this,G);
-}if(A.condition){C=function(J){if(A.condition.call(this,J)){return G.call(this,J);}return false;};}F=A.base||F;}var D=function(){return G.call(I);};var B=Element.NativeEvents[F]||0;
-if(B){if(B==2){D=function(J){J=new Event(J,I.getWindow());if(C.call(I,J)===false){J.stop();}};}this.addListener(F,D);}H[E].values.push(D);return this;},removeEvent:function(D,C){var B=this.retrieve("events");
-if(!B||!B[D]){return this;}var G=B[D].keys.indexOf(C);if(G==-1){return this;}var A=B[D].keys.splice(G,1)[0];var F=B[D].values.splice(G,1)[0];var E=Element.Events.get(D);
-if(E){if(E.onRemove){E.onRemove.call(this,C);}D=E.base||D;}return(Element.NativeEvents[D])?this.removeListener(D,F):this;},addEvents:function(A){for(var B in A){this.addEvent(B,A[B]);
-}return this;},removeEvents:function(B){var A=this.retrieve("events");if(!A){return this;}if(!B){for(var C in A){this.removeEvents(C);}A=null;}else{if(A[B]){while(A[B].keys[0]){this.removeEvent(B,A[B].keys[0]);
-}A[B]=null;}}return this;},fireEvent:function(D,B,A){var C=this.retrieve("events");if(!C||!C[D]){return this;}C[D].keys.each(function(E){E.create({bind:this,delay:A,"arguments":B})();
-},this);return this;},cloneEvents:function(D,A){D=$(D);var C=D.retrieve("events");if(!C){return this;}if(!A){for(var B in C){this.cloneEvents(D,B);}}else{if(C[A]){C[A].keys.each(function(E){this.addEvent(A,E);
-},this);}}return this;}});Element.NativeEvents={click:2,dblclick:2,mouseup:2,mousedown:2,contextmenu:2,mousewheel:2,DOMMouseScroll:2,mouseover:2,mouseout:2,mousemove:2,selectstart:2,selectend:2,keydown:2,keypress:2,keyup:2,focus:2,blur:2,change:2,reset:2,select:2,submit:2,load:1,unload:1,beforeunload:2,resize:1,move:1,DOMContentLoaded:1,readystatechange:1,error:1,abort:1,scroll:1};
-(function(){var A=function(B){var C=B.relatedTarget;if(C==undefined){return true;}if(C===false){return false;}return($type(this)!="document"&&C!=this&&C.prefix!="xul"&&!this.hasChild(C));
-};Element.Events=new Hash({mouseenter:{base:"mouseover",condition:A},mouseleave:{base:"mouseout",condition:A},mousewheel:{base:(Browser.Engine.gecko)?"DOMMouseScroll":"mousewheel"}});
-})();Element.Properties.styles={set:function(A){this.setStyles(A);}};Element.Properties.opacity={set:function(A,B){if(!B){if(A==0){if(this.style.visibility!="hidden"){this.style.visibility="hidden";
-}}else{if(this.style.visibility!="visible"){this.style.visibility="visible";}}}if(!this.currentStyle||!this.currentStyle.hasLayout){this.style.zoom=1;}if(Browser.Engine.trident){this.style.filter=(A==1)?"":"alpha(opacity="+A*100+")";
-}this.style.opacity=A;this.store("opacity",A);},get:function(){return this.retrieve("opacity",1);}};Element.implement({setOpacity:function(A){return this.set("opacity",A,true);
-},getOpacity:function(){return this.get("opacity");},setStyle:function(B,A){switch(B){case"opacity":return this.set("opacity",parseFloat(A));case"float":B=(Browser.Engine.trident)?"styleFloat":"cssFloat";
-}B=B.camelCase();if($type(A)!="string"){var C=(Element.Styles.get(B)||"@").split(" ");A=$splat(A).map(function(E,D){if(!C[D]){return"";}return($type(E)=="number")?C[D].replace("@",Math.round(E)):E;
-}).join(" ");}else{if(A==String(Number(A))){A=Math.round(A);}}this.style[B]=A;return this;},getStyle:function(G){switch(G){case"opacity":return this.get("opacity");
-case"float":G=(Browser.Engine.trident)?"styleFloat":"cssFloat";}G=G.camelCase();var A=this.style[G];if(!$chk(A)){A=[];for(var F in Element.ShortStyles){if(G!=F){continue;
-}for(var E in Element.ShortStyles[F]){A.push(this.getStyle(E));}return A.join(" ");}A=this.getComputedStyle(G);}if(A){A=String(A);var C=A.match(/rgba?\([\d\s,]+\)/);
-if(C){A=A.replace(C[0],C[0].rgbToHex());}}if(Browser.Engine.presto||(Browser.Engine.trident&&!$chk(parseInt(A)))){if(G.test(/^(height|width)$/)){var B=(G=="width")?["left","right"]:["top","bottom"],D=0;
-B.each(function(H){D+=this.getStyle("border-"+H+"-width").toInt()+this.getStyle("padding-"+H).toInt();},this);return this["offset"+G.capitalize()]-D+"px";
-}if(Browser.Engine.presto&&String(A).test("px")){return A;}if(G.test(/(border(.+)Width|margin|padding)/)){return"0px";}}return A;},setStyles:function(B){for(var A in B){this.setStyle(A,B[A]);
-}return this;},getStyles:function(){var A={};Array.each(arguments,function(B){A[B]=this.getStyle(B);},this);return A;}});Element.Styles=new Hash({left:"@px",top:"@px",bottom:"@px",right:"@px",width:"@px",height:"@px",maxWidth:"@px",maxHeight:"@px",minWidth:"@px",minHeight:"@px",backgroundColor:"rgb(@, @, @)",backgroundPosition:"@px @px",color:"rgb(@, @, @)",fontSize:"@px",letterSpacing:"@px",lineHeight:"@px",clip:"rect(@px @px @px @px)",margin:"@px @px @px @px",padding:"@px @px @px @px",border:"@px @ rgb(@, @, @) @px @ rgb(@, @, @) @px @ rgb(@, @, @)",borderWidth:"@px @px @px @px",borderStyle:"@ @ @ @",borderColor:"rgb(@, @, @) rgb(@, @, @) rgb(@, @, @) rgb(@, @, @)",zIndex:"@",zoom:"@",fontWeight:"@",textIndent:"@px",opacity:"@"});
-Element.ShortStyles={margin:{},padding:{},border:{},borderWidth:{},borderStyle:{},borderColor:{}};["Top","Right","Bottom","Left"].each(function(G){var F=Element.ShortStyles;
-var B=Element.Styles;["margin","padding"].each(function(H){var I=H+G;F[H][I]=B[I]="@px";});var E="border"+G;F.border[E]=B[E]="@px @ rgb(@, @, @)";var D=E+"Width",A=E+"Style",C=E+"Color";
-F[E]={};F.borderWidth[D]=F[E][D]=B[D]="@px";F.borderStyle[A]=F[E][A]=B[A]="@";F.borderColor[C]=F[E][C]=B[C]="rgb(@, @, @)";});(function(){Element.implement({scrollTo:function(H,I){if(B(this)){this.getWindow().scrollTo(H,I);
-}else{this.scrollLeft=H;this.scrollTop=I;}return this;},getSize:function(){if(B(this)){return this.getWindow().getSize();}return{x:this.offsetWidth,y:this.offsetHeight};
-},getScrollSize:function(){if(B(this)){return this.getWindow().getScrollSize();}return{x:this.scrollWidth,y:this.scrollHeight};},getScroll:function(){if(B(this)){return this.getWindow().getScroll();
-}return{x:this.scrollLeft,y:this.scrollTop};},getScrolls:function(){var I=this,H={x:0,y:0};while(I&&!B(I)){H.x+=I.scrollLeft;H.y+=I.scrollTop;I=I.parentNode;
-}return H;},getOffsetParent:function(){var H=this;if(B(H)){return null;}if(!Browser.Engine.trident){return H.offsetParent;}while((H=H.parentNode)&&!B(H)){if(D(H,"position")!="static"){return H;
-}}return null;},getOffsets:function(){var I=this,H={x:0,y:0};if(B(this)){return H;}while(I&&!B(I)){H.x+=I.offsetLeft;H.y+=I.offsetTop;if(Browser.Engine.gecko){if(!F(I)){H.x+=C(I);
-H.y+=G(I);}var J=I.parentNode;if(J&&D(J,"overflow")!="visible"){H.x+=C(J);H.y+=G(J);}}else{if(I!=this&&(Browser.Engine.trident||Browser.Engine.webkit)){H.x+=C(I);
-H.y+=G(I);}}I=I.offsetParent;if(Browser.Engine.trident){while(I&&!I.currentStyle.hasLayout){I=I.offsetParent;}}}if(Browser.Engine.gecko&&!F(this)){H.x-=C(this);
-H.y-=G(this);}return H;},getPosition:function(K){if(B(this)){return{x:0,y:0};}var L=this.getOffsets(),I=this.getScrolls();var H={x:L.x-I.x,y:L.y-I.y};var J=(K&&(K=$(K)))?K.getPosition():{x:0,y:0};
-return{x:H.x-J.x,y:H.y-J.y};},getCoordinates:function(J){if(B(this)){return this.getWindow().getCoordinates();}var H=this.getPosition(J),I=this.getSize();
-var K={left:H.x,top:H.y,width:I.x,height:I.y};K.right=K.left+K.width;K.bottom=K.top+K.height;return K;},computePosition:function(H){return{left:H.x-E(this,"margin-left"),top:H.y-E(this,"margin-top")};
-},position:function(H){return this.setStyles(this.computePosition(H));}});Native.implement([Document,Window],{getSize:function(){var I=this.getWindow();
-if(Browser.Engine.presto||Browser.Engine.webkit){return{x:I.innerWidth,y:I.innerHeight};}var H=A(this);return{x:H.clientWidth,y:H.clientHeight};},getScroll:function(){var I=this.getWindow();
-var H=A(this);return{x:I.pageXOffset||H.scrollLeft,y:I.pageYOffset||H.scrollTop};},getScrollSize:function(){var I=A(this);var H=this.getSize();return{x:Math.max(I.scrollWidth,H.x),y:Math.max(I.scrollHeight,H.y)};
-},getPosition:function(){return{x:0,y:0};},getCoordinates:function(){var H=this.getSize();return{top:0,left:0,bottom:H.y,right:H.x,height:H.y,width:H.x};
-}});var D=Element.getComputedStyle;function E(H,I){return D(H,I).toInt()||0;}function F(H){return D(H,"-moz-box-sizing")=="border-box";}function G(H){return E(H,"border-top-width");
-}function C(H){return E(H,"border-left-width");}function B(H){return(/^(?:body|html)$/i).test(H.tagName);}function A(H){var I=H.getDocument();return(!I.compatMode||I.compatMode=="CSS1Compat")?I.html:I.body;
-}})();Native.implement([Window,Document,Element],{getHeight:function(){return this.getSize().y;},getWidth:function(){return this.getSize().x;},getScrollTop:function(){return this.getScroll().y;
-},getScrollLeft:function(){return this.getScroll().x;},getScrollHeight:function(){return this.getScrollSize().y;},getScrollWidth:function(){return this.getScrollSize().x;
-},getTop:function(){return this.getPosition().y;},getLeft:function(){return this.getPosition().x;}});Native.implement([Document,Element],{getElements:function(H,G){H=H.split(",");
-var C,E={};for(var D=0,B=H.length;D<B;D++){var A=H[D],F=Selectors.Utils.search(this,A,E);if(D!=0&&F.item){F=$A(F);}C=(D==0)?F:(C.item)?$A(C).concat(F):C.concat(F);
-}return new Elements(C,{ddup:(H.length>1),cash:!G});}});Element.implement({match:function(B){if(!B){return true;}var D=Selectors.Utils.parseTagAndID(B);
-var A=D[0],E=D[1];if(!Selectors.Filters.byID(this,E)||!Selectors.Filters.byTag(this,A)){return false;}var C=Selectors.Utils.parseSelector(B);return(C)?Selectors.Utils.filter(this,C,{}):true;
-}});var Selectors={Cache:{nth:{},parsed:{}}};Selectors.RegExps={id:(/#([\w-]+)/),tag:(/^(\w+|\*)/),quick:(/^(\w+|\*)$/),splitter:(/\s*([+>~\s])\s*([a-zA-Z#.*:\[])/g),combined:(/\.([\w-]+)|\[(\w+)(?:([!*^$~|]?=)["']?(.*?)["']?)?\]|:([\w-]+)(?:\(["']?(.*?)?["']?\)|$)/g)};
-Selectors.Utils={chk:function(B,C){if(!C){return true;}var A=$uid(B);if(!C[A]){return C[A]=true;}return false;},parseNthArgument:function(F){if(Selectors.Cache.nth[F]){return Selectors.Cache.nth[F];
-}var C=F.match(/^([+-]?\d*)?([a-z]+)?([+-]?\d*)?$/);if(!C){return false;}var E=parseInt(C[1]);var B=(E||E===0)?E:1;var D=C[2]||false;var A=parseInt(C[3])||0;
-if(B!=0){A--;while(A<1){A+=B;}while(A>=B){A-=B;}}else{B=A;D="index";}switch(D){case"n":C={a:B,b:A,special:"n"};break;case"odd":C={a:2,b:0,special:"n"};
-break;case"even":C={a:2,b:1,special:"n"};break;case"first":C={a:0,special:"index"};break;case"last":C={special:"last-child"};break;case"only":C={special:"only-child"};
-break;default:C={a:(B-1),special:"index"};}return Selectors.Cache.nth[F]=C;},parseSelector:function(E){if(Selectors.Cache.parsed[E]){return Selectors.Cache.parsed[E];
-}var D,H={classes:[],pseudos:[],attributes:[]};while((D=Selectors.RegExps.combined.exec(E))){var I=D[1],G=D[2],F=D[3],B=D[4],C=D[5],J=D[6];if(I){H.classes.push(I);
-}else{if(C){var A=Selectors.Pseudo.get(C);if(A){H.pseudos.push({parser:A,argument:J});}else{H.attributes.push({name:C,operator:"=",value:J});}}else{if(G){H.attributes.push({name:G,operator:F,value:B});
-}}}}if(!H.classes.length){delete H.classes;}if(!H.attributes.length){delete H.attributes;}if(!H.pseudos.length){delete H.pseudos;}if(!H.classes&&!H.attributes&&!H.pseudos){H=null;
-}return Selectors.Cache.parsed[E]=H;},parseTagAndID:function(B){var A=B.match(Selectors.RegExps.tag);var C=B.match(Selectors.RegExps.id);return[(A)?A[1]:"*",(C)?C[1]:false];
-},filter:function(F,C,E){var D;if(C.classes){for(D=C.classes.length;D--;D){var G=C.classes[D];if(!Selectors.Filters.byClass(F,G)){return false;}}}if(C.attributes){for(D=C.attributes.length;
-D--;D){var B=C.attributes[D];if(!Selectors.Filters.byAttribute(F,B.name,B.operator,B.value)){return false;}}}if(C.pseudos){for(D=C.pseudos.length;D--;D){var A=C.pseudos[D];
-if(!Selectors.Filters.byPseudo(F,A.parser,A.argument,E)){return false;}}}return true;},getByTagAndID:function(B,A,D){if(D){var C=(B.getElementById)?B.getElementById(D,true):Element.getElementById(B,D,true);
-return(C&&Selectors.Filters.byTag(C,A))?[C]:[];}else{return B.getElementsByTagName(A);}},search:function(J,I,O){var B=[];var C=I.trim().replace(Selectors.RegExps.splitter,function(Z,Y,X){B.push(Y);
-return":)"+X;}).split(":)");var K,F,E,V;for(var U=0,Q=C.length;U<Q;U++){var T=C[U];if(U==0&&Selectors.RegExps.quick.test(T)){K=J.getElementsByTagName(T);
-continue;}var A=B[U-1];var L=Selectors.Utils.parseTagAndID(T);var W=L[0],M=L[1];if(U==0){K=Selectors.Utils.getByTagAndID(J,W,M);}else{var D={},H=[];for(var S=0,R=K.length;
-S<R;S++){H=Selectors.Getters[A](H,K[S],W,M,D);}K=H;}var G=Selectors.Utils.parseSelector(T);if(G){E=[];for(var P=0,N=K.length;P<N;P++){V=K[P];if(Selectors.Utils.filter(V,G,O)){E.push(V);
-}}K=E;}}return K;}};Selectors.Getters={" ":function(H,G,I,A,E){var D=Selectors.Utils.getByTagAndID(G,I,A);for(var C=0,B=D.length;C<B;C++){var F=D[C];if(Selectors.Utils.chk(F,E)){H.push(F);
-}}return H;},">":function(H,G,I,A,F){var C=Selectors.Utils.getByTagAndID(G,I,A);for(var E=0,D=C.length;E<D;E++){var B=C[E];if(B.parentNode==G&&Selectors.Utils.chk(B,F)){H.push(B);
-}}return H;},"+":function(C,B,A,E,D){while((B=B.nextSibling)){if(B.nodeType==1){if(Selectors.Utils.chk(B,D)&&Selectors.Filters.byTag(B,A)&&Selectors.Filters.byID(B,E)){C.push(B);
-}break;}}return C;},"~":function(C,B,A,E,D){while((B=B.nextSibling)){if(B.nodeType==1){if(!Selectors.Utils.chk(B,D)){break;}if(Selectors.Filters.byTag(B,A)&&Selectors.Filters.byID(B,E)){C.push(B);
-}}}return C;}};Selectors.Filters={byTag:function(B,A){return(A=="*"||(B.tagName&&B.tagName.toLowerCase()==A));},byID:function(A,B){return(!B||(A.id&&A.id==B));
-},byClass:function(B,A){return(B.className&&B.className.contains(A," "));},byPseudo:function(A,D,C,B){return D.call(A,C,B);},byAttribute:function(C,D,B,E){var A=Element.prototype.getProperty.call(C,D);
-if(!A){return false;}if(!B||E==undefined){return true;}switch(B){case"=":return(A==E);case"*=":return(A.contains(E));case"^=":return(A.substr(0,E.length)==E);
-case"$=":return(A.substr(A.length-E.length)==E);case"!=":return(A!=E);case"~=":return A.contains(E," ");case"|=":return A.contains(E,"-");}return false;
-}};Selectors.Pseudo=new Hash({empty:function(){return !(this.innerText||this.textContent||"").length;},not:function(A){return !Element.match(this,A);},contains:function(A){return(this.innerText||this.textContent||"").contains(A);
-},"first-child":function(){return Selectors.Pseudo.index.call(this,0);},"last-child":function(){var A=this;while((A=A.nextSibling)){if(A.nodeType==1){return false;
-}}return true;},"only-child":function(){var B=this;while((B=B.previousSibling)){if(B.nodeType==1){return false;}}var A=this;while((A=A.nextSibling)){if(A.nodeType==1){return false;
-}}return true;},"nth-child":function(G,E){G=(G==undefined)?"n":G;var C=Selectors.Utils.parseNthArgument(G);if(C.special!="n"){return Selectors.Pseudo[C.special].call(this,C.a,E);
-}var F=0;E.positions=E.positions||{};var D=$uid(this);if(!E.positions[D]){var B=this;while((B=B.previousSibling)){if(B.nodeType!=1){continue;}F++;var A=E.positions[$uid(B)];
-if(A!=undefined){F=A+F;break;}}E.positions[D]=F;}return(E.positions[D]%C.a==C.b);},index:function(A){var B=this,C=0;while((B=B.previousSibling)){if(B.nodeType==1&&++C>A){return false;
-}}return(C==A);},even:function(B,A){return Selectors.Pseudo["nth-child"].call(this,"2n+1",A);},odd:function(B,A){return Selectors.Pseudo["nth-child"].call(this,"2n",A);
-}});Element.Events.domready={onAdd:function(A){if(Browser.loaded){A.call(this);}}};(function(){var B=function(){if(Browser.loaded){return ;}Browser.loaded=true;
-window.fireEvent("domready");document.fireEvent("domready");};switch(Browser.Engine.name){case"webkit":(function(){(["loaded","complete"].contains(document.readyState))?B():arguments.callee.delay(50);
-})();break;case"trident":var A=document.createElement("div");(function(){($try(function(){A.doScroll("left");return $(A).inject(document.body).set("html","temp").dispose();
-}))?B():arguments.callee.delay(50);})();break;default:window.addEvent("load",B);document.addEvent("DOMContentLoaded",B);}})();var JSON=new Hash({encode:function(B){switch($type(B)){case"string":return'"'+B.replace(/[\x00-\x1f\\"]/g,JSON.$replaceChars)+'"';
-case"array":return"["+String(B.map(JSON.encode).filter($defined))+"]";case"object":case"hash":var A=[];Hash.each(B,function(E,D){var C=JSON.encode(E);if(C){A.push(JSON.encode(D)+":"+C);
-}});return"{"+A+"}";case"number":case"boolean":return String(B);case false:return"null";}return null;},$specialChars:{"\b":"\\b","\t":"\\t","\n":"\\n","\f":"\\f","\r":"\\r",'"':'\\"',"\\":"\\\\"},$replaceChars:function(A){return JSON.$specialChars[A]||"\\u00"+Math.floor(A.charCodeAt()/16).toString(16)+(A.charCodeAt()%16).toString(16);
-},decode:function(string,secure){if($type(string)!="string"||!string.length){return null;}if(secure&&!(/^[,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]*$/).test(string.replace(/\\./g,"@").replace(/"[^"\\\n\r]*"/g,""))){return null;
-}return eval("("+string+")");}});Native.implement([Hash,Array,String,Number],{toJSON:function(){return JSON.encode(this);}});var Cookie=new Class({Implements:Options,options:{path:false,domain:false,duration:false,secure:false,document:document},initialize:function(B,A){this.key=B;
-this.setOptions(A);},write:function(B){B=encodeURIComponent(B);if(this.options.domain){B+="; domain="+this.options.domain;}if(this.options.path){B+="; path="+this.options.path;
-}if(this.options.duration){var A=new Date();A.setTime(A.getTime()+this.options.duration*24*60*60*1000);B+="; expires="+A.toGMTString();}if(this.options.secure){B+="; secure";
-}this.options.document.cookie=this.key+"="+B;return this;},read:function(){var A=this.options.document.cookie.match("(?:^|;)\\s*"+this.key.escapeRegExp()+"=([^;]*)");
-return(A)?decodeURIComponent(A[1]):null;},dispose:function(){new Cookie(this.key,$merge(this.options,{duration:-1})).write("");return this;}});Cookie.write=function(B,C,A){return new Cookie(B,A).write(C);
-};Cookie.read=function(A){return new Cookie(A).read();};Cookie.dispose=function(B,A){return new Cookie(B,A).dispose();};var Fx=new Class({Implements:[Chain,Events,Options],options:{fps:50,unit:false,duration:500,link:"ignore",transition:function(A){return -(Math.cos(Math.PI*A)-1)/2;
-}},initialize:function(A){this.subject=this.subject||this;this.setOptions(A);this.options.duration=Fx.Durations[this.options.duration]||this.options.duration.toInt();
-var B=this.options.wait;if(B===false){this.options.link="cancel";}},step:function(){var A=$time();if(A<this.time+this.options.duration){var B=this.options.transition((A-this.time)/this.options.duration);
-this.set(this.compute(this.from,this.to,B));}else{this.set(this.compute(this.from,this.to,1));this.complete();}},set:function(A){return A;},compute:function(C,B,A){return Fx.compute(C,B,A);
-},check:function(A){if(!this.timer){return true;}switch(this.options.link){case"cancel":this.cancel();return true;case"chain":this.chain(A.bind(this,Array.slice(arguments,1)));
-return false;}return false;},start:function(B,A){if(!this.check(arguments.callee,B,A)){return this;}this.from=B;this.to=A;this.time=0;this.startTimer();
-this.onStart();return this;},complete:function(){if(this.stopTimer()){this.onComplete();}return this;},cancel:function(){if(this.stopTimer()){this.onCancel();
-}return this;},onStart:function(){this.fireEvent("start",this.subject);},onComplete:function(){this.fireEvent("complete",this.subject);if(!this.callChain()){this.fireEvent("chainComplete",this.subject);
-}},onCancel:function(){this.fireEvent("cancel",this.subject).clearChain();},pause:function(){this.stopTimer();return this;},resume:function(){this.startTimer();
-return this;},stopTimer:function(){if(!this.timer){return false;}this.time=$time()-this.time;this.timer=$clear(this.timer);return true;},startTimer:function(){if(this.timer){return false;
-}this.time=$time()-this.time;this.timer=this.step.periodical(Math.round(1000/this.options.fps),this);return true;}});Fx.compute=function(C,B,A){return(B-C)*A+C;
-};Fx.Durations={"short":250,normal:500,"long":1000};Fx.CSS=new Class({Extends:Fx,prepare:function(D,E,B){B=$splat(B);var C=B[1];if(!$chk(C)){B[1]=B[0];
-B[0]=D.getStyle(E);}var A=B.map(this.parse);return{from:A[0],to:A[1]};},parse:function(A){A=$lambda(A)();A=(typeof A=="string")?A.split(" "):$splat(A);
-return A.map(function(C){C=String(C);var B=false;Fx.CSS.Parsers.each(function(F,E){if(B){return ;}var D=F.parse(C);if($chk(D)){B={value:D,parser:F};}});
-B=B||{value:C,parser:Fx.CSS.Parsers.String};return B;});},compute:function(D,C,B){var A=[];(Math.min(D.length,C.length)).times(function(E){A.push({value:D[E].parser.compute(D[E].value,C[E].value,B),parser:D[E].parser});
-});A.$family={name:"fx:css:value"};return A;},serve:function(C,B){if($type(C)!="fx:css:value"){C=this.parse(C);}var A=[];C.each(function(D){A=A.concat(D.parser.serve(D.value,B));
-});return A;},render:function(A,D,C,B){A.setStyle(D,this.serve(C,B));},search:function(A){if(Fx.CSS.Cache[A]){return Fx.CSS.Cache[A];}var B={};Array.each(document.styleSheets,function(E,D){var C=E.href;
-if(C&&C.contains("://")&&!C.contains(document.domain)){return ;}var F=E.rules||E.cssRules;Array.each(F,function(I,G){if(!I.style){return ;}var H=(I.selectorText)?I.selectorText.replace(/^\w+/,function(J){return J.toLowerCase();
-}):null;if(!H||!H.test("^"+A+"$")){return ;}Element.Styles.each(function(K,J){if(!I.style[J]||Element.ShortStyles[J]){return ;}K=String(I.style[J]);B[J]=(K.test(/^rgb/))?K.rgbToHex():K;
-});});});return Fx.CSS.Cache[A]=B;}});Fx.CSS.Cache={};Fx.CSS.Parsers=new Hash({Color:{parse:function(A){if(A.match(/^#[0-9a-f]{3,6}$/i)){return A.hexToRgb(true);
-}return((A=A.match(/(\d+),\s*(\d+),\s*(\d+)/)))?[A[1],A[2],A[3]]:false;},compute:function(C,B,A){return C.map(function(E,D){return Math.round(Fx.compute(C[D],B[D],A));
-});},serve:function(A){return A.map(Number);}},Number:{parse:parseFloat,compute:Fx.compute,serve:function(B,A){return(A)?B+A:B;}},String:{parse:$lambda(false),compute:$arguments(1),serve:$arguments(0)}});
-Fx.Tween=new Class({Extends:Fx.CSS,initialize:function(B,A){this.element=this.subject=$(B);this.parent(A);},set:function(B,A){if(arguments.length==1){A=B;
-B=this.property||this.options.property;}this.render(this.element,B,A,this.options.unit);return this;},start:function(C,E,D){if(!this.check(arguments.callee,C,E,D)){return this;
-}var B=Array.flatten(arguments);this.property=this.options.property||B.shift();var A=this.prepare(this.element,this.property,B);return this.parent(A.from,A.to);
-}});Element.Properties.tween={set:function(A){var B=this.retrieve("tween");if(B){B.cancel();}return this.eliminate("tween").store("tween:options",$extend({link:"cancel"},A));
-},get:function(A){if(A||!this.retrieve("tween")){if(A||!this.retrieve("tween:options")){this.set("tween",A);}this.store("tween",new Fx.Tween(this,this.retrieve("tween:options")));
-}return this.retrieve("tween");}};Element.implement({tween:function(A,C,B){this.get("tween").start(arguments);return this;},fade:function(C){var E=this.get("tween"),D="opacity",A;
-C=$pick(C,"toggle");switch(C){case"in":E.start(D,1);break;case"out":E.start(D,0);break;case"show":E.set(D,1);break;case"hide":E.set(D,0);break;case"toggle":var B=this.retrieve("fade:flag",this.get("opacity")==1);
-E.start(D,(B)?0:1);this.store("fade:flag",!B);A=true;break;default:E.start(D,arguments);}if(!A){this.eliminate("fade:flag");}return this;},highlight:function(C,A){if(!A){A=this.retrieve("highlight:original",this.getStyle("background-color"));
-A=(A=="transparent")?"#fff":A;}var B=this.get("tween");B.start("background-color",C||"#ffff88",A).chain(function(){this.setStyle("background-color",this.retrieve("highlight:original"));
-B.callChain();}.bind(this));return this;}});Fx.Morph=new Class({Extends:Fx.CSS,initialize:function(B,A){this.element=this.subject=$(B);this.parent(A);},set:function(A){if(typeof A=="string"){A=this.search(A);
-}for(var B in A){this.render(this.element,B,A[B],this.options.unit);}return this;},compute:function(E,D,C){var A={};for(var B in E){A[B]=this.parent(E[B],D[B],C);
-}return A;},start:function(B){if(!this.check(arguments.callee,B)){return this;}if(typeof B=="string"){B=this.search(B);}var E={},D={};for(var C in B){var A=this.prepare(this.element,C,B[C]);
-E[C]=A.from;D[C]=A.to;}return this.parent(E,D);}});Element.Properties.morph={set:function(A){var B=this.retrieve("morph");if(B){B.cancel();}return this.eliminate("morph").store("morph:options",$extend({link:"cancel"},A));
-},get:function(A){if(A||!this.retrieve("morph")){if(A||!this.retrieve("morph:options")){this.set("morph",A);}this.store("morph",new Fx.Morph(this,this.retrieve("morph:options")));
-}return this.retrieve("morph");}};Element.implement({morph:function(A){this.get("morph").start(A);return this;}});(function(){var A=Fx.prototype.initialize;
-Fx.prototype.initialize=function(B){A.call(this,B);var C=this.options.transition;if(typeof C=="string"&&(C=C.split(":"))){var D=Fx.Transitions;D=D[C[0]]||D[C[0].capitalize()];
-if(C[1]){D=D["ease"+C[1].capitalize()+(C[2]?C[2].capitalize():"")];}this.options.transition=D;}};})();Fx.Transition=function(B,A){A=$splat(A);return $extend(B,{easeIn:function(C){return B(C,A);
-},easeOut:function(C){return 1-B(1-C,A);},easeInOut:function(C){return(C<=0.5)?B(2*C,A)/2:(2-B(2*(1-C),A))/2;}});};Fx.Transitions=new Hash({linear:$arguments(0)});
-Fx.Transitions.extend=function(A){for(var B in A){Fx.Transitions[B]=new Fx.Transition(A[B]);}};Fx.Transitions.extend({Pow:function(B,A){return Math.pow(B,A[0]||6);
-},Expo:function(A){return Math.pow(2,8*(A-1));},Circ:function(A){return 1-Math.sin(Math.acos(A));},Sine:function(A){return 1-Math.sin((1-A)*Math.PI/2);
-},Back:function(B,A){A=A[0]||1.618;return Math.pow(B,2)*((A+1)*B-A);},Bounce:function(D){var C;for(var B=0,A=1;1;B+=A,A/=2){if(D>=(7-4*B)/11){C=-Math.pow((11-6*B-11*D)/4,2)+A*A;
-break;}}return C;},Elastic:function(B,A){return Math.pow(2,10*--B)*Math.cos(20*B*Math.PI*(A[0]||1)/3);}});["Quad","Cubic","Quart","Quint"].each(function(B,A){Fx.Transitions[B]=new Fx.Transition(function(C){return Math.pow(C,[A+2]);
-});});var Request=new Class({Implements:[Chain,Events,Options],options:{url:"",data:"",headers:{"X-Requested-With":"XMLHttpRequest",Accept:"text/javascript, text/html, application/xml, text/xml, */*"},async:true,format:false,method:"post",link:"ignore",isSuccess:null,emulation:true,urlEncoded:true,encoding:"utf-8",evalScripts:false,evalResponse:false},initialize:function(A){this.xhr=new Browser.Request();
-this.setOptions(A);this.options.isSuccess=this.options.isSuccess||this.isSuccess;this.headers=new Hash(this.options.headers);},onStateChange:function(){if(this.xhr.readyState!=4||!this.running){return ;
-}this.running=false;this.status=0;$try(function(){this.status=this.xhr.status;}.bind(this));if(this.options.isSuccess.call(this,this.status)){this.response={text:this.xhr.responseText,xml:this.xhr.responseXML};
-this.success(this.response.text,this.response.xml);}else{this.response={text:null,xml:null};this.failure();}this.xhr.onreadystatechange=$empty;},isSuccess:function(){return((this.status>=200)&&(this.status<300));
-},processScripts:function(A){if(this.options.evalResponse||(/(ecma|java)script/).test(this.getHeader("Content-type"))){return $exec(A);}return A.stripScripts(this.options.evalScripts);
-},success:function(B,A){this.onSuccess(this.processScripts(B),A);},onSuccess:function(){this.fireEvent("complete",arguments).fireEvent("success",arguments).callChain();
-},failure:function(){this.onFailure();},onFailure:function(){this.fireEvent("complete").fireEvent("failure",this.xhr);},setHeader:function(A,B){this.headers.set(A,B);
-return this;},getHeader:function(A){return $try(function(){return this.xhr.getResponseHeader(A);}.bind(this));},check:function(A){if(!this.running){return true;
-}switch(this.options.link){case"cancel":this.cancel();return true;case"chain":this.chain(A.bind(this,Array.slice(arguments,1)));return false;}return false;
-},send:function(I){if(!this.check(arguments.callee,I)){return this;}this.running=true;var G=$type(I);if(G=="string"||G=="element"){I={data:I};}var D=this.options;
-I=$extend({data:D.data,url:D.url,method:D.method},I);var E=I.data,B=I.url,A=I.method;switch($type(E)){case"element":E=$(E).toQueryString();break;case"object":case"hash":E=Hash.toQueryString(E);
-}if(this.options.format){var H="format="+this.options.format;E=(E)?H+"&"+E:H;}if(this.options.emulation&&["put","delete"].contains(A)){var F="_method="+A;
-E=(E)?F+"&"+E:F;A="post";}if(this.options.urlEncoded&&A=="post"){var C=(this.options.encoding)?"; charset="+this.options.encoding:"";this.headers.set("Content-type","application/x-www-form-urlencoded"+C);
-}if(E&&A=="get"){B=B+(B.contains("?")?"&":"?")+E;E=null;}this.xhr.open(A.toUpperCase(),B,this.options.async);this.xhr.onreadystatechange=this.onStateChange.bind(this);
-this.headers.each(function(K,J){if(!$try(function(){this.xhr.setRequestHeader(J,K);return true;}.bind(this))){this.fireEvent("exception",[J,K]);}},this);
-this.fireEvent("request");this.xhr.send(E);if(!this.options.async){this.onStateChange();}return this;},cancel:function(){if(!this.running){return this;
-}this.running=false;this.xhr.abort();this.xhr.onreadystatechange=$empty;this.xhr=new Browser.Request();this.fireEvent("cancel");return this;}});(function(){var A={};
-["get","post","put","delete","GET","POST","PUT","DELETE"].each(function(B){A[B]=function(){var C=Array.link(arguments,{url:String.type,data:$defined});
-return this.send($extend(C,{method:B.toLowerCase()}));};});Request.implement(A);})();Element.Properties.send={set:function(A){var B=this.retrieve("send");
-if(B){B.cancel();}return this.eliminate("send").store("send:options",$extend({data:this,link:"cancel",method:this.get("method")||"post",url:this.get("action")},A));
-},get:function(A){if(A||!this.retrieve("send")){if(A||!this.retrieve("send:options")){this.set("send",A);}this.store("send",new Request(this.retrieve("send:options")));
-}return this.retrieve("send");}};Element.implement({send:function(A){var B=this.get("send");B.send({data:this,url:A||B.options.url});return this;}});Request.HTML=new Class({Extends:Request,options:{update:false,evalScripts:true,filter:false},processHTML:function(C){var B=C.match(/<body[^>]*>([\s\S]*?)<\/body>/i);
-C=(B)?B[1]:C;var A=new Element("div");return $try(function(){var D="<root>"+C+"</root>",G;if(Browser.Engine.trident){G=new ActiveXObject("Microsoft.XMLDOM");
-G.async=false;G.loadXML(D);}else{G=new DOMParser().parseFromString(D,"text/xml");}D=G.getElementsByTagName("root")[0];for(var F=0,E=D.childNodes.length;
-F<E;F++){var H=Element.clone(D.childNodes[F],true,true);if(H){A.grab(H);}}return A;})||A.set("html",C);},success:function(D){var C=this.options,B=this.response;
-B.html=D.stripScripts(function(E){B.javascript=E;});var A=this.processHTML(B.html);B.tree=A.childNodes;B.elements=A.getElements("*");if(C.filter){B.tree=B.elements.filter(C.filter);
-}if(C.update){$(C.update).empty().adopt(B.tree);}if(C.evalScripts){$exec(B.javascript);}this.onSuccess(B.tree,B.elements,B.html,B.javascript);}});Element.Properties.load={set:function(A){var B=this.retrieve("load");
-if(B){send.cancel();}return this.eliminate("load").store("load:options",$extend({data:this,link:"cancel",update:this,method:"get"},A));},get:function(A){if(A||!this.retrieve("load")){if(A||!this.retrieve("load:options")){this.set("load",A);
-}this.store("load",new Request.HTML(this.retrieve("load:options")));}return this.retrieve("load");}};Element.implement({load:function(){this.get("load").send(Array.link(arguments,{data:Object.type,url:String.type}));
-return this;}});Request.JSON=new Class({Extends:Request,options:{secure:true},initialize:function(A){this.parent(A);this.headers.extend({Accept:"application/json","X-Request":"JSON"});
-},success:function(A){this.response.json=JSON.decode(A,this.options.secure);this.onSuccess(this.response.json,A);}});
\ No newline at end of file

=== removed file 'loggerhead/static/javascript/mootools-1.2-more.js'
--- loggerhead/static/javascript/mootools-1.2-more.js	2008-06-18 07:49:45 +0000
+++ loggerhead/static/javascript/mootools-1.2-more.js	1970-01-01 00:00:00 +0000
@@ -1,97 +0,0 @@
-//MooTools More, <http://mootools.net/more>. Copyright (c) 2006-2008 Valerio Proietti, <http://mad4milk.net>, MIT Style License.
-
-Fx.Slide=new Class({Extends:Fx,options:{mode:"vertical"},initialize:function(B,A){this.addEvent("complete",function(){this.open=(this.wrapper["offset"+this.layout.capitalize()]!=0);
-if(this.open&&Browser.Engine.webkit419){this.element.dispose().inject(this.wrapper);}},true);this.element=this.subject=$(B);this.parent(A);var C=this.element.retrieve("wrapper");
-this.wrapper=C||new Element("div",{styles:$extend(this.element.getStyles("margin","position"),{overflow:"hidden"})}).wraps(this.element);this.element.store("wrapper",this.wrapper).setStyle("margin",0);
-this.now=[];this.open=true;},vertical:function(){this.margin="margin-top";this.layout="height";this.offset=this.element.offsetHeight;},horizontal:function(){this.margin="margin-left";
-this.layout="width";this.offset=this.element.offsetWidth;},set:function(A){this.element.setStyle(this.margin,A[0]);this.wrapper.setStyle(this.layout,A[1]);
-return this;},compute:function(E,D,C){var B=[];var A=2;A.times(function(F){B[F]=Fx.compute(E[F],D[F],C);});return B;},start:function(B,E){if(!this.check(arguments.callee,B,E)){return this;
-}this[E||this.options.mode]();var D=this.element.getStyle(this.margin).toInt();var C=this.wrapper.getStyle(this.layout).toInt();var A=[[D,C],[0,this.offset]];
-var G=[[D,C],[-this.offset,0]];var F;switch(B){case"in":F=A;break;case"out":F=G;break;case"toggle":F=(this.wrapper["offset"+this.layout.capitalize()]==0)?A:G;
-}return this.parent(F[0],F[1]);},slideIn:function(A){return this.start("in",A);},slideOut:function(A){return this.start("out",A);},hide:function(A){this[A||this.options.mode]();
-this.open=false;return this.set([-this.offset,0]);},show:function(A){this[A||this.options.mode]();this.open=true;return this.set([0,this.offset]);},toggle:function(A){return this.start("toggle",A);
-}});Element.Properties.slide={set:function(B){var A=this.retrieve("slide");if(A){A.cancel();}return this.eliminate("slide").store("slide:options",$extend({link:"cancel"},B));
-},get:function(A){if(A||!this.retrieve("slide")){if(A||!this.retrieve("slide:options")){this.set("slide",A);}this.store("slide",new Fx.Slide(this,this.retrieve("slide:options")));
-}return this.retrieve("slide");}};Element.implement({slide:function(D,E){D=D||"toggle";var B=this.get("slide"),A;switch(D){case"hide":B.hide(E);break;case"show":B.show(E);
-break;case"toggle":var C=this.retrieve("slide:flag",B.open);B[(C)?"slideOut":"slideIn"](E);this.store("slide:flag",!C);A=true;break;default:B.start(D,E);
-}if(!A){this.eliminate("slide:flag");}return this;}});Fx.Elements=new Class({Extends:Fx.CSS,initialize:function(B,A){this.elements=this.subject=$$(B);this.parent(A);
-},compute:function(G,H,I){var C={};for(var D in G){var A=G[D],E=H[D],F=C[D]={};for(var B in A){F[B]=this.parent(A[B],E[B],I);}}return C;},set:function(B){for(var C in B){var A=B[C];
-for(var D in A){this.render(this.elements[C],D,A[D],this.options.unit);}}return this;},start:function(C){if(!this.check(arguments.callee,C)){return this;
-}var H={},I={};for(var D in C){var F=C[D],A=H[D]={},G=I[D]={};for(var B in F){var E=this.prepare(this.elements[D],B,F[B]);A[B]=E.from;G[B]=E.to;}}return this.parent(H,I);
-}});var Drag=new Class({Implements:[Events,Options],options:{snap:6,unit:"px",grid:false,style:true,limit:false,handle:false,invert:false,preventDefault:false,modifiers:{x:"left",y:"top"}},initialize:function(){var B=Array.link(arguments,{options:Object.type,element:$defined});
-this.element=$(B.element);this.document=this.element.getDocument();this.setOptions(B.options||{});var A=$type(this.options.handle);this.handles=(A=="array"||A=="collection")?$$(this.options.handle):$(this.options.handle)||this.element;
-this.mouse={now:{},pos:{}};this.value={start:{},now:{}};this.selection=(Browser.Engine.trident)?"selectstart":"mousedown";this.bound={start:this.start.bind(this),check:this.check.bind(this),drag:this.drag.bind(this),stop:this.stop.bind(this),cancel:this.cancel.bind(this),eventStop:$lambda(false)};
-this.attach();},attach:function(){this.handles.addEvent("mousedown",this.bound.start);return this;},detach:function(){this.handles.removeEvent("mousedown",this.bound.start);
-return this;},start:function(C){if(this.options.preventDefault){C.preventDefault();}this.fireEvent("beforeStart",this.element);this.mouse.start=C.page;
-var A=this.options.limit;this.limit={x:[],y:[]};for(var D in this.options.modifiers){if(!this.options.modifiers[D]){continue;}if(this.options.style){this.value.now[D]=this.element.getStyle(this.options.modifiers[D]).toInt();
-}else{this.value.now[D]=this.element[this.options.modifiers[D]];}if(this.options.invert){this.value.now[D]*=-1;}this.mouse.pos[D]=C.page[D]-this.value.now[D];
-if(A&&A[D]){for(var B=2;B--;B){if($chk(A[D][B])){this.limit[D][B]=$lambda(A[D][B])();}}}}if($type(this.options.grid)=="number"){this.options.grid={x:this.options.grid,y:this.options.grid};
-}this.document.addEvents({mousemove:this.bound.check,mouseup:this.bound.cancel});this.document.addEvent(this.selection,this.bound.eventStop);},check:function(A){if(this.options.preventDefault){A.preventDefault();
-}var B=Math.round(Math.sqrt(Math.pow(A.page.x-this.mouse.start.x,2)+Math.pow(A.page.y-this.mouse.start.y,2)));if(B>this.options.snap){this.cancel();this.document.addEvents({mousemove:this.bound.drag,mouseup:this.bound.stop});
-this.fireEvent("start",this.element).fireEvent("snap",this.element);}},drag:function(A){if(this.options.preventDefault){A.preventDefault();}this.mouse.now=A.page;
-for(var B in this.options.modifiers){if(!this.options.modifiers[B]){continue;}this.value.now[B]=this.mouse.now[B]-this.mouse.pos[B];if(this.options.invert){this.value.now[B]*=-1;
-}if(this.options.limit&&this.limit[B]){if($chk(this.limit[B][1])&&(this.value.now[B]>this.limit[B][1])){this.value.now[B]=this.limit[B][1];}else{if($chk(this.limit[B][0])&&(this.value.now[B]<this.limit[B][0])){this.value.now[B]=this.limit[B][0];
-}}}if(this.options.grid[B]){this.value.now[B]-=(this.value.now[B]%this.options.grid[B]);}if(this.options.style){this.element.setStyle(this.options.modifiers[B],this.value.now[B]+this.options.unit);
-}else{this.element[this.options.modifiers[B]]=this.value.now[B];}}this.fireEvent("drag",this.element);},cancel:function(A){this.document.removeEvent("mousemove",this.bound.check);
-this.document.removeEvent("mouseup",this.bound.cancel);if(A){this.document.removeEvent(this.selection,this.bound.eventStop);this.fireEvent("cancel",this.element);
-}},stop:function(A){this.document.removeEvent(this.selection,this.bound.eventStop);this.document.removeEvent("mousemove",this.bound.drag);this.document.removeEvent("mouseup",this.bound.stop);
-if(A){this.fireEvent("complete",this.element);}}});Element.implement({makeResizable:function(A){return new Drag(this,$merge({modifiers:{x:"width",y:"height"}},A));
-}});Hash.Cookie=new Class({Extends:Cookie,options:{autoSave:true},initialize:function(B,A){this.parent(B,A);this.load();},save:function(){var A=JSON.encode(this.hash);
-if(!A||A.length>4096){return false;}if(A=="{}"){this.dispose();}else{this.write(A);}return true;},load:function(){this.hash=new Hash(JSON.decode(this.read(),true));
-return this;}});Hash.Cookie.implement((function(){var A={};Hash.each(Hash.prototype,function(C,B){A[B]=function(){var D=C.apply(this.hash,arguments);if(this.options.autoSave){this.save();
-}return D;};});return A;})());var Color=new Native({initialize:function(B,C){if(arguments.length>=3){C="rgb";B=Array.slice(arguments,0,3);}else{if(typeof B=="string"){if(B.match(/rgb/)){B=B.rgbToHex().hexToRgb(true);
-}else{if(B.match(/hsb/)){B=B.hsbToRgb();}else{B=B.hexToRgb(true);}}}}C=C||"rgb";switch(C){case"hsb":var A=B;B=B.hsbToRgb();B.hsb=A;break;case"hex":B=B.hexToRgb(true);
-break;}B.rgb=B.slice(0,3);B.hsb=B.hsb||B.rgbToHsb();B.hex=B.rgbToHex();return $extend(B,this);}});Color.implement({mix:function(){var A=Array.slice(arguments);
-var C=($type(A.getLast())=="number")?A.pop():50;var B=this.slice();A.each(function(D){D=new Color(D);for(var E=0;E<3;E++){B[E]=Math.round((B[E]/100*(100-C))+(D[E]/100*C));
-}});return new Color(B,"rgb");},invert:function(){return new Color(this.map(function(A){return 255-A;}));},setHue:function(A){return new Color([A,this.hsb[1],this.hsb[2]],"hsb");
-},setSaturation:function(A){return new Color([this.hsb[0],A,this.hsb[2]],"hsb");},setBrightness:function(A){return new Color([this.hsb[0],this.hsb[1],A],"hsb");
-}});function $RGB(C,B,A){return new Color([C,B,A],"rgb");}function $HSB(C,B,A){return new Color([C,B,A],"hsb");}function $HEX(A){return new Color(A,"hex");
-}Array.implement({rgbToHsb:function(){var B=this[0],C=this[1],J=this[2];var G,F,H;var I=Math.max(B,C,J),E=Math.min(B,C,J);var K=I-E;H=I/255;F=(I!=0)?K/I:0;
-if(F==0){G=0;}else{var D=(I-B)/K;var A=(I-C)/K;var L=(I-J)/K;if(B==I){G=L-A;}else{if(C==I){G=2+D-L;}else{G=4+A-D;}}G/=6;if(G<0){G++;}}return[Math.round(G*360),Math.round(F*100),Math.round(H*100)];
-},hsbToRgb:function(){var C=Math.round(this[2]/100*255);if(this[1]==0){return[C,C,C];}else{var A=this[0]%360;var E=A%60;var F=Math.round((this[2]*(100-this[1]))/10000*255);
-var D=Math.round((this[2]*(6000-this[1]*E))/600000*255);var B=Math.round((this[2]*(6000-this[1]*(60-E)))/600000*255);switch(Math.floor(A/60)){case 0:return[C,B,F];
-case 1:return[D,C,F];case 2:return[F,C,B];case 3:return[F,D,C];case 4:return[B,F,C];case 5:return[C,F,D];}}return false;}});String.implement({rgbToHsb:function(){var A=this.match(/\d{1,3}/g);
-return(A)?hsb.rgbToHsb():null;},hsbToRgb:function(){var A=this.match(/\d{1,3}/g);return(A)?A.hsbToRgb():null;}});var Group=new Class({initialize:function(){this.instances=Array.flatten(arguments);
-this.events={};this.checker={};},addEvent:function(B,A){this.checker[B]=this.checker[B]||{};this.events[B]=this.events[B]||[];if(this.events[B].contains(A)){return false;
-}else{this.events[B].push(A);}this.instances.each(function(C,D){C.addEvent(B,this.check.bind(this,[B,C,D]));},this);return this;},check:function(C,A,B){this.checker[C][B]=true;
-var D=this.instances.every(function(F,E){return this.checker[C][E]||false;},this);if(!D){return ;}this.checker[C]={};this.events[C].each(function(E){E.call(this,this.instances,A);
-},this);}});var Asset=new Hash({javascript:function(F,D){D=$extend({onload:$empty,document:document,check:$lambda(true)},D);var B=new Element("script",{src:F,type:"text/javascript"});
-var E=D.onload.bind(B),A=D.check,G=D.document;delete D.onload;delete D.check;delete D.document;B.addEvents({load:E,readystatechange:function(){if(["loaded","complete"].contains(this.readyState)){E();
-}}}).setProperties(D);if(Browser.Engine.webkit419){var C=(function(){if(!$try(A)){return ;}$clear(C);E();}).periodical(50);}return B.inject(G.head);},css:function(B,A){return new Element("link",$merge({rel:"stylesheet",media:"screen",type:"text/css",href:B},A)).inject(document.head);
-},image:function(C,B){B=$merge({onload:$empty,onabort:$empty,onerror:$empty},B);var D=new Image();var A=$(D)||new Element("img");["load","abort","error"].each(function(E){var F="on"+E;
-var G=B[F];delete B[F];D[F]=function(){if(!D){return ;}if(!A.parentNode){A.width=D.width;A.height=D.height;}D=D.onload=D.onabort=D.onerror=null;G.delay(1,A,A);
-A.fireEvent(E,A,1);};});D.src=A.src=C;if(D&&D.complete){D.onload.delay(1);}return A.setProperties(B);},images:function(D,C){C=$merge({onComplete:$empty,onProgress:$empty},C);
-if(!D.push){D=[D];}var A=[];var B=0;D.each(function(F){var E=new Asset.image(F,{onload:function(){C.onProgress.call(this,B,D.indexOf(F));B++;if(B==D.length){C.onComplete();
-}}});A.push(E);});return new Elements(A);}});var Tips=new Class({Implements:[Events,Options],options:{onShow:function(A){A.setStyle("visibility","visible");
-},onHide:function(A){A.setStyle("visibility","hidden");},showDelay:100,hideDelay:100,className:null,offsets:{x:16,y:16},fixed:false},initialize:function(){var C=Array.link(arguments,{options:Object.type,elements:$defined});
-this.setOptions(C.options||null);this.tip=new Element("div").inject(document.body);if(this.options.className){this.tip.addClass(this.options.className);
-}var B=new Element("div",{"class":"tip-top"}).inject(this.tip);this.container=new Element("div",{"class":"tip"}).inject(this.tip);var A=new Element("div",{"class":"tip-bottom"}).inject(this.tip);
-this.tip.setStyles({position:"absolute",top:0,left:0,visibility:"hidden"});if(C.elements){this.attach(C.elements);}},attach:function(A){$$(A).each(function(D){var G=D.retrieve("tip:title",D.get("title"));
-var F=D.retrieve("tip:text",D.get("rel")||D.get("href"));var E=D.retrieve("tip:enter",this.elementEnter.bindWithEvent(this,D));var C=D.retrieve("tip:leave",this.elementLeave.bindWithEvent(this,D));
-D.addEvents({mouseenter:E,mouseleave:C});if(!this.options.fixed){var B=D.retrieve("tip:move",this.elementMove.bindWithEvent(this,D));D.addEvent("mousemove",B);
-}D.store("tip:native",D.get("title"));D.erase("title");},this);return this;},detach:function(A){$$(A).each(function(C){C.removeEvent("mouseenter",C.retrieve("tip:enter")||$empty);
-C.removeEvent("mouseleave",C.retrieve("tip:leave")||$empty);C.removeEvent("mousemove",C.retrieve("tip:move")||$empty);C.eliminate("tip:enter").eliminate("tip:leave").eliminate("tip:move");
-var B=C.retrieve("tip:native");if(B){C.set("title",B);}});return this;},elementEnter:function(B,A){$A(this.container.childNodes).each(Element.dispose);
-var D=A.retrieve("tip:title");if(D){this.titleElement=new Element("div",{"class":"tip-title"}).inject(this.container);this.fill(this.titleElement,D);}var C=A.retrieve("tip:text");
-if(C){this.textElement=new Element("div",{"class":"tip-text"}).inject(this.container);this.fill(this.textElement,C);}this.timer=$clear(this.timer);this.timer=this.show.delay(this.options.showDelay,this);
-this.position((!this.options.fixed)?B:{page:A.getPosition()});},elementLeave:function(A){$clear(this.timer);this.timer=this.hide.delay(this.options.hideDelay,this);
-},elementMove:function(A){this.position(A);},position:function(D){var B=window.getSize(),A=window.getScroll();var E={x:this.tip.offsetWidth,y:this.tip.offsetHeight};
-var C={x:"left",y:"top"};for(var F in C){var G=D.page[F]+this.options.offsets[F];if((G+E[F]-A[F])>B[F]){G=D.page[F]-this.options.offsets[F]-E[F];}this.tip.setStyle(C[F],G);
-}},fill:function(A,B){(typeof B=="string")?A.set("html",B):A.adopt(B);},show:function(){this.fireEvent("show",this.tip);},hide:function(){this.fireEvent("hide",this.tip);
-}});var Slider=new Class({Implements:[Events,Options],options:{onTick:function(A){if(this.options.snap){A=this.toPosition(this.step);}this.knob.setStyle(this.property,A);
-},snap:false,offset:0,range:false,wheel:false,steps:100,mode:"horizontal"},initialize:function(E,A,D){this.setOptions(D);this.element=$(E);this.knob=$(A);
-this.previousChange=this.previousEnd=this.step=-1;this.element.addEvent("mousedown",this.clickedElement.bind(this));if(this.options.wheel){this.element.addEvent("mousewheel",this.scrolledElement.bindWithEvent(this));
-}var F,B={},C={x:false,y:false};switch(this.options.mode){case"vertical":this.axis="y";this.property="top";F="offsetHeight";break;case"horizontal":this.axis="x";
-this.property="left";F="offsetWidth";}this.half=this.knob[F]/2;this.full=this.element[F]-this.knob[F]+(this.options.offset*2);this.min=$chk(this.options.range[0])?this.options.range[0]:0;
-this.max=$chk(this.options.range[1])?this.options.range[1]:this.options.steps;this.range=this.max-this.min;this.steps=this.options.steps||this.full;this.stepSize=Math.abs(this.range)/this.steps;
-this.stepWidth=this.stepSize*this.full/Math.abs(this.range);this.knob.setStyle("position","relative").setStyle(this.property,-this.options.offset);C[this.axis]=this.property;
-B[this.axis]=[-this.options.offset,this.full-this.options.offset];this.drag=new Drag(this.knob,{snap:0,limit:B,modifiers:C,onDrag:this.draggedKnob.bind(this),onStart:this.draggedKnob.bind(this),onComplete:function(){this.draggedKnob();
-this.end();}.bind(this)});if(this.options.snap){this.drag.options.grid=Math.ceil(this.stepWidth);this.drag.options.limit[this.axis][1]=this.full;}},set:function(A){if(!((this.range>0)^(A<this.min))){A=this.min;
-}if(!((this.range>0)^(A>this.max))){A=this.max;}this.step=Math.round(A);this.checkStep();this.end();this.fireEvent("tick",this.toPosition(this.step));return this;
-},clickedElement:function(C){var B=this.range<0?-1:1;var A=C.page[this.axis]-this.element.getPosition()[this.axis]-this.half;A=A.limit(-this.options.offset,this.full-this.options.offset);
-this.step=Math.round(this.min+B*this.toStep(A));this.checkStep();this.end();this.fireEvent("tick",A);},scrolledElement:function(A){var B=(this.options.mode=="horizontal")?(A.wheel<0):(A.wheel>0);
-this.set(B?this.step-this.stepSize:this.step+this.stepSize);A.stop();},draggedKnob:function(){var B=this.range<0?-1:1;var A=this.drag.value.now[this.axis];
-A=A.limit(-this.options.offset,this.full-this.options.offset);this.step=Math.round(this.min+B*this.toStep(A));this.checkStep();},checkStep:function(){if(this.previousChange!=this.step){this.previousChange=this.step;
-this.fireEvent("change",this.step);}},end:function(){if(this.previousEnd!==this.step){this.previousEnd=this.step;this.fireEvent("complete",this.step+"");
-}},toStep:function(A){var B=(A+this.options.offset)*this.stepSize/this.full*this.steps;return this.options.steps?Math.round(B-=B%this.stepSize):B;},toPosition:function(A){return(this.full*Math.abs(this.min-A))/(this.steps*this.stepSize)-this.options.offset;
-}});
\ No newline at end of file

=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2009-02-18 10:36:48 +0000
+++ loggerhead/templatefunctions.py	2011-03-23 04:04:44 +0000
@@ -13,8 +13,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
+
 import os
+import urllib
+
+import pkg_resources
+
+import bzrlib
+
+import loggerhead
 from loggerhead.zptsupport import zpt
+from loggerhead.util import html_format
 
 
 templatefunctions = {}
@@ -37,19 +46,36 @@
 
 
 @templatefunc
-def file_change_summary(url, entry, modified_file_link):
+def file_change_summary(url, entry, file_changes, style='normal', currently_showing=None):
+    if style == 'fragment':
+        def file_link(filename):
+            if currently_showing and filename == currently_showing:
+                return html_format(
+                    '<b><a href="#%s">%s</a></b>',
+                    urllib.quote(filename.encode('utf-8')), filename)
+            else:
+                return revision_link(
+                    url, entry.revno, filename,
+                    '#' + urllib.quote(filename.encode('utf-8')))
+    else:
+        def file_link(filename):
+            return html_format(
+                '<a href="%s%s" title="View changes to %s in revision %s">'
+                '%s</a>',
+                url(['/revision', entry.revno]),
+                '#' + urllib.quote(filename.encode('utf-8')),
+                filename, entry.revno, filename)
     return _pt('revisionfilechanges').expand(
-        url=url, entry=entry, modified_file_link=modified_file_link,
-        **templatefunctions)
+        entry=entry, file_changes=file_changes, file_link=file_link, **templatefunctions)
 
 
 @templatefunc
-def revisioninfo(url, branch, entry, modified_file_link=None):
+def revisioninfo(url, branch, entry, file_changes=None, currently_showing=None, merged_in=None):
     from loggerhead import util
     return _pt('revisioninfo').expand(
         url=url, change=entry, branch=branch, util=util,
-        modified_file_link=modified_file_link,
-        **templatefunctions)
+        file_changes=file_changes, currently_showing=currently_showing,
+        merged_in=merged_in, **templatefunctions)
 
 
 @templatefunc
@@ -80,21 +106,7 @@
         extra = ' ' + entry.branch_nick
     else:
         extra = ''
-    return '(%s%s)'%(entry.revno, extra)
-
-
-@templatefunc
-def modified_file_link_rev(url, entry, item):
-    return _pt('modified-file-link-rev').expand(
-        url=url, entry=entry, item=item,
-        **templatefunctions)
-
-
-@templatefunc
-def modified_file_link_log(url, entry, item):
-    return _pt('modified-file-link-log').expand(
-        url=url, entry=entry, item=item,
-        **templatefunctions)
+    return '(%s%s)' % (entry.revno, extra)
 
 
 @templatefunc
@@ -112,3 +124,86 @@
 def menu(branch, url, fileview_active=False):
     return _pt('menu').expand(branch=branch, url=url,
         fileview_active=fileview_active, **templatefunctions)
+
+
+@templatefunc
+def view_link(url, revno, path):
+    return html_format(
+        '<a href="%s" title="Annotate %s">%s</a>',
+        url(['/view', revno, path]), path, path)
+
+
+@templatefunc
+def revision_link(url, revno, path, frag=''):
+    return html_format(
+        '<a href="%s%s" title="View changes to %s in revision %s">%s</a>',
+        url(['/revision', revno, path]), frag, path, revno, path)
+
+
+@templatefunc
+def loggerhead_version():
+    return loggerhead.__version__
+
+_cached_generator_string = None
+
+@templatefunc
+def generator_string():
+    global _cached_generator_string
+    if _cached_generator_string is None:
+        versions = []
+
+        # TODO: Errors -- e.g. from a missing/invalid __version__ attribute, or
+        # ValueError accessing Distribution.version -- should be non-fatal.
+
+        versions.append(('Loggerhead', loggerhead.__version__))
+
+        import sys
+        python_version = bzrlib._format_version_tuple(sys.version_info)
+        versions.append(('Python', python_version))
+
+        versions.append(('Bazaar', bzrlib.__version__))
+
+        Paste = pkg_resources.get_distribution('Paste')
+        versions.append(('Paste', Paste.version))
+
+        try:
+            PasteDeploy = pkg_resources.get_distribution('PasteDeploy')
+        except pkg_resources.DistributionNotFound:
+            pass
+        else:
+            versions.append(('PasteDeploy', PasteDeploy.version))
+
+        import simpletal
+        versions.append(('SimpleTAL', simpletal.__version__))
+
+        try:
+            import pygments
+        except ImportError:
+            pass
+        else:
+            versions.append(('Pygments', pygments.__version__))
+
+        try:
+            from bzrlib.plugins import search
+        except ImportError:
+            pass
+        else:
+            bzr_search_version = bzrlib._format_version_tuple(
+                search.version_info)
+            versions.append(('bzr-search', bzr_search_version))
+
+        # TODO: On old Python versions, elementtree may be used.
+
+        import simplejson
+        versions.append(('simplejson', simplejson.__version__))
+
+        try:
+            Dozer = pkg_resources.get_distribution('Dozer')
+        except pkg_resources.DistributionNotFound:
+            pass
+        else:
+            versions.append(('Dozer', Dozer.version))
+
+        version_strings = ("%s/%s" % t for t in versions)
+        _cached_generator_string = ' '.join(version_strings)
+    return _cached_generator_string

=== modified file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	2008-06-26 20:42:15 +0000
+++ loggerhead/templates/atom.pt	2010-05-05 18:42:58 +0000
@@ -3,29 +3,31 @@
   <title>
     bazaar changes for <tal:branch-name content="branch/friendly_name">branch name</tal:branch-name>
   </title>
-  <updated tal:content="updated">${updated}</updated>
-  <id tal:content="python:branch.url(['/atom'])">url</id>
-  <link rel="self" tal:attributes="href python:branch.url([''])" />
+  <updated tal:content="string:${updated}Z">${updated}</updated>
+  <id tal:content="python:branch.absolute_url(['/atom'])">url</id>
+  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.absolute_url(['/atom'])" />
+  <link rel="alternate" type="text/html" tal:attributes="href python:branch.url(['/changes'])" />
 
   <entry tal:repeat="entry changes">
-    <title tal:content="string: ${entry/revno}: ${entry/short_comment}">
+    <title tal:content="string:${entry/revno}: ${entry/short_comment}">
       ${entry/revno}: ${entry/short_comment}
     </title>
-    <updated tal:content="entry/date/isoformat">
-    Z
+    <updated tal:content="string:${entry/utc_date/isoformat}Z">
+      updated
     </updated>
-    <id tal:content="python:branch.url(['/revision', entry.revno])">
+    <!-- TODO: The feed validator will generate warnings because this is URL-encoded -->
+    <id tal:content="python:branch.absolute_url(['/revision', entry.revno])">
       ID
     </id>
-    <author>
-      <name tal:content="python:util.hide_email(entry.author)">
+    <author tal:repeat="author entry/authors">
+      <name tal:content="python:util.hide_email(author)">
         author
       </name>
     </author>
     <content type="text" tal:content="entry/comment">
       comment
     </content>
-    <link rel="alternate"
+    <link rel="alternate" type="text/html"
           tal:attributes="href python:branch.url(['/revision', entry.revno])" />
   </entry>
 </feed>

=== removed file 'loggerhead/templates/browse.pt'
--- loggerhead/templates/browse.pt	2008-10-18 00:23:09 +0000
+++ loggerhead/templates/browse.pt	1970-01-01 00:00:00 +0000
@@ -1,60 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml">
-  <head>
-    <title>loggerhead branches</title>
-    <link rel="stylesheet"
-          tal:attributes="href python:branch.static_url('/static/css/global.css')" />
-  </head>
-
-  <body>
-   <div id="loggerheadCont">
-    <div>
-      <div class="browse-project"
-           tal:repeat="project projects">
-        <h1 tal:content="project/friendly_name" />
-        <div class="browse-project-description"
-             tal:condition="python:len(project.long_description) > 0"
-             tal:content="structure project/long_description"></div>
-
-          <table id="logentries">
-            <tr class="logheader">
-              <th class="summarycell" colspan="2">Name</th>
-              <th class="summarycell">Description</th>
-              <th class="datecell" colspan="2">Last change</th>
-            </tr>
-            <tal:block repeat="view python:views_by_project[project]">
-              <tr tal:attributes="class string:blueRow1">
-                <td class="icocell">
-                  <img tal:attributes="src python:branch.static_url('/static/images/ico_branch.gif')" alt="Branch" />
-                </td>
-                <td class="autcell">
-                    <a tal:attributes="href python:branch.static_url(project.name + '/' +  view.name)" 
-                       tal:content="view/friendly_name" />
-                </td>
-                <td class="autcell"
-                    tal:content="view/description" />
-                <td class="date"
-                    tal:content="python:view.get_history().has_revisions and util._approximatedate(view.last_updated()) or '(no revisions)'" />
-                <td class="icocell">
-                  <a tal:attributes="href python:branch.static_url(project.name + '/' +  view.name + '/atom')"
-                     title="atom feed">
-                    <img tal:attributes="src python:branch.static_url('/static/images/ico_rss.gif')"
-                         alt="atom feed" />
-                  </a>
-                </td>
-              </tr>
-              <tr tal:condition="view/branch_url">
-                <td class="name">
-                </td>
-                <td class="description url" colspan="2">
-                  <a tal:attributes="href view/branch_url"
-                     tal:content="view/branch_url" />
-                </td>
-              </tr>
-            </tal:block>
-          </table>
-      </div>
-    </div>
-  </div>
-  </body>
-</html>

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2009-01-23 20:53:40 +0000
+++ loggerhead/templates/changelog.pt	2011-02-23 03:01:15 +0000
@@ -1,169 +1,168 @@
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<tal:block define="onload string:sortCollapseElements()">
-  <html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
-    <head>
-      <title metal:fill-slot="title"
-             tal:content="string:${branch/friendly_name} : changes"></title>
-      <metal:block fill-slot="header_extras">
-        <link rel="alternate" type="application/atom+xml"
-              tal:attributes="href python:url(['/atom']);
-                              title string:RSS feed for ${branch/friendly_name}" />
-      </metal:block>
-    </head>
-
-    <body>
-        <tal:block metal:fill-slot="heading">
-         <h1>
-           <tal:has-link condition="branch/branch_link">
-             <a tal:attributes="href branch/branch_link"
-                tal:content="branch/friendly_name">
-               nice/branch/name
-             </a>
-           </tal:has-link>
-           <tal:no-link condition="not: branch/branch_link">
-             <span metal:use-macro="breadcrumbs/directory"></span>
-           </tal:no-link>
-           <tal:block condition="changes">
-             : changes
-             <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>
-             <tal:block condition="start_revid">from revision
-               <span tal:content="python:history.get_revno(start_revid)"/>
-             </tal:block>
-             <tal:block condition="query">matching <tal:block content="query" /></tal:block>
-           </tal:block>
-         </h1>
-        </tal:block>
-
-        <div metal:fill-slot="content">
-          <tal:branch-info replace="structure python:branchinfo(branch)" />
-
-           <!-- Something nicer-looking should be done with search -->
-        <p tal:condition="search_failed">
-          Sorry, no results found for your search.
-        </p>
-
-        <p tal:condition="not:changes">
-          No revisions!
-        </p>
-
-        <p class="fr revnolink">From Revision <a tal:attributes="href #
-                                                            title python:'Show revision '+history.get_revno(revid)" 
-                                tal:content="python:history.get_revno(revid)"></a>
-          <tal:block tal:condition="python:navigation.last_in_page_revid is not None"> to
-          <a tal:attributes="href # 
+<html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
+  <head>
+    <title metal:fill-slot="title"
+           tal:content="string:${branch/friendly_name} : changes"></title>
+    <metal:block fill-slot="header_extras">
+      <link rel="alternate" type="application/atom+xml"
+            tal:attributes="href python:url(['/atom']);
+                            title string:RSS feed for ${branch/friendly_name}" />
+    <script type="text/javascript">
+      var revids = <tal:block content="data" />;
+    </script>
+      <script type="text/javascript"
+              tal:attributes="src python:branch.static_url('/static/javascript/changelog.js')"></script>
+    </metal:block>
+  </head>
+
+  <body>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+    <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
+    <tal:block metal:fill-slot="heading">
+        <div id="breadcrumbs">
+            <tal:has-link condition="branch/branch_link">
+              <a tal:attributes="href branch/branch_link"
+                 tal:content="branch/friendly_name">
+                nice/branch/name
+              </a>
+            </tal:has-link>
+            <tal:no-link condition="not: branch/branch_link">
+              <span metal:use-macro="breadcrumbs/directory"></span>
+            </tal:no-link>
+            <tal:block condition="changes">
+              &#187; Changes
+              <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>
+              <tal:block condition="start_revid">from revision
+                <span tal:content="python:history.get_revno(start_revid)"/>
+              </tal:block>
+              <tal:block condition="query">matching <tal:block content="query" /></tal:block>
+            </tal:block>
+        </div>
+    </tal:block>
+
+    <div metal:fill-slot="content">
+      <tal:branch-info replace="structure python:branchinfo(branch)" />
+
+      <!-- Something nicer-looking should be done with search -->
+      <p tal:condition="search_failed">
+        Sorry, no results found for your search.
+      </p>
+
+      <p tal:condition="not:changes">
+        No revisions!
+      </p>
+
+      <p class="fr revnolink">From Revision <a tal:attributes="href #
+                                                               title python:'Show revision '+history.get_revno(revid)"
+                                               tal:content="python:history.get_revno(revid)"></a>
+        <tal:block tal:condition="python:navigation.last_in_page_revid is not None"> to
+          <a tal:attributes="href #
                              title 'Show revision '+history.get_revno(navigation.last_in_page_revid)"
              tal:content="python:history.get_revno(navigation.last_in_page_revid)"></a>
-         </tal:block></p>
-        <p class="expand" id="expand_all"><a tal:attributes="href string:javascript:toggle_expand_all('open')">
-            <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"
-                  alt="expand all" /> expand all</a>
-        </p>
-        <p class="expand" id="collapse_all" style="display:none;"><a tal:attributes="href string:javascript:toggle_expand_all('close')">
-            <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png')"
-                  alt="collapse all" /> collapse all</a>
-        </p>
-        <!-- Table -->        
-        <table id="logentries">
-            <tr class="logheader">
-                <td class="revisionnumber">Rev</td>
-                <td class="expandcell">&nbsp;</td>
-                <td class="summarycell">Summary</td>
-                <td class="authorcell">Author</td>
-                <td class="datecell">Date</td>
-                <td class="diffcell">Diff</td>
-                <td class="downloadcell">Files</td>
-            </tr>
-            <tal:block tal:repeat="entry changes">
-            <a tal:attributes="name string:entry-${entry/revno}"/>
-                <tr tal:attributes="class string:blueRow${entry/parity} revision_log">
-                <td class="revnro revnolink"><a tal:attributes="title python:'Show revision '+entry.revno;
-                                                                href  python:url(['/revision', entry.revno], clear=1)"
-                                                tal:content="python:util.trunc(entry.revno)"></a>
-                </td>            
-                <td class="expcell">
-                    <div class="expand_revisioninfo">
-                        <!-- So, this is interesting. I'm using "alt" to have the correct URL for the image of the expanded icon
-                             and "title" tag for the contracted URL of the image. This is a bit of a hack, but it's better than
-                             other approaches I tried  :)  -->
-                        <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png');
-                                             alt python:branch.static_url('/static/images/treeExpanded.png');
-                                             title python:branch.static_url('/static/images/treeCollapsed.png')" 
-                             class="expand_icon" />
-                    </div>
-                </td>
-                <td class="summcell"><div tal:attributes="class string:short_description">
-                                        <a tal:attributes="title python:'Show revision '+entry.revno;
-                                                           href  python:url(['/revision', entry.revno], clear=1);
-                                                           class string:link"
-                                           tal:content="entry/short_comment"></a>
-                                     </div>
-                                     <div tal:attributes="class string:long_description;
-                                                          style string:display:none">
-                                        <a tal:attributes="title python:'Show revision '+entry.revno;
-                                                           href  python:url(['/revision', entry.revno], clear=1);
-                                                           class string:link"
-                                           tal:content="entry/comment"></a>
-                                     </div>
-                                <div class="revisioninfo" style="display:none;">
-                                    <ul class="expandrev">
-               	                        <li class="mfrom" tal:repeat="parent python:entry.parents[1:]">
-                                            <span class="revnolink">
-                                                <a tal:attributes="href python:url(['/changes', parent.revno])"
-                                                   tal:content="parent/revno"></a>
-                                            </span>
-                                            <a tal:condition="parent.branch_nick"
-                                               tal:attributes="href python:url(['/changes'], start_revid=parent.revno)"
-                                               tal:content="python:'(' + parent.branch_nick + ')'"
-                                                title="Show history" class="link"></a>
-                                        </li>
-               	                        <li class="mto" tal:repeat="merge_point entry/merge_points">
-                                            <a tal:attributes="href python:url(['/changes'], start_revid=merge_point.revno)"
-                                               tal:content="python:revno_with_nick(merge_point)"
-                                               title="Show history" class="link"></a>
-                                        </li>
-                                        <li class="committerli" tal:content="python:util.hide_email(entry.author)"></li>
-                                        <tal:block content="structure python:file_change_summary(url, entry, modified_file_link_log)" />
-                                    </ul>
-                                </div>
-                </td>
-                <td tal:content="python:util.trunc(util.hide_email(entry.author), 20)"
-                    class="autcell"></td>
-                <td class="date">
-                    <span tal:attributes="title python:util.date_time(entry.date)"
-                          tal:content="python:util._approximatedate(entry.date)"></span>
-                </td>
-                <td class="diffr"><a tal:attributes="title python:'Show diff at revision '+entry.revno;
-                                                     href python:url(['/revision', entry.revno], clear=1)">
-                                  <img tal:attributes="src python:branch.static_url('/static/images/ico_diff.gif')" alt="Diff" /></a></td>
-                <td class="downr"><a tal:attributes="href python:branch.url(['/files', entry.revno]);
-                                                     title string:Files at revision ${entry/revno}">
-                                   <img tal:attributes="src python:branch.static_url('/static/images/ico_file.gif')" alt="Files" /></a>
-                </td>
-            </tr>
-          </tal:block>
-        </table>
-
-        <ul tal:condition="python:navigation.prev_page_revid or navigation.next_page_revid"
-            id="pages">
-            <li tal:condition="navigation/prev_page_revid" 
-                class="previous">
-                <a tal:attributes="href navigation/prev_page_url">&laquo; Previous</a>
-            </li>
-            <!-- FIXME: Leaving this to eventually show page numbers. Can't show all of them,
-                        so some magic has to be done to just show the previous and next N page numbers
-
-            <li class="active">1</li>
-            <tal:block tal:repeat="page_number python:range(navigation.page_count)">
-            <li><a href="#"
-                   tal:content="page_number"></a></li>
-               </tal:block>-->
-            <li tal:condition="navigation/next_page_revid"
-                class="next">
-                <a tal:attributes="href navigation/next_page_url">Next &raquo;</a>
-            </li>
-        </ul>
-
-      </div>
-    </body>
-  </html>
-</tal:block>
+      </tal:block></p>
+      <p class="expand show_if_js" id="expand_all"><a href="#">
+          <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"
+               alt="expand all" /> expand all</a>
+      </p>
+      <p class="expand" id="collapse_all" style="display:none;"><a href="#">
+          <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png')"
+               alt="collapse all" /> collapse all</a>
+      </p>
+      <!-- Table -->
+      <table id="logentries">
+        <tr class="logheader">
+          <th class="revisionnumber">Rev</th>
+          <th class="expandcell show_if_js">&nbsp;</th>
+          <th class="summarycell">Summary</th>
+          <th class="authorcell">Authors</th>
+          <th tal:condition="show_tag_col" class="tagscell">Tags</th>
+          <th class="datecell">Date</th>
+          <th class="diffcell">Diff</th>
+          <th class="downloadcell">Files</th>
+        </tr>
+        <tal:block tal:repeat="entry changes">
+          <a tal:attributes="name string:entry-${entry/revno}"/>
+          <tr tal:attributes="class string:blueRow${entry/parity} revision_log; id string:log-${entry/index}">
+            <td class="revnro revnolink"><a tal:attributes="title python:'Show revision '+entry.revno;
+                                                            href  python:url(['/revision', entry.revno], clear=1)"
+                                            tal:content="python:util.trunc(entry.revno)"></a>
+            </td>
+            <td class="expcell show_if_js">
+              <div class="expand_revisioninfo">
+                <a href="#">
+                  <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"
+                       class="expand_icon" />
+                </a>
+              </div>
+            </td>
+            <td class="summcell">
+              <div class="short_description">
+                <img tal:condition="python:entry.parents[1:]"
+                     tal:attributes="src python:branch.static_url('/static/images/ico_mergefrom.gif')" />
+                <a tal:attributes="title python:'Show revision '+entry.revno;
+                                   href  python:url(['/revision', entry.revno], clear=1);
+                                   class string:link"
+                   tal:content="entry/short_comment"></a>
+              </div>
+              <div class="long_description" style="display: none">
+                <img tal:condition="python:entry.parents[1:]"
+                     tal:attributes="src python:branch.static_url('/static/images/ico_mergefrom.gif')" />
+                <a tal:attributes="title python:'Show revision '+entry.revno;
+                                   href  python:url(['/revision', entry.revno], clear=1);
+                                   class string:link"
+                   tal:content="structure python:util.fixed_width(entry.comment)"></a>
+                <div class="loading">
+                  <img tal:attributes="src python:branch.static_url('/static/images/spinner.gif')" />
+                </div>
+              </div>
+            </td>
+            <td tal:content="python:util.trunc(util.hide_email(entry.authors[0]), 20)"
+                class="autcell"></td>
+            <td tal:condition="show_tag_col" tal:content="string:${entry/tags}"
+                class="tagcell"></td>
+            <td class="date">
+              <span tal:attributes="title python:util.date_time(entry.date)"
+                    tal:content="python:util._approximatedate(entry.date)"></span>
+            </td>
+            <td class="diffr"><a tal:attributes="title python:'Show diff at revision '+entry.revno;
+                                                 href python:url(['/revision', entry.revno], clear=1)">
+                <img tal:attributes="src python:branch.static_url('/static/images/ico_diff.gif')" alt="Diff" /></a></td>
+            <td class="downr"><a tal:attributes="href python:branch.url(['/files', entry.revno]);
+                                                 title string:Files at revision ${entry/revno}">
+                <img tal:attributes="src python:branch.static_url('/static/images/ico_file.gif')" alt="Files" /></a>
+            </td>
+          </tr>
+        </tal:block>
+      </table>
+
+      <ul tal:condition="python:navigation.prev_page_revid or navigation.next_page_revid"
+          id="pages">
+        <li tal:condition="navigation/prev_page_revid"
+            class="previous">
+          <a tal:attributes="href navigation/prev_page_url">&laquo; Newer</a>
+        </li>
+        <!-- FIXME: Leaving this to eventually show page numbers. Can't show all of them,
+             so some magic has to be done to just show the previous and next N page numbers
+
+             <li class="active">1</li>
+             <tal:block tal:repeat="page_number python:range(navigation.page_count)">
+               <li><a href="#"
+                      tal:content="page_number"></a></li>
+             </tal:block>-->
+        <li tal:condition="navigation/next_page_revid"
+            class="next">
+          <a tal:attributes="href navigation/next_page_url">Older &raquo;</a>
+        </li>
+      </ul>
+
+    </div>
+  </body>
+</html>

=== modified file 'loggerhead/templates/directory.pt'
--- loggerhead/templates/directory.pt	2008-10-18 00:23:09 +0000
+++ loggerhead/templates/directory.pt	2011-02-17 19:51:25 +0000
@@ -4,6 +4,8 @@
     <title tal:content="string:Browsing ${name}"></title>
     <link rel="stylesheet"
           tal:attributes="href python:static_url('/static/css/global.css')" />
+    <link rel="shortcut icon"
+          tal:attributes="href python:branch.static_url('/static/images/favicon.ico')" /> 
   </head>
   <body>
     <div id="loggerheadCont">
@@ -19,13 +21,13 @@
             <th class="datecell">Latest Rev</th>
             <th class="datecell">Last Changed</th>
           </tr>
-          
+
           <tr class="blueRow0" tal:condition="python:name != '/'">
             <td class="icocell">
-                <a href=".."><img tal:attributes="src python:static_url('/static/images/ico_folder_up.gif')" /></a>
+                <a href="../"><img tal:attributes="src python:static_url('/static/images/ico_folder_up.gif')" /></a>
             </td>
             <td class="summcell" colspan="3">
-                <a href="..">..</a>
+                <a href="../">..</a>
             </td>
           </tr>
           <tal:block repeat="dir dirs">
@@ -50,12 +52,12 @@
             <tal:non-branch-row tal:condition="not:dir/branch">
               <tr tal:attributes="class string:blueRow${dir/parity}">
                 <td class="icocell">
-                    <a tal:attributes="href string:${dir/dirname}">
+                    <a tal:attributes="href string:${dir/dirname}/">
                         <img tal:attributes="src python:static_url('/static/images/ico_folder.gif')" alt="Folder" />
                     </a>
                 </td>
                 <td class="autcell">
-                  <a tal:attributes="href string:${dir/dirname}" tal:content="dir/dirname" /></td>
+                  <a tal:attributes="href string:${dir/dirname}/" tal:content="dir/dirname" /></td>
                 <td class="date"></td>
                 <td class="date" tal:content="dir/last_change"></td>
               </tr>

=== added file 'loggerhead/templates/filediff.pt'
--- loggerhead/templates/filediff.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/filediff.pt	2009-04-01 15:28:21 +0000
@@ -0,0 +1,25 @@
+<div>
+  <div class="pseudotable unified"
+       tal:repeat="chunk chunks">
+
+    <tal:block condition="not:repeat/chunk/start">
+      <div class="pseudorow context-row">
+        <div class="lineNumber separate"></div>
+        <div class="lineNumber second separate"></div>
+        <div class="code separate"></div>
+        <div class="clear"><!-- --></div>
+      </div>
+    </tal:block>
+
+    <div tal:repeat="line chunk/diff"
+         tal:attributes="class string:pseudorow ${line/type}-row">
+      <div class="lineNumber first"
+           tal:content="structure python:util.fill_div(line.old_lineno)"></div>
+      <div class="lineNumber second"
+           tal:content="structure python:util.fill_div(line.new_lineno)"></div>
+      <div tal:attributes="class string:code ${line/type}"
+           tal:content="structure python:util.fill_div(util.html_clean(line.line))"></div>
+      <div class="clear"><!-- --></div>
+    </div>
+  </div>
+</div>

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2009-02-10 04:25:11 +0000
+++ loggerhead/templates/inventory.pt	2011-02-23 03:01:15 +0000
@@ -9,23 +9,34 @@
     </metal:block>
   </head>
   <body>
-
-    <h1 metal:fill-slot="heading">
-      <tal:has-link condition="branch/branch_link">
-        <a tal:attributes="href branch/branch_link"
-           tal:content="branch/friendly_name">
-          nice/branch/name
-        </a>
-      </tal:has-link>
-      <span tal:condition="not: branch/branch_link">
-        <span metal:use-macro="breadcrumbs/directory" />
-      </span>
-      <tal:block condition="python:change">
-        : viewing
-        <span metal:use-macro="breadcrumbs/branch" />
-        for revision <span tal:content="change/revno"/>
-      </tal:block>
-    </h1>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
+      <tal:block metal:fill-slot="heading">
+      <div id="breadcrumbs">
+          <tal:has-link condition="branch/branch_link">
+            <a tal:attributes="href branch/branch_link"
+               tal:content="branch/friendly_name">
+              nice/branch/name
+            </a>
+          </tal:has-link>
+          <span tal:condition="not: branch/branch_link">
+            <span metal:use-macro="breadcrumbs/directory" />
+          </span>
+          <tal:block condition="python:change">
+            &#187; Viewing
+            <span metal:use-macro="breadcrumbs/branch" />
+            for revision <span tal:content="change/revno"/>
+          </tal:block>
+        </div>
+      </tal:block>
 
     <div metal:fill-slot="content">
         <tal:branch-info replace="structure python:branchinfo(branch)" />
@@ -70,7 +81,7 @@
                                              title string:Go to ${file/filename}" />
                     </a>
 
-                 <a tal:attributes="href python:url(['/files', revno_url, file.absolutepath])" 
+                 <a tal:attributes="href python:url(['/files', revno_url, file.absolutepath])"
                                         tal:content="file/filename" class="link"></a>
                  </td>
                  <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
@@ -89,11 +100,11 @@
 
             <!-- Show this if it's a symlink -->
             <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='symlink'">
-                <td class="autcell"><a tal:attributes="href python:url(['/annotate', change.revno, file.absolutepath])">
-                        <img tal:attributes="src python:branch.static_url('/static/images/ico_flecha.gif')" alt="Symlink" />
+                <td class="autcell"><a tal:attributes="href python:url(['/view', change.revno, file.absolutepath])">
+                        <img tal:attributes="src python:branch.static_url('/static/images/ico_file_flecha.gif')" alt="Symlink" />
                     </a>
 
-                 <a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])" 
+                 <a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])"
                                         tal:content="file/filename" class="link"></a>
                  </td>
                  <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
@@ -113,17 +124,17 @@
 
             <!-- Show this if it's a regular file -->
             <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='file'">
-                <td class="autcell"><a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])">
+                <td class="autcell"><a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])">
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_file.gif');
-                                             title string:Annotate ${file/filename}"
+                                             title string:View ${file/filename}"
                              tal:condition="python:file.executable is False" />
                         <!-- Show a different icon id the file executable -->
-                        <img tal:attributes="src python:branch.static_url('/static/images/ico_file_modify.gif'); 
-                                             title string:Annotate ${file/filename}"
+                        <img tal:attributes="src python:branch.static_url('/static/images/ico_file_modify.gif');
+                                             title string:View ${file/filename}"
                              tal:condition="python:file.executable is True" alt="File" />
                     </a>
 
-              <a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])"
+              <a tal:attributes="href python:url(['/view', revno_url, file.absolutepath])"
                                      tal:content="file/filename" class="link"></a></td>
               <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
                                                   title string:Show revision ${file/change/revno}"
@@ -131,8 +142,8 @@
               </td>
               <td class="date" tal:content="python:util.date_time(file.change.date)"></td>
               <td class="timedate2" tal:content="python:util.human_size(file.size)"></td>
-              <td class="expcell"><a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath]);
-                                                     title string:Annotate ${file/filename}">
+              <td class="expcell"><a tal:attributes="href python:url(['/view', revno_url, file.absolutepath]);
+                                                     title string:View ${file/filename}">
                                       <img tal:attributes="src python:branch.static_url('/static/images/ico_planilla.gif')" alt="Diff" />
                                      </a>
               </td>

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2008-10-25 18:41:18 +0000
+++ loggerhead/templates/macros.pt	2011-03-03 10:58:57 +0000
@@ -3,46 +3,63 @@
 <html xmlns="http://www.w3.org/1999/xhtml" metal:define-macro="main">
   <head>
     <meta content="text/html; charset=utf-8" http-equiv="Content-Type" />
+    <meta name="generator"
+          tal:attributes="content generator_string" />
 
     <title metal:define-slot="title"></title>
     <link rel="stylesheet"
           tal:attributes="href python:branch.static_url('/static/css/global.css')" />
-    <script type="text/javascript"
-            tal:attributes="src python:branch.static_url('/static/javascript/mootools-1.2-core.js')"></script>
-    <script type="text/javascript"
-            tal:attributes="src python:branch.static_url('/static/javascript/mootools-1.2-more.js')"></script>
+    <link rel="shortcut icon"
+          tal:attributes="href python:branch.static_url('/static/images/favicon.png')" />
+    <tal:comment condition="nothing">
+      <script type="text/javascript"
+              src="http://getfirebug.com/releases/lite/1.2/firebug-lite-compressed.js"></script>
+    </tal:comment>
+    <script type="text/javascript">
+      var global_path = <tal:block content="python:'\''+branch.url('/')+'\''" />;
+      var collapsed_icon_path = <tal:block content="python:'\''+branch.static_url('/static/images/treeCollapsed.png')+'\''" />;
+      var expanded_icon_path = <tal:block content="python:'\''+branch.static_url('/static/images/treeExpanded.png')+'\''" />;
+    </script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('yui/yui-min.js')"></script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('oop/oop-min.js')"></script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('event/event-min.js')"></script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('attribute/attribute-min.js')"></script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('base/base-min.js')"></script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('dom/dom-min.js')"></script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('node/node-min.js')"></script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('anim/anim-min.js')"></script>
+    <script type="text/javascript"
+            tal:attributes="src python:branch.yui_url('io/io-base-min.js')"></script>
     <script type="text/javascript"
             tal:attributes="src python:branch.static_url('/static/javascript/custom.js')"></script>
-      <script type="text/javascript">
-          var global_path = <tal:block content="python:'\''+branch.url('/')+'\''" />;
-      </script>
-      <script type="text/javascript"
-              tal:attributes="src
-              python:branch.static_url('/static/javascript/yui/build/yui/yui-min.js')"></script>
     <metal:block metal:define-slot="header_extras" />
   </head>
 
 <body>
     <!-- Loggerhead Content Area -->
-      <div id="finderBox">
-         <tal:search-box define="navigation navigation"
-                        replace="structure python:search_box(branch,
-                                 navigation)" />
-        <div>
-          <tal:feed-link replace="structure python:feed_link(branch, url)" />
-        </div>
-      </div>
-
+      
+      <div metal:define-slot="backlink" />
+      
+      <h1 class="branch-name">
+        <span metal:define-slot="branchname" />
+      </h1>
       <tal:menu define="fileview_active fileview_active"
                 replace="structure python:menu(branch, url, fileview_active)" />
       <div id="loggerheadCont">
 
         <div id="search_terms"></div>
-
-        <h1 metal:define-slot="heading"></h1>
+        <div metal:define-slot="heading" />
         <div metal:define-slot="content"></div>
 
-        <p class="fl">Loggerhead is a web-based interface for <a href="http://bazaar-vcs.org/">Bazaar</a> branches</p>
+        <p id="footer" class="fl">Loggerhead <tal:version content="loggerhead_version" /> is a web-based interface for <a href="http://bazaar-vcs.org/">Bazaar</a> branches</p>
       </div>
   </body>
 </html>

=== removed file 'loggerhead/templates/modified-file-link-rev.pt'
--- loggerhead/templates/modified-file-link-rev.pt	2008-06-19 21:39:15 +0000
+++ loggerhead/templates/modified-file-link-rev.pt	1970-01-01 00:00:00 +0000
@@ -1,7 +0,0 @@
-<tal:block>
-<li class="files"><a 
-     tal:attributes="href  string:#${item/filename}-s;
-                     title string:Jump to ${item/filename} below"
-     tal:content="item/filename" >
-</a></li>
-</tal:block>

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2009-02-17 04:06:27 +0000
+++ loggerhead/templates/revision.pt	2011-02-23 03:01:15 +0000
@@ -9,13 +9,29 @@
       <metal:block fill-slot="header_extras">
         <link rel="stylesheet" type="text/css" media="all"
               tal:attributes="href python:branch.static_url('/static/css/diff.css')"/>
+        <script type="text/javascript"
+                tal:attributes="src python:branch.static_url('/static/javascript/diff.js')"></script>
+        <script type="text/javascript">
+          var link_data = <tal:b content="link_data" />;
+          var specific_path = <tal:b content="json_specific_path" />;
+          var path_to_id = <tal:b content="path_to_id" />;
+        </script>
       </metal:block>
     </head>
 
     <body>
-
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
       <tal:block metal:fill-slot="heading">
-        <h1>
+        <div id="breadcrumbs">
           <tal:has-link condition="branch/branch_link">
             <a tal:attributes="href branch/branch_link"
                tal:content="branch/friendly_name">
@@ -25,12 +41,31 @@
           <tal:no-link condition="not: branch/branch_link">
             <span metal:use-macro="breadcrumbs/directory"></span>
           </tal:no-link>
-          <span>: <tal:revno content="change/revno"></tal:revno>
-            <tal:compare-to condition="python:compare_revid is not None">
+          <span class="breadcrumb">&#187; Revision
+            <tal:revno condition="not:specific_path" content="change/revno"></tal:revno>
+            <a tal:condition="specific_path" tal:content="change/revno"
+               tal:attributes="href python:url(['/revision', change.revno])"
+               title="View changes to all files"></a>
+                      <tal:compare-to condition="python:compare_revid is not None">
               (compared to revision <tal:block content="python:history.get_revno(compare_revid)" />)
             </tal:compare-to>
-        </span></h1>
+          </span>
+          <span class="breadcrumb" tal:condition="specific_path">
+            : <tal:annotate content="structure python:view_link(url, change.revno, specific_path)" />
+          </span>
+        </div>
         <tal:branch-info replace="structure python:branchinfo(branch)" />
+        <h1 tal:condition="not:specific_path">
+          Viewing all changes in revision <tal:revno content="change/revno" />.
+        </h1>
+        <p tal:condition="specific_path">
+          <a tal:attributes="href python:url(['/revision', change.revno])">
+            &#xAB; back to all changes in this revision
+          </a>
+        </p>
+        <p tal:condition="specific_path">
+          Viewing changes to <tal:annotate content="structure python:view_link(url, change.revno, specific_path)" />
+        </p>
         <ul id="submenuTabs">
           <li id="first"><a tal:attributes="href python:url(['/files', change.revno]);
                                             title string:browse files at revision ${change/revno}"
@@ -70,16 +105,19 @@
           </ul>
         </tal:we-are-comparing>
 
-        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, modified_file_link_rev)" />
-        <p class="expand" id="expand_all" style="display:none;"><a tal:attributes="href string:javascript:toggle_expand_all_revisionview('open')">
-            <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"
-                 alt="expand all" /> expand all</a>
-        </p>
-        <p class="expand" id="collapse_all"><a tal:attributes="href string:javascript:toggle_expand_all_revisionview('close')">
-            <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png')"
-                 alt="collapse all" /> collapse all</a>
-        </p>
+        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, file_changes, specific_path, merged_in)" />
+        <tal:specific-path condition="not:specific_path">
+          <p class="expand show_if_js" id="expand_all"><a href="#">
+              <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"
+                   alt="expand all" /> expand all</a>
+          </p>
+          <p class="expand" id="collapse_all" style="display:none;"><a href="#">
+              <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png')"
+                   alt="collapse all" /> collapse all</a>
+          </p>
+        </tal:specific-path>
         <!-- Table -->
+        <p class="expand show_if_js"><a id="toggle_unified_sbs" href="#">Show diffs side-by-side</a></p>
         <p class="codin"><img tal:attributes="src python:branch.static_url('/static/images/newCode.gif')" alt="added" /> added</p>
         <p class="codin"><img tal:attributes="src python:branch.static_url('/static/images/deleteCode.gif')" alt="removed" /> removed</p>
         <div class="clear"><!-- --></div>
@@ -87,79 +125,81 @@
       </tal:block>
 
       <div metal:fill-slot="content">
-        <div class="diff"
-             tal:repeat="item change/changes/modified">
-
-          <div class="diffBox">
-            <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png');
-                                 title python:branch.static_url('/static/images/treeCollapsed.png');
-                                 alt python:branch.static_url('/static/images/treeExpanded.png')"
-                 class="expand_diff" />
-            <a tal:attributes="href python:url(['/annotate', change.revno], clear=1, file_id=item.file_id);
-                               id string:${item/filename}-s;
-                               title string:Annotate ${item/filename}"
-               tal:content="item/filename">
-            </a>
-          </div>
-          <div class="psuedotable diffinfo"
-               tal:repeat="chunk item/sbs_chunks">
-
-            <tal:block condition="not:repeat/chunk/start">
-              <div class="lineNumber separate"></div>
-              <div class="lineCode separate"></div>
-              <div class="lineNumber separate"></div>
-              <div class="lineCode separate"></div>
-              <div class="clear"><!-- --></div>
-            </tal:block>
-
-            <div class="pseudorow"
-                 tal:repeat="line chunk/diff">
-              <div class="lineNumber"
-                   tal:content="structure python:util.fill_div(line.old_lineno)"></div>
-              <div tal:attributes="class string:lineCode ${line/old_type}"
-                   tal:content="structure python:util.fill_div(line.old_line)"></div>
-
-              <div class="lineNumber"
-                   tal:content="structure python:util.fill_div(line.new_lineno)"></div>
-              <div tal:attributes="class string:lineCode ${line/new_type}"
-                   tal:content="structure python:util.fill_div(line.new_line)"></div>
-              <div class="clear"><!-- --></div>
-            </div>
-          </div>
-
-          <tal:comment condition="nothing">
-            <!-- here's how to do unified diffs -->
-            <div class="psuedotable diffinfo"
-                 tal:repeat="chunk item/chunks">
-
-              <tal:block condition="not:repeat/chunk/start">
-                <div class="lineNumber separate"></div>
-                <div class="lineCode separate"></div>
-                <div class="lineNumber separate"></div>
-                <div class="lineCode separate"></div>
-                <div class="clear"><!-- --></div>
-              </tal:block>
-
-              <div class="pseudorow"
-                   tal:repeat="line chunk/diff">
-                <div class="lineNumber"
-                     tal:content="structure python:util.fill_div(line.old_lineno)"></div>
-
-                <div class="lineNumber"
-                     tal:content="structure python:util.fill_div(line.new_lineno)"></div>
-                <div tal:attributes="class string:lineCode-u ${line/type}"
-                     tal:content="structure python:util.fill_div(util.html_clean(line.line))"></div>
-                <div class="clear"><!-- --></div>
-              </div>
-            </div>
-          </tal:comment>
-        </div>
+        <tal:block condition="specific_path">
+          <div class="diff">
+
+            <div  class="diffBox">
+              <a tal:attributes="href python:url(['/revision', change.revno, specific_path], clear=1);
+                                 id string:${specific_path};
+                                 title string:View changes to ${specific_path} only"
+                 class="the-link">
+                <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png')"
+                     class="expand_diff"/>
+                <tal:b content="specific_path" />
+              </a>
+            </div>
+            <div style="overflow: hidden">
+              <div class="container">
+                <div class="loading" style="display:none">
+                  <img tal:attributes="src python:branch.static_url('/static/images/spinner.gif')" />
+                </div>
+                <div class="diffinfo">
+                  <div class="pseudotable unified"
+                       tal:repeat="chunk diff_chunks">
+
+                    <tal:block condition="not:repeat/chunk/start">
+                      <div class="pseudorow context-row">
+                        <div class="lineNumber separate"></div>
+                        <div class="lineNumber second separate"></div>
+                        <div class="code separate"></div>
+                        <div class="clear"><!-- --></div>
+                      </div>
+                    </tal:block>
+
+                    <div tal:repeat="line chunk/diff"
+                         tal:attributes="class string:pseudorow ${line/type}-row">
+                      <div class="lineNumber first"
+                           tal:content="structure python:util.fill_div(line.old_lineno)"></div>
+                      <div class="lineNumber second"
+                           tal:content="structure python:util.fill_div(line.new_lineno)"></div>
+                      <div tal:attributes="class string:code ${line/type}"
+                           tal:content="structure python:util.fill_div(util.html_clean(line.line))"></div>
+                      <div class="clear"><!-- --></div>
+                    </div>
+                  </div>
+                </div>
+              </div>
+            </div>
+          </div>
+        </tal:block>
+
+        <tal:block condition="not:specific_path">
+          <div class="diff"
+               tal:repeat="item file_changes/text_changes" tal:attributes="id string:diff-${item/index}">
+
+            <div  class="diffBox">
+              <a tal:attributes="href python:url(['/revision', change.revno, item.filename], clear=1);
+                                 id string:${item/filename};
+                                 title string:View changes to ${item/filename} only"
+                 class="the-link">
+                <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"
+                     class="expand_diff" />
+                <tal:b content="item/filename" />
+              </a>
+            </div>
+            <div class="diffinfo"  style="display:none">
+              <div class="loading">
+                <img tal:attributes="src python:branch.static_url('/static/images/spinner.gif')" />
+              </div>
+            </div>
+          </div>
+        </tal:block>
 
         <ul tal:condition="python:navigation.prev_page_revid or navigation.next_page_revid"
             id="pages">
           <li tal:condition="navigation/prev_page_revid"
               class="previous">
-            <a tal:attributes="href navigation/prev_page_url">&laquo; Previous</a>
+            <a tal:attributes="href navigation/prev_page_url">&laquo; Newer</a>
           </li>
           <!-- FIXME: Leaving this to eventually show page numbers. Can't show all of them,
                so some magic has to be done to just show the previous and next N page numbers
@@ -171,7 +211,7 @@
                </tal:block>-->
           <li tal:condition="navigation/next_page_revid"
               class="next">
-            <a tal:attributes="href navigation/next_page_url">Next &raquo;</a>
+            <a tal:attributes="href navigation/next_page_url">Older &raquo;</a>
           </li>
         </ul>
 

=== modified file 'loggerhead/templates/revisionfilechanges.pt'
--- loggerhead/templates/revisionfilechanges.pt	2008-07-23 21:31:07 +0000
+++ loggerhead/templates/revisionfilechanges.pt	2009-03-19 00:05:41 +0000
@@ -1,58 +1,50 @@
 <tal:block>
-<ul tal:repeat="added entry/changes/added">
+  <ul tal:repeat="added file_changes/added">
     <tal:block condition="repeat/added/start">
-        <li class="desc">files added:</li>
+      <li class="desc">files added:</li>
     </tal:block>
-    <li class="files"
-        tal:define="filename python:added[0];
-                    file_id  python:added[1]">
-        <a tal:attributes="href  python:url(['/annotate', entry.revno], file_id=file_id);
-                           title string:Annotate ${filename}"
-           tal:content="filename" class="link"></a>
+    <li class="files">
+      <tal:revision condition="python:added.kind == 'file'"
+                    content="structure python:file_link(added.filename)" />
+      <tal:revision condition="python:added.kind != 'file'"
+                    content="added/filename" />
     </li>
-</ul>
+  </ul>
 
-<ul tal:repeat="removed entry/changes/removed">
+  <ul tal:repeat="removed file_changes/removed">
     <tal:block condition="repeat/removed/start">
-        <li class="desc">files removed:</li>
+      <li class="desc">files removed:</li>
     </tal:block>
-    <li class="files"
-        tal:define="filename python:removed[0];
-                    file_id  python:removed[1]">
-      <tal:block content="filename" />
+    <li class="files">
+      <tal:revision condition="python:removed.kind == 'file'"
+                    content="structure python:file_link(removed.filename)" />
+      <tal:revision condition="python:removed.kind != 'file'"
+                    content="removed/filename" />
     </li>
-</ul>
+  </ul>
 
-<ul tal:repeat="renamed entry/changes/renamed">
+  <ul tal:repeat="renamed file_changes/renamed">
     <tal:block condition="repeat/renamed/start">
-        <li class="desc">files renamed:</li>
+      <li class="desc">files renamed:</li>
     </tal:block>
-    <li class="files"
-        tal:define="old_filename python:renamed[0];
-                    new_filename python:renamed[1];
-                    file_id      python:renamed[2]">
-      <a tal:attributes="href  python:url(['/annotate', entry.revno], file_id=file_id);
-                         title string:Annotate ${old_filename}"
-         tal:content="old_filename" class="filename">
+    <li class="files">
+      <tal:old-name content="renamed/old_filename" class="filename">
         old_filename
-      </a>
-      =>
-      <a tal:attributes="href  python:url(['/annotate', entry.revno], file_id=file_id);
-                         title string:Annotate ${new_filename}"
-         tal:content="new_filename">
+      </tal:old-name>
+      =&gt;
+      <tal:revision condition="renamed/text_modified" content="structure python:file_link(renamed.new_filename)" />
+      <tal:old-name condition="not:renamed/text_modified" content="renamed/new_filename" class="filename">
         new_filename
-      </a>
+      </tal:old-name>
     </li>
-</ul>
+  </ul>
 
-<ul tal:repeat="item entry/changes/modified">
+  <ul tal:repeat="item file_changes/modified">
     <tal:block condition="repeat/item/start">
-        <li class="desc">files modified:</li>
+      <li class="desc">files modified:</li>
     </tal:block>
     <li class="files">
-      <a tal:attributes="href  python:url(['/annotate', entry.revno], file_id=item.file_id);
-                         title string:Annotate ${item/filename}"
-                         tal:content="item/filename"/>
+      <tal:revision content="structure python:file_link(item.filename)" />
     </li>
-</ul>
+  </ul>
 </tal:block>

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2009-01-23 19:19:39 +0000
+++ loggerhead/templates/revisioninfo.pt	2011-03-16 12:37:17 +0000
@@ -1,23 +1,58 @@
 <div class="infoContainer">
     <div id="infTxt">
-        <div class="committer" tal:content="python:util.hide_email(change.author)"></div>
-        <div class="timer" tal:content="python:util.date_time(change.date)"></div>
-        <div class="mfrom" tal:repeat="parent python:change.parents[1:]">
-            <a tal:attributes="title string:Merged From ${parent/revno};
-                               href python:url(['/revision', parent.revno], clear=1)"
-               tal:content="python:revno_with_nick(parent)"></a>
-        </div>
-        <div class="mto" tal:repeat="merge_point change/merge_points">
-            <a tal:attributes="title string:Merged In ${merge_point/revno};
-                               href python:url(['/revision', merge_point.revno], clear=1)"
-               tal:content="python:revno_with_nick(merge_point)"></a>
-        </div>
-        <div class="revid" tal:content="string:Revision ID: ${change/revid}"></div>
-        <div class="information" tal:content="python:util.fixed_width(change.comment)"></div>
+        <ul>
+            <li class="committer">
+                <strong>Committer:</strong>
+                <span tal:content="python:util.hide_email(change.committer)"></span>
+            </li>
+            <li class="authors" tal:condition="python:len(change.authors) > 1 or change.authors[0] != change.committer">
+                <strong>Author(s):</strong>
+                <span tal:content="python:', '.join(util.hide_emails(change.authors))"></span>
+            </li>
+            <li class="timer">
+                <strong>Date:</strong>
+                <span tal:content="python:util.date_time(change.date)"></span>
+            </li>
+            <li class="mfrom" tal:condition="python:change.parents[1:]">
+                <strong>mfrom:</strong>
+                <span tal:repeat="parent python:change.parents[1:]">
+                    <a tal:attributes="title string:Merged From ${parent/revno};
+                                       href python:url(['/revision', parent.revno], clear=1)"
+                       tal:content="python:revno_with_nick(parent)"></a>
+                </span>
+            </li>
+            <li class="mto" tal:condition="change/merge_points">
+                <strong>mto:</strong>
+                <span tal:repeat="merge_point change/merge_points">
+                    <a tal:attributes="title string:Merged In ${merge_point/revno};
+                                       href python:url(['/revision', merge_point.revno], clear=1)"
+                       tal:content="python:revno_with_nick(merge_point)"></a>
+                </span>
+            </li>
+            <li class="mto" tal:condition="merged_in">
+                <strong>mto:</strong>
+                  This revision was merged to the branch mainline in
+                  <a tal:attributes="href python:url(['/revision', merged_in], clear=1)">revision
+                    <tal:b content="merged_in"/></a>.
+            </li>
+            <li class="revid">
+                <strong>Revision ID:</strong>
+                <span tal:content="change/revid"></span>
+            </li>
+        </ul>
+	<div tal:attributes="class string:foreign-${change/foreign_vcs}"
+	     tal:condition="change/foreign_vcs" 
+	     tal:repeat="revid_part python:change.foreign_revid.keys()"
+	     tal:content="python:'%s: %s' % (revid_part, change.foreign_revid[revid_part])"/>
+        <div class="tags" tal:condition="change/tags" tal:content="string:Tags: ${change/tags}"></div>
+        <div class="clear"><!-- --></div>
+        <div class="bug" tal:repeat="bug change/bugs">
+            <a tal:attributes="href bug" tal:content="bug"></a>
+        </div>
+        <div class="information" tal:content="structure python:util.fixed_width(change.comment)"></div>
     </div>
-    <ul id="list-files">
-        <tal:block condition="nocall:modified_file_link">
-        <tal:block content="structure python:file_change_summary(url, change, modified_file_link)" /></tal:block>
+    <ul id="list-files" tal:condition="file_changes">
+      <tal:block content="structure python:file_change_summary(url, change, file_changes, 'fragment', currently_showing)" />
     </ul>
     <div class="clear"><!-- --></div>
 </div>

=== added file 'loggerhead/templates/revlog.pt'
--- loggerhead/templates/revlog.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/revlog.pt	2009-03-31 15:23:54 +0000
@@ -0,0 +1,17 @@
+<div class="revisioninfo">
+  <ul class="expandrev">
+    <li class="mfrom" tal:repeat="parent python:entry.parents[1:]">
+      <span class="revnolink">
+        <a tal:attributes="href python:url(['/changes', parent.revno])"
+           tal:content="python:revno_with_nick(parent)"></a>
+      </span>
+    </li>
+    <li class="mto" tal:repeat="merge_point entry/merge_points">
+      <a tal:attributes="href python:url(['/changes', merge_point.revno])"
+         tal:content="python:revno_with_nick(merge_point)"
+         title="Show history" class="link"></a>
+    </li>
+    <li class="committerli" tal:content="python:', '.join(util.hide_emails(entry.authors))"></li>
+    <tal:block content="structure python:file_change_summary(url, entry, file_changes)" />
+  </ul>
+</div>

=== renamed file 'loggerhead/templates/annotate.pt' => 'loggerhead/templates/view.pt'
--- loggerhead/templates/annotate.pt	2009-01-23 20:53:40 +0000
+++ loggerhead/templates/view.pt	2011-02-23 03:01:15 +0000
@@ -3,29 +3,42 @@
 <html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
   <head>
     <title metal:fill-slot="title"
-           tal:content="string:${branch/friendly_name} : contents of ${path}
+           tal:content="string:${branch/friendly_name} : contents of ${file_path}
                         at revision ${change/revno}">
     </title>
     <metal:block fill-slot="header_extras">
         <link rel="stylesheet" type="text/css" media="all"
-              tal:attributes="href python:branch.static_url('/static/css/annotate.css')"/>
+              tal:attributes="href python:branch.static_url('/static/css/view.css')"/>
+        <link rel="stylesheet" type="text/css" media="all"
+              tal:attributes="href python:branch.static_url('/static/css/highlight.css')"/>
     </metal:block>
   </head>
 
   <body>
-
-    <h1 metal:fill-slot="heading">
-      <tal:has-link condition="branch/branch_link">
-        <a tal:attributes="href branch/branch_link"
-           tal:content="branch/friendly_name">
-          nice/branch/name
-        </a>
-      </tal:has-link>
-      <tal:no-link condition="not: branch/branch_link">
-        <span metal:use-macro="breadcrumbs/directory"></span>
-      </tal:no-link>
-      <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
-    </h1>
+      <tal:block metal:fill-slot="backlink">
+          <tal:has-link condition="branch/branch_link">
+            <div class="black-link">
+                <a tal:attributes="href branch/branch_link">
+                  &#8592; Back to branch summary
+                </a>
+            </div>
+          </tal:has-link>
+      </tal:block>
+      <tal:block metal:fill-slot="branchname" tal:content="string:${branch/friendly_name}" />
+          <tal:block metal:fill-slot="heading">
+          <div id="breadcrumbs">
+          <tal:has-link condition="branch/branch_link">
+            <a tal:attributes="href branch/branch_link"
+               tal:content="branch/friendly_name">
+              nice/branch/name
+            </a>
+          </tal:has-link>
+          <tal:no-link condition="not: branch/branch_link">
+            <span metal:use-macro="breadcrumbs/directory"></span>
+          </tal:no-link>
+          <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
+        </div>
+      </tal:block>
 
     <div metal:fill-slot="content">
       <tal:branch-info replace="structure python:branchinfo(branch)" />
@@ -34,12 +47,19 @@
             <li id="first">
                 <a tal:attributes="href python:url(['/files', change.revno], clear=1)">browse files</a>
             </li>
-            <li>
-                <a tal:attributes="href python:url(['/revision'], clear=1, start_revid=change.revno)">view revision</a>
-            </li>
-            <li>
-                <a tal:attributes="href python:url(['/changes'], clear=1, 
-                                    start_revid=change.revno, filter_file_id=file_id)"
+            <li tal:condition="not:annotated">
+              <a tal:attributes="href python:url(['/annotate', revno_url, file_path], clear=1)"
+              >view revision numbers per line</a>
+            </li>
+            <li tal:condition="annotated">
+              <a tal:attributes="href python:url(['/view', revno_url, file_path], clear=1)"
+              >view without revision numbers</a>
+            </li>
+            <li>
+                <a tal:attributes="href python:url(['/revision', change.revno], clear=1)">view revision</a>
+            </li>
+            <li>
+                <a tal:attributes="href python:url(['/changes'], clear=1, filter_file_id=file_id)"
                 >view changes to this file</a>
             </li>
             <li id="last">
@@ -49,25 +69,30 @@
             </li>
         </ul>
 
-      <div class="annotate">
+      <div class="view">
         <table id="logentries">
           <tr class="logheader">
-            <td class="annoLineTit">Line</td>
-            <td class="annoRevTit">Revision</td>
-            <td class="annoContTit">Contents</td>
+            <th class="viewLineTit">Line</th>
+            <th class="viewRevTit" tal:condition="annotated">Revision</th>
+            <th class="viewContTit">Contents</th>
           </tr>
 
-          <tr tal:repeat="line contents" 
-              tal:attributes="class string:blueRow${line/parity}">
-            <td class="annoLine"><a tal:attributes="id string:L${line/lineno}" tal:content="line/lineno">1</a></td>
-            <td class="annoRev">
-              <a tal:condition="python:line.status=='changed'"
-                 tal:content="python:util.trunc(line.change.revno)"
-                 tal:attributes="href python:url(['/revision'], clear=1, start_revid=line.change.revno, filter_file_id=file_id);
-                                 title python:'%s by %s, on %s (%s)'%(line.change.revno, util.hide_email(line.change.author), line.change.date.strftime('%d %b %Y %H:%M'), util.date_time(line.change.date))"></a>
-            </td>
-            <td class="annoCont" tal:content="line/text"></td>
-          </tr>
+          <tal:rep tal:repeat="line contents">
+            <tr tal:define="anno python:annotated and annotated.next()"
+                tal:attributes="class python:anno and 'blueRow' + str(anno.parity) or None">
+              <td class="viewLine">
+                <a tal:attributes="id string:L${repeat/line/number}; href string:#L${repeat/line/number}"
+                   tal:content="repeat/line/number">1</a>
+              </td>
+              <td class="viewRev" tal:condition="annotated">
+                <a tal:condition="python:anno.new_rev"
+                   tal:content="python:util.trunc(anno.change.revno)"
+                   tal:attributes="href python:url(['/revision', anno.change.revno], clear=1);
+                                   title python:'%s by %s, on %s (%s)' % (anno.change.revno, ', '.join(util.hide_emails(anno.change.authors)), anno.change.date.strftime('%d %b %Y %H:%M'), util.date_time(anno.change.date))"></a>
+              </td>
+              <td class="viewCont"><pre tal:content="structure line"></pre></td>
+            </tr>
+          </tal:rep>
         </table>
       </div>
 

=== modified file 'loggerhead/tests/__init__.py'
--- loggerhead/tests/__init__.py	2006-12-11 06:44:19 +0000
+++ loggerhead/tests/__init__.py	2011-03-23 03:59:38 +0000
@@ -0,0 +1,31 @@
+# Copyright 2006, 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+def load_tests(standard_tests, module, loader):
+    standard_tests.addTests(loader.loadTestsFromModuleNames([
+        (__name__ + '.' + x) for x in [
+            'test_controllers',
+            'test_corners',
+            'test_history',
+            'test_http_head',
+            'test_load_test',
+            'test_simple',
+            'test_revision_ui',
+            'test_templating',
+            'test_util',
+        ]]))
+    return standard_tests

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2009-02-17 00:56:57 +0000
+++ loggerhead/tests/test_controllers.py	2011-03-15 02:56:59 +0000
@@ -1,4 +1,12 @@
+from cStringIO import StringIO
+import logging
+
+from paste.httpexceptions import HTTPServerError
+
+from bzrlib import errors
+
 from loggerhead.apps.branch import BranchWSGIApp
+from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
 from loggerhead.controllers.revision_ui import RevisionUI
 from loggerhead.tests.test_simple import BasicTests
@@ -14,14 +22,49 @@
         tree.commit('')
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
+        branch_app = BranchWSGIApp(tree.branch, '')
+        branch_app.log.setLevel(logging.CRITICAL)
+        # These are usually set in BranchWSGIApp.app(), which is set from env
+        # settings set by BranchesFromTransportRoot, so we fake it.
+        branch_app._static_url_base = '/'
+        branch_app._url_base = '/'
         return tree.branch, InventoryUI(branch_app, branch_app.get_history)
 
+    def consume_app(self, app, extra_environ=None):
+        env = {'SCRIPT_NAME': '/files', 'PATH_INFO': ''}
+        if extra_environ is not None:
+            env.update(extra_environ)
+        body = StringIO()
+        start = []
+        def start_response(status, headers, exc_info=None):
+            start.append((status, headers, exc_info))
+            return body.write
+        extra_content = list(app(env, start_response))
+        body.writelines(extra_content)
+        return start[0], body.getvalue()
+
     def test_get_filelist(self):
         bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
             ['filename'])
         inv = bzrbranch.repository.get_inventory(bzrbranch.last_revision())
-        self.assertEqual(1, len(inv_ui.get_filelist(inv, '')))
+        self.assertEqual(1, len(inv_ui.get_filelist(inv, '', 'filename', 'head')))
+
+    def test_smoke(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        start, content = self.consume_app(inv_ui)
+        self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
+                         start)
+        self.assertContainsRe(content, 'filename')
+
+    def test_no_content_for_HEAD(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        start, content = self.consume_app(inv_ui,
+                            extra_environ={'REQUEST_METHOD': 'HEAD'})
+        self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
+                         start)
+        self.assertEqual('', content)
 
 
 class TestRevisionUI(BasicTests):
@@ -31,8 +74,8 @@
         self.build_tree_contents(shape1)
         tree.smart_add([])
         tree.commit('')
-        tree.smart_add([])
         self.build_tree_contents(shape2)
+        tree.smart_add([])
         tree.commit('')
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
@@ -52,5 +95,33 @@
         rev_ui.args = ['2']
         util.set_context({})
         self.assertIsInstance(
-            rev_ui.get_values('2', {}, []),
+            rev_ui.get_values('', {}, []),
             dict)
+
+
+class TestAnnotateUI(BasicTests):
+
+    def make_annotate_ui_for_file_history(self, file_id, rev_ids_texts):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('filename', '')])
+        tree.add(['filename'], [file_id])
+        for rev_id, text in rev_ids_texts:
+            self.build_tree_contents([('filename', text)])
+            tree.commit(rev_id=rev_id, message='.')
+        tree.branch.lock_read()
+        self.addCleanup(tree.branch.unlock)
+        branch_app = BranchWSGIApp(tree.branch, friendly_name='test_name')
+        return AnnotateUI(branch_app, branch_app.get_history)
+
+    def test_annotate_file(self):
+        history = [('rev1', 'old\nold\n'), ('rev2', 'new\nold\n')]
+        ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
+        # A lot of this state is set up by __call__, but we'll do it directly
+        # here.
+        ann_ui.args = ['rev2']
+        annotate_info = ann_ui.get_values('filename',
+            kwargs={'file_id': 'file_id'}, headers={})
+        annotated = list(annotate_info['annotated'])
+        self.assertEqual(2, len(annotated))
+        self.assertEqual('2', annotated[0].change.revno)
+        self.assertEqual('1', annotated[1].change.revno)

=== modified file 'loggerhead/tests/test_corners.py'
--- loggerhead/tests/test_corners.py	2009-02-10 01:58:31 +0000
+++ loggerhead/tests/test_corners.py	2009-04-02 20:32:12 +0000
@@ -34,6 +34,16 @@
         res = app.get('/revision/'+newrevid)
         res.mustcontain('executable')
 
+    def test_revision_escapes_commit_message(self):
+        """XXX."""
+        self.createBranch()
+
+        msg = '<b>hi</b>'
+        self.addFileAndCommit('myfilename', msg)
+        app = self.setUpLoggerhead()
+        res = app.get('/revision/1')
+        self.assertFalse(msg in res.body)
+
     def test_empty_commit_message(self):
         """Check that an empty commit message does not break the rendering."""
         self.createBranch()
@@ -46,7 +56,7 @@
         res = app.get('/changes')
         # It's not much of an assertion, but we only really care about
         # "assert not crashed".
-        res.mustcontain('myfilename')
+        res.mustcontain('1')
 
     def test_whitespace_only_commit_message(self):
         """Check that a whitespace-only commit message does not break the
@@ -61,4 +71,4 @@
         res = app.get('/changes')
         # It's not much of an assertion, but we only really care about
         # "assert not crashed".
-        res.mustcontain('myfilename')
+        res.mustcontain('1')

=== removed file 'loggerhead/tests/test_filechangecache.py'
--- loggerhead/tests/test_filechangecache.py	2008-10-24 02:26:05 +0000
+++ loggerhead/tests/test_filechangecache.py	1970-01-01 00:00:00 +0000
@@ -1,98 +0,0 @@
-"""Tests for the FileChangeCache in loggerhead.changecache."""
-
-import shutil
-import tempfile
-
-from loggerhead.changecache import FileChangeCache
-
-
-class MockEntry(object):
-
-    def __init__(self, revid):
-        self.revid = revid
-
-
-class MockHistory(object):
-
-    def __init__(self):
-        self.fetched_revids = set()
-
-    def get_file_changes_uncached(self, entries):
-        output = []
-        for entry in entries:
-            self.fetched_revids.add(entry.revid)
-            output.append(entry.revid)
-        return output
-
-
-class TestFileChangeCache(object):
-
-    # setup_method and teardown_method are so i can run the tests with
-    # py.test and take advantage of the error reporting.
-
-    def setup_method(self, meth):
-        self.setUp()
-
-    def teardown_method(self, meth):
-        self.tearDown()
-
-    def setUp(self):
-        self.cache_folders = []
-
-    def tearDown(self):
-        for folder in self.cache_folders:
-            shutil.rmtree(folder)
-
-    def makeHistoryAndEntriesForRevids(self, revids, fill_cache_with=[]):
-        cache_folder = tempfile.mkdtemp()
-        self.cache_folders.append(cache_folder)
-        self.history = MockHistory()
-        self.cache = FileChangeCache(self.history, cache_folder)
-
-        self.entries = [MockEntry(revid) for revid in revids]
-
-        self.cache.get_file_changes([entry for entry in self.entries
-                                     if entry.revid in fill_cache_with])
-        self.history.fetched_revids.clear()
-
-    def test_empty_cache(self):
-        """An empty cache passes all the revids through to the history object.
-        """
-        revids = ['a', 'b']
-        self.makeHistoryAndEntriesForRevids(revids)
-
-        result = self.cache.get_file_changes(self.entries)
-
-        assert result == revids
-        assert self.history.fetched_revids == set(revids)
-
-    def test_full_cache(self):
-        """A full cache passes none of the revids through to the history
-        object.
-        """
-        revids = ['a', 'b']
-        self.makeHistoryAndEntriesForRevids(revids, fill_cache_with=revids)
-
-        result = self.cache.get_file_changes(self.entries)
-
-        assert result == revids
-        assert self.history.fetched_revids == set()
-
-    def test_partial_cache(self):
-        """A partially full cache passes some of the revids through to the
-        history object, and preserves the ordering of the argument list.
-        """
-        # To test the preservation of argument order code, we put the uncached
-        # revid at the beginning, middle and then end of the list of revids
-        # being asked for.
-        for i in range(3):
-            cached_revids = ['a', 'b']
-            revids = cached_revids[:]
-            revids.insert(i, 'uncached')
-            self.makeHistoryAndEntriesForRevids(
-                revids, fill_cache_with=cached_revids)
-
-            result = self.cache.get_file_changes(self.entries)
-            assert result == revids
-
-            assert self.history.fetched_revids == set(['uncached'])

=== added file 'loggerhead/tests/test_history.py'
--- loggerhead/tests/test_history.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_history.py	2011-03-16 14:43:36 +0000
@@ -0,0 +1,293 @@
+# Copyright (C) 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+"""Direct tests of the loggerhead/history.py module"""
+
+from datetime import datetime
+from bzrlib import tests
+
+from loggerhead import history as _mod_history
+
+
+class TestCaseWithExamples(tests.TestCaseWithMemoryTransport):
+
+    def make_linear_ancestry(self):
+        # Time goes up
+        # rev-3
+        #  |
+        # rev-2
+        #  |
+        # rev-1
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('rev-1', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        builder.build_snapshot('rev-2', ['rev-1'], [])
+        builder.build_snapshot('rev-3', ['rev-2'], [])
+        builder.finish_series()
+        b = builder.get_branch()
+        self.addCleanup(b.lock_read().unlock)
+        return _mod_history.History(b, {})
+
+    def make_long_linear_ancestry(self):
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        for r in "BCDEFGHIJKLMNOPQRSTUVWXYZ":
+            builder.build_snapshot(r, None, [])
+        builder.finish_series()
+        b = builder.get_branch()
+        self.addCleanup(b.lock_read().unlock)
+        return _mod_history.History(b, {})
+
+    def make_merged_ancestry(self):
+        # Time goes up
+        # rev-3
+        #  |  \
+        #  |  rev-2
+        #  |  /
+        # rev-1
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('rev-1', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        builder.build_snapshot('rev-2', ['rev-1'], [])
+        builder.build_snapshot('rev-3', ['rev-1', 'rev-2'], [])
+        builder.finish_series()
+        b = builder.get_branch()
+        self.addCleanup(b.lock_read().unlock)
+        return _mod_history.History(b, {})
+
+    def make_deep_merged_ancestry(self):
+        # Time goes up
+        # F
+        # |\
+        # | E
+        # | |\
+        # | | D
+        # | |/
+        # B C
+        # |/
+        # A
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        builder.build_snapshot('B', ['A'], [])
+        builder.build_snapshot('C', ['A'], [])
+        builder.build_snapshot('D', ['C'], [])
+        builder.build_snapshot('E', ['C', 'D'], [])
+        builder.build_snapshot('F', ['B', 'E'], [])
+        builder.finish_series()
+        b = builder.get_branch()
+        self.addCleanup(b.lock_read().unlock)
+        return _mod_history.History(b, {})
+
+    def assertRevidsFrom(self, expected, history, search_revs, tip_rev):
+        self.assertEqual(expected,
+                         list(history.get_revids_from(search_revs, tip_rev)))
+
+
+class _DictProxy(object):
+
+    def __init__(self, d):
+        self._d = d
+        self._accessed = set()
+        self.__setitem__ = d.__setitem__
+
+    def __getitem__(self, name):
+        self._accessed.add(name)
+        return self._d[name]
+
+    def __len__(self):
+        return len(self._d)
+
+
+def track_rev_info_accesses(h):
+    """Track __getitem__ access to History._rev_info,
+
+    :return: set of items accessed
+    """
+    h._rev_info = _DictProxy(h._rev_info)
+    return h._rev_info._accessed
+
+
+class TestHistoryGetRevidsFrom(TestCaseWithExamples):
+
+    def test_get_revids_from_simple_mainline(self):
+        history = self.make_linear_ancestry()
+        self.assertRevidsFrom(['rev-3', 'rev-2', 'rev-1'],
+                              history, None, 'rev-3')
+
+    def test_get_revids_from_merged_mainline(self):
+        history = self.make_merged_ancestry()
+        self.assertRevidsFrom(['rev-3', 'rev-1'],
+                              history, None, 'rev-3')
+
+    def test_get_revids_given_one_rev(self):
+        history = self.make_merged_ancestry()
+        # rev-3 was the first mainline revision to see rev-2.
+        self.assertRevidsFrom(['rev-3'], history, ['rev-2'], 'rev-3')
+
+    def test_get_revids_deep_ancestry(self):
+        history = self.make_deep_merged_ancestry()
+        self.assertRevidsFrom(['F'], history, ['F'], 'F')
+        self.assertRevidsFrom(['F'], history, ['E'], 'F')
+        self.assertRevidsFrom(['F'], history, ['D'], 'F')
+        self.assertRevidsFrom(['F'], history, ['C'], 'F')
+        self.assertRevidsFrom(['B'], history, ['B'], 'F')
+        self.assertRevidsFrom(['A'], history, ['A'], 'F')
+
+    def test_get_revids_doesnt_over_produce_simple_mainline(self):
+        # get_revids_from shouldn't walk the whole ancestry just to get the
+        # answers for the first few revisions.
+        history = self.make_long_linear_ancestry()
+        accessed = track_rev_info_accesses(history)
+        result = history.get_revids_from(None, 'Z')
+        self.assertEqual(set(), accessed)
+        self.assertEqual('Z', result.next())
+        # We already know 'Z' because we passed it in.
+        self.assertEqual(set(), accessed)
+        self.assertEqual('Y', result.next())
+        self.assertEqual(set([history._rev_indices['Z']]), accessed)
+
+    def test_get_revids_doesnt_over_produce_for_merges(self):
+        # get_revids_from shouldn't walk the whole ancestry just to get the
+        # answers for the first few revisions.
+        history = self.make_long_linear_ancestry()
+        accessed = track_rev_info_accesses(history)
+        result = history.get_revids_from(['X', 'V'], 'Z')
+        self.assertEqual(set(), accessed)
+        self.assertEqual('X', result.next())
+        # We access 'W' because we are checking that W wasn't merged into X.
+        # The important bit is that we aren't getting the whole ancestry.
+        self.assertEqual(set([history._rev_indices[x] for x in 'ZYXW']),
+                         accessed)
+        self.assertEqual('V', result.next())
+        self.assertEqual(set([history._rev_indices[x] for x in 'ZYXWVU']),
+                         accessed)
+        self.assertRaises(StopIteration, result.next)
+        self.assertEqual(set([history._rev_indices[x] for x in 'ZYXWVU']),
+                         accessed)
+
+
+
+class TestHistoryChangeFromRevision(tests.TestCaseWithTransport):
+
+    def make_single_commit(self):
+        tree = self.make_branch_and_tree('test')
+        rev_id = tree.commit('Commit Message', timestamp=1299838474.317,
+            timezone=3600, committer='Joe Example <joe@example.com>',
+            revprops={})
+        self.addCleanup(tree.branch.lock_write().unlock)
+        rev = tree.branch.repository.get_revision(rev_id)
+        history = _mod_history.History(tree.branch, {})
+        return history, rev
+
+    def test_simple(self):
+        history, rev = self.make_single_commit()
+        change = history._change_from_revision(rev)
+        self.assertEqual(rev.revision_id, change.revid)
+        self.assertEqual(datetime.fromtimestamp(1299838474.317),
+                         change.date)
+        self.assertEqual(datetime.utcfromtimestamp(1299838474.317),
+                         change.utc_date)
+        self.assertEqual(['Joe Example <joe@example.com>'],
+                         change.authors)
+        self.assertEqual('test', change.branch_nick)
+        self.assertEqual('Commit Message', change.short_comment)
+        self.assertEqual('Commit Message', change.comment)
+        self.assertEqual(['Commit&nbsp;Message'], change.comment_clean)
+        self.assertEqual([], change.parents)
+        self.assertEqual([], change.bugs)
+        self.assertEqual(None, change.tags)
+
+    def test_tags(self):
+        history, rev = self.make_single_commit()
+        b = history._branch
+        b.tags.set_tag('tag-1', rev.revision_id)
+        b.tags.set_tag('tag-2', rev.revision_id)
+        b.tags.set_tag('Tag-10', rev.revision_id)
+        change = history._change_from_revision(rev)
+        # tags are 'naturally' sorted, sorting numbers in order, and ignoring
+        # case, etc.
+        self.assertEqual('tag-1, tag-2, Tag-10', change.tags)
+
+    def test_committer_vs_authors(self):
+        tree = self.make_branch_and_tree('test')
+        rev_id = tree.commit('Commit Message', timestamp=1299838474.317,
+            timezone=3600, committer='Joe Example <joe@example.com>',
+            revprops={'authors': u'A Author <aauthor@example.com>\n'
+                                 u'B Author <bauthor@example.com>'})
+        self.addCleanup(tree.branch.lock_write().unlock)
+        rev = tree.branch.repository.get_revision(rev_id)
+        history = _mod_history.History(tree.branch, {})
+        change = history._change_from_revision(rev)
+        self.assertEqual(u'Joe Example <joe@example.com>',
+                         change.committer)
+        self.assertEqual([u'A Author <aauthor@example.com>',
+                          u'B Author <bauthor@example.com>'],
+                         change.authors)
+
+
+class TestHistory_IterateSufficiently(tests.TestCase):
+
+    def assertIterate(self, expected, iterable, stop_at, extra_rev_count):
+        self.assertEqual(expected, _mod_history.History._iterate_sufficiently(
+            iterable, stop_at, extra_rev_count))
+
+    def test_iter_no_extra(self):
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(['a', 'b', 'c'], iter(lst), 'c', 0)
+        self.assertIterate(['a', 'b', 'c', 'd'], iter(lst), 'd', 0)
+
+    def test_iter_not_found(self):
+        # If the key in question isn't found, we just exhaust the list
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(lst, iter(lst), 'not-there', 0)
+
+    def test_iter_with_extra(self):
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(['a', 'b', 'c'], iter(lst), 'b', 1)
+        self.assertIterate(['a', 'b', 'c', 'd', 'e'], iter(lst), 'c', 2)
+
+    def test_iter_with_too_many_extra(self):
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(lst, iter(lst), 'y', 10)
+        self.assertIterate(lst, iter(lst), 'z', 10)
+
+    def test_iter_with_extra_None(self):
+        lst = list('abcdefghijklmnopqrstuvwxyz')
+        self.assertIterate(lst, iter(lst), 'z', None)
+
+
+
+class TestHistoryGetView(TestCaseWithExamples):
+
+    def test_get_view_limited_history(self):
+        # get_view should only load enough history to serve the result, not all
+        # history.
+        history = self.make_long_linear_ancestry()
+        accessed = track_rev_info_accesses(history)
+        revid, start_revid, revid_list = history.get_view('Z', 'Z', None,
+                                                      extra_rev_count=5)
+        self.assertEqual(['Z', 'Y', 'X', 'W', 'V', 'U'], revid_list)
+        self.assertEqual('Z', revid)
+        self.assertEqual('Z', start_revid)
+        self.assertEqual(set([history._rev_indices[x] for x in 'ZYXWVU']),
+                         accessed)

=== added file 'loggerhead/tests/test_http_head.py'
--- loggerhead/tests/test_http_head.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_http_head.py	2011-02-10 02:33:15 +0000
@@ -0,0 +1,92 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the HeadMiddleware app."""
+
+from cStringIO import StringIO
+
+from bzrlib import tests
+
+from loggerhead.apps import http_head
+
+
+content = ["<html>",
+           "<head><title>Listed</title></head>",
+           "<body>Content</body>",
+           "</html>",
+          ]
+headers = {'X-Ignored-Header': 'Value'}
+
+def yielding_app(environ, start_response):
+    writer = start_response('200 OK', headers)
+    for chunk in content:
+        yield chunk
+
+
+def list_app(environ, start_response):
+    writer = start_response('200 OK', headers)
+    return content
+
+
+def writer_app(environ, start_response):
+    writer = start_response('200 OK', headers)
+    for chunk in content:
+        writer(chunk)
+    return []
+
+
+class TestHeadMiddleware(tests.TestCase):
+
+    def _trap_start_response(self, status, response_headers, exc_info=None):
+        self._write_buffer = StringIO()
+        self._start_response_passed = (status, response_headers, exc_info)
+        return self._write_buffer.write
+
+    def _consume_app(self, app, request_method):
+        environ = {'REQUEST_METHOD': request_method}
+        value = list(app(environ, self._trap_start_response))
+        self._write_buffer.writelines(value)
+
+    def _verify_get_passthrough(self, app):
+        app = http_head.HeadMiddleware(app)
+        self._consume_app(app, 'GET')
+        self.assertEqual(('200 OK', headers, None), self._start_response_passed)
+        self.assertEqualDiff(''.join(content), self._write_buffer.getvalue())
+
+    def _verify_head_no_body(self, app):
+        app = http_head.HeadMiddleware(app)
+        self._consume_app(app, 'HEAD')
+        self.assertEqual(('200 OK', headers, None), self._start_response_passed)
+        self.assertEqualDiff('', self._write_buffer.getvalue())
+
+    def test_get_passthrough_yielding(self):
+        self._verify_get_passthrough(yielding_app)
+
+    def test_head_passthrough_yielding(self):
+        self._verify_head_no_body(yielding_app)
+
+    def test_get_passthrough_list(self):
+        self._verify_get_passthrough(list_app)
+
+    def test_head_passthrough_list(self):
+        self._verify_head_no_body(list_app)
+
+    def test_get_passthrough_writer(self):
+        self._verify_get_passthrough(writer_app)
+
+    def test_head_passthrough_writer(self):
+        self._verify_head_no_body(writer_app)
+

=== added file 'loggerhead/tests/test_load_test.py'
--- loggerhead/tests/test_load_test.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_load_test.py	2011-02-10 00:49:06 +0000
@@ -0,0 +1,337 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#
+
+"""Tests for the load testing code."""
+
+import socket
+import time
+import threading
+import Queue
+
+from bzrlib import tests
+from bzrlib.tests import http_server
+
+from bzrlib.plugins.loggerhead.loggerhead import load_test
+
+
+empty_script = """{
+    "parameters": {},
+    "requests": []
+}"""
+
+class TestRequestDescription(tests.TestCase):
+
+    def test_init_from_dict(self):
+        rd = load_test.RequestDescription({'thread': '10', 'relpath': '/foo'})
+        self.assertEqual('10', rd.thread)
+        self.assertEqual('/foo', rd.relpath)
+
+    def test_default_thread_is_1(self):
+        rd = load_test.RequestDescription({'relpath': '/bar'})
+        self.assertEqual('1', rd.thread)
+        self.assertEqual('/bar', rd.relpath)
+
+
+_cur_time = time.time()
+def one_sec_timer():
+    """Every time this timer is called, it increments by 1 second."""
+    global _cur_time
+    _cur_time += 1.0
+    return _cur_time
+
+
+class NoopRequestWorker(load_test.RequestWorker):
+
+    # Every call to _timer will increment by one
+    _timer = staticmethod(one_sec_timer)
+
+    # Ensure that process never does anything
+    def process(self, url):
+        return True
+
+
+class TestRequestWorkerInfrastructure(tests.TestCase):
+    """Tests various infrastructure bits, without doing actual requests."""
+
+    def test_step_next_tracks_time(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        self.assertTrue(rt.queue.empty())
+        self.assertEqual([('item', True, 1.0)], rt.stats)
+
+    def test_step_multiple_items(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        rt.queue.put('next-item')
+        rt.step_next()
+        self.assertTrue(rt.queue.empty())
+        self.assertEqual([('item', True, 1.0), ('next-item', True, 1.0)],
+                         rt.stats)
+
+    def test_step_next_does_nothing_for_noop(self):
+        rt = NoopRequestWorker('id')
+        rt.queue.put('item')
+        rt.step_next()
+        rt.queue.put('<noop>')
+        rt.step_next()
+        self.assertEqual([('item', True, 1.0)], rt.stats)
+
+    def test_step_next_will_timeout(self):
+        # We don't want step_next to block forever
+        rt = NoopRequestWorker('id', blocking_time=0.001)
+        self.assertRaises(Queue.Empty, rt.step_next)
+
+    def test_run_stops_for_stop_event(self):
+        rt = NoopRequestWorker('id', blocking_time=0.001, _queue_size=2)
+        rt.queue.put('item1')
+        rt.queue.put('item2')
+        event = threading.Event()
+        t = threading.Thread(target=rt.run, args=(event,))
+        t.start()
+        # Wait for the queue to be processed
+        rt.queue.join()
+        # Now we can queue up another one, and wait for it
+        rt.queue.put('item3')
+        rt.queue.join()
+        # Now set the stopping event
+        event.set()
+        # Add another item to the queue, which might get processed, but the
+        # next item won't
+        rt.queue.put('item4')
+        rt.queue.put('item5')
+        t.join()
+        self.assertEqual([('item1', True, 1.0), ('item2', True, 1.0),
+                          ('item3', True, 1.0)],
+                         rt.stats[:3])
+        # The last event might be item4 or might be item3, the important thing
+        # is that even though there are still queued events, we won't
+        # process anything past the first
+        self.assertNotEqual('item5', rt.stats[-1][0])
+
+
+class TestRequestWorker(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestRequestWorker, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_request_items(self):
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        self.build_tree(['file1', 'file2'])
+        readonly_url1 = self.get_readonly_url('file1')
+        self.assertStartsWith(readonly_url1, 'http://')
+        readonly_url2 = self.get_readonly_url('file2')
+        rt.queue.put(readonly_url1)
+        rt.queue.put(readonly_url2)
+        rt.step_next()
+        rt.step_next()
+        self.assertEqual(readonly_url1, rt.stats[0][0])
+        self.assertEqual(readonly_url2, rt.stats[1][0])
+
+    def test_request_nonexistant_items(self):
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        readonly_url1 = self.get_readonly_url('no-file1')
+        rt.queue.put(readonly_url1)
+        rt.step_next()
+        self.assertEqual(readonly_url1, rt.stats[0][0])
+        self.assertEqual(False, rt.stats[0][1])
+
+    def test_no_server(self):
+        s = socket.socket()
+        # Bind to a port, but don't listen on it
+        s.bind(('localhost', 0))
+        url = 'http://%s:%s/path' % s.getsockname()
+        rt = load_test.RequestWorker('id', blocking_time=0.01, _queue_size=2)
+        rt.queue.put(url)
+        rt.step_next()
+        self.assertEqual((url, False), rt.stats[0][:2])
+
+
+class NoActionScript(load_test.ActionScript):
+
+    _thread_class = NoopRequestWorker
+    _default_blocking_timeout = 0.01
+
+
+class TestActionScriptInfrastructure(tests.TestCase):
+
+    def test_parse_requires_parameters_and_requests(self):
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{}')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{"parameters": {}}')
+        self.assertRaises(ValueError,
+            load_test.ActionScript.parse, '{"requests": []}')
+        load_test.ActionScript.parse(
+            '{"parameters": {}, "requests": [], "comment": "section"}')
+        script = load_test.ActionScript.parse(
+            empty_script)
+        self.assertIsNot(None, script)
+
+    def test_parse_default_base_url(self):
+        script = load_test.ActionScript.parse(empty_script)
+        self.assertEqual('http://localhost:8080', script.base_url)
+
+    def test_parse_find_base_url(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {"base_url": "http://example.com"},'
+            ' "requests": []}')
+        self.assertEqual('http://example.com', script.base_url)
+
+    def test_parse_default_blocking_timeout(self):
+        script = load_test.ActionScript.parse(empty_script)
+        self.assertEqual(60.0, script.blocking_timeout)
+
+    def test_parse_find_blocking_timeout(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {"blocking_timeout": 10.0},'
+            ' "requests": []}'
+        )
+        self.assertEqual(10.0, script.blocking_timeout)
+
+    def test_parse_finds_requests(self):
+        script = load_test.ActionScript.parse(
+            '{"parameters": {}, "requests": ['
+            ' {"relpath": "/foo"},'
+            ' {"relpath": "/bar"}'
+            ' ]}')
+        self.assertEqual(2, len(script._requests))
+        self.assertEqual("/foo", script._requests[0].relpath)
+        self.assertEqual("/bar", script._requests[1].relpath)
+
+    def test__get_worker(self):
+        script = NoActionScript()
+        # If an id is found, then we should create it
+        self.assertEqual({}, script._threads)
+        worker = script._get_worker('id')
+        self.assertTrue('id' in script._threads)
+        # We should have set the blocking timeout
+        self.assertEqual(script.blocking_timeout / 10.0,
+                         worker.blocking_time)
+
+        # Another request will return the identical object
+        self.assertIs(worker, script._get_worker('id'))
+
+        # And the stop event will stop the thread
+        script.stop_and_join()
+
+    def test__full_url(self):
+        script = NoActionScript()
+        self.assertEqual('http://localhost:8080/path',
+                         script._full_url('/path'))
+        self.assertEqual('http://localhost:8080/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com'
+        self.assertEqual('http://example.com/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com/base'
+        self.assertEqual('http://example.com/base/path/to/foo',
+                         script._full_url('/path/to/foo'))
+        script.base_url = 'http://example.com'
+        self.assertEqual('http://example.com:8080/path',
+                         script._full_url(':8080/path'))
+
+    def test_single_threaded(self):
+        script = NoActionScript.parse("""{
+            "parameters": {"base_url": ""},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "1", "relpath": "second"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "1", "relpath": "fourth"}
+            ]}""")
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual(["first", "second", "third", "fourth"],
+                         [s[0] for s in worker.stats])
+
+    def test_two_threads(self):
+        script = NoActionScript.parse("""{
+            "parameters": {"base_url": ""},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "2", "relpath": "second"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "2", "relpath": "fourth"}
+            ]}""")
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual(["first", "third"],
+                         [s[0] for s in worker.stats])
+        worker = script._get_worker("2")
+        self.assertEqual(["second", "fourth"],
+                         [s[0] for s in worker.stats])
+
+
+class TestActionScriptIntegration(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestActionScriptIntegration, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_full_integration(self):
+        self.build_tree(['first', 'second', 'third', 'fourth'])
+        url = self.get_readonly_url()
+        script = load_test.ActionScript.parse("""{
+            "parameters": {"base_url": "%s", "blocking_timeout": 0.1},
+            "requests": [
+                {"thread": "1", "relpath": "first"},
+                {"thread": "2", "relpath": "second"},
+                {"thread": "1", "relpath": "no-this"},
+                {"thread": "2", "relpath": "or-this"},
+                {"thread": "1", "relpath": "third"},
+                {"thread": "2", "relpath": "fourth"}
+            ]}""" % (url,))
+        script.run()
+        worker = script._get_worker("1")
+        self.assertEqual([("first", True), ('no-this', False),
+                          ("third", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+        worker = script._get_worker("2")
+        self.assertEqual([("second", True), ('or-this', False),
+                          ("fourth", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+
+
+class TestRunScript(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestRunScript, self).setUp()
+        self.transport_readonly_server = http_server.HttpServer
+
+    def test_run_script(self):
+        self.build_tree(['file1', 'file2', 'file3', 'file4'])
+        url = self.get_readonly_url()
+        self.build_tree_contents([('localhost.script', """{
+    "parameters": {"base_url": "%s", "blocking_timeout": 0.1},
+    "requests": [
+        {"thread": "1", "relpath": "file1"},
+        {"thread": "2", "relpath": "file2"},
+        {"thread": "1", "relpath": "file3"},
+        {"thread": "2", "relpath": "file4"}
+    ]
+}""" % (url,))])
+        script = load_test.run_script('localhost.script')
+        worker = script._threads["1"][0]
+        self.assertEqual([("file1", True), ('file3', True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])
+        worker = script._threads["2"][0]
+        self.assertEqual([("file2", True), ("file4", True)],
+                         [(s[0].rsplit('/', 1)[1], s[1])
+                          for s in worker.stats])

=== added file 'loggerhead/tests/test_revision_ui.py'
--- loggerhead/tests/test_revision_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_revision_ui.py	2011-03-16 12:37:17 +0000
@@ -0,0 +1,46 @@
+# Copyright (C) 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+
+from loggerhead.tests.test_simple import BasicTests
+
+
+class TestRevisionUI(BasicTests):
+
+    def test_authors_vs_committer(self):
+        self.createBranch()
+        self.tree.commit('First', committer="Joe Example <joe@example.com>",
+            revprops={'authors': u'A Author <aauthor@example.com>\n'
+                                 u'B Author <bauthor@example.com>'})
+        app = self.setUpLoggerhead()
+        res = app.get('/revision/1')
+        # We would like to assert that Joe Example is connected to Committer,
+        # and the Authors are connected. However, that requires asserting the
+        # exact HTML connections, which I wanted to avoid.
+        res.mustcontain('Committer', 'Joe Example',
+                        'Author(s)', 'A Author, B Author')
+
+    def test_author_is_committer(self):
+        self.createBranch()
+        self.tree.commit('First', committer="Joe Example <joe@example.com>")
+        app = self.setUpLoggerhead()
+        res = app.get('/revision/1')
+        # We would like to assert that Joe Example is connected to Committer,
+        # and the Authors are connected. However, that requires asserting the
+        # exact HTML connections, which I wanted to avoid.
+        res.mustcontain('Committer', 'Joe Example')
+        self.assertFalse('Author(s)' in res.body)

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2009-02-18 10:40:22 +0000
+++ loggerhead/tests/test_simple.py	2011-03-23 03:36:23 +0000
@@ -1,21 +1,36 @@
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
 import cgi
 import logging
+import re
 
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.util.configobj.configobj import ConfigObj
+try:
+    from bzrlib.util.configobj.configobj import ConfigObj
+except ImportError:
+    from configobj import ConfigObj
+from bzrlib import config
 
 from loggerhead.apps.branch import BranchWSGIApp
+from loggerhead.apps.http_head import HeadMiddleware
 from paste.fixture import TestApp
-from paste.httpexceptions import HTTPExceptionHandler
-
-
-
-def test_config_root():
-    from loggerhead.apps.config import Root
-    config = ConfigObj()
-    app = TestApp(HTTPExceptionHandler(Root(config)))
-    res = app.get('/')
-    res.mustcontain('loggerhead branches')
+from paste.httpexceptions import HTTPExceptionHandler, HTTPMovedPermanently
+
 
 
 class BasicTests(TestCaseWithTransport):
@@ -41,9 +56,11 @@
 
         self.filecontents = ('some\nmultiline\ndata\n'
                              'with<htmlspecialchars\n')
+        filenames = ['myfilename', 'anotherfile<']
         self.build_tree_contents(
-            [('myfilename', self.filecontents)])
-        self.tree.add('myfilename')
+            (filename, self.filecontents) for filename in filenames)
+        for filename in filenames:
+            self.tree.add(filename, '%s-id' % filename)
         self.fileid = self.tree.path2id('myfilename')
         self.msg = 'a very exciting commit message <'
         self.revid = self.tree.commit(message=self.msg)
@@ -53,6 +70,11 @@
         res = app.get('/changes')
         res.mustcontain(cgi.escape(self.msg))
 
+    def test_changes_for_file(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes?filter_file_id=myfilename-id')
+        res.mustcontain(cgi.escape(self.msg))
+
     def test_changes_branch_from(self):
         app = self.setUpLoggerhead(served_url="lp:loggerhead")
         res = app.get('/changes')
@@ -70,8 +92,20 @@
     def test_annotate(self):
         app = self.setUpLoggerhead()
         res = app.get('/annotate', params={'file_id': self.fileid})
+        # If pygments is installed, it inserts <span class="pyg" content into
+        # the output, to trigger highlighting. And it specifically highlights
+        # the &lt; that we are interested in seeing in the output.
+        # Without pygments we have a simple: 'with&lt;htmlspecialchars'
+        # With it, we have
+        # '<span class='pyg-n'>with</span><span class='pyg-o'>&lt;</span>'
+        # '<span class='pyg-n'>htmlspecialchars</span>
+        # So we pre-filter the body, to make sure remove spans of that type.
+        body_no_span = re.sub(r'<span class="pyg-.">', '', res.body)
+        body_no_span = body_no_span.replace('</span>', '')
         for line in self.filecontents.splitlines():
-            res.mustcontain(cgi.escape(line))
+            escaped = cgi.escape(line)
+            self.assertTrue(escaped in body_no_span,
+                            "did not find %r in %r" % (escaped, body_no_span))
 
     def test_inventory(self):
         app = self.setUpLoggerhead()
@@ -99,10 +133,13 @@
     def test_revision(self):
         app = self.setUpLoggerhead()
         res = app.get('/revision/1')
+        res.mustcontain(no=['anotherfile<'])
+        res.mustcontain('anotherfile&lt;')
         res.mustcontain('myfilename')
 
 
 class TestEmptyBranch(BasicTests):
+    """Test that an empty branch doesn't break"""
 
     def setUp(self):
         BasicTests.setUp(self)
@@ -118,3 +155,90 @@
         res = app.get('/files')
         res.mustcontain('No revisions!')
 
+
+class TestHiddenBranch(BasicTests):
+    """
+    Test that hidden branches aren't shown
+    FIXME: not tested that it doesn't show up on listings
+    """
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+        locations = config.locations_config_filename()
+        config.ensure_config_dir_exists()
+        open(locations, 'wb').write('[%s]\nhttp_serve = False'
+                                    % (self.tree.branch.base,))
+
+    def test_no_access(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes', status=404)
+
+
+class TestControllerRedirects(BasicTests):
+    """
+    Test that a file under /files redirects to /view,
+    and a directory under /view redirects to /files.
+    """
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+        self.build_tree(('file', 'folder/', 'folder/file'))
+        self.tree.smart_add([])
+        self.tree.commit('')
+
+    def test_view_folder(self):
+        app = TestApp(BranchWSGIApp(self.tree.branch, '').app)
+
+        e = self.assertRaises(HTTPMovedPermanently, app.get, '/view/head:/folder')
+        self.assertEqual(e.location(), '/files/head:/folder')
+
+    def test_files_file(self):
+        app = TestApp(BranchWSGIApp(self.tree.branch, '').app)
+
+        e = self.assertRaises(HTTPMovedPermanently, app.get, '/files/head:/folder/file')
+        self.assertEqual(e.location(), '/view/head:/folder/file')
+        e = self.assertRaises(HTTPMovedPermanently, app.get, '/files/head:/file')
+        self.assertEqual(e.location(), '/view/head:/file')
+
+
+class TestHeadMiddleware(BasicTests):
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+        self.msg = 'trivial commit message'
+        self.revid = self.tree.commit(message=self.msg)
+
+    def setUpLoggerhead(self, **kw):
+        branch_app = BranchWSGIApp(self.tree.branch, '', **kw).app
+        return TestApp(HTTPExceptionHandler(HeadMiddleware(branch_app)))
+
+    def test_get(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes')
+        res.mustcontain(self.msg)
+        self.assertEqual('text/html', res.header('Content-Type'))
+
+    def test_head(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/changes', extra_environ={'REQUEST_METHOD': 'HEAD'})
+        self.assertEqual('text/html', res.header('Content-Type'))
+        self.assertEqualDiff('', res.body)
+
+
+#class TestGlobalConfig(BasicTests):
+#    """
+#    Test that global config settings are respected
+#    """
+
+#    def setUp(self):
+#        BasicTests.setUp(self)
+#        self.createBranch()
+#        config.GlobalConfig().set_user_option('http_version', 'True')
+
+#    def test_setting_respected(self):
+        #FIXME: Figure out how to test this properly
+#        app = self.setUpLoggerhead()
+#        res = app.get('/changes', status=200)

=== added file 'loggerhead/tests/test_util.py'
--- loggerhead/tests/test_util.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_util.py	2011-03-23 04:27:37 +0000
@@ -0,0 +1,33 @@
+# Copyright 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib import tests
+
+from loggerhead.util import html_escape, html_format
+
+
+class TestHTMLEscaping(tests.TestCase):
+
+    def test_html_escape(self):
+        self.assertEqual(
+            "foo &quot;&#39;&lt;&gt;&amp;",
+            html_escape("foo \"'<>&"))
+
+    def test_html_format(self):
+        self.assertEqual(
+            '<foo bar="baz&quot;&#39;">&lt;baz&gt;&amp;</foo>',
+            html_format(
+                '<foo bar="%s">%s</foo>', "baz\"'", "<baz>&"))

=== removed file 'loggerhead/trace.py'
--- loggerhead/trace.py	2008-10-24 02:26:05 +0000
+++ loggerhead/trace.py	1970-01-01 00:00:00 +0000
@@ -1,75 +0,0 @@
-#
-# Copyright (C) 2008  Canonical Ltd.
-#                     (Authored by Martin Albisetti <argentina@gmail.com)
-# Copyright (C) 2008  Guillermo Gonzalez <guillo.gonzo@gmail.com>
-# Copyright (C) 2006  Robey Pointer <robey@lag.net>
-# Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
-
-import os
-import logging
-import sys
-
-
-def make_handler(config, filename):
-    roll = config.get('log.roll', 'never')
-    if roll == 'daily':
-        h = logging.handlers.TimedRotatingFileHandler(filename,
-                                                      'midnight',
-                                                      1,
-                                                      100)
-    elif roll == 'weekly':
-        h = logging.handlers.TimedRotatingFileHandler(filename, 'W0', 1, 100)
-    else:
-        h = logging.FileHandler(filename)
-    return h
-
-
-def setup_logging(log_folder, config, foreground):
-    # i hate that stupid logging config format, so just set up logging here.
-
-    if not os.path.exists(log_folder):
-        os.mkdir(log_folder)
-
-    f = logging.Formatter('%(levelname)-.3s [%(asctime)s.%(msecs)03d]'
-                          ' %(name)s: %(message)s',
-                          '%Y%m%d-%H:%M:%S')
-    debug_log = make_handler(config, os.path.join(log_folder, 'debug.log'))
-    debug_log.setLevel(logging.DEBUG)
-    debug_log.setFormatter(f)
-    if foreground:
-        stdout_log = logging.StreamHandler(sys.stdout)
-        stdout_log.setLevel(logging.DEBUG)
-        stdout_log.setFormatter(f)
-    f = logging.Formatter('[%(asctime)s.%(msecs)03d] %(message)s',
-                          '%Y%m%d-%H:%M:%S')
-    access_log = make_handler(config, os.path.join(log_folder, 'access.log'))
-    access_log.setLevel(logging.INFO)
-    access_log.setFormatter(f)
-
-    error_log = make_handler(config, os.path.join(log_folder, 'error.log'))
-    error_log.setLevel(logging.ERROR)
-    error_log.setFormatter(f)
-
-    logging.getLogger('').setLevel(logging.DEBUG)
-    logging.getLogger('').addHandler(debug_log)
-    logging.getLogger('loggerhead.access').addHandler(access_log)
-    logging.getLogger('loggerhead').setLevel(logging.ERROR)
-    logging.getLogger('loggerhead').addHandler(error_log)
-
-    if foreground:
-        logging.getLogger('').addHandler(stdout_log)

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-10-24 02:26:05 +0000
+++ loggerhead/util.py	2011-03-23 05:21:34 +0000
@@ -19,13 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-try:
-    from xml.etree import ElementTree as ET
-except ImportError:
-    from elementtree import ElementTree as ET
-
 import base64
-import cgi
 import datetime
 import logging
 import re
@@ -36,6 +30,15 @@
 import os
 import subprocess
 
+try:
+    from xml.etree import ElementTree as ET
+except ImportError:
+    from elementtree import ElementTree as ET
+
+from bzrlib import urlutils
+
+from simpletal.simpleTALUtils import HTMLStructureCleaner
+
 log = logging.getLogger("loggerhead.controllers")
 
 
@@ -64,7 +67,7 @@
 
 def date_time(value):
     if value is not None:
-        return value.strftime('%Y-%m-%d %T')
+        return value.strftime('%Y-%m-%d %H:%M:%S')
     else:
         return 'N/A'
 
@@ -127,12 +130,13 @@
     return _wrap_with_date_time_title(date, _displaydate(date))
 
 
-class Container (object):
+class Container(object):
     """
     Convert a dict into an object with attributes.
     """
 
     def __init__(self, _dict=None, **kw):
+        self._properties = {}
         if _dict is not None:
             for key, value in _dict.iteritems():
                 setattr(self, key, value)
@@ -149,6 +153,24 @@
         out += '}'
         return out
 
+    def __getattr__(self, attr):
+        """Used for handling things that aren't already available."""
+        if attr.startswith('_') or attr not in self._properties:
+            raise AttributeError('No attribute: %s' % (attr,))
+        val = self._properties[attr](self, attr)
+        setattr(self, attr, val)
+        return val
+
+    def _set_property(self, attr, prop_func):
+        """Set a function that will be called when an attribute is desired.
+
+        We will cache the return value, so the function call should be
+        idempotent. We will pass 'self' and the 'attr' name when triggered.
+        """
+        if attr.startswith('_'):
+            raise ValueError("Cannot create properties that start with _")
+        self._properties[attr] = prop_func
+
 
 def trunc(text, limit=10):
     if len(text) <= limit:
@@ -179,20 +201,59 @@
         return '%s at %s' % (username, domains[-2])
     return '%s at %s' % (username, domains[0])
 
+def hide_emails(emails):
+    """
+    try to obscure any email address in a list of bazaar committers' names.
+    """
+    result = []
+    for email in emails:
+        result.append(hide_email(email))
+    return result
 
 # only do this if unicode turns out to be a problem
 #_BADCHARS_RE = re.compile(ur'[\u007f-\uffff]')
 
+# Can't be a dict; &amp; needs to be done first.
+html_entity_subs = [
+    ("&", "&amp;"),
+    ('"', "&quot;"),
+    ("'", "&#39;"), # &apos; is defined in XML, but not HTML.
+    (">", "&gt;"),
+    ("<", "&lt;"),
+    ]
+
+
+def html_escape(s):
+    """Transform dangerous (X)HTML characters into entities.
+
+    Like cgi.escape, except also escaping " and '. This makes it safe to use
+    in both attribute and element content.
+
+    If you want to safely fill a format string with escaped values, use
+    html_format instead
+    """
+    for char, repl in html_entity_subs:
+        s = s.replace(char, repl)
+    return s
+
+
+def html_format(template, *args):
+    """Safely format an HTML template string, escaping the arguments.
+
+    The template string must not be user-controlled; it will not be escaped.
+    """
+    return template % tuple(html_escape(arg) for arg in args)
+
+
 # FIXME: get rid of this method; use fixed_width() and avoid XML().
 
-
 def html_clean(s):
     """
     clean up a string for html display.  expand any tabs, encode any html
     entities, and replace spaces with '&nbsp;'.  this is primarily for use
     in displaying monospace text.
     """
-    s = cgi.escape(s.expandtabs())
+    s = html_escape(s.expandtabs())
     s = s.replace(' ', '&nbsp;')
     return s
 
@@ -207,8 +268,6 @@
 
     return: the same value recieved if not empty, and a '&nbsp;' if it is.
     """
-
-
     if s is None:
         return '&nbsp;'
     elif isinstance(s, int):
@@ -222,6 +281,7 @@
             s = s.decode('iso-8859-15')
         return s
 
+HSC = HTMLStructureCleaner()
 
 def fixed_width(s):
     """
@@ -238,7 +298,10 @@
             s = s.decode('utf-8')
         except UnicodeDecodeError:
             s = s.decode('iso-8859-15')
-    return s.expandtabs().replace(' ', NONBREAKING_SPACE)
+
+    s = html_escape(s).expandtabs().replace(' ', NONBREAKING_SPACE)
+
+    return HSC.clean(s).replace('\n', '<br/>')
 
 
 def fake_permissions(kind, executable):
@@ -302,7 +365,7 @@
 
     out = str(base)
     if (base < 100) and (dot != 0):
-        out += '.%d' % (dot)
+        out += '.%d' % (dot,)
     if divisor == KILO:
         out += 'K'
     elif divisor == MEG:
@@ -312,6 +375,14 @@
     return out
 
 
+def local_path_from_url(url):
+    """Convert Bazaar URL to local path, ignoring readonly+ prefix"""
+    readonly_prefix = 'readonly+'
+    if url.startswith(readonly_prefix):
+        url = url[len(readonly_prefix):]
+    return urlutils.local_path_from_url(url)
+
+
 def fill_in_navigation(navigation):
     """
     given a navigation block (used by the template for the page header), fill
@@ -370,8 +441,6 @@
     """
     # Is our root directory itself a branch?
     if is_root:
-        if view == 'directory':
-            directory = 'files'
         breadcrumbs = [{
             'dir_name': path,
             'path': '',
@@ -435,25 +504,6 @@
     return new_decorator
 
 
-# common threading-lock decorator
-
-
-def with_lock(lockname, debug_name=None):
-    if debug_name is None:
-        debug_name = lockname
-
-    @decorator
-    def _decorator(unbound):
-
-        def locked(self, *args, **kw):
-            getattr(self, lockname).acquire()
-            try:
-                return unbound(self, *args, **kw)
-            finally:
-                getattr(self, lockname).release()
-        return locked
-    return _decorator
-
 
 @decorator
 def lsprof(f):
@@ -470,7 +520,7 @@
         now = time.time()
         msec = int(now * 1000) % 1000
         timestr = time.strftime('%Y%m%d%H%M%S',
-                                time.localtime(now)) + ('%03d' % msec)
+                                time.localtime(now)) + ('%03d' % (msec,))
         filename = f.__name__ + '-' + timestr + '.lsprof'
         cPickle.dump(stats, open(filename, 'w'), 2)
         return ret
@@ -542,7 +592,7 @@
     _reloader_environ_key = 'PYTHON_RELOADER_SHOULD_RUN'
 
     @classmethod
-    def _turn_sigterm_into_systemexit(self):
+    def _turn_sigterm_into_systemexit(cls):
         """
         Attempts to turn a SIGTERM exception into a SystemExit exception.
         """
@@ -556,26 +606,26 @@
         signal.signal(signal.SIGTERM, handle_term)
 
     @classmethod
-    def is_installed(self):
-        return os.environ.get(self._reloader_environ_key)
+    def is_installed(cls):
+        return os.environ.get(cls._reloader_environ_key)
 
     @classmethod
-    def install(self):
+    def install(cls):
         from paste import reloader
         reloader.install(int(1))
 
     @classmethod
-    def restart_with_reloader(self):
+    def restart_with_reloader(cls):
         """Based on restart_with_monitor from paste.script.serve."""
         print 'Starting subprocess with file monitor'
-        while 1:
+        while True:
             args = [sys.executable] + sys.argv
             new_environ = os.environ.copy()
-            new_environ[self._reloader_environ_key] = 'true'
+            new_environ[cls._reloader_environ_key] = 'true'
             proc = None
             try:
                 try:
-                    self._turn_sigterm_into_systemexit()
+                    cls._turn_sigterm_into_systemexit()
                     proc = subprocess.Popen(args, env=new_environ)
                     exit_code = proc.wait()
                     proc = None
@@ -584,7 +634,7 @@
                     return 1
             finally:
                 if (proc is not None
-                    and hasattr(os, 'kill')):
+                    and getattr(os, 'kill', None) is not None):
                     import signal
                     try:
                         os.kill(proc.pid, signal.SIGTERM)
@@ -596,3 +646,20 @@
             if exit_code != 3:
                 return exit_code
             print '-'*20, 'Restarting', '-'*20
+
+
+def convert_file_errors(application):
+    """WSGI wrapper to convert some file errors to Paste exceptions"""
+    def new_application(environ, start_response):
+        try:
+            return application(environ, start_response)
+        except (IOError, OSError), e:
+            import errno
+            from paste import httpexceptions
+            if e.errno == errno.ENOENT:
+                raise httpexceptions.HTTPNotFound()
+            elif e.errno == errno.EACCES:
+                raise httpexceptions.HTTPForbidden()
+            else:
+                raise
+    return new_application

=== modified file 'loggerhead/wholehistory.py'
--- loggerhead/wholehistory.py	2008-12-05 20:09:14 +0000
+++ loggerhead/wholehistory.py	2010-04-26 19:37:26 +0000
@@ -1,4 +1,5 @@
-# Cache the whole history data needed by loggerhead about a branch.
+#
+# Copyright (C) 2008, 2009 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,6 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
+"""Cache the whole history data needed by loggerhead about a branch."""
 
 import logging
 import time
@@ -30,50 +32,54 @@
     # Filter ghosts, and null:
     if NULL_REVISION in revision_graph:
         del revision_graph[NULL_REVISION]
-    for key, parents in revision_graph.items():
+    for key, parents in revision_graph.iteritems():
         revision_graph[key] = tuple(parent for parent in parents if parent
             in revision_graph)
     return revision_graph
 
 
 def compute_whole_history_data(branch):
+    """Compute _rev_info and _rev_indices for a branch.
+
+    See History.__doc__ for what these data structures mean.
+    """
     z = time.time()
 
     last_revid = branch.last_revision()
 
     log = logging.getLogger('loggerhead.%s' %
-                            branch.get_config().get_nickname())
+                            (branch.get_config().get_nickname(),))
 
     graph = branch.repository.get_graph()
-    parent_map = dict(((key, value) for key, value in
-         graph.iter_ancestry([last_revid]) if value is not None))
+    parent_map = dict((key, value) for key, value in
+        graph.iter_ancestry([last_revid]) if value is not None)
 
     _revision_graph = _strip_NULL_ghosts(parent_map)
-    _full_history = []
-    _revision_info = {}
-    _revno_revid = {}
+
+    _rev_info = []
+    _rev_indices = {}
+
     if is_null(last_revid):
         _merge_sort = []
     else:
         _merge_sort = merge_sort(
             _revision_graph, last_revid, generate_revno=True)
 
-    for (seq, revid, merge_depth, revno, end_of_merge) in _merge_sort:
-        _full_history.append(revid)
+    for info in _merge_sort:
+        seq, revid, merge_depth, revno, end_of_merge = info
         revno_str = '.'.join(str(n) for n in revno)
-        _revno_revid[revno_str] = revid
-        _revision_info[revid] = (
-            seq, revid, merge_depth, revno_str, end_of_merge)
-
-    _where_merged = {}
-
-    for revid in _revision_graph.keys():
-        if _revision_info[revid][2] == 0:
+        parents = _revision_graph[revid]
+        _rev_indices[revid] = len(_rev_info)
+        _rev_info.append([(seq, revid, merge_depth, revno_str, end_of_merge), (), parents])
+
+    for revid in _revision_graph.iterkeys():
+        if _rev_info[_rev_indices[revid]][0][2] == 0:
             continue
         for parent in _revision_graph[revid]:
-            _where_merged.setdefault(parent, set()).add(revid)
-
-    log.info('built revision graph cache: %r secs' % (time.time() - z))
-
-    return (_revision_graph, _full_history, _revision_info,
-            _revno_revid, _merge_sort, _where_merged)
+            c = _rev_info[_rev_indices[parent]]
+            if revid not in c[1]:
+                c[1] = c[1] + (revid,)
+
+    log.info('built revision graph cache: %.3f secs' % (time.time() - z,))
+
+    return (_rev_info, _rev_indices)

=== modified file 'loggerhead/zptsupport.py'
--- loggerhead/zptsupport.py	2008-10-24 02:26:05 +0000
+++ loggerhead/zptsupport.py	2010-05-12 15:01:44 +0000
@@ -18,14 +18,11 @@
 import logging
 import os
 import pkg_resources
+import re
 import StringIO
 
 from simpletal import simpleTAL, simpleTALES
 
-logging.getLogger("simpleTAL").setLevel(logging.INFO)
-logging.getLogger("simpleTALES").setLevel(logging.INFO)
-
-
 _zpt_cache = {}
 
 
@@ -33,8 +30,11 @@
     tinstance = _zpt_cache.get(tfile)
     stat = os.stat(tfile)
     if tinstance is None or tinstance.stat != stat:
+        text = open(tfile).read()
+        text = re.sub(r'\s*\n\s*', '\n', text)
+        text = re.sub(r'[ \t]+', ' ', text)
         tinstance = _zpt_cache[tfile] = TemplateWrapper(
-            simpleTAL.compileXMLTemplate(open(tfile)), tfile, stat)
+            simpleTAL.compileXMLTemplate(text), tfile, stat)
     return tinstance
 
 

=== modified file 'loggerheadd'
--- loggerheadd	2008-11-24 20:26:26 +0000
+++ loggerheadd	2009-04-30 10:39:05 +0000
@@ -16,9 +16,9 @@
 
 LHUSER=loggerhead
 
-if [ `whoami` = "$LHUSER" ]; then                                                                                   
+if [ `whoami` = "$LHUSER" ]; then
     SUDO=""
-else                                                                                                                
+else
     SUDO="sudo -H -u $LHUSER"
 fi
 

=== added file 'serve-branches'
--- serve-branches	1970-01-01 00:00:00 +0000
+++ serve-branches	2009-06-26 09:42:43 +0000
@@ -0,0 +1,27 @@
+#!/usr/bin/env python
+#
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Search for branches underneath a directory and serve them all."""
+
+import sys
+
+from loggerhead.main import main
+
+
+if __name__ == "__main__":
+    main(sys.argv[1:])

=== modified file 'setup.py' (properties changed: -x to +x)
--- setup.py	2008-11-20 16:55:47 +0000
+++ setup.py	2011-03-10 14:22:12 +0000
@@ -18,8 +18,8 @@
 
 """Loggerhead is a web viewer for projects in bazaar"""
 
-
 from distutils.core import setup
+
 import loggerhead
 
 
@@ -30,11 +30,13 @@
     license = "GNU GPL v2 or later",
     maintainer = "Michael Hudson",
     maintainer_email = "michael.hudson@canonical.com",
-    scripts = ["start-loggerhead", "stop-loggerhead", "serve-branches"],
+    scripts = ["serve-branches"],
     packages = ["loggerhead",
                 "loggerhead/apps",
                 "loggerhead/controllers",
-                "loggerhead/templates"],
+                "loggerhead/templates",
+                "bzrlib.plugins.loggerhead"],
+    package_dir={'bzrlib.plugins.loggerhead':'.'},
     package_data = {"loggerhead": ["templates/*.pt",
                                    "static/css/*.css",
                                    "static/javascript/*.js",
@@ -62,9 +64,6 @@
                                    "static/javascript/yui/build/yui-base/*",
                                    "static/images/*"]},
     data_files = [
-        ('share/man/man1', ['start-loggerhead.1',
-                            'stop-loggerhead.1',
-                            'serve-branches.1']),
-        ('share/doc/loggerhead', ['loggerhead.conf.example']),
+        ('share/man/man1', ['serve-branches.1']),
         ],
     )

=== removed file 'start-loggerhead'
--- start-loggerhead	2009-02-10 01:25:34 +0000
+++ start-loggerhead	1970-01-01 00:00:00 +0000
@@ -1,134 +0,0 @@
-#!/usr/bin/env python
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""A script for starting the loggerhead process."""
-
-
-import logging
-import logging.handlers
-from optparse import OptionParser
-import os
-import sys
-import urlparse
-
-from bzrlib.util.configobj.configobj import ConfigObj
-
-from paste import httpserver
-from paste.httpexceptions import make_middleware
-from paste.translogger import make_filter
-
-from loggerhead import daemon
-from loggerhead.apps.config import Root
-from loggerhead.trace import make_handler, setup_logging
-from loggerhead.apps.error import ErrorHandlerApp
-
-
-def main():
-    home = os.path.realpath(os.path.dirname(__file__))
-    default_pidfile = os.path.join(home, 'loggerhead.pid')
-    default_configfile = os.path.join(home, 'loggerhead.conf')
-    default_log_folder = os.path.join(home, 'logs')
-    parser = OptionParser(usage='usage: %prog [options]', version='%prog')
-    parser.add_option('-f', '--foreground',
-                      action='store_true', dest='foreground', default=False,
-                      help="run in the foreground; don't daemonize")
-    parser.add_option('-C', '--check', action='store_true',
-                      dest='check', default=False,
-                      help=("only start if not already running (useful for "
-                            "cron jobs)"))
-    parser.add_option('-p', '--pidfile', dest="pidfile",
-                      default=default_pidfile,
-                      type=str, help="override pidfile location")
-    parser.add_option('-c', '--config-file', dest="configfile",
-                      default=default_configfile,
-                      type=str, help="override configuration file location")
-    parser.add_option('-L', '--log-folder', dest="log_folder",
-                      default=default_log_folder,
-                      type=str, help="override log file directory")
-    options, args = parser.parse_args()
-    if len(args) > 0:
-        parser.error('No filename arguments are used, only options.')
-
-    if options.check:
-        if daemon.is_running(options.pidfile):
-            sys.exit(0)
-        sys.stderr.write('Did not find loggerhead running in %r;' % (
-                         options.pidfile))
-        sys.stderr.write(' restarting...\n')
-
-    # read loggerhead config
-
-    config = ConfigObj(options.configfile, encoding='utf-8')
-    extra_path = config.get('bzrpath', None)
-    if extra_path:
-        sys.path.insert(0, extra_path)
-
-    potential_overrides = [('server.socket_port', int),
-                           ('server.webpath', str),
-                           ('server.thread_pool', int),
-                           ('server.socket_host', str)]
-    server_port = int(config.get('server.socket_port', 8080))
-    nworkers = int(config.get('server.thread_pool', 10))
-    server_host = config.get('server.socket_host', '0.0.0.0')
-    webpath = config.get('server.webpath', None)
-
-    for key, keytype in potential_overrides:
-        value = config.get(key, None)
-        if value is not None:
-            value = keytype(value)
-
-    if not options.foreground:
-        sys.stderr.write('\n')
-        sys.stderr.write('Launching loggerhead into the background.\n')
-        sys.stderr.write('PID file: %s\n' % options.pidfile)
-        sys.stderr.write('\n')
-
-        daemon.daemonize(options.pidfile, home)
-
-    setup_logging(options.log_folder, config, foreground=options.foreground)
-
-    log = logging.getLogger('loggerhead')
-    log.info('Starting up...')
-
-    app = Root(config)
-
-    app = app
-    app = make_middleware(app)
-    app = make_filter(app, None, logger_name=log.name+'.access')
-    app = ErrorHandlerApp(app)
-
-    if webpath:
-        scheme, netloc, path, blah, blah, blah = urlparse.urlparse(webpath)
-
-        def app(environ, start_response, orig=app):
-            environ['SCRIPT_NAME'] = path
-            environ['HTTP_HOST'] = netloc
-            environ['wsgi.url_scheme'] = scheme
-            return orig(environ, start_response)
-
-    try:
-        httpserver.serve(
-            app, host=server_host, port=server_port,
-            threadpool_workers=nworkers)
-    finally:
-        log.info('Shutdown.')
-        try:
-            os.remove(options.pidfile)
-        except OSError:
-            pass
-
-
-if __name__ == '__main__':
-    main()

=== removed file 'start-loggerhead.1'
--- start-loggerhead.1	2008-08-06 18:33:20 +0000
+++ start-loggerhead.1	1970-01-01 00:00:00 +0000
@@ -1,38 +0,0 @@
-.TH START-LOGGERHEAD "1" "July 2008" "start-loggerhead 1.2.1" "User Commands"
-.SH NAME
-start-loggerhead \- start a loggerhead server
-.SH SYNOPSIS
-.B start-loggerhead
-[\fIoptions\fR]
-.SH DESCRIPTION
-Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
-a branch history, annotate files, view patches, perform searches, etc. It's 
-heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
-.PP
-The start-loggerhead command starts a new standalone loggerhead server.
-.SH OPTIONS
-.TP
-\fB\-\-version\fR
-show program's version number and exit
-.TP
-\fB\-h\fR, \fB\-\-help\fR
-show this help message and exit
-.TP
-\fB\-f\fR, \fB\-\-foreground\fR
-run in the foreground; don't daemonize
-.TP
-\fB\-C\fR, \fB\-\-check\fR
-only start if not already running (useful for cron jobs)
-.TP
-\fB\-p\fR, \fB\-\-pidfile=PIDFILE\fR
-override pid file location
-.TP
-\fB\-c\fR, \fB\-\-config-file=CONFIGFILE\fR
-override configuration file location
-.TP
-\fB\-L\fR, \fB\-\-log-folder=LOG-FOLDER\fR
-override log directory
-.SH "LICENSE"
-GNU GPLv2 or later.
-.SH "SEE ALSO"
-https://launchpad.net/loggerhead

=== removed file 'stop-loggerhead'
--- stop-loggerhead	2008-10-24 02:26:05 +0000
+++ stop-loggerhead	1970-01-01 00:00:00 +0000
@@ -1,51 +0,0 @@
-#!/usr/bin/env python
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""A script for stopping the loggerhead process."""
-
-import os
-import sys
-from optparse import OptionParser
-
-home = os.path.realpath(os.path.dirname(__file__))
-default_pidfile = os.path.join(home, 'loggerhead.pid')
-parser = OptionParser(usage='usage: %prog [options]', version='%prog')
-parser.add_option('-p', '--pidfile', dest="pidfile", default=default_pidfile,
-                  help="override pidfile location")
-
-options, args = parser.parse_args()
-if len(args) > 0:
-    parser.error('No filename arguments are used, only options.')
-
-try:
-    f = open(options.pidfile, 'r')
-except IOError, e:
-    print 'Pid file %s not found.' % options.pidfile
-    sys.exit(1)
-
-pid = int(f.readline())
-
-try:
-    os.kill(pid, 0)
-except OSError, e:
-    print 'Stale pid file; server is not running.'
-    sys.exit(1)
-
-print
-print 'Shutting down previous server @ pid %d.' % pid
-print
-
-import signal
-os.kill(pid, signal.SIGINT)

=== removed file 'stop-loggerhead.1'
--- stop-loggerhead.1	2008-08-06 18:39:15 +0000
+++ stop-loggerhead.1	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-.TH STOP-LOGGERHEAD "1" "July 2008" "stop-loggerhead 1.2.1" "User Commands"
-.SH NAME
-stop-loggerhead \- stop a loggerhead server
-.SH SYNOPSIS
-.B stop-loggerhead
-.SH DESCRIPTION
-Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
-a branch history, annotate files, view patches, perform searches, etc. It's 
-heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
-.SH OPTIONS
-.TP
-\fB\-p\fR, \fB\-\-pidfile=PIDFILE\fR
-override pid file location
-.PP
-The stop-loggerhead command stops the currently running loggerhead server.
-.SH "LICENSE"
-GNU GPLv2 or later.
-.SH "SEE ALSO"
-https://launchpad.net/loggerhead

